{-# OPTIONS_GHC -O0 #-}
module Network.BSD.ServiceDB(
         ProtocolName
       , ServiceName
       , ServiceEntry(..)
       , serviceDB
       )
 where

import Data.Typeable
import Data.Word
import Network.Socket.Internal(PortNumber(..))

type ProtocolName = String
type ServiceName = String

data ServiceEntry = ServiceEntry {
    serviceName     :: ServiceName
  , serviceAliases  :: [ServiceName]
  , servicePort     :: PortNumber
  , serviceProtocol :: ProtocolName
  }
 deriving (Show, Typeable)

makeService :: String -> Word16 -> String -> [String] -> ServiceEntry
makeService n po pr al = ServiceEntry n al (PortNum po) pr

serviceDB :: [ServiceEntry]
serviceDB = [
    makeService "tcpmux"         1    "tcp" []
  , makeService "tcpmux"         1    "udp" []
  , makeService "rje"            5    "tcp" []
  , makeService "rje"            5    "udp" []
  , makeService "echo"           7    "tcp" []
  , makeService "echo"           7    "udp" []
  , makeService "discard"        9    "tcp"  ["sink","null"]
  , makeService "discard"        9    "udp"  ["sink","null"]
  , makeService "systat"         11    "tcp" ["users"]
  , makeService "systat"         11    "udp" ["users"]
  , makeService "daytime"        13    "tcp" []
  , makeService "daytime"        13    "udp" []
  , makeService "qotd"           17    "tcp" ["quote"]
  , makeService "qotd"           17    "udp" ["quote"]
  , makeService "msp"            18    "tcp" []
  , makeService "msp"            18    "udp" []
  , makeService "chargen"        19    "tcp" ["ttytst","source"]
  , makeService "chargen"        19    "udp" ["ttytst","source"]
  , makeService "ftp-data"       20    "tcp" []
  , makeService "ftp-data"       20    "udp" []
  , makeService "ftp"            21    "tcp" []
  , makeService "ftp"            21    "udp"  ["fsp","fspd"]
  , makeService "ssh"            22    "tcp" []
  , makeService "ssh"            22    "udp" []
  , makeService "telnet"         23    "tcp" []
  , makeService "telnet"         23    "udp" []
  , makeService "lmtp"           24    "tcp" []
  , makeService "lmtp"           24    "udp" []
  , makeService "smtp"           25    "tcp" ["mail"]
  , makeService "smtp"           25    "udp" ["mail"]
  , makeService "time"           37    "tcp" ["timserver"]
  , makeService "time"           37    "udp" ["timserver"]
  , makeService "rlp"            39    "tcp" ["resource"]
  , makeService "rlp"            39    "udp" ["resource"]
  , makeService "nameserver"     42    "tcp" ["name"]
  , makeService "nameserver"     42    "udp" ["name"]
  , makeService "nicname"        43    "tcp" ["whois"]
  , makeService "nicname"        43    "udp" ["whois"]
  , makeService "tacacs"         49    "tcp" []
  , makeService "tacacs"         49    "udp" []
  , makeService "re-mail-ck"     50    "tcp" []
  , makeService "re-mail-ck"     50    "udp" []
  , makeService "domain"         53    "tcp" []
  , makeService "domain"         53    "udp" []
  , makeService "whois++"        63    "tcp" ["whoispp"]
  , makeService "whois++"        63    "udp" ["whoispp"]
  , makeService "bootps"         67    "tcp" []
  , makeService "bootps"         67    "udp" []
  , makeService "bootpc"         68    "tcp" ["dhcpc"]
  , makeService "bootpc"         68    "udp" ["dhcpc"]
  , makeService "tftp"           69    "tcp" []
  , makeService "tftp"           69    "udp" []
  , makeService "gopher"         70    "tcp" []
  , makeService "gopher"         70    "udp" []
  , makeService "netrjs-1"       71    "tcp" []
  , makeService "netrjs-1"       71    "udp" []
  , makeService "netrjs-2"       72    "tcp" []
  , makeService "netrjs-2"       72    "udp" []
  , makeService "netrjs-3"       73    "tcp" []
  , makeService "netrjs-3"       73    "udp" []
  , makeService "netrjs-4"       74    "tcp" []
  , makeService "netrjs-4"       74    "udp" []
  , makeService "finger"         79    "tcp" []
  , makeService "finger"         79    "udp" []
  , makeService "http"           80    "tcp" ["www","www-http"]
  , makeService "http"           80    "udp" ["www","www-http"]
  , makeService "kerberos"       88    "tcp" ["kerberos5","krb5"]
  , makeService "kerberos"       88    "udp" ["kerberos5","krb5"]
  , makeService "supdup"         95    "tcp" []
  , makeService "supdup"         95    "udp" []
  , makeService "hostname"       101    "tcp" ["hostnames"]
  , makeService "hostname"       101    "udp" ["hostnames"]
  , makeService "iso-tsap"       102    "tcp" ["tsap"]
  , makeService "csnet-ns"       105    "tcp" ["cso"]
  , makeService "csnet-ns"       105    "udp" ["cso"]
  , makeService "rtelnet"        107    "tcp" []
  , makeService "rtelnet"        107    "udp" []
  , makeService "pop2"           109    "tcp" ["pop-2","postoffice"]
  , makeService "pop2"           109    "udp" ["pop-2"]
  , makeService "pop3"           110    "tcp" ["pop-3"]
  , makeService "pop3"           110    "udp" ["pop-3"]
  , makeService "sunrpc"         111    "tcp" ["portmapper","rpcbind"]
  , makeService "sunrpc"         111    "udp" ["portmapper","rpcbind"]
  , makeService "auth"           113    "tcp" ["authentication","tap","ident"]
  , makeService "auth"           113    "udp" ["authentication","tap","ident"]
  , makeService "sftp"           115    "tcp" []
  , makeService "sftp"           115    "udp" []
  , makeService "uucp-path"      117    "tcp" []
  , makeService "uucp-path"      117    "udp" []
  , makeService "nntp"           119    "tcp" ["readnews","untp"]
  , makeService "nntp"           119    "udp" ["readnews","untp"]
  , makeService "ntp"            123    "tcp" []
  , makeService "ntp"            123    "udp" []
  , makeService "netbios-ns"     137    "tcp" []
  , makeService "netbios-ns"     137    "udp" []
  , makeService "netbios-dgm"    138    "tcp" []
  , makeService "netbios-dgm"    138    "udp" []
  , makeService "netbios-ssn"    139    "tcp" []
  , makeService "netbios-ssn"    139    "udp" []
  , makeService "imap"           143    "tcp" ["imap2"]
  , makeService "imap"           143    "udp" ["imap2"]
  , makeService "snmp"           161    "tcp" []
  , makeService "snmp"           161    "udp" []
  , makeService "snmptrap"       162    "tcp" []
  , makeService "snmptrap"       162    "udp" ["snmp-trap"]
  , makeService "cmip-man"       163    "tcp" []
  , makeService "cmip-man"       163    "udp" []
  , makeService "cmip-agent"     164    "tcp" []
  , makeService "cmip-agent"     164    "udp" []
  , makeService "mailq"          174    "tcp" []
  , makeService "mailq"          174    "udp" []
  , makeService "xdmcp"          177    "tcp" []
  , makeService "xdmcp"          177    "udp" []
  , makeService "nextstep"       178    "tcp" ["NeXTStep","NextStep"]
  , makeService "nextstep"       178    "udp" ["NeXTStep","NextStep"]
  , makeService "bgp"            179    "tcp" []
  , makeService "bgp"            179    "udp" []
  , makeService "prospero"       191    "tcp" []
  , makeService "prospero"       191    "udp" []
  , makeService "irc"            194    "tcp" []
  , makeService "irc"            194    "udp" []
  , makeService "smux"           199    "tcp" []
  , makeService "smux"           199    "udp" []
  , makeService "at-rtmp"        201    "tcp" []
  , makeService "at-rtmp"        201    "udp" []
  , makeService "at-nbp"         202    "tcp" []
  , makeService "at-nbp"         202    "udp" []
  , makeService "at-echo"        204    "tcp" []
  , makeService "at-echo"        204    "udp" []
  , makeService "at-zis"         206    "tcp" []
  , makeService "at-zis"         206    "udp" []
  , makeService "qmtp"           209    "tcp" []
  , makeService "qmtp"           209    "udp" []
  , makeService "z39.50"         210    "tcp" ["z3950","z39-50","wais"]
  , makeService "z39.50"         210    "udp" ["z3950","z39-50","wais"]
  , makeService "ipx"            213    "tcp" []
  , makeService "ipx"            213    "udp" []
  , makeService "imap3"          220    "tcp" []
  , makeService "imap3"          220    "udp" []
  , makeService "link"           245    "tcp" ["ttylink"]
  , makeService "link"           245    "udp" ["ttylink"]
  , makeService "gist"           270    "udp" []
  , makeService "fatserv"        347    "tcp" []
  , makeService "fatserv"        347    "udp" []
  , makeService "rsvp_tunnel"    363    "tcp" ["rsvp-tunnel"]
  , makeService "rsvp_tunnel"    363    "udp" ["rsvp-tunnel"]
  , makeService "odmr"           366    "tcp" []
  , makeService "odmr"           366    "udp" []
  , makeService "rpc2portmap"    369    "tcp" []
  , makeService "rpc2portmap"    369    "udp" []
  , makeService "codaauth2"      370    "tcp" []
  , makeService "codaauth2"      370    "udp" []
  , makeService "ulistproc"      372    "tcp" ["ulistserv"]
  , makeService "ulistproc"      372    "udp" ["ulistserv"]
  , makeService "ldap"           389    "tcp" []
  , makeService "ldap"           389    "udp" []
  , makeService "osb-sd"         400    "tcp" []
  , makeService "osb-sd"         400    "udp" []
  , makeService "svrloc"         427    "tcp" []
  , makeService "svrloc"         427    "udp" []
  , makeService "mobileip-agent" 434    "tcp" []
  , makeService "mobileip-agent" 434    "udp" []
  , makeService "mobilip-mn"     435    "tcp" []
  , makeService "mobilip-mn"     435    "udp" []
  , makeService "https"          443    "tcp" []
  , makeService "https"          443    "udp" []
  , makeService "snpp"           444    "tcp" []
  , makeService "snpp"           444    "udp" []
  , makeService "microsoft-ds"   445    "tcp" []
  , makeService "microsoft-ds"   445    "udp" []
  , makeService "kpasswd"        464    "tcp" ["kpwd"]
  , makeService "kpasswd"        464    "udp" ["kpwd"]
  , makeService "photuris"       468    "tcp" []
  , makeService "photuris"       468    "udp" []
  , makeService "saft"           487    "tcp" []
  , makeService "saft"           487    "udp" []
  , makeService "gss-http"       488    "tcp" []
  , makeService "gss-http"       488    "udp" []
  , makeService "pim-rp-disc"    496    "tcp" []
  , makeService "pim-rp-disc"    496    "udp" []
  , makeService "isakmp"         500    "tcp" []
  , makeService "isakmp"         500    "udp" []
  , makeService "gdomap"         538    "tcp" []
  , makeService "gdomap"         538    "udp" []
  , makeService "iiop"           535    "tcp" []
  , makeService "iiop"           535    "udp" []
  , makeService "dhcpv6-client"  546    "tcp" []
  , makeService "dhcpv6-client"  546    "udp" []
  , makeService "dhcpv6-server"  547    "tcp" []
  , makeService "dhcpv6-server"  547    "udp" []
  , makeService "rtsp"           554    "tcp" []
  , makeService "rtsp"           554    "udp" []
  , makeService "nntps"          563    "tcp" []
  , makeService "nntps"          563    "udp" []
  , makeService "whoami"         565    "tcp" []
  , makeService "whoami"         565    "udp" []
  , makeService "submission"     587    "tcp" ["msa"]
  , makeService "submission"     587    "udp" ["msa"]
  , makeService "npmp-local"     610    "tcp" ["dqs313_qmaster"]
  , makeService "npmp-local"     610    "udp" ["dqs313_qmaster"]
  , makeService "npmp-gui"       611    "tcp" ["dqs313_execd"]
  , makeService "npmp-gui"       611    "udp" ["dqs313_execd"]
  , makeService "hmmp-ind"       612    "tcp" ["dqs313_intercell"]
  , makeService "hmmp-ind"       612    "udp" ["dqs313_intercell"]
  , makeService "ipp"            631    "tcp" []
  , makeService "ipp"            631    "udp" []
  , makeService "ldaps"          636    "tcp" []
  , makeService "ldaps"          636    "udp" []
  , makeService "acap"           674    "tcp" []
  , makeService "acap"           674    "udp" []
  , makeService "ha-cluster"     694    "tcp" []
  , makeService "ha-cluster"     694    "udp" []
  , makeService "kerberos-adm"   749    "tcp" []
  , makeService "kerberos-adm"   749    "udp" []
  , makeService "kerberos-iv"    750    "udp" ["kerberos4","kerberos-sec","kdc","loadav"]
  , makeService "kerberos-iv"    750    "tcp" ["kerberos4","kerberos-sec","kdc","rfile"]
  , makeService "webster"        765    "tcp" []
  , makeService "webster"        765    "udp" []
  , makeService "phonebook"      767    "tcp" []
  , makeService "phonebook"      767    "udp" []
  , makeService "rsync"          873    "tcp" []
  , makeService "rsync"          873    "udp" []
  , makeService "rquotad"        875    "tcp" []
  , makeService "rquotad"        875    "udp" []
  , makeService "telnets"        992    "tcp" []
  , makeService "telnets"        992    "udp" []
  , makeService "imaps"          993    "tcp" []
  , makeService "imaps"          993    "udp" []
  , makeService "pop3s"          995    "tcp" []
  , makeService "pop3s"          995    "udp" []
  , makeService "exec"           512    "tcp" []
  , makeService "biff"           512    "udp" ["comsat"]
  , makeService "login"          513    "tcp" []
  , makeService "who"            513    "udp" ["whod"]
  , makeService "shell"          514    "tcp" ["cmd"]
  , makeService "syslog"         514    "udp" []
  , makeService "printer"        515    "tcp" ["spooler"]
  , makeService "printer"        515    "udp" ["spooler"]
  , makeService "talk"           517    "udp" []
  , makeService "ntalk"          518    "udp" []
  , makeService "utime"          519    "tcp" ["unixtime"]
  , makeService "utime"          519    "udp" ["unixtime"]
  , makeService "efs"            520    "tcp" []
  , makeService "router"         520    "udp" ["route","routed"]
  , makeService "ripng"          521    "tcp" []
  , makeService "ripng"          521    "udp" []
  , makeService "timed"          525    "tcp" ["timeserver"]
  , makeService "timed"          525    "udp" ["timeserver"]
  , makeService "tempo"          526    "tcp" ["newdate"]
  , makeService "courier"        530    "tcp" ["rpc"]
  , makeService "conference"     531    "tcp" ["chat"]
  , makeService "netnews"        532    "tcp" []
  , makeService "netwall"        533    "udp" []
  , makeService "uucp"           540    "tcp" ["uucpd"]
  , makeService "klogin"         543    "tcp" []
  , makeService "kshell"         544    "tcp" ["krcmd"]
  , makeService "afpovertcp"     548    "tcp" []
  , makeService "afpovertcp"     548    "udp" []
  , makeService "remotefs"       556    "tcp" ["rfs_server","rfs"]
  , makeService "socks"          1080    "tcp" []
  , makeService "socks"          1080    "udp" []
  , makeService "bvcontrol"      1236    "tcp" ["rmtcfg"]
  , makeService "bvcontrol"      1236    "udp" []
  , makeService "h323hostcallsc" 1300    "tcp" []
  , makeService "h323hostcallsc" 1300    "udp" []
  , makeService "ms-sql-s"       1433    "tcp" []
  , makeService "ms-sql-s"       1433    "udp" []
  , makeService "ms-sql-m"       1434    "tcp" []
  , makeService "ms-sql-m"       1434    "udp" []
  , makeService "ica"            1494    "tcp" []
  , makeService "ica"            1494    "udp" []
  , makeService "wins"           1512    "tcp" []
  , makeService "wins"           1512    "udp" []
  , makeService "ingreslock"     1524    "tcp" []
  , makeService "ingreslock"     1524    "udp" []
  , makeService "prospero-np"    1525    "tcp" ["orasrv"]
  , makeService "prospero-np"    1525    "udp" ["orasrv"]
  , makeService "datametrics"    1645    "tcp" ["old-radius","sightline"]
  , makeService "datametrics"    1645    "udp" ["old-radius","sightline"]
  , makeService "sa-msg-port"    1646    "tcp" ["old-radacct"]
  , makeService "sa-msg-port"    1646    "udp" ["old-radacct"]
  , makeService "kermit"         1649    "tcp" []
  , makeService "kermit"         1649    "udp" []
  , makeService "l2tp"           1701    "tcp" ["l2f"]
  , makeService "l2tp"           1701    "udp" ["l2f"]
  , makeService "h323gatedisc"   1718    "tcp" []
  , makeService "h323gatedisc"   1718    "udp" []
  , makeService "h323gatestat"   1719    "tcp" []
  , makeService "h323gatestat"   1719    "udp" []
  , makeService "h323hostcall"   1720    "tcp" []
  , makeService "h323hostcall"   1720    "udp" []
  , makeService "tftp-mcast"     1758    "tcp" []
  , makeService "tftp-mcast"     1758    "udp" []
  , makeService "mtftp"          1759    "udp" ["spss-lm"]
  , makeService "hello"          1789    "tcp" []
  , makeService "hello"          1789    "udp" []
  , makeService "radius"         1812    "tcp" []
  , makeService "radius"         1812    "udp" []
  , makeService "radius-acct"    1813    "tcp" ["radacct"]
  , makeService "radius-acct"    1813    "udp" ["radacct"]
  , makeService "mtp"            1911    "tcp" []
  , makeService "mtp"            1911    "udp" []
  , makeService "hsrp"           1985    "tcp" []
  , makeService "hsrp"           1985    "udp" []
  , makeService "licensedaemon"  1986    "tcp" []
  , makeService "licensedaemon"  1986    "udp" []
  , makeService "gdp-port"       1997    "tcp" []
  , makeService "gdp-port"       1997    "udp" []
  , makeService "sieve-filter"   2000    "tcp" ["cisco-sccp"]
  , makeService "sieve-filter"   2000    "udp" ["cisco-sccp"]
  , makeService "nfs"            2049    "tcp" ["nfsd","shilp"]
  , makeService "nfs"            2049    "udp" ["nfsd","shilp"]
  , makeService "zephyr-srv"     2102    "tcp" []
  , makeService "zephyr-srv"     2102    "udp" []
  , makeService "zephyr-clt"     2103    "tcp" []
  , makeService "zephyr-clt"     2103    "udp" []
  , makeService "zephyr-hm"      2104    "tcp" []
  , makeService "zephyr-hm"      2104    "udp" []
  , makeService "cvspserver"     2401    "tcp" []
  , makeService "cvspserver"     2401    "udp" []
  , makeService "venus"          2430    "tcp" []
  , makeService "venus"          2430    "udp" []
  , makeService "venus-se"       2431    "tcp" []
  , makeService "venus-se"       2431    "udp" []
  , makeService "codasrv"        2432    "tcp" []
  , makeService "codasrv"        2432    "udp" []
  , makeService "codasrv-se"     2433    "tcp" []
  , makeService "codasrv-se"     2433    "udp" []
  , makeService "hpstgmgr"       2600    "tcp" ["zebrasrv"]
  , makeService "hpstgmgr"       2600    "udp" []
  , makeService "discp-client"   2601    "tcp" ["zebra"]
  , makeService "discp-client"   2601    "udp" []
  , makeService "discp-server"   2602    "tcp" ["ripd"]
  , makeService "discp-server"   2602    "udp" []
  , makeService "servicemeter"   2603    "tcp" ["ripngd"]
  , makeService "servicemeter"   2603    "udp" []
  , makeService "nsc-ccs"        2604    "tcp" ["ospfd"]
  , makeService "nsc-ccs"        2604    "udp" []
  , makeService "nsc-posa"       2605    "tcp" ["bgpd"]
  , makeService "nsc-posa"       2605    "udp" []
  , makeService "netmon"         2606    "tcp" ["ospf6d"]
  , makeService "netmon"         2606    "udp" []
  , makeService "dict"           2628    "tcp" []
  , makeService "dict"           2628    "udp" []
  , makeService "corbaloc"       2809    "tcp" []
  , makeService "icpv2"          3130    "tcp" []
  , makeService "icpv2"          3130    "udp" []
  , makeService "mysql"          3306    "tcp" []
  , makeService "mysql"          3306    "udp" []
  , makeService "trnsprntproxy"  3346    "tcp" []
  , makeService "trnsprntproxy"  3346    "udp" []
  , makeService "pxe"            4011    "udp" ["altserviceboot"]
  , makeService "fud"            4201    "udp" []
  , makeService "rwhois"         4321    "tcp" []
  , makeService "rwhois"         4321    "udp" []
  , makeService "krb524"         4444    "tcp" ["nv-video"]
  , makeService "krb524"         4444    "udp" ["nv-video"]
  , makeService "rfe"            5002    "tcp" []
  , makeService "rfe"            5002    "udp" []
  , makeService "cfengine"       5308    "tcp" []
  , makeService "cfengine"       5308    "udp" []
  , makeService "cvsup"          5999    "tcp" ["CVSup"]
  , makeService "cvsup"          5999    "udp" ["CVSup"]
  , makeService "x11"            6000    "tcp" ["X"]
  , makeService "afs3-fileserver"7000    "tcp" []
  , makeService "afs3-fileserver"7000    "udp" []
  , makeService "afs3-callback"  7001    "tcp" []
  , makeService "afs3-callback"  7001    "udp" []
  , makeService "afs3-prserver"  7002    "tcp" []
  , makeService "afs3-prserver"  7002    "udp" []
  , makeService "afs3-vlserver"  7003    "tcp" []
  , makeService "afs3-vlserver"  7003    "udp" []
  , makeService "afs3-kaserver"  7004    "tcp" []
  , makeService "afs3-kaserver"  7004    "udp" []
  , makeService "afs3-volser"    7005    "tcp" []
  , makeService "afs3-volser"    7005    "udp" []
  , makeService "afs3-errors"    7006    "tcp" []
  , makeService "afs3-errors"    7006    "udp" []
  , makeService "afs3-bos"       7007    "tcp" []
  , makeService "afs3-bos"       7007    "udp" []
  , makeService "afs3-update"    7008    "tcp" []
  , makeService "afs3-update"    7008    "udp" []
  , makeService "afs3-rmtsys"    7009    "tcp" []
  , makeService "afs3-rmtsys"    7009    "udp" []
  , makeService "amanda"         10080    "tcp" []
  , makeService "amanda"         10080    "udp" []
  , makeService "pgpkeyserver"   11371    "tcp" ["hkp"]
  , makeService "pgpkeyserver"   11371    "udp" ["hkp"]
  , makeService "asgcypresstcps" 11489    "tcp" []
  , makeService "h323callsigalt" 11720    "tcp" []
  , makeService "h323callsigalt" 11720    "udp" []
  , makeService "bprd"           13720    "tcp" []
  , makeService "bprd"           13720    "udp" []
  , makeService "bpdbm"          13721    "tcp" []
  , makeService "bpdbm"          13721    "udp" []
  , makeService "bpjava-msvc"    13722    "tcp" []
  , makeService "bpjava-msvc"    13722    "udp" []
  , makeService "vnetd"          13724    "tcp" []
  , makeService "vnetd"          13724    "udp" []
  , makeService "bpcd"           13782    "tcp" []
  , makeService "bpcd"           13782    "udp" []
  , makeService "vopied"         13783    "tcp" []
  , makeService "vopied"         13783    "udp" []
  , makeService "wnn6"           22273    "tcp" ["wnn4"]
  , makeService "wnn6"           22273    "udp" ["wnn4"]
  , makeService "quake"          26000    "tcp" []
  , makeService "quake"          26000    "udp" []
  , makeService "wnn6-ds"        26208    "tcp" []
  , makeService "wnn6-ds"        26208    "udp" []
  , makeService "traceroute"     33434    "tcp" []
  , makeService "traceroute"     33434    "udp" []
  , makeService "kerberos_master"751    "udp" ["pump"]
  , makeService "kerberos_master"751    "tcp" ["pump"]
  , makeService "passwd_server"  752    "udp" ["qrh"]
  , makeService "krbupdate"      760    "tcp" ["kreg","ns"]
  , makeService "kpop"           1109    "tcp" []
  , makeService "knetd"          2053    "tcp" ["lot105-ds-upd"]
  , makeService "krb5_prop"      754    "tcp" ["tell"]
  , makeService "eklogin"        2105    "tcp" ["minipay"]
  , makeService "supfilesrv"     871    "tcp" []
  , makeService "supfiledbg"     1127    "tcp" ["kwdb-commn"]
  , makeService "netstat"        15    "tcp" []
  , makeService "poppassd"       106    "tcp" []
  , makeService "poppassd"       106    "udp" []
  , makeService "omirr"          808    "tcp" ["omirrd"]
  , makeService "omirr"          808    "udp" ["omirrd"]
  , makeService "swat"           901    "tcp" ["smpnameres"]
  , makeService "rndc"           953    "tcp" []
  , makeService "rndc"           953    "udp" []
  , makeService "skkserv"        1178    "tcp" ["sgi-storman"]
  , makeService "xtel"           1313    "tcp" ["bmc_patroldb","bmc-patroldb"]
  , makeService "support"        1529    "tcp" ["prmsd","gnatsd","coauthor"]
  , makeService "cfinger"        2003    "tcp" ["brutus"]
  , makeService "ninstall"       2150    "tcp" ["dynamic3d"]
  , makeService "ninstall"       2150    "udp" ["dynamic3d"]
  , makeService "afbackup"       2988    "tcp" ["hippad"]
  , makeService "afbackup"       2988    "udp" ["hippad"]
  , makeService "squid"          3128    "tcp" ["ndl-aas"]
  , makeService "prsvp"          3455    "tcp" []
  , makeService "prsvp"          3455    "udp" []
  , makeService "distcc"         3632    "tcp" []
  , makeService "svn"            3690    "tcp" []
  , makeService "svn"            3690    "udp" []
  , makeService "postgres"       5432    "tcp" ["postgresql"]
  , makeService "postgres"       5432    "udp" ["postgresql"]
  , makeService "fax"            4557    "tcp" []
  , makeService "hylafax"        4559    "tcp" []
  , makeService "sgi-dgl"        5232    "tcp" ["csedaemon"]
  , makeService "sgi-dgl"        5232    "udp" []
  , makeService "hostmon"        5355    "tcp" ["llmnr"]
  , makeService "hostmon"        5355    "udp" ["llmnr"]
  , makeService "canna"          5680    "tcp" ["auriga-router"]
  , makeService "x11-ssh-offset" 6010    "tcp" []
  , makeService "xfs"            7100    "tcp" ["font-service"]
  , makeService "tircproxy"      7666    "tcp" []
  , makeService "webcache"       8080    "tcp" ["http-alt"]
  , makeService "webcache"       8080    "udp" ["http-alt"]
  , makeService "tproxy"         8081    "tcp" ["sunproxyadmin"]
  , makeService "tproxy"         8081    "udp" ["sunproxyadmin"]
  , makeService "jetdirect"      9100    "tcp" ["laserjet","hplj","hp-pdl-datastr","pdl-datastream"]
  , makeService "mandelspawn"    9359    "udp" ["mandelbrot"]
  , makeService "kamanda"        10081    "tcp" ["famdc"]
  , makeService "kamanda"        10081    "udp" ["famdc"]
  , makeService "amandaidx"      10082    "tcp" []
  , makeService "amidxtape"      10083    "tcp" []
  , makeService "isdnlog"        20011    "tcp" []
  , makeService "isdnlog"        20011    "udp" []
  , makeService "wnn4_Kr"        22305    "tcp" ["cis"]
  , makeService "wnn4_Cn"        22289    "tcp" []
  , makeService "wnn4_Tw"        22321    "tcp" []
  , makeService "binkp"          24554    "tcp" []
  , makeService "binkp"          24554    "udp" []
  , makeService "canditv"        24676    "tcp" []
  , makeService "canditv"        24676    "udp" []
  , makeService "asp"            27374    "tcp" []
  , makeService "asp"            27374    "udp" []
  , makeService "tfido"          60177    "tcp" []
  , makeService "tfido"          60177    "udp" []
  , makeService "fido"           60179    "tcp" []
  , makeService "fido"           60179    "udp" []
  , makeService "compressnet"    2    "tcp" []
  , makeService "compressnet"    2    "udp" []
  , makeService "nsw-fe"         27    "tcp" []
  , makeService "nsw-fe"         27    "udp" []
  , makeService "msg-icp"        29    "tcp" []
  , makeService "msg-icp"        29    "udp" []
  , makeService "msg-auth"       31    "tcp" []
  , makeService "msg-auth"       31    "udp" []
  , makeService "dsp"            33    "tcp" []
  , makeService "dsp"            33    "udp" []
  , makeService "rap"            38    "tcp" []
  , makeService "rap"            38    "udp" []
  , makeService "graphics"       41    "tcp" []
  , makeService "graphics"       41    "udp" []
  , makeService "mpm-flags"      44    "tcp" []
  , makeService "mpm-flags"      44    "udp" []
  , makeService "mpm"            45    "tcp" []
  , makeService "mpm"            45    "udp" []
  , makeService "mpm-snd"        46    "tcp" []
  , makeService "mpm-snd"        46    "udp" []
  , makeService "ni-ftp"         47    "tcp" []
  , makeService "ni-ftp"         47    "udp" []
  , makeService "auditd"         48    "tcp" []
  , makeService "auditd"         48    "udp" []
  , makeService "la-maint"       51    "tcp" []
  , makeService "la-maint"       51    "udp" []
  , makeService "xns-time"       52    "tcp" []
  , makeService "xns-time"       52    "udp" []
  , makeService "xns-ch"         54    "tcp" []
  , makeService "xns-ch"         54    "udp" []
  , makeService "isi-gl"         55    "tcp" []
  , makeService "isi-gl"         55    "udp" []
  , makeService "xns-auth"       56    "tcp" []
  , makeService "xns-auth"       56    "udp" []
  , makeService "xns-mail"       58    "tcp" []
  , makeService "xns-mail"       58    "udp" []
  , makeService "ni-mail"        61    "tcp" []
  , makeService "ni-mail"        61    "udp" []
  , makeService "acas"           62    "tcp" []
  , makeService "acas"           62    "udp" []
  , makeService "covia"          64    "tcp" []
  , makeService "covia"          64    "udp" []
  , makeService "tacacs-ds"      65    "tcp" []
  , makeService "tacacs-ds"      65    "udp" []
  , makeService "sql*net"        66    "tcp" []
  , makeService "sql*net"        66    "udp" []
  , makeService "deos"           76    "tcp" []
  , makeService "deos"           76    "udp" []
  , makeService "vettcp"         78    "tcp" []
  , makeService "vettcp"         78    "udp" []
  , makeService "xfer"           82    "tcp" []
  , makeService "xfer"           82    "udp" []
  , makeService "mit-ml-dev"     83    "tcp" []
  , makeService "mit-ml-dev"     83    "udp" []
  , makeService "ctf"            84    "tcp" []
  , makeService "ctf"            84    "udp" []
  , makeService "mfcobol"        86    "tcp" []
  , makeService "mfcobol"        86    "udp" []
  , makeService "su-mit-tg"      89    "tcp" []
  , makeService "su-mit-tg"      89    "udp" []
  , makeService "dnsix"          90    "tcp" []
  , makeService "dnsix"          90    "udp" []
  , makeService "mit-dov"        91    "tcp" []
  , makeService "mit-dov"        91    "udp" []
  , makeService "npp"            92    "tcp" []
  , makeService "npp"            92    "udp" []
  , makeService "dcp"            93    "tcp" []
  , makeService "dcp"            93    "udp" []
  , makeService "objcall"        94    "tcp" []
  , makeService "objcall"        94    "udp" []
  , makeService "dixie"          96    "tcp" []
  , makeService "dixie"          96    "udp" []
  , makeService "swift-rvf"      97    "tcp" []
  , makeService "swift-rvf"      97    "udp" []
  , makeService "tacnews"        98    "tcp" []
  , makeService "tacnews"        98    "udp" []
  , makeService "metagram"       99    "tcp" []
  , makeService "metagram"       99    "udp" []
  , makeService "newacct"        100    "tcp" []
  , makeService "iso-tsap"       102    "udp" []
  , makeService "gppitnp"        103    "tcp" []
  , makeService "gppitnp"        103    "udp" []
  , makeService "acr-nema"       104    "tcp" []
  , makeService "acr-nema"       104    "udp" []
  , makeService "snagas"         108    "tcp" []
  , makeService "snagas"         108    "udp" []
  , makeService "mcidas"         112    "tcp" []
  , makeService "mcidas"         112    "udp" []
  , makeService "ansanotify"     116    "tcp" []
  , makeService "ansanotify"     116    "udp" []
  , makeService "sqlserv"        118    "tcp" []
  , makeService "sqlserv"        118    "udp" []
  , makeService "cfdptkt"        120    "tcp" []
  , makeService "cfdptkt"        120    "udp" []
  , makeService "erpc"           121    "tcp" []
  , makeService "erpc"           121    "udp" []
  , makeService "smakynet"       122    "tcp" []
  , makeService "smakynet"       122    "udp" []
  , makeService "ansatrader"     124    "tcp" []
  , makeService "ansatrader"     124    "udp" []
  , makeService "locus-map"      125    "tcp" []
  , makeService "locus-map"      125    "udp" []
  , makeService "nxedit"         126    "tcp" []
  , makeService "nxedit"         126    "udp" []
  , makeService "locus-con"      127    "tcp" []
  , makeService "locus-con"      127    "udp" []
  , makeService "gss-xlicen"     128    "tcp" []
  , makeService "gss-xlicen"     128    "udp" []
  , makeService "pwdgen"         129    "tcp" []
  , makeService "pwdgen"         129    "udp" []
  , makeService "cisco-fna"      130    "tcp" []
  , makeService "cisco-fna"      130    "udp" []
  , makeService "cisco-tna"      131    "tcp" []
  , makeService "cisco-tna"      131    "udp" []
  , makeService "cisco-sys"      132    "tcp" []
  , makeService "cisco-sys"      132    "udp" []
  , makeService "statsrv"        133    "tcp" []
  , makeService "statsrv"        133    "udp" []
  , makeService "ingres-net"     134    "tcp" []
  , makeService "ingres-net"     134    "udp" []
  , makeService "epmap"          135    "tcp" []
  , makeService "epmap"          135    "udp" []
  , makeService "profile"        136    "tcp" []
  , makeService "profile"        136    "udp" []
  , makeService "emfis-data"     140    "tcp" []
  , makeService "emfis-data"     140    "udp" []
  , makeService "emfis-cntl"     141    "tcp" []
  , makeService "emfis-cntl"     141    "udp" []
  , makeService "bl-idm"         142    "tcp" []
  , makeService "bl-idm"         142    "udp" []
  , makeService "uma"            144    "tcp" []
  , makeService "uma"            144    "udp" []
  , makeService "uaac"           145    "tcp" []
  , makeService "uaac"           145    "udp" []
  , makeService "iso-tp0"        146    "tcp" []
  , makeService "iso-tp0"        146    "udp" []
  , makeService "iso-ip"         147    "tcp" []
  , makeService "iso-ip"         147    "udp" []
  , makeService "jargon"         148    "tcp" []
  , makeService "jargon"         148    "udp" []
  , makeService "aed-512"        149    "tcp" []
  , makeService "aed-512"        149    "udp" []
  , makeService "sql-net"        150    "tcp" []
  , makeService "sql-net"        150    "udp" []
  , makeService "hems"           151    "tcp" []
  , makeService "hems"           151    "udp" []
  , makeService "bftp"           152    "tcp" []
  , makeService "bftp"           152    "udp" []
  , makeService "sgmp"           153    "tcp" []
  , makeService "sgmp"           153    "udp" []
  , makeService "netsc-prod"     154    "tcp" []
  , makeService "netsc-prod"     154    "udp" []
  , makeService "netsc-dev"      155    "tcp" []
  , makeService "netsc-dev"      155    "udp" []
  , makeService "sqlsrv"         156    "tcp" []
  , makeService "sqlsrv"         156    "udp" []
  , makeService "knet-cmp"       157    "tcp" []
  , makeService "knet-cmp"       157    "udp" []
  , makeService "pcmail-srv"     158    "tcp" []
  , makeService "pcmail-srv"     158    "udp" []
  , makeService "nss-routing"    159    "tcp" []
  , makeService "nss-routing"    159    "udp" []
  , makeService "sgmp-traps"     160    "tcp" []
  , makeService "sgmp-traps"     160    "udp" []
  , makeService "xns-courier"    165    "tcp" []
  , makeService "xns-courier"    165    "udp" []
  , makeService "s-net"          166    "tcp" []
  , makeService "s-net"          166    "udp" []
  , makeService "namp"           167    "tcp" []
  , makeService "namp"           167    "udp" []
  , makeService "rsvd"           168    "tcp" []
  , makeService "rsvd"           168    "udp" []
  , makeService "send"           169    "tcp" []
  , makeService "send"           169    "udp" []
  , makeService "print-srv"      170    "tcp" []
  , makeService "print-srv"      170    "udp" []
  , makeService "multiplex"      171    "tcp" []
  , makeService "multiplex"      171    "udp" []
  , makeService "cl/1"           172    "tcp" ["cl-1"]
  , makeService "cl/1"           172    "udp" ["cl-1"]
  , makeService "xyplex-mux"     173    "tcp" []
  , makeService "xyplex-mux"     173    "udp" []
  , makeService "vmnet"          175    "tcp" []
  , makeService "vmnet"          175    "udp" []
  , makeService "genrad-mux"     176    "tcp" []
  , makeService "genrad-mux"     176    "udp" []
  , makeService "ris"            180    "tcp" []
  , makeService "ris"            180    "udp" []
  , makeService "unify"          181    "tcp" []
  , makeService "unify"          181    "udp" []
  , makeService "audit"          182    "tcp" []
  , makeService "audit"          182    "udp" []
  , makeService "ocbinder"       183    "tcp" []
  , makeService "ocbinder"       183    "udp" []
  , makeService "ocserver"       184    "tcp" []
  , makeService "ocserver"       184    "udp" []
  , makeService "remote-kis"     185    "tcp" []
  , makeService "remote-kis"     185    "udp" []
  , makeService "kis"            186    "tcp" []
  , makeService "kis"            186    "udp" []
  , makeService "aci"            187    "tcp" []
  , makeService "aci"            187    "udp" []
  , makeService "mumps"          188    "tcp" []
  , makeService "mumps"          188    "udp" []
  , makeService "qft"            189    "tcp" []
  , makeService "qft"            189    "udp" []
  , makeService "gacp"           190    "tcp" []
  , makeService "gacp"           190    "udp" []
  , makeService "osu-nms"        192    "tcp" []
  , makeService "osu-nms"        192    "udp" []
  , makeService "srmp"           193    "tcp" []
  , makeService "srmp"           193    "udp" []
  , makeService "dn6-nlm-aud"    195    "tcp" []
  , makeService "dn6-nlm-aud"    195    "udp" []
  , makeService "dn6-smm-red"    196    "tcp" []
  , makeService "dn6-smm-red"    196    "udp" []
  , makeService "dls"            197    "tcp" []
  , makeService "dls"            197    "udp" []
  , makeService "dls-mon"        198    "tcp" []
  , makeService "dls-mon"        198    "udp" []
  , makeService "src"            200    "tcp" []
  , makeService "src"            200    "udp" []
  , makeService "at-3"           203    "tcp" []
  , makeService "at-3"           203    "udp" []
  , makeService "at-5"           205    "tcp" []
  , makeService "at-5"           205    "udp" []
  , makeService "at-7"           207    "tcp" []
  , makeService "at-7"           207    "udp" []
  , makeService "at-8"           208    "tcp" []
  , makeService "at-8"           208    "udp" []
  , makeService "914c/g"         211    "tcp" ["914c-g"]
  , makeService "914c/g"         211    "udp" ["914c-g"]
  , makeService "anet"           212    "tcp" []
  , makeService "anet"           212    "udp" []
  , makeService "vmpwscs"        214    "tcp" []
  , makeService "vmpwscs"        214    "udp" []
  , makeService "softpc"         215    "tcp" []
  , makeService "softpc"         215    "udp" []
  , makeService "CAIlic"         216    "tcp" []
  , makeService "CAIlic"         216    "udp" []
  , makeService "dbase"          217    "tcp" []
  , makeService "dbase"          217    "udp" []
  , makeService "mpp"            218    "tcp" []
  , makeService "mpp"            218    "udp" []
  , makeService "uarps"          219    "tcp" []
  , makeService "uarps"          219    "udp" []
  , makeService "fln-spx"        221    "tcp" []
  , makeService "fln-spx"        221    "udp" []
  , makeService "rsh-spx"        222    "tcp" []
  , makeService "rsh-spx"        222    "udp" []
  , makeService "cdc"            223    "tcp" []
  , makeService "cdc"            223    "udp" []
  , makeService "masqdialer"     224    "tcp" []
  , makeService "masqdialer"     224    "udp" []
  , makeService "direct"         242    "tcp" []
  , makeService "direct"         242    "udp" []
  , makeService "sur-meas"       243    "tcp" []
  , makeService "sur-meas"       243    "udp" []
  , makeService "inbusiness"     244    "tcp" []
  , makeService "inbusiness"     244    "udp" []
  , makeService "dsp3270"        246    "tcp" []
  , makeService "dsp3270"        246    "udp" []
  , makeService "subntbcst_tftp" 247    "tcp" ["subntbcst-tftp"]
  , makeService "subntbcst_tftp" 247    "udp" ["subntbcst-tftp"]
  , makeService "bhfhs"          248    "tcp" []
  , makeService "bhfhs"          248    "udp" []
  , makeService "set"            257    "tcp" []
  , makeService "set"            257    "udp" []
  , makeService "esro-gen"       259    "tcp" []
  , makeService "esro-gen"       259    "udp" []
  , makeService "openport"       260    "tcp" []
  , makeService "openport"       260    "udp" []
  , makeService "nsiiops"        261    "tcp" []
  , makeService "nsiiops"        261    "udp" []
  , makeService "arcisdms"       262    "tcp" []
  , makeService "arcisdms"       262    "udp" []
  , makeService "hdap"           263    "tcp" []
  , makeService "hdap"           263    "udp" []
  , makeService "bgmp"           264    "tcp" []
  , makeService "bgmp"           264    "udp" []
  , makeService "x-bone-ctl"     265    "tcp" []
  , makeService "x-bone-ctl"     265    "udp" []
  , makeService "sst"            266    "tcp" []
  , makeService "sst"            266    "udp" []
  , makeService "td-service"     267    "tcp" []
  , makeService "td-service"     267    "udp" []
  , makeService "td-replica"     268    "tcp" []
  , makeService "td-replica"     268    "udp" []
  , makeService "manet"          269    "tcp" []
  , makeService "manet"          269    "udp" []
  , makeService "http-mgmt"      280    "tcp" []
  , makeService "http-mgmt"      280    "udp" []
  , makeService "personal-link"  281    "tcp" []
  , makeService "personal-link"  281    "udp" []
  , makeService "cableport-ax"   282    "tcp" []
  , makeService "cableport-ax"   282    "udp" []
  , makeService "rescap"         283    "tcp" []
  , makeService "rescap"         283    "udp" []
  , makeService "corerjd"        284    "tcp" []
  , makeService "corerjd"        284    "udp" []
  , makeService "fxp"            286    "tcp" []
  , makeService "fxp"            286    "udp" []
  , makeService "k-block"        287    "tcp" []
  , makeService "k-block"        287    "udp" []
  , makeService "novastorbakcup" 308    "tcp" []
  , makeService "novastorbakcup" 308    "udp" []
  , makeService "entrusttime"    309    "tcp" []
  , makeService "entrusttime"    309    "udp" []
  , makeService "bhmds"          310    "tcp" []
  , makeService "bhmds"          310    "udp" []
  , makeService "asip-webadmin"  311    "tcp" []
  , makeService "asip-webadmin"  311    "udp" []
  , makeService "vslmp"          312    "tcp" []
  , makeService "vslmp"          312    "udp" []
  , makeService "magenta-logic"  313    "tcp" []
  , makeService "magenta-logic"  313    "udp" []
  , makeService "opalis-robot"   314    "tcp" []
  , makeService "opalis-robot"   314    "udp" []
  , makeService "dpsi"           315    "tcp" []
  , makeService "dpsi"           315    "udp" []
  , makeService "decauth"        316    "tcp" []
  , makeService "decauth"        316    "udp" []
  , makeService "zannet"         317    "tcp" []
  , makeService "zannet"         317    "udp" []
  , makeService "pkix-timestamp" 318    "tcp" []
  , makeService "pkix-timestamp" 318    "udp" []
  , makeService "ptp-event"      319    "tcp" []
  , makeService "ptp-event"      319    "udp" []
  , makeService "ptp-general"    320    "tcp" []
  , makeService "ptp-general"    320    "udp" []
  , makeService "pip"            321    "tcp" []
  , makeService "pip"            321    "udp" []
  , makeService "rtsps"          322    "tcp" []
  , makeService "rtsps"          322    "udp" []
  , makeService "rpki-rtr"       323    "tcp" []
  , makeService "rpki-rtr-tls"   324    "tcp" []
  , makeService "texar"          333    "tcp" []
  , makeService "texar"          333    "udp" []
  , makeService "pdap"           344    "tcp" []
  , makeService "pdap"           344    "udp" []
  , makeService "pawserv"        345    "tcp" []
  , makeService "pawserv"        345    "udp" []
  , makeService "zserv"          346    "tcp" []
  , makeService "zserv"          346    "udp" []
  , makeService "csi-sgwp"       348    "tcp" []
  , makeService "csi-sgwp"       348    "udp" []
  , makeService "mftp"           349    "tcp" []
  , makeService "mftp"           349    "udp" []
  , makeService "matip-type-a"   350    "tcp" []
  , makeService "matip-type-a"   350    "udp" []
  , makeService "matip-type-b"   351    "tcp" ["bhoetty"]
  , makeService "matip-type-b"   351    "udp" ["bhoetty"]
  , makeService "dtag-ste-sb"    352    "tcp" ["bhoedap4"]
  , makeService "dtag-ste-sb"    352    "udp" ["bhoedap4"]
  , makeService "ndsauth"        353    "tcp" []
  , makeService "ndsauth"        353    "udp" []
  , makeService "bh611"          354    "tcp" []
  , makeService "bh611"          354    "udp" []
  , makeService "datex-asn"      355    "tcp" []
  , makeService "datex-asn"      355    "udp" []
  , makeService "cloanto-net-1"  356    "tcp" []
  , makeService "cloanto-net-1"  356    "udp" []
  , makeService "bhevent"        357    "tcp" []
  , makeService "bhevent"        357    "udp" []
  , makeService "shrinkwrap"     358    "tcp" []
  , makeService "shrinkwrap"     358    "udp" []
  , makeService "nsrmp"          359    "tcp" []
  , makeService "nsrmp"          359    "udp" []
  , makeService "scoi2odialog"   360    "tcp" []
  , makeService "scoi2odialog"   360    "udp" []
  , makeService "semantix"       361    "tcp" []
  , makeService "semantix"       361    "udp" []
  , makeService "srssend"        362    "tcp" []
  , makeService "srssend"        362    "udp" []
  , makeService "aurora-cmgr"    364    "tcp" []
  , makeService "aurora-cmgr"    364    "udp" []
  , makeService "dtk"            365    "tcp" []
  , makeService "dtk"            365    "udp" []
  , makeService "mortgageware"   367    "tcp" []
  , makeService "mortgageware"   367    "udp" []
  , makeService "qbikgdp"        368    "tcp" []
  , makeService "qbikgdp"        368    "udp" []
  , makeService "clearcase"      371    "tcp" []
  , makeService "clearcase"      371    "udp" []
  , makeService "legent-1"       373    "tcp" []
  , makeService "legent-1"       373    "udp" []
  , makeService "legent-2"       374    "tcp" []
  , makeService "legent-2"       374    "udp" []
  , makeService "hassle"         375    "tcp" []
  , makeService "hassle"         375    "udp" []
  , makeService "nip"            376    "tcp" []
  , makeService "nip"            376    "udp" []
  , makeService "tnETOS"         377    "tcp" []
  , makeService "tnETOS"         377    "udp" []
  , makeService "dsETOS"         378    "tcp" []
  , makeService "dsETOS"         378    "udp" []
  , makeService "is99c"          379    "tcp" []
  , makeService "is99c"          379    "udp" []
  , makeService "is99s"          380    "tcp" []
  , makeService "is99s"          380    "udp" []
  , makeService "hp-collector"   381    "tcp" []
  , makeService "hp-collector"   381    "udp" []
  , makeService "hp-managed-node"382    "tcp" []
  , makeService "hp-managed-node"382    "udp" []
  , makeService "hp-alarm-mgr"   383    "tcp" []
  , makeService "hp-alarm-mgr"   383    "udp" []
  , makeService "arns"           384    "tcp" []
  , makeService "arns"           384    "udp" []
  , makeService "ibm-app"        385    "tcp" []
  , makeService "ibm-app"        385    "udp" []
  , makeService "asa"            386    "tcp" []
  , makeService "asa"            386    "udp" []
  , makeService "aurp"           387    "tcp" []
  , makeService "aurp"           387    "udp" []
  , makeService "unidata-ldm"    388    "tcp" []
  , makeService "unidata-ldm"    388    "udp" []
  , makeService "uis"            390    "tcp" []
  , makeService "uis"            390    "udp" []
  , makeService "synotics-relay" 391    "tcp" []
  , makeService "synotics-relay" 391    "udp" []
  , makeService "synotics-broker"392    "tcp" []
  , makeService "synotics-broker"392    "udp" []
  , makeService "meta5"          393    "tcp" []
  , makeService "meta5"          393    "udp" []
  , makeService "embl-ndt"       394    "tcp" []
  , makeService "embl-ndt"       394    "udp" []
  , makeService "netcp"          395    "tcp" []
  , makeService "netcp"          395    "udp" []
  , makeService "netware-ip"     396    "tcp" []
  , makeService "netware-ip"     396    "udp" []
  , makeService "mptn"           397    "tcp" []
  , makeService "mptn"           397    "udp" []
  , makeService "kryptolan"      398    "tcp" []
  , makeService "kryptolan"      398    "udp" []
  , makeService "iso-tsap-c2"    399    "tcp" []
  , makeService "iso-tsap-c2"    399    "udp" []
  , makeService "ups"            401    "tcp" []
  , makeService "ups"            401    "udp" []
  , makeService "genie"          402    "tcp" []
  , makeService "genie"          402    "udp" []
  , makeService "decap"          403    "tcp" []
  , makeService "decap"          403    "udp" []
  , makeService "nced"           404    "tcp" []
  , makeService "nced"           404    "udp" []
  , makeService "ncld"           405    "tcp" []
  , makeService "ncld"           405    "udp" []
  , makeService "imsp"           406    "tcp" []
  , makeService "imsp"           406    "udp" []
  , makeService "timbuktu"       407    "tcp" []
  , makeService "timbuktu"       407    "udp" []
  , makeService "prm-sm"         408    "tcp" []
  , makeService "prm-sm"         408    "udp" []
  , makeService "prm-nm"         409    "tcp" []
  , makeService "prm-nm"         409    "udp" []
  , makeService "decladebug"     410    "tcp" []
  , makeService "decladebug"     410    "udp" []
  , makeService "rmt"            411    "tcp" []
  , makeService "rmt"            411    "udp" []
  , makeService "synoptics-trap" 412    "tcp" []
  , makeService "synoptics-trap" 412    "udp" []
  , makeService "smsp"           413    "tcp" []
  , makeService "smsp"           413    "udp" []
  , makeService "infoseek"       414    "tcp" []
  , makeService "infoseek"       414    "udp" []
  , makeService "bnet"           415    "tcp" []
  , makeService "bnet"           415    "udp" []
  , makeService "silverplatter"  416    "tcp" []
  , makeService "silverplatter"  416    "udp" []
  , makeService "onmux"          417    "tcp" []
  , makeService "onmux"          417    "udp" []
  , makeService "hyper-g"        418    "tcp" []
  , makeService "hyper-g"        418    "udp" []
  , makeService "ariel1"         419    "tcp" []
  , makeService "ariel1"         419    "udp" []
  , makeService "smpte"          420    "tcp" []
  , makeService "smpte"          420    "udp" []
  , makeService "ariel2"         421    "tcp" []
  , makeService "ariel2"         421    "udp" []
  , makeService "ariel3"         422    "tcp" []
  , makeService "ariel3"         422    "udp" []
  , makeService "opc-job-start"  423    "tcp" []
  , makeService "opc-job-start"  423    "udp" []
  , makeService "opc-job-track"  424    "tcp" []
  , makeService "opc-job-track"  424    "udp" []
  , makeService "icad-el"        425    "tcp" []
  , makeService "icad-el"        425    "udp" []
  , makeService "smartsdp"       426    "tcp" []
  , makeService "smartsdp"       426    "udp" []
  , makeService "ocs_cmu"        428    "tcp" []
  , makeService "ocs_cmu"        428    "udp" []
  , makeService "ocs_amu"        429    "tcp" []
  , makeService "ocs_amu"        429    "udp" []
  , makeService "utmpsd"         430    "tcp" []
  , makeService "utmpsd"         430    "udp" []
  , makeService "utmpcd"         431    "tcp" []
  , makeService "utmpcd"         431    "udp" []
  , makeService "iasd"           432    "tcp" []
  , makeService "iasd"           432    "udp" []
  , makeService "nnsp"           433    "tcp" []
  , makeService "nnsp"           433    "udp" []
  , makeService "dna-cml"        436    "tcp" []
  , makeService "dna-cml"        436    "udp" []
  , makeService "comscm"         437    "tcp" []
  , makeService "comscm"         437    "udp" []
  , makeService "dsfgw"          438    "tcp" []
  , makeService "dsfgw"          438    "udp" []
  , makeService "dasp"           439    "tcp" []
  , makeService "dasp"           439    "udp" []
  , makeService "sgcp"           440    "tcp" []
  , makeService "sgcp"           440    "udp" []
  , makeService "decvms-sysmgt"  441    "tcp" []
  , makeService "decvms-sysmgt"  441    "udp" []
  , makeService "cvc_hostd"      442    "tcp" []
  , makeService "cvc_hostd"      442    "udp" []
  , makeService "ddm-rdb"        446    "tcp" []
  , makeService "ddm-rdb"        446    "udp" []
  , makeService "ddm-dfm"        447    "tcp" []
  , makeService "ddm-dfm"        447    "udp" []
  , makeService "ddm-ssl"        448    "tcp" []
  , makeService "ddm-ssl"        448    "udp" []
  , makeService "as-servermap"   449    "tcp" []
  , makeService "as-servermap"   449    "udp" []
  , makeService "tserver"        450    "tcp" []
  , makeService "tserver"        450    "udp" []
  , makeService "sfs-smp-net"    451    "tcp" []
  , makeService "sfs-smp-net"    451    "udp" []
  , makeService "sfs-config"     452    "tcp" []
  , makeService "sfs-config"     452    "udp" []
  , makeService "creativeserver" 453    "tcp" []
  , makeService "creativeserver" 453    "udp" []
  , makeService "contentserver"  454    "tcp" []
  , makeService "contentserver"  454    "udp" []
  , makeService "creativepartnr" 455    "tcp" []
  , makeService "creativepartnr" 455    "udp" []
  , makeService "macon-tcp"      456    "tcp" []
  , makeService "macon-udp"      456    "udp" []
  , makeService "scohelp"        457    "tcp" []
  , makeService "scohelp"        457    "udp" []
  , makeService "appleqtc"       458    "tcp" []
  , makeService "appleqtc"       458    "udp" []
  , makeService "ampr-rcmd"      459    "tcp" []
  , makeService "ampr-rcmd"      459    "udp" []
  , makeService "skronk"         460    "tcp" []
  , makeService "skronk"         460    "udp" []
  , makeService "datasurfsrv"    461    "tcp" []
  , makeService "datasurfsrv"    461    "udp" []
  , makeService "datasurfsrvsec" 462    "tcp" []
  , makeService "datasurfsrvsec" 462    "udp" []
  , makeService "alpes"          463    "tcp" []
  , makeService "alpes"          463    "udp" []
  , makeService "urd"            465    "tcp" ["smtps"]
  , makeService "igmpv3lite"     465    "udp" []
  , makeService "digital-vrc"    466    "tcp" []
  , makeService "digital-vrc"    466    "udp" []
  , makeService "mylex-mapd"     467    "tcp" []
  , makeService "mylex-mapd"     467    "udp" []
  , makeService "rcp"            469    "tcp" []
  , makeService "rcp"            469    "udp" []
  , makeService "scx-proxy"      470    "tcp" []
  , makeService "scx-proxy"      470    "udp" []
  , makeService "mondex"         471    "tcp" []
  , makeService "mondex"         471    "udp" []
  , makeService "ljk-login"      472    "tcp" []
  , makeService "ljk-login"      472    "udp" []
  , makeService "hybrid-pop"     473    "tcp" []
  , makeService "hybrid-pop"     473    "udp" []
  , makeService "tn-tl-w1"       474    "tcp" []
  , makeService "tn-tl-w2"       474    "udp" []
  , makeService "tcpnethaspsrv"  475    "tcp" []
  , makeService "tcpnethaspsrv"  475    "udp" []
  , makeService "tn-tl-fd1"      476    "tcp" []
  , makeService "tn-tl-fd1"      476    "udp" []
  , makeService "ss7ns"          477    "tcp" []
  , makeService "ss7ns"          477    "udp" []
  , makeService "spsc"           478    "tcp" []
  , makeService "spsc"           478    "udp" []
  , makeService "iafserver"      479    "tcp" []
  , makeService "iafserver"      479    "udp" []
  , makeService "iafdbase"       480    "tcp" []
  , makeService "iafdbase"       480    "udp" []
  , makeService "ph"             481    "tcp" []
  , makeService "ph"             481    "udp" []
  , makeService "bgs-nsi"        482    "tcp" []
  , makeService "bgs-nsi"        482    "udp" []
  , makeService "ulpnet"         483    "tcp" []
  , makeService "ulpnet"         483    "udp" []
  , makeService "integra-sme"    484    "tcp" []
  , makeService "integra-sme"    484    "udp" []
  , makeService "powerburst"     485    "tcp" []
  , makeService "powerburst"     485    "udp" []
  , makeService "avian"          486    "tcp" []
  , makeService "avian"          486    "udp" []
  , makeService "nest-protocol"  489    "tcp" []
  , makeService "nest-protocol"  489    "udp" []
  , makeService "micom-pfs"      490    "tcp" []
  , makeService "micom-pfs"      490    "udp" []
  , makeService "go-login"       491    "tcp" []
  , makeService "go-login"       491    "udp" []
  , makeService "ticf-1"         492    "tcp" []
  , makeService "ticf-1"         492    "udp" []
  , makeService "ticf-2"         493    "tcp" []
  , makeService "ticf-2"         493    "udp" []
  , makeService "pov-ray"        494    "tcp" []
  , makeService "pov-ray"        494    "udp" []
  , makeService "intecourier"    495    "tcp" []
  , makeService "intecourier"    495    "udp" []
  , makeService "retrospect"     497    "tcp" []
  , makeService "retrospect"     497    "udp" []
  , makeService "siam"           498    "tcp" []
  , makeService "siam"           498    "udp" []
  , makeService "iso-ill"        499    "tcp" []
  , makeService "iso-ill"        499    "udp" []
  , makeService "stmf"           501    "tcp" []
  , makeService "stmf"           501    "udp" []
  , makeService "asa-appl-proto" 502    "tcp" []
  , makeService "asa-appl-proto" 502    "udp" []
  , makeService "intrinsa"       503    "tcp" []
  , makeService "intrinsa"       503    "udp" []
  , makeService "citadel"        504    "tcp" []
  , makeService "citadel"        504    "udp" []
  , makeService "mailbox-lm"     505    "tcp" []
  , makeService "mailbox-lm"     505    "udp" []
  , makeService "ohimsrv"        506    "tcp" []
  , makeService "ohimsrv"        506    "udp" []
  , makeService "crs"            507    "tcp" []
  , makeService "crs"            507    "udp" []
  , makeService "xvttp"          508    "tcp" []
  , makeService "xvttp"          508    "udp" []
  , makeService "snare"          509    "tcp" []
  , makeService "snare"          509    "udp" []
  , makeService "fcp"            510    "tcp" []
  , makeService "fcp"            510    "udp" []
  , makeService "passgo"         511    "tcp" []
  , makeService "passgo"         511    "udp" []
  , makeService "videotex"       516    "tcp" []
  , makeService "videotex"       516    "udp" []
  , makeService "talk"           517    "tcp" []
  , makeService "ntalk"          518    "tcp" []
  , makeService "ulp"            522    "tcp" []
  , makeService "ulp"            522    "udp" []
  , makeService "ibm-db2"        523    "tcp" []
  , makeService "ibm-db2"        523    "udp" []
  , makeService "ncp"            524    "tcp" []
  , makeService "ncp"            524    "udp" []
  , makeService "tempo"          526    "udp" []
  , makeService "stx"            527    "tcp" []
  , makeService "stx"            527    "udp" []
  , makeService "custix"         528    "tcp" []
  , makeService "custix"         528    "udp" []
  , makeService "irc-serv"       529    "tcp" []
  , makeService "irc-serv"       529    "udp" []
  , makeService "courier"        530    "udp" []
  , makeService "conference"     531    "udp" []
  , makeService "netnews"        532    "udp" []
  , makeService "netwall"        533    "tcp" []
  , makeService "windream"       534    "tcp" []
  , makeService "windream"       534    "udp" []
  , makeService "opalis-rdv"     536    "tcp" []
  , makeService "opalis-rdv"     536    "udp" []
  , makeService "nmsp"           537    "tcp" []
  , makeService "nmsp"           537    "udp" []
  , makeService "apertus-ldp"    539    "tcp" []
  , makeService "apertus-ldp"    539    "udp" []
  , makeService "uucp"           540    "udp" []
  , makeService "uucp-rlogin"    541    "tcp" []
  , makeService "uucp-rlogin"    541    "udp" []
  , makeService "commerce"       542    "tcp" []
  , makeService "commerce"       542    "udp" []
  , makeService "klogin"         543    "udp" []
  , makeService "kshell"         544    "udp" []
  , makeService "appleqtcsrvr"   545    "tcp" []
  , makeService "appleqtcsrvr"   545    "udp" []
  , makeService "idfp"           549    "tcp" []
  , makeService "idfp"           549    "udp" []
  , makeService "new-rwho"       550    "tcp" []
  , makeService "new-rwho"       550    "udp" []
  , makeService "cybercash"      551    "tcp" []
  , makeService "cybercash"      551    "udp" []
  , makeService "devshr-nts"     552    "tcp" []
  , makeService "devshr-nts"     552    "udp" []
  , makeService "pirp"           553    "tcp" []
  , makeService "pirp"           553    "udp" []
  , makeService "dsf"            555    "tcp" []
  , makeService "dsf"            555    "udp" []
  , makeService "remotefs"       556    "udp" []
  , makeService "openvms-sysipc" 557    "tcp" []
  , makeService "openvms-sysipc" 557    "udp" []
  , makeService "sdnskmp"        558    "tcp" []
  , makeService "sdnskmp"        558    "udp" []
  , makeService "teedtap"        559    "tcp" []
  , makeService "teedtap"        559    "udp" []
  , makeService "rmonitor"       560    "tcp" []
  , makeService "rmonitor"       560    "udp" []
  , makeService "monitor"        561    "tcp" []
  , makeService "monitor"        561    "udp" []
  , makeService "chshell"        562    "tcp" []
  , makeService "chshell"        562    "udp" []
  , makeService "9pfs"           564    "tcp" []
  , makeService "9pfs"           564    "udp" []
  , makeService "streettalk"     566    "tcp" []
  , makeService "streettalk"     566    "udp" []
  , makeService "banyan-rpc"     567    "tcp" []
  , makeService "banyan-rpc"     567    "udp" []
  , makeService "ms-shuttle"     568    "tcp" []
  , makeService "ms-shuttle"     568    "udp" []
  , makeService "ms-rome"        569    "tcp" []
  , makeService "ms-rome"        569    "udp" []
  , makeService "meter"          570    "tcp" []
  , makeService "meter"          570    "udp" []
  , makeService "sonar"          572    "tcp" []
  , makeService "sonar"          572    "udp" []
  , makeService "banyan-vip"     573    "tcp" []
  , makeService "banyan-vip"     573    "udp" []
  , makeService "ftp-agent"      574    "tcp" []
  , makeService "ftp-agent"      574    "udp" []
  , makeService "vemmi"          575    "tcp" []
  , makeService "vemmi"          575    "udp" []
  , makeService "ipcd"           576    "tcp" []
  , makeService "ipcd"           576    "udp" []
  , makeService "vnas"           577    "tcp" []
  , makeService "vnas"           577    "udp" []
  , makeService "ipdd"           578    "tcp" []
  , makeService "ipdd"           578    "udp" []
  , makeService "decbsrv"        579    "tcp" []
  , makeService "decbsrv"        579    "udp" []
  , makeService "sntp-heartbeat" 580    "tcp" []
  , makeService "sntp-heartbeat" 580    "udp" []
  , makeService "bdp"            581    "tcp" []
  , makeService "bdp"            581    "udp" []
  , makeService "scc-security"   582    "tcp" []
  , makeService "scc-security"   582    "udp" []
  , makeService "philips-vc"     583    "tcp" []
  , makeService "philips-vc"     583    "udp" []
  , makeService "keyserver"      584    "tcp" []
  , makeService "keyserver"      584    "udp" []
  , makeService "password-chg"   586    "tcp" []
  , makeService "password-chg"   586    "udp" []
  , makeService "cal"            588    "tcp" []
  , makeService "cal"            588    "udp" []
  , makeService "eyelink"        589    "tcp" []
  , makeService "eyelink"        589    "udp" []
  , makeService "tns-cml"        590    "tcp" []
  , makeService "tns-cml"        590    "udp" []
  , makeService "eudora-set"     592    "tcp" []
  , makeService "eudora-set"     592    "udp" []
  , makeService "http-rpc-epmap" 593    "tcp" []
  , makeService "http-rpc-epmap" 593    "udp" []
  , makeService "tpip"           594    "tcp" []
  , makeService "tpip"           594    "udp" []
  , makeService "cab-protocol"   595    "tcp" []
  , makeService "cab-protocol"   595    "udp" []
  , makeService "smsd"           596    "tcp" []
  , makeService "smsd"           596    "udp" []
  , makeService "ptcnameservice" 597    "tcp" []
  , makeService "ptcnameservice" 597    "udp" []
  , makeService "sco-websrvrmg3" 598    "tcp" []
  , makeService "sco-websrvrmg3" 598    "udp" []
  , makeService "acp"            599    "tcp" []
  , makeService "acp"            599    "udp" []
  , makeService "ipcserver"      600    "tcp" []
  , makeService "ipcserver"      600    "udp" []
  , makeService "syslog-conn"    601    "tcp" []
  , makeService "syslog-conn"    601    "udp" []
  , makeService "xmlrpc-beep"    602    "tcp" []
  , makeService "xmlrpc-beep"    602    "udp" []
  , makeService "idxp"           603    "tcp" []
  , makeService "idxp"           603    "udp" []
  , makeService "tunnel"         604    "tcp" []
  , makeService "tunnel"         604    "udp" []
  , makeService "soap-beep"      605    "tcp" []
  , makeService "soap-beep"      605    "udp" []
  , makeService "urm"            606    "tcp" []
  , makeService "urm"            606    "udp" []
  , makeService "nqs"            607    "tcp" []
  , makeService "nqs"            607    "udp" []
  , makeService "sift-uft"       608    "tcp" []
  , makeService "sift-uft"       608    "udp" []
  , makeService "npmp-trap"      609    "tcp" []
  , makeService "npmp-trap"      609    "udp" []
  , makeService "hmmp-op"        613    "tcp" []
  , makeService "hmmp-op"        613    "udp" []
  , makeService "sshell"         614    "tcp" []
  , makeService "sshell"         614    "udp" []
  , makeService "sco-inetmgr"    615    "tcp" []
  , makeService "sco-inetmgr"    615    "udp" []
  , makeService "sco-sysmgr"     616    "tcp" ["gii"]
  , makeService "sco-sysmgr"     616    "udp" []
  , makeService "sco-dtmgr"      617    "tcp" []
  , makeService "sco-dtmgr"      617    "udp" []
  , makeService "dei-icda"       618    "tcp" []
  , makeService "dei-icda"       618    "udp" []
  , makeService "compaq-evm"     619    "tcp" []
  , makeService "compaq-evm"     619    "udp" []
  , makeService "sco-websrvrmgr" 620    "tcp" []
  , makeService "sco-websrvrmgr" 620    "udp" []
  , makeService "escp-ip"        621    "tcp" []
  , makeService "escp-ip"        621    "udp" []
  , makeService "collaborator"   622    "tcp" []
  , makeService "collaborator"   622    "udp" []
  , makeService "oob-ws-http"    623    "tcp" []
  , makeService "asf-rmcp"       623    "udp" []
  , makeService "cryptoadmin"    624    "tcp" []
  , makeService "cryptoadmin"    624    "udp" []
  , makeService "dec_dlm"        625    "tcp" ["dec-dlm"]
  , makeService "dec_dlm"        625    "udp" ["dec-dlm"]
  , makeService "asia"           626    "tcp" []
  , makeService "asia"           626    "udp" []
  , makeService "passgo-tivoli"  627    "tcp" []
  , makeService "passgo-tivoli"  627    "udp" []
  , makeService "qmqp"           628    "tcp" []
  , makeService "qmqp"           628    "udp" []
  , makeService "3com-amp3"      629    "tcp" []
  , makeService "3com-amp3"      629    "udp" []
  , makeService "rda"            630    "tcp" []
  , makeService "rda"            630    "udp" []
  , makeService "bmpp"           632    "tcp" []
  , makeService "bmpp"           632    "udp" []
  , makeService "servstat"       633    "tcp" []
  , makeService "servstat"       633    "udp" []
  , makeService "ginad"          634    "tcp" []
  , makeService "ginad"          634    "udp" []
  , makeService "rlzdbase"       635    "tcp" []
  , makeService "rlzdbase"       635    "udp" []
  , makeService "lanserver"      637    "tcp" []
  , makeService "lanserver"      637    "udp" []
  , makeService "mcns-sec"       638    "tcp" []
  , makeService "mcns-sec"       638    "udp" []
  , makeService "msdp"           639    "tcp" []
  , makeService "msdp"           639    "udp" []
  , makeService "entrust-sps"    640    "tcp" []
  , makeService "entrust-sps"    640    "udp" []
  , makeService "repcmd"         641    "tcp" []
  , makeService "repcmd"         641    "udp" []
  , makeService "esro-emsdp"     642    "tcp" []
  , makeService "esro-emsdp"     642    "udp" []
  , makeService "sanity"         643    "tcp" []
  , makeService "sanity"         643    "udp" []
  , makeService "dwr"            644    "tcp" []
  , makeService "dwr"            644    "udp" []
  , makeService "pssc"           645    "tcp" []
  , makeService "pssc"           645    "udp" []
  , makeService "ldp"            646    "tcp" []
  , makeService "ldp"            646    "udp" []
  , makeService "dhcp-failover"  647    "tcp" []
  , makeService "dhcp-failover"  647    "udp" []
  , makeService "rrp"            648    "tcp" []
  , makeService "rrp"            648    "udp" []
  , makeService "cadview-3d"     649    "tcp" []
  , makeService "cadview-3d"     649    "udp" []
  , makeService "obex"           650    "tcp" []
  , makeService "obex"           650    "udp" []
  , makeService "ieee-mms"       651    "tcp" []
  , makeService "ieee-mms"       651    "udp" []
  , makeService "hello-port"     652    "tcp" []
  , makeService "hello-port"     652    "udp" []
  , makeService "repscmd"        653    "tcp" []
  , makeService "repscmd"        653    "udp" []
  , makeService "aodv"           654    "tcp" []
  , makeService "aodv"           654    "udp" []
  , makeService "tinc"           655    "tcp" []
  , makeService "tinc"           655    "udp" []
  , makeService "spmp"           656    "tcp" []
  , makeService "spmp"           656    "udp" []
  , makeService "rmc"            657    "tcp" []
  , makeService "rmc"            657    "udp" []
  , makeService "tenfold"        658    "tcp" []
  , makeService "tenfold"        658    "udp" []
  , makeService "mac-srvr-admin" 660    "tcp" []
  , makeService "mac-srvr-admin" 660    "udp" []
  , makeService "hap"            661    "tcp" []
  , makeService "hap"            661    "udp" []
  , makeService "pftp"           662    "tcp" []
  , makeService "pftp"           662    "udp" []
  , makeService "purenoise"      663    "tcp" []
  , makeService "purenoise"      663    "udp" []
  , makeService "oob-ws-https"   664    "tcp" []
  , makeService "asf-secure-rmcp"664    "udp" []
  , makeService "sun-dr"         665    "tcp" []
  , makeService "sun-dr"         665    "udp" []
  , makeService "mdqs"           666    "tcp" ["doom"]
  , makeService "mdqs"           666    "udp" ["doom"]
  , makeService "disclose"       667    "tcp" []
  , makeService "disclose"       667    "udp" []
  , makeService "mecomm"         668    "tcp" []
  , makeService "mecomm"         668    "udp" []
  , makeService "meregister"     669    "tcp" []
  , makeService "meregister"     669    "udp" []
  , makeService "vacdsm-sws"     670    "tcp" []
  , makeService "vacdsm-sws"     670    "udp" []
  , makeService "vacdsm-app"     671    "tcp" []
  , makeService "vacdsm-app"     671    "udp" []
  , makeService "vpps-qua"       672    "tcp" []
  , makeService "vpps-qua"       672    "udp" []
  , makeService "cimplex"        673    "tcp" []
  , makeService "cimplex"        673    "udp" []
  , makeService "dctp"           675    "tcp" []
  , makeService "dctp"           675    "udp" []
  , makeService "vpps-via"       676    "tcp" []
  , makeService "vpps-via"       676    "udp" []
  , makeService "vpp"            677    "tcp" []
  , makeService "vpp"            677    "udp" []
  , makeService "ggf-ncp"        678    "tcp" []
  , makeService "ggf-ncp"        678    "udp" []
  , makeService "mrm"            679    "tcp" []
  , makeService "mrm"            679    "udp" []
  , makeService "entrust-aaas"   680    "tcp" []
  , makeService "entrust-aaas"   680    "udp" []
  , makeService "entrust-aams"   681    "tcp" []
  , makeService "entrust-aams"   681    "udp" []
  , makeService "xfr"            682    "tcp" []
  , makeService "xfr"            682    "udp" []
  , makeService "corba-iiop"     683    "tcp" []
  , makeService "corba-iiop"     683    "udp" []
  , makeService "corba-iiop-ssl" 684    "tcp" []
  , makeService "corba-iiop-ssl" 684    "udp" []
  , makeService "mdc-portmapper" 685    "tcp" []
  , makeService "mdc-portmapper" 685    "udp" []
  , makeService "hcp-wismar"     686    "tcp" []
  , makeService "hcp-wismar"     686    "udp" []
  , makeService "asipregistry"   687    "tcp" []
  , makeService "asipregistry"   687    "udp" []
  , makeService "realm-rusd"     688    "tcp" []
  , makeService "realm-rusd"     688    "udp" []
  , makeService "nmap"           689    "tcp" []
  , makeService "nmap"           689    "udp" []
  , makeService "vatp"           690    "tcp" []
  , makeService "vatp"           690    "udp" []
  , makeService "msexch-routing" 691    "tcp" []
  , makeService "msexch-routing" 691    "udp" []
  , makeService "hyperwave-isp"  692    "tcp" []
  , makeService "hyperwave-isp"  692    "udp" []
  , makeService "connendp"       693    "tcp" []
  , makeService "connendp"       693    "udp" []
  , makeService "ieee-mms-ssl"   695    "tcp" []
  , makeService "ieee-mms-ssl"   695    "udp" []
  , makeService "rushd"          696    "tcp" []
  , makeService "rushd"          696    "udp" []
  , makeService "uuidgen"        697    "tcp" []
  , makeService "uuidgen"        697    "udp" []
  , makeService "olsr"           698    "tcp" []
  , makeService "olsr"           698    "udp" []
  , makeService "accessnetwork"  699    "tcp" []
  , makeService "accessnetwork"  699    "udp" []
  , makeService "epp"            700    "tcp" []
  , makeService "epp"            700    "udp" []
  , makeService "lmp"            701    "tcp" []
  , makeService "lmp"            701    "udp" []
  , makeService "iris-beep"      702    "tcp" []
  , makeService "iris-beep"      702    "udp" []
  , makeService "elcsd"          704    "tcp" []
  , makeService "elcsd"          704    "udp" []
  , makeService "agentx"         705    "tcp" []
  , makeService "agentx"         705    "udp" []
  , makeService "silc"           706    "tcp" []
  , makeService "silc"           706    "udp" []
  , makeService "borland-dsj"    707    "tcp" []
  , makeService "borland-dsj"    707    "udp" []
  , makeService "entrust-kmsh"   709    "tcp" []
  , makeService "entrust-kmsh"   709    "udp" []
  , makeService "entrust-ash"    710    "tcp" []
  , makeService "entrust-ash"    710    "udp" []
  , makeService "cisco-tdp"      711    "tcp" []
  , makeService "cisco-tdp"      711    "udp" []
  , makeService "tbrpf"          712    "tcp" []
  , makeService "tbrpf"          712    "udp" []
  , makeService "iris-xpc"       713    "tcp" []
  , makeService "iris-xpc"       713    "udp" []
  , makeService "iris-xpcs"      714    "tcp" []
  , makeService "iris-xpcs"      714    "udp" []
  , makeService "iris-lwz"       715    "tcp" []
  , makeService "iris-lwz"       715    "udp" []
  , makeService "pana"           716    "udp" []
  , makeService "netviewdm1"     729    "tcp" []
  , makeService "netviewdm1"     729    "udp" []
  , makeService "netviewdm2"     730    "tcp" []
  , makeService "netviewdm2"     730    "udp" []
  , makeService "netviewdm3"     731    "tcp" []
  , makeService "netviewdm3"     731    "udp" []
  , makeService "netgw"          741    "tcp" []
  , makeService "netgw"          741    "udp" []
  , makeService "netrcs"         742    "tcp" []
  , makeService "netrcs"         742    "udp" []
  , makeService "flexlm"         744    "tcp" []
  , makeService "flexlm"         744    "udp" []
  , makeService "fujitsu-dev"    747    "tcp" []
  , makeService "fujitsu-dev"    747    "udp" []
  , makeService "ris-cm"         748    "tcp" []
  , makeService "ris-cm"         748    "udp" []
  , makeService "qrh"            752    "tcp" []
  , makeService "rrh"            753    "tcp" []
  , makeService "rrh"            753    "udp" []
  , makeService "tell"           754    "udp" []
  , makeService "nlogin"         758    "tcp" []
  , makeService "nlogin"         758    "udp" []
  , makeService "con"            759    "tcp" []
  , makeService "con"            759    "udp" []
  , makeService "ns"             760    "udp" []
  , makeService "rxe"            761    "tcp" []
  , makeService "rxe"            761    "udp" []
  , makeService "quotad"         762    "tcp" []
  , makeService "quotad"         762    "udp" []
  , makeService "cycleserv"      763    "tcp" []
  , makeService "cycleserv"      763    "udp" []
  , makeService "omserv"         764    "tcp" []
  , makeService "omserv"         764    "udp" []
  , makeService "vid"            769    "tcp" []
  , makeService "vid"            769    "udp" []
  , makeService "cadlock"        770    "tcp" []
  , makeService "cadlock"        770    "udp" []
  , makeService "rtip"           771    "tcp" []
  , makeService "rtip"           771    "udp" []
  , makeService "cycleserv2"     772    "tcp" []
  , makeService "cycleserv2"     772    "udp" []
  , makeService "submit"         773    "tcp" []
  , makeService "notify"         773    "udp" []
  , makeService "rpasswd"        774    "tcp" []
  , makeService "acmaint_dbd"    774    "udp" ["acmaint-dbd"]
  , makeService "entomb"         775    "tcp" []
  , makeService "acmaint_transd" 775    "udp" ["acmaint-transd"]
  , makeService "wpages"         776    "tcp" []
  , makeService "wpages"         776    "udp" []
  , makeService "multiling-http" 777    "tcp" []
  , makeService "multiling-http" 777    "udp" []
  , makeService "wpgs"           780    "tcp" []
  , makeService "wpgs"           780    "udp" []
  , makeService "mdbs_daemon"    800    "tcp" ["mdbs-daemon"]
  , makeService "mdbs_daemon"    800    "udp" ["mdbs-daemon"]
  , makeService "device"         801    "tcp" []
  , makeService "device"         801    "udp" []
  , makeService "fcp-udp"        810    "tcp" []
  , makeService "fcp-udp"        810    "udp" []
  , makeService "itm-mcell-s"    828    "tcp" []
  , makeService "itm-mcell-s"    828    "udp" []
  , makeService "pkix-3-ca-ra"   829    "tcp" []
  , makeService "pkix-3-ca-ra"   829    "udp" []
  , makeService "netconf-ssh"    830    "tcp" []
  , makeService "netconf-ssh"    830    "udp" []
  , makeService "netconf-beep"   831    "tcp" []
  , makeService "netconf-beep"   831    "udp" []
  , makeService "netconfsoaphttp"832    "tcp" []
  , makeService "netconfsoaphttp"832    "udp" []
  , makeService "netconfsoapbeep"833    "tcp" []
  , makeService "netconfsoapbeep"833    "udp" []
  , makeService "dhcp-failover2" 847    "tcp" []
  , makeService "dhcp-failover2" 847    "udp" []
  , makeService "gdoi"           848    "tcp" []
  , makeService "gdoi"           848    "udp" []
  , makeService "iscsi"          860    "tcp" []
  , makeService "iscsi"          860    "udp" []
  , makeService "owamp-control"  861    "tcp" []
  , makeService "owamp-control"  861    "udp" []
  , makeService "twamp-control"  862    "tcp" []
  , makeService "twamp-control"  862    "udp" []
  , makeService "iclcnet-locate" 886    "tcp" []
  , makeService "iclcnet-locate" 886    "udp" []
  , makeService "iclcnet_svinfo" 887    "tcp" ["iclcnet-svinfo"]
  , makeService "iclcnet_svinfo" 887    "udp" ["iclcnet-svinfo"]
  , makeService "cddbp"          888    "tcp" []
  , makeService "omginitialrefs" 900    "tcp" []
  , makeService "omginitialrefs" 900    "udp" []
  , makeService "smpnameres"     901    "udp" []
  , makeService "ideafarm-door"  902    "tcp" []
  , makeService "ideafarm-door"  902    "udp" []
  , makeService "ideafarm-panic" 903    "tcp" []
  , makeService "ideafarm-panic" 903    "udp" []
  , makeService "kink"           910    "tcp" []
  , makeService "kink"           910    "udp" []
  , makeService "xact-backup"    911    "tcp" []
  , makeService "xact-backup"    911    "udp" []
  , makeService "apex-mesh"      912    "tcp" []
  , makeService "apex-mesh"      912    "udp" []
  , makeService "apex-edge"      913    "tcp" []
  , makeService "apex-edge"      913    "udp" []
  , makeService "ftps-data"      989    "tcp" []
  , makeService "ftps-data"      989    "udp" []
  , makeService "ftps"           990    "tcp" []
  , makeService "ftps"           990    "udp" []
  , makeService "nas"            991    "tcp" []
  , makeService "nas"            991    "udp" []
  , makeService "vsinet"         996    "tcp" []
  , makeService "vsinet"         996    "udp" []
  , makeService "maitrd"         997    "tcp" []
  , makeService "maitrd"         997    "udp" []
  , makeService "busboy"         998    "tcp" []
  , makeService "puparp"         998    "udp" []
  , makeService "garcon"         999    "tcp" []
  , makeService "applix"         999    "udp" []
  , makeService "cadlock2"       1000    "tcp" []
  , makeService "cadlock2"       1000    "udp" []
  , makeService "surf"           1010    "tcp" []
  , makeService "surf"           1010    "udp" []
  , makeService "exp1"           1021    "tcp" []
  , makeService "exp1"           1021    "udp" []
  , makeService "exp2"           1022    "tcp" []
  , makeService "exp2"           1022    "udp" []
  , makeService "blackjack"      1025    "tcp" []
  , makeService "blackjack"      1025    "udp" []
  , makeService "cap"            1026    "tcp" []
  , makeService "cap"            1026    "udp" []
  , makeService "6a44"           1027    "udp" []
  , makeService "solid-mux"      1029    "tcp" []
  , makeService "solid-mux"      1029    "udp" []
  , makeService "iad1"           1030    "tcp" []
  , makeService "iad1"           1030    "udp" []
  , makeService "iad2"           1031    "tcp" []
  , makeService "iad2"           1031    "udp" []
  , makeService "iad3"           1032    "tcp" []
  , makeService "iad3"           1032    "udp" []
  , makeService "netinfo-local"  1033    "tcp" []
  , makeService "netinfo-local"  1033    "udp" []
  , makeService "activesync"     1034    "tcp" []
  , makeService "activesync"     1034    "udp" []
  , makeService "mxxrlogin"      1035    "tcp" []
  , makeService "mxxrlogin"      1035    "udp" []
  , makeService "nsstp"          1036    "tcp" []
  , makeService "nsstp"          1036    "udp" []
  , makeService "ams"            1037    "tcp" []
  , makeService "ams"            1037    "udp" []
  , makeService "mtqp"           1038    "tcp" []
  , makeService "mtqp"           1038    "udp" []
  , makeService "sbl"            1039    "tcp" []
  , makeService "sbl"            1039    "udp" []
  , makeService "netarx"         1040    "tcp" []
  , makeService "netarx"         1040    "udp" []
  , makeService "danf-ak2"       1041    "tcp" []
  , makeService "danf-ak2"       1041    "udp" []
  , makeService "afrog"          1042    "tcp" []
  , makeService "afrog"          1042    "udp" []
  , makeService "boinc-client"   1043    "tcp" []
  , makeService "boinc-client"   1043    "udp" []
  , makeService "dcutility"      1044    "tcp" []
  , makeService "dcutility"      1044    "udp" []
  , makeService "fpitp"          1045    "tcp" []
  , makeService "fpitp"          1045    "udp" []
  , makeService "wfremotertm"    1046    "tcp" []
  , makeService "wfremotertm"    1046    "udp" []
  , makeService "neod1"          1047    "tcp" []
  , makeService "neod1"          1047    "udp" []
  , makeService "neod2"          1048    "tcp" []
  , makeService "neod2"          1048    "udp" []
  , makeService "td-postman"     1049    "tcp" []
  , makeService "td-postman"     1049    "udp" []
  , makeService "cma"            1050    "tcp" []
  , makeService "cma"            1050    "udp" []
  , makeService "optima-vnet"    1051    "tcp" []
  , makeService "optima-vnet"    1051    "udp" []
  , makeService "ddt"            1052    "tcp" []
  , makeService "ddt"            1052    "udp" []
  , makeService "remote-as"      1053    "tcp" []
  , makeService "remote-as"      1053    "udp" []
  , makeService "brvread"        1054    "tcp" []
  , makeService "brvread"        1054    "udp" []
  , makeService "ansyslmd"       1055    "tcp" []
  , makeService "ansyslmd"       1055    "udp" []
  , makeService "vfo"            1056    "tcp" []
  , makeService "vfo"            1056    "udp" []
  , makeService "startron"       1057    "tcp" []
  , makeService "startron"       1057    "udp" []
  , makeService "nim"            1058    "tcp" []
  , makeService "nim"            1058    "udp" []
  , makeService "nimreg"         1059    "tcp" []
  , makeService "nimreg"         1059    "udp" []
  , makeService "polestar"       1060    "tcp" []
  , makeService "polestar"       1060    "udp" []
  , makeService "kiosk"          1061    "tcp" []
  , makeService "kiosk"          1061    "udp" []
  , makeService "veracity"       1062    "tcp" []
  , makeService "veracity"       1062    "udp" []
  , makeService "kyoceranetdev"  1063    "tcp" []
  , makeService "kyoceranetdev"  1063    "udp" []
  , makeService "jstel"          1064    "tcp" []
  , makeService "jstel"          1064    "udp" []
  , makeService "syscomlan"      1065    "tcp" []
  , makeService "syscomlan"      1065    "udp" []
  , makeService "fpo-fns"        1066    "tcp" []
  , makeService "fpo-fns"        1066    "udp" []
  , makeService "instl_boots"    1067    "tcp" ["instl-boots"]
  , makeService "instl_boots"    1067    "udp" ["instl-boots"]
  , makeService "instl_bootc"    1068    "tcp" ["instl-bootc"]
  , makeService "instl_bootc"    1068    "udp" ["instl-bootc"]
  , makeService "cognex-insight" 1069    "tcp" []
  , makeService "cognex-insight" 1069    "udp" []
  , makeService "gmrupdateserv"  1070    "tcp" []
  , makeService "gmrupdateserv"  1070    "udp" []
  , makeService "bsquare-voip"   1071    "tcp" []
  , makeService "bsquare-voip"   1071    "udp" []
  , makeService "cardax"         1072    "tcp" []
  , makeService "cardax"         1072    "udp" []
  , makeService "bridgecontrol"  1073    "tcp" []
  , makeService "bridgecontrol"  1073    "udp" []
  , makeService "warmspotMgmt"   1074    "tcp" []
  , makeService "warmspotMgmt"   1074    "udp" []
  , makeService "rdrmshc"        1075    "tcp" []
  , makeService "rdrmshc"        1075    "udp" []
  , makeService "dab-sti-c"      1076    "tcp" []
  , makeService "dab-sti-c"      1076    "udp" []
  , makeService "imgames"        1077    "tcp" []
  , makeService "imgames"        1077    "udp" []
  , makeService "avocent-proxy"  1078    "tcp" []
  , makeService "avocent-proxy"  1078    "udp" []
  , makeService "asprovatalk"    1079    "tcp" []
  , makeService "asprovatalk"    1079    "udp" []
  , makeService "pvuniwien"      1081    "tcp" []
  , makeService "pvuniwien"      1081    "udp" []
  , makeService "amt-esd-prot"   1082    "tcp" []
  , makeService "amt-esd-prot"   1082    "udp" []
  , makeService "ansoft-lm-1"    1083    "tcp" []
  , makeService "ansoft-lm-1"    1083    "udp" []
  , makeService "ansoft-lm-2"    1084    "tcp" []
  , makeService "ansoft-lm-2"    1084    "udp" []
  , makeService "webobjects"     1085    "tcp" []
  , makeService "webobjects"     1085    "udp" []
  , makeService "cplscrambler-lg"1086    "tcp" []
  , makeService "cplscrambler-lg"1086    "udp" []
  , makeService "cplscrambler-in"1087    "tcp" []
  , makeService "cplscrambler-in"1087    "udp" []
  , makeService "cplscrambler-al"1088    "tcp" []
  , makeService "cplscrambler-al"1088    "udp" []
  , makeService "ff-annunc"      1089    "tcp" []
  , makeService "ff-annunc"      1089    "udp" []
  , makeService "ff-fms"         1090    "tcp" []
  , makeService "ff-fms"         1090    "udp" []
  , makeService "ff-sm"          1091    "tcp" []
  , makeService "ff-sm"          1091    "udp" []
  , makeService "obrpd"          1092    "tcp" []
  , makeService "obrpd"          1092    "udp" []
  , makeService "proofd"         1093    "tcp" []
  , makeService "proofd"         1093    "udp" []
  , makeService "rootd"          1094    "tcp" []
  , makeService "rootd"          1094    "udp" []
  , makeService "nicelink"       1095    "tcp" []
  , makeService "nicelink"       1095    "udp" []
  , makeService "cnrprotocol"    1096    "tcp" []
  , makeService "cnrprotocol"    1096    "udp" []
  , makeService "sunclustermgr"  1097    "tcp" []
  , makeService "sunclustermgr"  1097    "udp" []
  , makeService "rmiactivation"  1098    "tcp" []
  , makeService "rmiactivation"  1098    "udp" []
  , makeService "rmiregistry"    1099    "tcp" []
  , makeService "rmiregistry"    1099    "udp" []
  , makeService "mctp"           1100    "tcp" []
  , makeService "mctp"           1100    "udp" []
  , makeService "pt2-discover"   1101    "tcp" []
  , makeService "pt2-discover"   1101    "udp" []
  , makeService "adobeserver-1"  1102    "tcp" []
  , makeService "adobeserver-1"  1102    "udp" []
  , makeService "adobeserver-2"  1103    "tcp" []
  , makeService "adobeserver-2"  1103    "udp" []
  , makeService "xrl"            1104    "tcp" []
  , makeService "xrl"            1104    "udp" []
  , makeService "ftranhc"        1105    "tcp" []
  , makeService "ftranhc"        1105    "udp" []
  , makeService "isoipsigport-1" 1106    "tcp" []
  , makeService "isoipsigport-1" 1106    "udp" []
  , makeService "isoipsigport-2" 1107    "tcp" []
  , makeService "isoipsigport-2" 1107    "udp" []
  , makeService "ratio-adp"      1108    "tcp" []
  , makeService "ratio-adp"      1108    "udp" []
  , makeService "webadmstart"    1110    "tcp" []
  , makeService "nfsd-keepalive" 1110    "udp" []
  , makeService "lmsocialserver" 1111    "tcp" []
  , makeService "lmsocialserver" 1111    "udp" []
  , makeService "icp"            1112    "tcp" []
  , makeService "icp"            1112    "udp" []
  , makeService "ltp-deepspace"  1113    "tcp" []
  , makeService "ltp-deepspace"  1113    "udp" []
  , makeService "mini-sql"       1114    "tcp" []
  , makeService "mini-sql"       1114    "udp" []
  , makeService "ardus-trns"     1115    "tcp" []
  , makeService "ardus-trns"     1115    "udp" []
  , makeService "ardus-cntl"     1116    "tcp" []
  , makeService "ardus-cntl"     1116    "udp" []
  , makeService "ardus-mtrns"    1117    "tcp" []
  , makeService "ardus-mtrns"    1117    "udp" []
  , makeService "sacred"         1118    "tcp" []
  , makeService "sacred"         1118    "udp" []
  , makeService "bnetgame"       1119    "tcp" []
  , makeService "bnetgame"       1119    "udp" []
  , makeService "bnetfile"       1120    "tcp" []
  , makeService "bnetfile"       1120    "udp" []
  , makeService "rmpp"           1121    "tcp" []
  , makeService "rmpp"           1121    "udp" []
  , makeService "availant-mgr"   1122    "tcp" []
  , makeService "availant-mgr"   1122    "udp" []
  , makeService "murray"         1123    "tcp" []
  , makeService "murray"         1123    "udp" []
  , makeService "hpvmmcontrol"   1124    "tcp" []
  , makeService "hpvmmcontrol"   1124    "udp" []
  , makeService "hpvmmagent"     1125    "tcp" []
  , makeService "hpvmmagent"     1125    "udp" []
  , makeService "hpvmmdata"      1126    "tcp" []
  , makeService "hpvmmdata"      1126    "udp" []
  , makeService "kwdb-commn"     1127    "udp" []
  , makeService "saphostctrl"    1128    "tcp" []
  , makeService "saphostctrl"    1128    "udp" []
  , makeService "saphostctrls"   1129    "tcp" []
  , makeService "saphostctrls"   1129    "udp" []
  , makeService "casp"           1130    "tcp" []
  , makeService "casp"           1130    "udp" []
  , makeService "caspssl"        1131    "tcp" []
  , makeService "caspssl"        1131    "udp" []
  , makeService "kvm-via-ip"     1132    "tcp" []
  , makeService "kvm-via-ip"     1132    "udp" []
  , makeService "dfn"            1133    "tcp" []
  , makeService "dfn"            1133    "udp" []
  , makeService "aplx"           1134    "tcp" []
  , makeService "aplx"           1134    "udp" []
  , makeService "omnivision"     1135    "tcp" []
  , makeService "omnivision"     1135    "udp" []
  , makeService "hhb-gateway"    1136    "tcp" []
  , makeService "hhb-gateway"    1136    "udp" []
  , makeService "trim"           1137    "tcp" []
  , makeService "trim"           1137    "udp" []
  , makeService "encrypted_admin"1138    "tcp" ["encrypted-admin"]
  , makeService "encrypted_admin"1138    "udp" ["encrypted-admin"]
  , makeService "evm"            1139    "tcp" []
  , makeService "evm"            1139    "udp" []
  , makeService "autonoc"        1140    "tcp" []
  , makeService "autonoc"        1140    "udp" []
  , makeService "mxomss"         1141    "tcp" []
  , makeService "mxomss"         1141    "udp" []
  , makeService "edtools"        1142    "tcp" []
  , makeService "edtools"        1142    "udp" []
  , makeService "imyx"           1143    "tcp" []
  , makeService "imyx"           1143    "udp" []
  , makeService "fuscript"       1144    "tcp" []
  , makeService "fuscript"       1144    "udp" []
  , makeService "x9-icue"        1145    "tcp" []
  , makeService "x9-icue"        1145    "udp" []
  , makeService "audit-transfer" 1146    "tcp" []
  , makeService "audit-transfer" 1146    "udp" []
  , makeService "capioverlan"    1147    "tcp" []
  , makeService "capioverlan"    1147    "udp" []
  , makeService "elfiq-repl"     1148    "tcp" []
  , makeService "elfiq-repl"     1148    "udp" []
  , makeService "bvtsonar"       1149    "tcp" []
  , makeService "bvtsonar"       1149    "udp" []
  , makeService "blaze"          1150    "tcp" []
  , makeService "blaze"          1150    "udp" []
  , makeService "unizensus"      1151    "tcp" []
  , makeService "unizensus"      1151    "udp" []
  , makeService "winpoplanmess"  1152    "tcp" []
  , makeService "winpoplanmess"  1152    "udp" []
  , makeService "c1222-acse"     1153    "tcp" []
  , makeService "c1222-acse"     1153    "udp" []
  , makeService "resacommunity"  1154    "tcp" []
  , makeService "resacommunity"  1154    "udp" []
  , makeService "nfa"            1155    "tcp" []
  , makeService "nfa"            1155    "udp" []
  , makeService "iascontrol-oms" 1156    "tcp" []
  , makeService "iascontrol-oms" 1156    "udp" []
  , makeService "iascontrol"     1157    "tcp" []
  , makeService "iascontrol"     1157    "udp" []
  , makeService "dbcontrol-oms"  1158    "tcp" []
  , makeService "dbcontrol-oms"  1158    "udp" []
  , makeService "oracle-oms"     1159    "tcp" []
  , makeService "oracle-oms"     1159    "udp" []
  , makeService "olsv"           1160    "tcp" []
  , makeService "olsv"           1160    "udp" []
  , makeService "health-polling" 1161    "tcp" []
  , makeService "health-polling" 1161    "udp" []
  , makeService "health-trap"    1162    "tcp" []
  , makeService "health-trap"    1162    "udp" []
  , makeService "sddp"           1163    "tcp" []
  , makeService "sddp"           1163    "udp" []
  , makeService "qsm-proxy"      1164    "tcp" []
  , makeService "qsm-proxy"      1164    "udp" []
  , makeService "qsm-gui"        1165    "tcp" []
  , makeService "qsm-gui"        1165    "udp" []
  , makeService "qsm-remote"     1166    "tcp" []
  , makeService "qsm-remote"     1166    "udp" []
  , makeService "cisco-ipsla"    1167    "tcp" []
  , makeService "cisco-ipsla"    1167    "udp" []
  , makeService "vchat"          1168    "tcp" []
  , makeService "vchat"          1168    "udp" []
  , makeService "tripwire"       1169    "tcp" []
  , makeService "tripwire"       1169    "udp" []
  , makeService "atc-lm"         1170    "tcp" []
  , makeService "atc-lm"         1170    "udp" []
  , makeService "atc-appserver"  1171    "tcp" []
  , makeService "atc-appserver"  1171    "udp" []
  , makeService "dnap"           1172    "tcp" []
  , makeService "dnap"           1172    "udp" []
  , makeService "d-cinema-rrp"   1173    "tcp" []
  , makeService "d-cinema-rrp"   1173    "udp" []
  , makeService "fnet-remote-ui" 1174    "tcp" []
  , makeService "fnet-remote-ui" 1174    "udp" []
  , makeService "dossier"        1175    "tcp" []
  , makeService "dossier"        1175    "udp" []
  , makeService "indigo-server"  1176    "tcp" []
  , makeService "indigo-server"  1176    "udp" []
  , makeService "dkmessenger"    1177    "tcp" []
  , makeService "dkmessenger"    1177    "udp" []
  , makeService "sgi-storman"    1178    "udp" []
  , makeService "b2n"            1179    "tcp" []
  , makeService "b2n"            1179    "udp" []
  , makeService "mc-client"      1180    "tcp" []
  , makeService "mc-client"      1180    "udp" []
  , makeService "3comnetman"     1181    "tcp" []
  , makeService "3comnetman"     1181    "udp" []
  , makeService "accelenet"      1182    "tcp" []
  , makeService "accelenet-data" 1182    "udp" []
  , makeService "llsurfup-http"  1183    "tcp" []
  , makeService "llsurfup-http"  1183    "udp" []
  , makeService "llsurfup-https" 1184    "tcp" []
  , makeService "llsurfup-https" 1184    "udp" []
  , makeService "catchpole"      1185    "tcp" []
  , makeService "catchpole"      1185    "udp" []
  , makeService "mysql-cluster"  1186    "tcp" []
  , makeService "mysql-cluster"  1186    "udp" []
  , makeService "alias"          1187    "tcp" []
  , makeService "alias"          1187    "udp" []
  , makeService "hp-webadmin"    1188    "tcp" []
  , makeService "hp-webadmin"    1188    "udp" []
  , makeService "unet"           1189    "tcp" []
  , makeService "unet"           1189    "udp" []
  , makeService "commlinx-avl"   1190    "tcp" []
  , makeService "commlinx-avl"   1190    "udp" []
  , makeService "gpfs"           1191    "tcp" []
  , makeService "gpfs"           1191    "udp" []
  , makeService "caids-sensor"   1192    "tcp" []
  , makeService "caids-sensor"   1192    "udp" []
  , makeService "fiveacross"     1193    "tcp" []
  , makeService "fiveacross"     1193    "udp" []
  , makeService "openvpn"        1194    "tcp" []
  , makeService "openvpn"        1194    "udp" []
  , makeService "rsf-1"          1195    "tcp" []
  , makeService "rsf-1"          1195    "udp" []
  , makeService "netmagic"       1196    "tcp" []
  , makeService "netmagic"       1196    "udp" []
  , makeService "carrius-rshell" 1197    "tcp" []
  , makeService "carrius-rshell" 1197    "udp" []
  , makeService "cajo-discovery" 1198    "tcp" []
  , makeService "cajo-discovery" 1198    "udp" []
  , makeService "dmidi"          1199    "tcp" []
  , makeService "dmidi"          1199    "udp" []
  , makeService "scol"           1200    "tcp" []
  , makeService "scol"           1200    "udp" []
  , makeService "nucleus-sand"   1201    "tcp" []
  , makeService "nucleus-sand"   1201    "udp" []
  , makeService "caiccipc"       1202    "tcp" []
  , makeService "caiccipc"       1202    "udp" []
  , makeService "ssslic-mgr"     1203    "tcp" []
  , makeService "ssslic-mgr"     1203    "udp" []
  , makeService "ssslog-mgr"     1204    "tcp" []
  , makeService "ssslog-mgr"     1204    "udp" []
  , makeService "accord-mgc"     1205    "tcp" []
  , makeService "accord-mgc"     1205    "udp" []
  , makeService "anthony-data"   1206    "tcp" []
  , makeService "anthony-data"   1206    "udp" []
  , makeService "metasage"       1207    "tcp" []
  , makeService "metasage"       1207    "udp" []
  , makeService "seagull-ais"    1208    "tcp" []
  , makeService "seagull-ais"    1208    "udp" []
  , makeService "ipcd3"          1209    "tcp" []
  , makeService "ipcd3"          1209    "udp" []
  , makeService "eoss"           1210    "tcp" []
  , makeService "eoss"           1210    "udp" []
  , makeService "groove-dpp"     1211    "tcp" []
  , makeService "groove-dpp"     1211    "udp" []
  , makeService "lupa"           1212    "tcp" []
  , makeService "lupa"           1212    "udp" []
  , makeService "mpc-lifenet"    1213    "tcp" []
  , makeService "mpc-lifenet"    1213    "udp" []
  , makeService "kazaa"          1214    "tcp" []
  , makeService "kazaa"          1214    "udp" []
  , makeService "scanstat-1"     1215    "tcp" []
  , makeService "scanstat-1"     1215    "udp" []
  , makeService "etebac5"        1216    "tcp" []
  , makeService "etebac5"        1216    "udp" []
  , makeService "hpss-ndapi"     1217    "tcp" []
  , makeService "hpss-ndapi"     1217    "udp" []
  , makeService "aeroflight-ads" 1218    "tcp" []
  , makeService "aeroflight-ads" 1218    "udp" []
  , makeService "aeroflight-ret" 1219    "tcp" []
  , makeService "aeroflight-ret" 1219    "udp" []
  , makeService "qt-serveradmin" 1220    "tcp" []
  , makeService "qt-serveradmin" 1220    "udp" []
  , makeService "sweetware-apps" 1221    "tcp" []
  , makeService "sweetware-apps" 1221    "udp" []
  , makeService "nerv"           1222    "tcp" []
  , makeService "nerv"           1222    "udp" []
  , makeService "tgp"            1223    "tcp" []
  , makeService "tgp"            1223    "udp" []
  , makeService "vpnz"           1224    "tcp" []
  , makeService "vpnz"           1224    "udp" []
  , makeService "slinkysearch"   1225    "tcp" []
  , makeService "slinkysearch"   1225    "udp" []
  , makeService "stgxfws"        1226    "tcp" []
  , makeService "stgxfws"        1226    "udp" []
  , makeService "dns2go"         1227    "tcp" []
  , makeService "dns2go"         1227    "udp" []
  , makeService "florence"       1228    "tcp" []
  , makeService "florence"       1228    "udp" []
  , makeService "zented"         1229    "tcp" []
  , makeService "zented"         1229    "udp" []
  , makeService "periscope"      1230    "tcp" []
  , makeService "periscope"      1230    "udp" []
  , makeService "menandmice-lpm" 1231    "tcp" []
  , makeService "menandmice-lpm" 1231    "udp" []
  , makeService "first-defense"  1232    "tcp" []
  , makeService "first-defense"  1232    "udp" []
  , makeService "univ-appserver" 1233    "tcp" []
  , makeService "univ-appserver" 1233    "udp" []
  , makeService "search-agent"   1234    "tcp" []
  , makeService "search-agent"   1234    "udp" []
  , makeService "mosaicsyssvc1"  1235    "tcp" []
  , makeService "mosaicsyssvc1"  1235    "udp" []
  , makeService "tsdos390"       1237    "tcp" []
  , makeService "tsdos390"       1237    "udp" []
  , makeService "hacl-qs"        1238    "tcp" []
  , makeService "hacl-qs"        1238    "udp" []
  , makeService "nmsd"           1239    "tcp" []
  , makeService "nmsd"           1239    "udp" []
  , makeService "instantia"      1240    "tcp" []
  , makeService "instantia"      1240    "udp" []
  , makeService "nessus"         1241    "tcp" []
  , makeService "nessus"         1241    "udp" []
  , makeService "nmasoverip"     1242    "tcp" []
  , makeService "nmasoverip"     1242    "udp" []
  , makeService "serialgateway"  1243    "tcp" []
  , makeService "serialgateway"  1243    "udp" []
  , makeService "isbconference1" 1244    "tcp" []
  , makeService "isbconference1" 1244    "udp" []
  , makeService "isbconference2" 1245    "tcp" []
  , makeService "isbconference2" 1245    "udp" []
  , makeService "payrouter"      1246    "tcp" []
  , makeService "payrouter"      1246    "udp" []
  , makeService "visionpyramid"  1247    "tcp" []
  , makeService "visionpyramid"  1247    "udp" []
  , makeService "hermes"         1248    "tcp" []
  , makeService "hermes"         1248    "udp" []
  , makeService "mesavistaco"    1249    "tcp" []
  , makeService "mesavistaco"    1249    "udp" []
  , makeService "swldy-sias"     1250    "tcp" []
  , makeService "swldy-sias"     1250    "udp" []
  , makeService "servergraph"    1251    "tcp" []
  , makeService "servergraph"    1251    "udp" []
  , makeService "bspne-pcc"      1252    "tcp" []
  , makeService "bspne-pcc"      1252    "udp" []
  , makeService "q55-pcc"        1253    "tcp" []
  , makeService "q55-pcc"        1253    "udp" []
  , makeService "de-noc"         1254    "tcp" []
  , makeService "de-noc"         1254    "udp" []
  , makeService "de-cache-query" 1255    "tcp" []
  , makeService "de-cache-query" 1255    "udp" []
  , makeService "de-server"      1256    "tcp" []
  , makeService "de-server"      1256    "udp" []
  , makeService "shockwave2"     1257    "tcp" []
  , makeService "shockwave2"     1257    "udp" []
  , makeService "opennl"         1258    "tcp" []
  , makeService "opennl"         1258    "udp" []
  , makeService "opennl-voice"   1259    "tcp" []
  , makeService "opennl-voice"   1259    "udp" []
  , makeService "ibm-ssd"        1260    "tcp" []
  , makeService "ibm-ssd"        1260    "udp" []
  , makeService "mpshrsv"        1261    "tcp" []
  , makeService "mpshrsv"        1261    "udp" []
  , makeService "qnts-orb"       1262    "tcp" []
  , makeService "qnts-orb"       1262    "udp" []
  , makeService "dka"            1263    "tcp" []
  , makeService "dka"            1263    "udp" []
  , makeService "prat"           1264    "tcp" []
  , makeService "prat"           1264    "udp" []
  , makeService "dssiapi"        1265    "tcp" []
  , makeService "dssiapi"        1265    "udp" []
  , makeService "dellpwrappks"   1266    "tcp" []
  , makeService "dellpwrappks"   1266    "udp" []
  , makeService "epc"            1267    "tcp" []
  , makeService "epc"            1267    "udp" []
  , makeService "propel-msgsys"  1268    "tcp" []
  , makeService "propel-msgsys"  1268    "udp" []
  , makeService "watilapp"       1269    "tcp" []
  , makeService "watilapp"       1269    "udp" []
  , makeService "opsmgr"         1270    "tcp" []
  , makeService "opsmgr"         1270    "udp" []
  , makeService "excw"           1271    "tcp" []
  , makeService "excw"           1271    "udp" []
  , makeService "cspmlockmgr"    1272    "tcp" []
  , makeService "cspmlockmgr"    1272    "udp" []
  , makeService "emc-gateway"    1273    "tcp" []
  , makeService "emc-gateway"    1273    "udp" []
  , makeService "t1distproc"     1274    "tcp" []
  , makeService "t1distproc"     1274    "udp" []
  , makeService "ivcollector"    1275    "tcp" []
  , makeService "ivcollector"    1275    "udp" []
  , makeService "ivmanager"      1276    "tcp" []
  , makeService "ivmanager"      1276    "udp" []
  , makeService "miva-mqs"       1277    "tcp" []
  , makeService "miva-mqs"       1277    "udp" []
  , makeService "dellwebadmin-1" 1278    "tcp" []
  , makeService "dellwebadmin-1" 1278    "udp" []
  , makeService "dellwebadmin-2" 1279    "tcp" []
  , makeService "dellwebadmin-2" 1279    "udp" []
  , makeService "pictrography"   1280    "tcp" []
  , makeService "pictrography"   1280    "udp" []
  , makeService "healthd"        1281    "tcp" []
  , makeService "healthd"        1281    "udp" []
  , makeService "emperion"       1282    "tcp" []
  , makeService "emperion"       1282    "udp" []
  , makeService "productinfo"    1283    "tcp" []
  , makeService "productinfo"    1283    "udp" []
  , makeService "iee-qfx"        1284    "tcp" []
  , makeService "iee-qfx"        1284    "udp" []
  , makeService "neoiface"       1285    "tcp" []
  , makeService "neoiface"       1285    "udp" []
  , makeService "netuitive"      1286    "tcp" []
  , makeService "netuitive"      1286    "udp" []
  , makeService "routematch"     1287    "tcp" []
  , makeService "routematch"     1287    "udp" []
  , makeService "navbuddy"       1288    "tcp" []
  , makeService "navbuddy"       1288    "udp" []
  , makeService "jwalkserver"    1289    "tcp" []
  , makeService "jwalkserver"    1289    "udp" []
  , makeService "winjaserver"    1290    "tcp" []
  , makeService "winjaserver"    1290    "udp" []
  , makeService "seagulllms"     1291    "tcp" []
  , makeService "seagulllms"     1291    "udp" []
  , makeService "dsdn"           1292    "tcp" []
  , makeService "dsdn"           1292    "udp" []
  , makeService "pkt-krb-ipsec"  1293    "tcp" []
  , makeService "pkt-krb-ipsec"  1293    "udp" []
  , makeService "cmmdriver"      1294    "tcp" []
  , makeService "cmmdriver"      1294    "udp" []
  , makeService "ehtp"           1295    "tcp" []
  , makeService "ehtp"           1295    "udp" []
  , makeService "dproxy"         1296    "tcp" []
  , makeService "dproxy"         1296    "udp" []
  , makeService "sdproxy"        1297    "tcp" []
  , makeService "sdproxy"        1297    "udp" []
  , makeService "lpcp"           1298    "tcp" []
  , makeService "lpcp"           1298    "udp" []
  , makeService "hp-sci"         1299    "tcp" []
  , makeService "hp-sci"         1299    "udp" []
  , makeService "ci3-software-1" 1301    "tcp" []
  , makeService "ci3-software-1" 1301    "udp" []
  , makeService "ci3-software-2" 1302    "tcp" []
  , makeService "ci3-software-2" 1302    "udp" []
  , makeService "sftsrv"         1303    "tcp" []
  , makeService "sftsrv"         1303    "udp" []
  , makeService "boomerang"      1304    "tcp" []
  , makeService "boomerang"      1304    "udp" []
  , makeService "pe-mike"        1305    "tcp" []
  , makeService "pe-mike"        1305    "udp" []
  , makeService "re-conn-proto"  1306    "tcp" []
  , makeService "re-conn-proto"  1306    "udp" []
  , makeService "pacmand"        1307    "tcp" []
  , makeService "pacmand"        1307    "udp" []
  , makeService "odsi"           1308    "tcp" []
  , makeService "odsi"           1308    "udp" []
  , makeService "jtag-server"    1309    "tcp" []
  , makeService "jtag-server"    1309    "udp" []
  , makeService "husky"          1310    "tcp" []
  , makeService "husky"          1310    "udp" []
  , makeService "rxmon"          1311    "tcp" []
  , makeService "rxmon"          1311    "udp" []
  , makeService "sti-envision"   1312    "tcp" []
  , makeService "sti-envision"   1312    "udp" []
  , makeService "bmc_patroldb"   1313    "udp" ["bmc-patroldb"]
  , makeService "pdps"           1314    "tcp" []
  , makeService "pdps"           1314    "udp" []
  , makeService "els"            1315    "tcp" []
  , makeService "els"            1315    "udp" []
  , makeService "exbit-escp"     1316    "tcp" []
  , makeService "exbit-escp"     1316    "udp" []
  , makeService "vrts-ipcserver" 1317    "tcp" []
  , makeService "vrts-ipcserver" 1317    "udp" []
  , makeService "krb5gatekeeper" 1318    "tcp" []
  , makeService "krb5gatekeeper" 1318    "udp" []
  , makeService "amx-icsp"       1319    "tcp" []
  , makeService "amx-icsp"       1319    "udp" []
  , makeService "amx-axbnet"     1320    "tcp" []
  , makeService "amx-axbnet"     1320    "udp" []
  , makeService "novation"       1322    "tcp" []
  , makeService "novation"       1322    "udp" []
  , makeService "brcd"           1323    "tcp" []
  , makeService "brcd"           1323    "udp" []
  , makeService "delta-mcp"      1324    "tcp" []
  , makeService "delta-mcp"      1324    "udp" []
  , makeService "dx-instrument"  1325    "tcp" []
  , makeService "dx-instrument"  1325    "udp" []
  , makeService "wimsic"         1326    "tcp" []
  , makeService "wimsic"         1326    "udp" []
  , makeService "ultrex"         1327    "tcp" []
  , makeService "ultrex"         1327    "udp" []
  , makeService "ewall"          1328    "tcp" []
  , makeService "ewall"          1328    "udp" []
  , makeService "netdb-export"   1329    "tcp" []
  , makeService "netdb-export"   1329    "udp" []
  , makeService "streetperfect"  1330    "tcp" []
  , makeService "streetperfect"  1330    "udp" []
  , makeService "intersan"       1331    "tcp" []
  , makeService "intersan"       1331    "udp" []
  , makeService "pcia-rxp-b"     1332    "tcp" []
  , makeService "pcia-rxp-b"     1332    "udp" []
  , makeService "passwrd-policy" 1333    "tcp" []
  , makeService "passwrd-policy" 1333    "udp" []
  , makeService "writesrv"       1334    "tcp" []
  , makeService "writesrv"       1334    "udp" []
  , makeService "digital-notary" 1335    "tcp" []
  , makeService "digital-notary" 1335    "udp" []
  , makeService "ischat"         1336    "tcp" []
  , makeService "ischat"         1336    "udp" []
  , makeService "menandmice-dns" 1337    "tcp" []
  , makeService "menandmice-dns" 1337    "udp" []
  , makeService "wmc-log-svc"    1338    "tcp" []
  , makeService "wmc-log-svc"    1338    "udp" []
  , makeService "kjtsiteserver"  1339    "tcp" []
  , makeService "kjtsiteserver"  1339    "udp" []
  , makeService "naap"           1340    "tcp" []
  , makeService "naap"           1340    "udp" []
  , makeService "qubes"          1341    "tcp" []
  , makeService "qubes"          1341    "udp" []
  , makeService "esbroker"       1342    "tcp" []
  , makeService "esbroker"       1342    "udp" []
  , makeService "re101"          1343    "tcp" []
  , makeService "re101"          1343    "udp" []
  , makeService "icap"           1344    "tcp" []
  , makeService "icap"           1344    "udp" []
  , makeService "vpjp"           1345    "tcp" []
  , makeService "vpjp"           1345    "udp" []
  , makeService "alta-ana-lm"    1346    "tcp" []
  , makeService "alta-ana-lm"    1346    "udp" []
  , makeService "bbn-mmc"        1347    "tcp" []
  , makeService "bbn-mmc"        1347    "udp" []
  , makeService "bbn-mmx"        1348    "tcp" []
  , makeService "bbn-mmx"        1348    "udp" []
  , makeService "sbook"          1349    "tcp" []
  , makeService "sbook"          1349    "udp" []
  , makeService "editbench"      1350    "tcp" []
  , makeService "editbench"      1350    "udp" []
  , makeService "equationbuilder"1351    "tcp" []
  , makeService "equationbuilder"1351    "udp" []
  , makeService "lotusnote"      1352    "tcp" []
  , makeService "lotusnote"      1352    "udp" []
  , makeService "relief"         1353    "tcp" []
  , makeService "relief"         1353    "udp" []
  , makeService "XSIP-network"   1354    "tcp" []
  , makeService "XSIP-network"   1354    "udp" []
  , makeService "intuitive-edge" 1355    "tcp" []
  , makeService "intuitive-edge" 1355    "udp" []
  , makeService "cuillamartin"   1356    "tcp" []
  , makeService "cuillamartin"   1356    "udp" []
  , makeService "pegboard"       1357    "tcp" []
  , makeService "pegboard"       1357    "udp" []
  , makeService "connlcli"       1358    "tcp" []
  , makeService "connlcli"       1358    "udp" []
  , makeService "ftsrv"          1359    "tcp" []
  , makeService "ftsrv"          1359    "udp" []
  , makeService "mimer"          1360    "tcp" []
  , makeService "mimer"          1360    "udp" []
  , makeService "linx"           1361    "tcp" []
  , makeService "linx"           1361    "udp" []
  , makeService "timeflies"      1362    "tcp" []
  , makeService "timeflies"      1362    "udp" []
  , makeService "ndm-requester"  1363    "tcp" []
  , makeService "ndm-requester"  1363    "udp" []
  , makeService "ndm-server"     1364    "tcp" []
  , makeService "ndm-server"     1364    "udp" []
  , makeService "adapt-sna"      1365    "tcp" []
  , makeService "adapt-sna"      1365    "udp" []
  , makeService "netware-csp"    1366    "tcp" []
  , makeService "netware-csp"    1366    "udp" []
  , makeService "dcs"            1367    "tcp" []
  , makeService "dcs"            1367    "udp" []
  , makeService "screencast"     1368    "tcp" []
  , makeService "screencast"     1368    "udp" []
  , makeService "gv-us"          1369    "tcp" []
  , makeService "gv-us"          1369    "udp" []
  , makeService "us-gv"          1370    "tcp" []
  , makeService "us-gv"          1370    "udp" []
  , makeService "fc-cli"         1371    "tcp" []
  , makeService "fc-cli"         1371    "udp" []
  , makeService "fc-ser"         1372    "tcp" []
  , makeService "fc-ser"         1372    "udp" []
  , makeService "chromagrafx"    1373    "tcp" []
  , makeService "chromagrafx"    1373    "udp" []
  , makeService "molly"          1374    "tcp" []
  , makeService "molly"          1374    "udp" []
  , makeService "bytex"          1375    "tcp" []
  , makeService "bytex"          1375    "udp" []
  , makeService "ibm-pps"        1376    "tcp" []
  , makeService "ibm-pps"        1376    "udp" []
  , makeService "cichlid"        1377    "tcp" []
  , makeService "cichlid"        1377    "udp" []
  , makeService "elan"           1378    "tcp" []
  , makeService "elan"           1378    "udp" []
  , makeService "dbreporter"     1379    "tcp" []
  , makeService "dbreporter"     1379    "udp" []
  , makeService "telesis-licman" 1380    "tcp" []
  , makeService "telesis-licman" 1380    "udp" []
  , makeService "apple-licman"   1381    "tcp" []
  , makeService "apple-licman"   1381    "udp" []
  , makeService "udt_os"         1382    "tcp" ["udt-os"]
  , makeService "udt_os"         1382    "udp" ["udt-os"]
  , makeService "gwha"           1383    "tcp" []
  , makeService "gwha"           1383    "udp" []
  , makeService "os-licman"      1384    "tcp" []
  , makeService "os-licman"      1384    "udp" []
  , makeService "atex_elmd"      1385    "tcp" ["atex-elmd"]
  , makeService "atex_elmd"      1385    "udp" ["atex-elmd"]
  , makeService "checksum"       1386    "tcp" []
  , makeService "checksum"       1386    "udp" []
  , makeService "cadsi-lm"       1387    "tcp" []
  , makeService "cadsi-lm"       1387    "udp" []
  , makeService "objective-dbc"  1388    "tcp" []
  , makeService "objective-dbc"  1388    "udp" []
  , makeService "iclpv-dm"       1389    "tcp" []
  , makeService "iclpv-dm"       1389    "udp" []
  , makeService "iclpv-sc"       1390    "tcp" []
  , makeService "iclpv-sc"       1390    "udp" []
  , makeService "iclpv-sas"      1391    "tcp" []
  , makeService "iclpv-sas"      1391    "udp" []
  , makeService "iclpv-pm"       1392    "tcp" []
  , makeService "iclpv-pm"       1392    "udp" []
  , makeService "iclpv-nls"      1393    "tcp" []
  , makeService "iclpv-nls"      1393    "udp" []
  , makeService "iclpv-nlc"      1394    "tcp" []
  , makeService "iclpv-nlc"      1394    "udp" []
  , makeService "iclpv-wsm"      1395    "tcp" []
  , makeService "iclpv-wsm"      1395    "udp" []
  , makeService "dvl-activemail" 1396    "tcp" []
  , makeService "dvl-activemail" 1396    "udp" []
  , makeService "audio-activmail"1397    "tcp" []
  , makeService "audio-activmail"1397    "udp" []
  , makeService "video-activmail"1398    "tcp" []
  , makeService "video-activmail"1398    "udp" []
  , makeService "cadkey-licman"  1399    "tcp" []
  , makeService "cadkey-licman"  1399    "udp" []
  , makeService "cadkey-tablet"  1400    "tcp" []
  , makeService "cadkey-tablet"  1400    "udp" []
  , makeService "goldleaf-licman"1401    "tcp" []
  , makeService "goldleaf-licman"1401    "udp" []
  , makeService "prm-sm-np"      1402    "tcp" []
  , makeService "prm-sm-np"      1402    "udp" []
  , makeService "prm-nm-np"      1403    "tcp" []
  , makeService "prm-nm-np"      1403    "udp" []
  , makeService "igi-lm"         1404    "tcp" []
  , makeService "igi-lm"         1404    "udp" []
  , makeService "ibm-res"        1405    "tcp" []
  , makeService "ibm-res"        1405    "udp" []
  , makeService "netlabs-lm"     1406    "tcp" []
  , makeService "netlabs-lm"     1406    "udp" []
  , makeService "dbsa-lm"        1407    "tcp" []
  , makeService "dbsa-lm"        1407    "udp" []
  , makeService "sophia-lm"      1408    "tcp" []
  , makeService "sophia-lm"      1408    "udp" []
  , makeService "here-lm"        1409    "tcp" []
  , makeService "here-lm"        1409    "udp" []
  , makeService "hiq"            1410    "tcp" []
  , makeService "hiq"            1410    "udp" []
  , makeService "af"             1411    "tcp" []
  , makeService "af"             1411    "udp" []
  , makeService "innosys"        1412    "tcp" []
  , makeService "innosys"        1412    "udp" []
  , makeService "innosys-acl"    1413    "tcp" []
  , makeService "innosys-acl"    1413    "udp" []
  , makeService "ibm-mqseries"   1414    "tcp" []
  , makeService "ibm-mqseries"   1414    "udp" []
  , makeService "dbstar"         1415    "tcp" []
  , makeService "dbstar"         1415    "udp" []
  , makeService "novell-lu6.2"   1416    "tcp" ["novell-lu6-2"]
  , makeService "novell-lu6.2"   1416    "udp" ["novell-lu6-2"]
  , makeService "timbuktu-srv1"  1417    "tcp" []
  , makeService "timbuktu-srv1"  1417    "udp" []
  , makeService "timbuktu-srv2"  1418    "tcp" []
  , makeService "timbuktu-srv2"  1418    "udp" []
  , makeService "timbuktu-srv3"  1419    "tcp" []
  , makeService "timbuktu-srv3"  1419    "udp" []
  , makeService "timbuktu-srv4"  1420    "tcp" []
  , makeService "timbuktu-srv4"  1420    "udp" []
  , makeService "gandalf-lm"     1421    "tcp" []
  , makeService "gandalf-lm"     1421    "udp" []
  , makeService "autodesk-lm"    1422    "tcp" []
  , makeService "autodesk-lm"    1422    "udp" []
  , makeService "essbase"        1423    "tcp" []
  , makeService "essbase"        1423    "udp" []
  , makeService "hybrid"         1424    "tcp" []
  , makeService "hybrid"         1424    "udp" []
  , makeService "zion-lm"        1425    "tcp" []
  , makeService "zion-lm"        1425    "udp" []
  , makeService "sais"           1426    "tcp" []
  , makeService "sais"           1426    "udp" []
  , makeService "mloadd"         1427    "tcp" []
  , makeService "mloadd"         1427    "udp" []
  , makeService "informatik-lm"  1428    "tcp" []
  , makeService "informatik-lm"  1428    "udp" []
  , makeService "nms"            1429    "tcp" []
  , makeService "nms"            1429    "udp" []
  , makeService "tpdu"           1430    "tcp" []
  , makeService "tpdu"           1430    "udp" []
  , makeService "rgtp"           1431    "tcp" []
  , makeService "rgtp"           1431    "udp" []
  , makeService "blueberry-lm"   1432    "tcp" []
  , makeService "blueberry-lm"   1432    "udp" []
  , makeService "ibm-cics"       1435    "tcp" []
  , makeService "ibm-cics"       1435    "udp" []
  , makeService "saism"          1436    "tcp" []
  , makeService "saism"          1436    "udp" []
  , makeService "tabula"         1437    "tcp" []
  , makeService "tabula"         1437    "udp" []
  , makeService "eicon-server"   1438    "tcp" []
  , makeService "eicon-server"   1438    "udp" []
  , makeService "eicon-x25"      1439    "tcp" []
  , makeService "eicon-x25"      1439    "udp" []
  , makeService "eicon-slp"      1440    "tcp" []
  , makeService "eicon-slp"      1440    "udp" []
  , makeService "cadis-1"        1441    "tcp" []
  , makeService "cadis-1"        1441    "udp" []
  , makeService "cadis-2"        1442    "tcp" []
  , makeService "cadis-2"        1442    "udp" []
  , makeService "ies-lm"         1443    "tcp" []
  , makeService "ies-lm"         1443    "udp" []
  , makeService "marcam-lm"      1444    "tcp" []
  , makeService "marcam-lm"      1444    "udp" []
  , makeService "proxima-lm"     1445    "tcp" []
  , makeService "proxima-lm"     1445    "udp" []
  , makeService "ora-lm"         1446    "tcp" []
  , makeService "ora-lm"         1446    "udp" []
  , makeService "apri-lm"        1447    "tcp" []
  , makeService "apri-lm"        1447    "udp" []
  , makeService "oc-lm"          1448    "tcp" []
  , makeService "oc-lm"          1448    "udp" []
  , makeService "peport"         1449    "tcp" []
  , makeService "peport"         1449    "udp" []
  , makeService "dwf"            1450    "tcp" []
  , makeService "dwf"            1450    "udp" []
  , makeService "infoman"        1451    "tcp" []
  , makeService "infoman"        1451    "udp" []
  , makeService "gtegsc-lm"      1452    "tcp" []
  , makeService "gtegsc-lm"      1452    "udp" []
  , makeService "genie-lm"       1453    "tcp" []
  , makeService "genie-lm"       1453    "udp" []
  , makeService "interhdl_elmd"  1454    "tcp" ["interhdl-elmd"]
  , makeService "interhdl_elmd"  1454    "udp" ["interhdl-elmd"]
  , makeService "esl-lm"         1455    "tcp" []
  , makeService "esl-lm"         1455    "udp" []
  , makeService "dca"            1456    "tcp" []
  , makeService "dca"            1456    "udp" []
  , makeService "valisys-lm"     1457    "tcp" []
  , makeService "valisys-lm"     1457    "udp" []
  , makeService "nrcabq-lm"      1458    "tcp" []
  , makeService "nrcabq-lm"      1458    "udp" []
  , makeService "proshare1"      1459    "tcp" []
  , makeService "proshare1"      1459    "udp" []
  , makeService "proshare2"      1460    "tcp" []
  , makeService "proshare2"      1460    "udp" []
  , makeService "ibm_wrless_lan" 1461    "tcp" ["ibm-wrless-lan"]
  , makeService "ibm_wrless_lan" 1461    "udp" ["ibm-wrless-lan"]
  , makeService "world-lm"       1462    "tcp" []
  , makeService "world-lm"       1462    "udp" []
  , makeService "nucleus"        1463    "tcp" []
  , makeService "nucleus"        1463    "udp" []
  , makeService "msl_lmd"        1464    "tcp" ["msl-lmd"]
  , makeService "msl_lmd"        1464    "udp" ["msl-lmd"]
  , makeService "pipes"          1465    "tcp" []
  , makeService "pipes"          1465    "udp" []
  , makeService "oceansoft-lm"   1466    "tcp" []
  , makeService "oceansoft-lm"   1466    "udp" []
  , makeService "csdmbase"       1467    "tcp" []
  , makeService "csdmbase"       1467    "udp" []
  , makeService "csdm"           1468    "tcp" []
  , makeService "csdm"           1468    "udp" []
  , makeService "aal-lm"         1469    "tcp" []
  , makeService "aal-lm"         1469    "udp" []
  , makeService "uaiact"         1470    "tcp" []
  , makeService "uaiact"         1470    "udp" []
  , makeService "openmath"       1473    "tcp" []
  , makeService "openmath"       1473    "udp" []
  , makeService "telefinder"     1474    "tcp" []
  , makeService "telefinder"     1474    "udp" []
  , makeService "taligent-lm"    1475    "tcp" []
  , makeService "taligent-lm"    1475    "udp" []
  , makeService "clvm-cfg"       1476    "tcp" []
  , makeService "clvm-cfg"       1476    "udp" []
  , makeService "ms-sna-server"  1477    "tcp" []
  , makeService "ms-sna-server"  1477    "udp" []
  , makeService "ms-sna-base"    1478    "tcp" []
  , makeService "ms-sna-base"    1478    "udp" []
  , makeService "dberegister"    1479    "tcp" []
  , makeService "dberegister"    1479    "udp" []
  , makeService "pacerforum"     1480    "tcp" []
  , makeService "pacerforum"     1480    "udp" []
  , makeService "airs"           1481    "tcp" []
  , makeService "airs"           1481    "udp" []
  , makeService "miteksys-lm"    1482    "tcp" []
  , makeService "miteksys-lm"    1482    "udp" []
  , makeService "afs"            1483    "tcp" []
  , makeService "afs"            1483    "udp" []
  , makeService "confluent"      1484    "tcp" []
  , makeService "confluent"      1484    "udp" []
  , makeService "lansource"      1485    "tcp" []
  , makeService "lansource"      1485    "udp" []
  , makeService "nms_topo_serv"  1486    "tcp" ["nms-topo-serv"]
  , makeService "nms_topo_serv"  1486    "udp" ["nms-topo-serv"]
  , makeService "localinfosrvr"  1487    "tcp" []
  , makeService "localinfosrvr"  1487    "udp" []
  , makeService "docstor"        1488    "tcp" []
  , makeService "docstor"        1488    "udp" []
  , makeService "dmdocbroker"    1489    "tcp" []
  , makeService "dmdocbroker"    1489    "udp" []
  , makeService "insitu-conf"    1490    "tcp" []
  , makeService "insitu-conf"    1490    "udp" []
  , makeService "stone-design-1" 1492    "tcp" []
  , makeService "stone-design-1" 1492    "udp" []
  , makeService "netmap_lm"      1493    "tcp" ["netmap-lm"]
  , makeService "netmap_lm"      1493    "udp" ["netmap-lm"]
  , makeService "cvc"            1495    "tcp" []
  , makeService "cvc"            1495    "udp" []
  , makeService "liberty-lm"     1496    "tcp" []
  , makeService "liberty-lm"     1496    "udp" []
  , makeService "rfx-lm"         1497    "tcp" []
  , makeService "rfx-lm"         1497    "udp" []
  , makeService "sybase-sqlany"  1498    "tcp" []
  , makeService "sybase-sqlany"  1498    "udp" []
  , makeService "fhc"            1499    "tcp" []
  , makeService "fhc"            1499    "udp" []
  , makeService "vlsi-lm"        1500    "tcp" []
  , makeService "vlsi-lm"        1500    "udp" []
  , makeService "saiscm"         1501    "tcp" []
  , makeService "saiscm"         1501    "udp" []
  , makeService "shivadiscovery" 1502    "tcp" []
  , makeService "shivadiscovery" 1502    "udp" []
  , makeService "imtc-mcs"       1503    "tcp" []
  , makeService "imtc-mcs"       1503    "udp" []
  , makeService "evb-elm"        1504    "tcp" []
  , makeService "evb-elm"        1504    "udp" []
  , makeService "funkproxy"      1505    "tcp" []
  , makeService "funkproxy"      1505    "udp" []
  , makeService "utcd"           1506    "tcp" []
  , makeService "utcd"           1506    "udp" []
  , makeService "symplex"        1507    "tcp" []
  , makeService "symplex"        1507    "udp" []
  , makeService "diagmond"       1508    "tcp" []
  , makeService "diagmond"       1508    "udp" []
  , makeService "robcad-lm"      1509    "tcp" []
  , makeService "robcad-lm"      1509    "udp" []
  , makeService "mvx-lm"         1510    "tcp" []
  , makeService "mvx-lm"         1510    "udp" []
  , makeService "3l-l1"          1511    "tcp" []
  , makeService "3l-l1"          1511    "udp" []
  , makeService "fujitsu-dtc"    1513    "tcp" []
  , makeService "fujitsu-dtc"    1513    "udp" []
  , makeService "fujitsu-dtcns"  1514    "tcp" []
  , makeService "fujitsu-dtcns"  1514    "udp" []
  , makeService "ifor-protocol"  1515    "tcp" []
  , makeService "ifor-protocol"  1515    "udp" []
  , makeService "vpad"           1516    "tcp" []
  , makeService "vpad"           1516    "udp" []
  , makeService "vpac"           1517    "tcp" []
  , makeService "vpac"           1517    "udp" []
  , makeService "vpvd"           1518    "tcp" []
  , makeService "vpvd"           1518    "udp" []
  , makeService "vpvc"           1519    "tcp" []
  , makeService "vpvc"           1519    "udp" []
  , makeService "atm-zip-office" 1520    "tcp" []
  , makeService "atm-zip-office" 1520    "udp" []
  , makeService "ncube-lm"       1521    "tcp" []
  , makeService "ncube-lm"       1521    "udp" []
  , makeService "ricardo-lm"     1522    "tcp" []
  , makeService "ricardo-lm"     1522    "udp" []
  , makeService "cichild-lm"     1523    "tcp" []
  , makeService "cichild-lm"     1523    "udp" []
  , makeService "pdap-np"        1526    "tcp" []
  , makeService "pdap-np"        1526    "udp" []
  , makeService "tlisrv"         1527    "tcp" []
  , makeService "tlisrv"         1527    "udp" []
  , makeService "coauthor"       1529    "udp" []
  , makeService "rap-service"    1530    "tcp" []
  , makeService "rap-service"    1530    "udp" []
  , makeService "rap-listen"     1531    "tcp" []
  , makeService "rap-listen"     1531    "udp" []
  , makeService "miroconnect"    1532    "tcp" []
  , makeService "miroconnect"    1532    "udp" []
  , makeService "virtual-places" 1533    "tcp" []
  , makeService "virtual-places" 1533    "udp" []
  , makeService "micromuse-lm"   1534    "tcp" []
  , makeService "micromuse-lm"   1534    "udp" []
  , makeService "ampr-info"      1535    "tcp" []
  , makeService "ampr-info"      1535    "udp" []
  , makeService "ampr-inter"     1536    "tcp" []
  , makeService "ampr-inter"     1536    "udp" []
  , makeService "sdsc-lm"        1537    "tcp" []
  , makeService "sdsc-lm"        1537    "udp" []
  , makeService "3ds-lm"         1538    "tcp" []
  , makeService "3ds-lm"         1538    "udp" []
  , makeService "intellistor-lm" 1539    "tcp" []
  , makeService "intellistor-lm" 1539    "udp" []
  , makeService "rds"            1540    "tcp" []
  , makeService "rds"            1540    "udp" []
  , makeService "rds2"           1541    "tcp" []
  , makeService "rds2"           1541    "udp" []
  , makeService "gridgen-elmd"   1542    "tcp" []
  , makeService "gridgen-elmd"   1542    "udp" []
  , makeService "simba-cs"       1543    "tcp" []
  , makeService "simba-cs"       1543    "udp" []
  , makeService "aspeclmd"       1544    "tcp" []
  , makeService "aspeclmd"       1544    "udp" []
  , makeService "vistium-share"  1545    "tcp" []
  , makeService "vistium-share"  1545    "udp" []
  , makeService "abbaccuray"     1546    "tcp" []
  , makeService "abbaccuray"     1546    "udp" []
  , makeService "laplink"        1547    "tcp" []
  , makeService "laplink"        1547    "udp" []
  , makeService "axon-lm"        1548    "tcp" []
  , makeService "axon-lm"        1548    "udp" []
  , makeService "shivahose"      1549    "tcp" []
  , makeService "shivasound"     1549    "udp" []
  , makeService "3m-image-lm"    1550    "tcp" []
  , makeService "3m-image-lm"    1550    "udp" []
  , makeService "hecmtl-db"      1551    "tcp" []
  , makeService "hecmtl-db"      1551    "udp" []
  , makeService "pciarray"       1552    "tcp" []
  , makeService "pciarray"       1552    "udp" []
  , makeService "sna-cs"         1553    "tcp" []
  , makeService "sna-cs"         1553    "udp" []
  , makeService "caci-lm"        1554    "tcp" []
  , makeService "caci-lm"        1554    "udp" []
  , makeService "livelan"        1555    "tcp" []
  , makeService "livelan"        1555    "udp" []
  , makeService "veritas_pbx"    1556    "tcp" ["veritas-pbx"]
  , makeService "veritas_pbx"    1556    "udp" ["veritas-pbx"]
  , makeService "arbortext-lm"   1557    "tcp" []
  , makeService "arbortext-lm"   1557    "udp" []
  , makeService "xingmpeg"       1558    "tcp" []
  , makeService "xingmpeg"       1558    "udp" []
  , makeService "web2host"       1559    "tcp" []
  , makeService "web2host"       1559    "udp" []
  , makeService "asci-val"       1560    "tcp" []
  , makeService "asci-val"       1560    "udp" []
  , makeService "facilityview"   1561    "tcp" []
  , makeService "facilityview"   1561    "udp" []
  , makeService "pconnectmgr"    1562    "tcp" []
  , makeService "pconnectmgr"    1562    "udp" []
  , makeService "cadabra-lm"     1563    "tcp" []
  , makeService "cadabra-lm"     1563    "udp" []
  , makeService "pay-per-view"   1564    "tcp" []
  , makeService "pay-per-view"   1564    "udp" []
  , makeService "winddlb"        1565    "tcp" []
  , makeService "winddlb"        1565    "udp" []
  , makeService "corelvideo"     1566    "tcp" []
  , makeService "corelvideo"     1566    "udp" []
  , makeService "jlicelmd"       1567    "tcp" []
  , makeService "jlicelmd"       1567    "udp" []
  , makeService "tsspmap"        1568    "tcp" []
  , makeService "tsspmap"        1568    "udp" []
  , makeService "ets"            1569    "tcp" []
  , makeService "ets"            1569    "udp" []
  , makeService "orbixd"         1570    "tcp" []
  , makeService "orbixd"         1570    "udp" []
  , makeService "rdb-dbs-disp"   1571    "tcp" []
  , makeService "rdb-dbs-disp"   1571    "udp" []
  , makeService "chip-lm"        1572    "tcp" []
  , makeService "chip-lm"        1572    "udp" []
  , makeService "itscomm-ns"     1573    "tcp" []
  , makeService "itscomm-ns"     1573    "udp" []
  , makeService "mvel-lm"        1574    "tcp" []
  , makeService "mvel-lm"        1574    "udp" []
  , makeService "oraclenames"    1575    "tcp" []
  , makeService "oraclenames"    1575    "udp" []
  , makeService "moldflow-lm"    1576    "tcp" []
  , makeService "moldflow-lm"    1576    "udp" []
  , makeService "hypercube-lm"   1577    "tcp" []
  , makeService "hypercube-lm"   1577    "udp" []
  , makeService "jacobus-lm"     1578    "tcp" []
  , makeService "jacobus-lm"     1578    "udp" []
  , makeService "ioc-sea-lm"     1579    "tcp" []
  , makeService "ioc-sea-lm"     1579    "udp" []
  , makeService "tn-tl-r1"       1580    "tcp" []
  , makeService "tn-tl-r2"       1580    "udp" []
  , makeService "mil-2045-47001" 1581    "tcp" []
  , makeService "mil-2045-47001" 1581    "udp" []
  , makeService "msims"          1582    "tcp" []
  , makeService "msims"          1582    "udp" []
  , makeService "simbaexpress"   1583    "tcp" []
  , makeService "simbaexpress"   1583    "udp" []
  , makeService "tn-tl-fd2"      1584    "tcp" []
  , makeService "tn-tl-fd2"      1584    "udp" []
  , makeService "intv"           1585    "tcp" []
  , makeService "intv"           1585    "udp" []
  , makeService "ibm-abtact"     1586    "tcp" []
  , makeService "ibm-abtact"     1586    "udp" []
  , makeService "pra_elmd"       1587    "tcp" ["pra-elmd"]
  , makeService "pra_elmd"       1587    "udp" ["pra-elmd"]
  , makeService "triquest-lm"    1588    "tcp" []
  , makeService "triquest-lm"    1588    "udp" []
  , makeService "vqp"            1589    "tcp" []
  , makeService "vqp"            1589    "udp" []
  , makeService "gemini-lm"      1590    "tcp" []
  , makeService "gemini-lm"      1590    "udp" []
  , makeService "ncpm-pm"        1591    "tcp" []
  , makeService "ncpm-pm"        1591    "udp" []
  , makeService "commonspace"    1592    "tcp" []
  , makeService "commonspace"    1592    "udp" []
  , makeService "mainsoft-lm"    1593    "tcp" []
  , makeService "mainsoft-lm"    1593    "udp" []
  , makeService "sixtrak"        1594    "tcp" []
  , makeService "sixtrak"        1594    "udp" []
  , makeService "radio"          1595    "tcp" []
  , makeService "radio"          1595    "udp" []
  , makeService "radio-sm"       1596    "tcp" []
  , makeService "radio-bc"       1596    "udp" []
  , makeService "orbplus-iiop"   1597    "tcp" []
  , makeService "orbplus-iiop"   1597    "udp" []
  , makeService "picknfs"        1598    "tcp" []
  , makeService "picknfs"        1598    "udp" []
  , makeService "simbaservices"  1599    "tcp" []
  , makeService "simbaservices"  1599    "udp" []
  , makeService "issd"           1600    "tcp" []
  , makeService "issd"           1600    "udp" []
  , makeService "aas"            1601    "tcp" []
  , makeService "aas"            1601    "udp" []
  , makeService "inspect"        1602    "tcp" []
  , makeService "inspect"        1602    "udp" []
  , makeService "picodbc"        1603    "tcp" []
  , makeService "picodbc"        1603    "udp" []
  , makeService "icabrowser"     1604    "tcp" []
  , makeService "icabrowser"     1604    "udp" []
  , makeService "slp"            1605    "tcp" []
  , makeService "slp"            1605    "udp" []
  , makeService "slm-api"        1606    "tcp" []
  , makeService "slm-api"        1606    "udp" []
  , makeService "stt"            1607    "tcp" []
  , makeService "stt"            1607    "udp" []
  , makeService "smart-lm"       1608    "tcp" []
  , makeService "smart-lm"       1608    "udp" []
  , makeService "isysg-lm"       1609    "tcp" []
  , makeService "isysg-lm"       1609    "udp" []
  , makeService "taurus-wh"      1610    "tcp" []
  , makeService "taurus-wh"      1610    "udp" []
  , makeService "ill"            1611    "tcp" []
  , makeService "ill"            1611    "udp" []
  , makeService "netbill-trans"  1612    "tcp" []
  , makeService "netbill-trans"  1612    "udp" []
  , makeService "netbill-keyrep" 1613    "tcp" []
  , makeService "netbill-keyrep" 1613    "udp" []
  , makeService "netbill-cred"   1614    "tcp" []
  , makeService "netbill-cred"   1614    "udp" []
  , makeService "netbill-auth"   1615    "tcp" []
  , makeService "netbill-auth"   1615    "udp" []
  , makeService "netbill-prod"   1616    "tcp" []
  , makeService "netbill-prod"   1616    "udp" []
  , makeService "nimrod-agent"   1617    "tcp" []
  , makeService "nimrod-agent"   1617    "udp" []
  , makeService "skytelnet"      1618    "tcp" []
  , makeService "skytelnet"      1618    "udp" []
  , makeService "xs-openstorage" 1619    "tcp" []
  , makeService "xs-openstorage" 1619    "udp" []
  , makeService "faxportwinport" 1620    "tcp" []
  , makeService "faxportwinport" 1620    "udp" []
  , makeService "softdataphone"  1621    "tcp" []
  , makeService "softdataphone"  1621    "udp" []
  , makeService "ontime"         1622    "tcp" []
  , makeService "ontime"         1622    "udp" []
  , makeService "jaleosnd"       1623    "tcp" []
  , makeService "jaleosnd"       1623    "udp" []
  , makeService "udp-sr-port"    1624    "tcp" []
  , makeService "udp-sr-port"    1624    "udp" []
  , makeService "svs-omagent"    1625    "tcp" []
  , makeService "svs-omagent"    1625    "udp" []
  , makeService "shockwave"      1626    "tcp" []
  , makeService "shockwave"      1626    "udp" []
  , makeService "t128-gateway"   1627    "tcp" []
  , makeService "t128-gateway"   1627    "udp" []
  , makeService "lontalk-norm"   1628    "tcp" []
  , makeService "lontalk-norm"   1628    "udp" []
  , makeService "lontalk-urgnt"  1629    "tcp" []
  , makeService "lontalk-urgnt"  1629    "udp" []
  , makeService "oraclenet8cman" 1630    "tcp" []
  , makeService "oraclenet8cman" 1630    "udp" []
  , makeService "visitview"      1631    "tcp" []
  , makeService "visitview"      1631    "udp" []
  , makeService "pammratc"       1632    "tcp" []
  , makeService "pammratc"       1632    "udp" []
  , makeService "pammrpc"        1633    "tcp" []
  , makeService "pammrpc"        1633    "udp" []
  , makeService "loaprobe"       1634    "tcp" []
  , makeService "loaprobe"       1634    "udp" []
  , makeService "edb-server1"    1635    "tcp" []
  , makeService "edb-server1"    1635    "udp" []
  , makeService "isdc"           1636    "tcp" []
  , makeService "isdc"           1636    "udp" []
  , makeService "islc"           1637    "tcp" []
  , makeService "islc"           1637    "udp" []
  , makeService "ismc"           1638    "tcp" []
  , makeService "ismc"           1638    "udp" []
  , makeService "cert-initiator" 1639    "tcp" []
  , makeService "cert-initiator" 1639    "udp" []
  , makeService "cert-responder" 1640    "tcp" []
  , makeService "cert-responder" 1640    "udp" []
  , makeService "invision"       1641    "tcp" []
  , makeService "invision"       1641    "udp" []
  , makeService "isis-am"        1642    "tcp" []
  , makeService "isis-am"        1642    "udp" []
  , makeService "isis-ambc"      1643    "tcp" []
  , makeService "isis-ambc"      1643    "udp" []
  , makeService "saiseh"         1644    "tcp" []
  , makeService "saiseh"         1644    "udp" []
  , makeService "rsap"           1647    "tcp" []
  , makeService "rsap"           1647    "udp" []
  , makeService "concurrent-lm"  1648    "tcp" []
  , makeService "concurrent-lm"  1648    "udp" []
  , makeService "nkd"            1650    "tcp" []
  , makeService "nkd"            1650    "udp" []
  , makeService "shiva_confsrvr" 1651    "tcp" ["shiva-confsrvr"]
  , makeService "shiva_confsrvr" 1651    "udp" ["shiva-confsrvr"]
  , makeService "xnmp"           1652    "tcp" []
  , makeService "xnmp"           1652    "udp" []
  , makeService "alphatech-lm"   1653    "tcp" []
  , makeService "alphatech-lm"   1653    "udp" []
  , makeService "stargatealerts" 1654    "tcp" []
  , makeService "stargatealerts" 1654    "udp" []
  , makeService "dec-mbadmin"    1655    "tcp" []
  , makeService "dec-mbadmin"    1655    "udp" []
  , makeService "dec-mbadmin-h"  1656    "tcp" []
  , makeService "dec-mbadmin-h"  1656    "udp" []
  , makeService "fujitsu-mmpdc"  1657    "tcp" []
  , makeService "fujitsu-mmpdc"  1657    "udp" []
  , makeService "sixnetudr"      1658    "tcp" []
  , makeService "sixnetudr"      1658    "udp" []
  , makeService "sg-lm"          1659    "tcp" []
  , makeService "sg-lm"          1659    "udp" []
  , makeService "skip-mc-gikreq" 1660    "tcp" []
  , makeService "skip-mc-gikreq" 1660    "udp" []
  , makeService "netview-aix-1"  1661    "tcp" []
  , makeService "netview-aix-1"  1661    "udp" []
  , makeService "netview-aix-2"  1662    "tcp" []
  , makeService "netview-aix-2"  1662    "udp" []
  , makeService "netview-aix-3"  1663    "tcp" []
  , makeService "netview-aix-3"  1663    "udp" []
  , makeService "netview-aix-4"  1664    "tcp" []
  , makeService "netview-aix-4"  1664    "udp" []
  , makeService "netview-aix-5"  1665    "tcp" []
  , makeService "netview-aix-5"  1665    "udp" []
  , makeService "netview-aix-6"  1666    "tcp" []
  , makeService "netview-aix-6"  1666    "udp" []
  , makeService "netview-aix-7"  1667    "tcp" []
  , makeService "netview-aix-7"  1667    "udp" []
  , makeService "netview-aix-8"  1668    "tcp" []
  , makeService "netview-aix-8"  1668    "udp" []
  , makeService "netview-aix-9"  1669    "tcp" []
  , makeService "netview-aix-9"  1669    "udp" []
  , makeService "netview-aix-10" 1670    "tcp" []
  , makeService "netview-aix-10" 1670    "udp" []
  , makeService "netview-aix-11" 1671    "tcp" []
  , makeService "netview-aix-11" 1671    "udp" []
  , makeService "netview-aix-12" 1672    "tcp" []
  , makeService "netview-aix-12" 1672    "udp" []
  , makeService "proshare-mc-1"  1673    "tcp" []
  , makeService "proshare-mc-1"  1673    "udp" []
  , makeService "proshare-mc-2"  1674    "tcp" []
  , makeService "proshare-mc-2"  1674    "udp" []
  , makeService "pdp"            1675    "tcp" []
  , makeService "pdp"            1675    "udp" []
  , makeService "netcomm1"       1676    "tcp" []
  , makeService "netcomm2"       1676    "udp" []
  , makeService "groupwise"      1677    "tcp" []
  , makeService "groupwise"      1677    "udp" []
  , makeService "prolink"        1678    "tcp" []
  , makeService "prolink"        1678    "udp" []
  , makeService "darcorp-lm"     1679    "tcp" []
  , makeService "darcorp-lm"     1679    "udp" []
  , makeService "microcom-sbp"   1680    "tcp" []
  , makeService "microcom-sbp"   1680    "udp" []
  , makeService "sd-elmd"        1681    "tcp" []
  , makeService "sd-elmd"        1681    "udp" []
  , makeService "lanyon-lantern" 1682    "tcp" []
  , makeService "lanyon-lantern" 1682    "udp" []
  , makeService "ncpm-hip"       1683    "tcp" []
  , makeService "ncpm-hip"       1683    "udp" []
  , makeService "snaresecure"    1684    "tcp" []
  , makeService "snaresecure"    1684    "udp" []
  , makeService "n2nremote"      1685    "tcp" []
  , makeService "n2nremote"      1685    "udp" []
  , makeService "cvmon"          1686    "tcp" []
  , makeService "cvmon"          1686    "udp" []
  , makeService "nsjtp-ctrl"     1687    "tcp" []
  , makeService "nsjtp-ctrl"     1687    "udp" []
  , makeService "nsjtp-data"     1688    "tcp" []
  , makeService "nsjtp-data"     1688    "udp" []
  , makeService "firefox"        1689    "tcp" []
  , makeService "firefox"        1689    "udp" []
  , makeService "ng-umds"        1690    "tcp" []
  , makeService "ng-umds"        1690    "udp" []
  , makeService "empire-empuma"  1691    "tcp" []
  , makeService "empire-empuma"  1691    "udp" []
  , makeService "sstsys-lm"      1692    "tcp" []
  , makeService "sstsys-lm"      1692    "udp" []
  , makeService "rrirtr"         1693    "tcp" []
  , makeService "rrirtr"         1693    "udp" []
  , makeService "rrimwm"         1694    "tcp" []
  , makeService "rrimwm"         1694    "udp" []
  , makeService "rrilwm"         1695    "tcp" []
  , makeService "rrilwm"         1695    "udp" []
  , makeService "rrifmm"         1696    "tcp" []
  , makeService "rrifmm"         1696    "udp" []
  , makeService "rrisat"         1697    "tcp" []
  , makeService "rrisat"         1697    "udp" []
  , makeService "rsvp-encap-1"   1698    "tcp" []
  , makeService "rsvp-encap-1"   1698    "udp" []
  , makeService "rsvp-encap-2"   1699    "tcp" []
  , makeService "rsvp-encap-2"   1699    "udp" []
  , makeService "mps-raft"       1700    "tcp" []
  , makeService "mps-raft"       1700    "udp" []
  , makeService "deskshare"      1702    "tcp" []
  , makeService "deskshare"      1702    "udp" []
  , makeService "hb-engine"      1703    "tcp" []
  , makeService "hb-engine"      1703    "udp" []
  , makeService "bcs-broker"     1704    "tcp" []
  , makeService "bcs-broker"     1704    "udp" []
  , makeService "slingshot"      1705    "tcp" []
  , makeService "slingshot"      1705    "udp" []
  , makeService "jetform"        1706    "tcp" []
  , makeService "jetform"        1706    "udp" []
  , makeService "vdmplay"        1707    "tcp" []
  , makeService "vdmplay"        1707    "udp" []
  , makeService "gat-lmd"        1708    "tcp" []
  , makeService "gat-lmd"        1708    "udp" []
  , makeService "centra"         1709    "tcp" []
  , makeService "centra"         1709    "udp" []
  , makeService "impera"         1710    "tcp" []
  , makeService "impera"         1710    "udp" []
  , makeService "pptconference"  1711    "tcp" []
  , makeService "pptconference"  1711    "udp" []
  , makeService "registrar"      1712    "tcp" []
  , makeService "registrar"      1712    "udp" []
  , makeService "conferencetalk" 1713    "tcp" []
  , makeService "conferencetalk" 1713    "udp" []
  , makeService "sesi-lm"        1714    "tcp" []
  , makeService "sesi-lm"        1714    "udp" []
  , makeService "houdini-lm"     1715    "tcp" []
  , makeService "houdini-lm"     1715    "udp" []
  , makeService "xmsg"           1716    "tcp" []
  , makeService "xmsg"           1716    "udp" []
  , makeService "fj-hdnet"       1717    "tcp" []
  , makeService "fj-hdnet"       1717    "udp" []
  , makeService "caicci"         1721    "tcp" []
  , makeService "caicci"         1721    "udp" []
  , makeService "hks-lm"         1722    "tcp" []
  , makeService "hks-lm"         1722    "udp" []
  , makeService "pptp"           1723    "tcp" []
  , makeService "pptp"           1723    "udp" []
  , makeService "csbphonemaster" 1724    "tcp" []
  , makeService "csbphonemaster" 1724    "udp" []
  , makeService "iden-ralp"      1725    "tcp" []
  , makeService "iden-ralp"      1725    "udp" []
  , makeService "iberiagames"    1726    "tcp" []
  , makeService "iberiagames"    1726    "udp" []
  , makeService "winddx"         1727    "tcp" []
  , makeService "winddx"         1727    "udp" []
  , makeService "telindus"       1728    "tcp" []
  , makeService "telindus"       1728    "udp" []
  , makeService "citynl"         1729    "tcp" []
  , makeService "citynl"         1729    "udp" []
  , makeService "roketz"         1730    "tcp" []
  , makeService "roketz"         1730    "udp" []
  , makeService "msiccp"         1731    "tcp" []
  , makeService "msiccp"         1731    "udp" []
  , makeService "proxim"         1732    "tcp" []
  , makeService "proxim"         1732    "udp" []
  , makeService "siipat"         1733    "tcp" []
  , makeService "siipat"         1733    "udp" []
  , makeService "cambertx-lm"    1734    "tcp" []
  , makeService "cambertx-lm"    1734    "udp" []
  , makeService "privatechat"    1735    "tcp" []
  , makeService "privatechat"    1735    "udp" []
  , makeService "street-stream"  1736    "tcp" []
  , makeService "street-stream"  1736    "udp" []
  , makeService "ultimad"        1737    "tcp" []
  , makeService "ultimad"        1737    "udp" []
  , makeService "gamegen1"       1738    "tcp" []
  , makeService "gamegen1"       1738    "udp" []
  , makeService "webaccess"      1739    "tcp" []
  , makeService "webaccess"      1739    "udp" []
  , makeService "encore"         1740    "tcp" []
  , makeService "encore"         1740    "udp" []
  , makeService "cisco-net-mgmt" 1741    "tcp" []
  , makeService "cisco-net-mgmt" 1741    "udp" []
  , makeService "3Com-nsd"       1742    "tcp" []
  , makeService "3Com-nsd"       1742    "udp" []
  , makeService "cinegrfx-lm"    1743    "tcp" []
  , makeService "cinegrfx-lm"    1743    "udp" []
  , makeService "ncpm-ft"        1744    "tcp" []
  , makeService "ncpm-ft"        1744    "udp" []
  , makeService "remote-winsock" 1745    "tcp" []
  , makeService "remote-winsock" 1745    "udp" []
  , makeService "ftrapid-1"      1746    "tcp" []
  , makeService "ftrapid-1"      1746    "udp" []
  , makeService "ftrapid-2"      1747    "tcp" []
  , makeService "ftrapid-2"      1747    "udp" []
  , makeService "oracle-em1"     1748    "tcp" []
  , makeService "oracle-em1"     1748    "udp" []
  , makeService "aspen-services" 1749    "tcp" []
  , makeService "aspen-services" 1749    "udp" []
  , makeService "sslp"           1750    "tcp" []
  , makeService "sslp"           1750    "udp" []
  , makeService "swiftnet"       1751    "tcp" []
  , makeService "swiftnet"       1751    "udp" []
  , makeService "lofr-lm"        1752    "tcp" []
  , makeService "lofr-lm"        1752    "udp" []
  , makeService "predatar-comms" 1753    "tcp" []
  , makeService "oracle-em2"     1754    "tcp" []
  , makeService "oracle-em2"     1754    "udp" []
  , makeService "ms-streaming"   1755    "tcp" []
  , makeService "ms-streaming"   1755    "udp" []
  , makeService "capfast-lmd"    1756    "tcp" []
  , makeService "capfast-lmd"    1756    "udp" []
  , makeService "cnhrp"          1757    "tcp" []
  , makeService "cnhrp"          1757    "udp" []
  , makeService "spss-lm"        1759    "tcp" []
  , makeService "www-ldap-gw"    1760    "tcp" []
  , makeService "www-ldap-gw"    1760    "udp" []
  , makeService "cft-0"          1761    "tcp" []
  , makeService "cft-0"          1761    "udp" []
  , makeService "cft-1"          1762    "tcp" []
  , makeService "cft-1"          1762    "udp" []
  , makeService "cft-2"          1763    "tcp" []
  , makeService "cft-2"          1763    "udp" []
  , makeService "cft-3"          1764    "tcp" []
  , makeService "cft-3"          1764    "udp" []
  , makeService "cft-4"          1765    "tcp" []
  , makeService "cft-4"          1765    "udp" []
  , makeService "cft-5"          1766    "tcp" []
  , makeService "cft-5"          1766    "udp" []
  , makeService "cft-6"          1767    "tcp" []
  , makeService "cft-6"          1767    "udp" []
  , makeService "cft-7"          1768    "tcp" []
  , makeService "cft-7"          1768    "udp" []
  , makeService "bmc-net-adm"    1769    "tcp" []
  , makeService "bmc-net-adm"    1769    "udp" []
  , makeService "bmc-net-svc"    1770    "tcp" []
  , makeService "bmc-net-svc"    1770    "udp" []
  , makeService "vaultbase"      1771    "tcp" []
  , makeService "vaultbase"      1771    "udp" []
  , makeService "essweb-gw"      1772    "tcp" []
  , makeService "essweb-gw"      1772    "udp" []
  , makeService "kmscontrol"     1773    "tcp" []
  , makeService "kmscontrol"     1773    "udp" []
  , makeService "global-dtserv"  1774    "tcp" []
  , makeService "global-dtserv"  1774    "udp" []
  , makeService "femis"          1776    "tcp" []
  , makeService "femis"          1776    "udp" []
  , makeService "powerguardian"  1777    "tcp" []
  , makeService "powerguardian"  1777    "udp" []
  , makeService "prodigy-intrnet"1778    "tcp" []
  , makeService "prodigy-intrnet"1778    "udp" []
  , makeService "pharmasoft"     1779    "tcp" []
  , makeService "pharmasoft"     1779    "udp" []
  , makeService "dpkeyserv"      1780    "tcp" []
  , makeService "dpkeyserv"      1780    "udp" []
  , makeService "answersoft-lm"  1781    "tcp" []
  , makeService "answersoft-lm"  1781    "udp" []
  , makeService "hp-hcip"        1782    "tcp" []
  , makeService "hp-hcip"        1782    "udp" []
  , makeService "finle-lm"       1784    "tcp" []
  , makeService "finle-lm"       1784    "udp" []
  , makeService "windlm"         1785    "tcp" []
  , makeService "windlm"         1785    "udp" []
  , makeService "funk-logger"    1786    "tcp" []
  , makeService "funk-logger"    1786    "udp" []
  , makeService "funk-license"   1787    "tcp" []
  , makeService "funk-license"   1787    "udp" []
  , makeService "psmond"         1788    "tcp" []
  , makeService "psmond"         1788    "udp" []
  , makeService "ea1"            1791    "tcp" []
  , makeService "ea1"            1791    "udp" []
  , makeService "ibm-dt-2"       1792    "tcp" []
  , makeService "ibm-dt-2"       1792    "udp" []
  , makeService "rsc-robot"      1793    "tcp" []
  , makeService "rsc-robot"      1793    "udp" []
  , makeService "cera-bcm"       1794    "tcp" []
  , makeService "cera-bcm"       1794    "udp" []
  , makeService "dpi-proxy"      1795    "tcp" []
  , makeService "dpi-proxy"      1795    "udp" []
  , makeService "vocaltec-admin" 1796    "tcp" []
  , makeService "vocaltec-admin" 1796    "udp" []
  , makeService "etp"            1798    "tcp" []
  , makeService "etp"            1798    "udp" []
  , makeService "netrisk"        1799    "tcp" []
  , makeService "netrisk"        1799    "udp" []
  , makeService "ansys-lm"       1800    "tcp" []
  , makeService "ansys-lm"       1800    "udp" []
  , makeService "msmq"           1801    "tcp" []
  , makeService "msmq"           1801    "udp" []
  , makeService "concomp1"       1802    "tcp" []
  , makeService "concomp1"       1802    "udp" []
  , makeService "hp-hcip-gwy"    1803    "tcp" []
  , makeService "hp-hcip-gwy"    1803    "udp" []
  , makeService "enl"            1804    "tcp" []
  , makeService "enl"            1804    "udp" []
  , makeService "enl-name"       1805    "tcp" []
  , makeService "enl-name"       1805    "udp" []
  , makeService "musiconline"    1806    "tcp" []
  , makeService "musiconline"    1806    "udp" []
  , makeService "fhsp"           1807    "tcp" []
  , makeService "fhsp"           1807    "udp" []
  , makeService "oracle-vp2"     1808    "tcp" []
  , makeService "oracle-vp2"     1808    "udp" []
  , makeService "oracle-vp1"     1809    "tcp" []
  , makeService "oracle-vp1"     1809    "udp" []
  , makeService "jerand-lm"      1810    "tcp" []
  , makeService "jerand-lm"      1810    "udp" []
  , makeService "scientia-sdb"   1811    "tcp" []
  , makeService "scientia-sdb"   1811    "udp" []
  , makeService "tdp-suite"      1814    "tcp" []
  , makeService "tdp-suite"      1814    "udp" []
  , makeService "mmpft"          1815    "tcp" []
  , makeService "mmpft"          1815    "udp" []
  , makeService "harp"           1816    "tcp" []
  , makeService "harp"           1816    "udp" []
  , makeService "rkb-oscs"       1817    "tcp" []
  , makeService "rkb-oscs"       1817    "udp" []
  , makeService "etftp"          1818    "tcp" []
  , makeService "etftp"          1818    "udp" []
  , makeService "plato-lm"       1819    "tcp" []
  , makeService "plato-lm"       1819    "udp" []
  , makeService "mcagent"        1820    "tcp" []
  , makeService "mcagent"        1820    "udp" []
  , makeService "donnyworld"     1821    "tcp" []
  , makeService "donnyworld"     1821    "udp" []
  , makeService "es-elmd"        1822    "tcp" []
  , makeService "es-elmd"        1822    "udp" []
  , makeService "unisys-lm"      1823    "tcp" []
  , makeService "unisys-lm"      1823    "udp" []
  , makeService "metrics-pas"    1824    "tcp" []
  , makeService "metrics-pas"    1824    "udp" []
  , makeService "direcpc-video"  1825    "tcp" []
  , makeService "direcpc-video"  1825    "udp" []
  , makeService "ardt"           1826    "tcp" []
  , makeService "ardt"           1826    "udp" []
  , makeService "asi"            1827    "tcp" []
  , makeService "asi"            1827    "udp" []
  , makeService "itm-mcell-u"    1828    "tcp" []
  , makeService "itm-mcell-u"    1828    "udp" []
  , makeService "optika-emedia"  1829    "tcp" []
  , makeService "optika-emedia"  1829    "udp" []
  , makeService "net8-cman"      1830    "tcp" []
  , makeService "net8-cman"      1830    "udp" []
  , makeService "myrtle"         1831    "tcp" []
  , makeService "myrtle"         1831    "udp" []
  , makeService "tht-treasure"   1832    "tcp" []
  , makeService "tht-treasure"   1832    "udp" []
  , makeService "udpradio"       1833    "tcp" []
  , makeService "udpradio"       1833    "udp" []
  , makeService "ardusuni"       1834    "tcp" []
  , makeService "ardusuni"       1834    "udp" []
  , makeService "ardusmul"       1835    "tcp" []
  , makeService "ardusmul"       1835    "udp" []
  , makeService "ste-smsc"       1836    "tcp" []
  , makeService "ste-smsc"       1836    "udp" []
  , makeService "csoft1"         1837    "tcp" []
  , makeService "csoft1"         1837    "udp" []
  , makeService "talnet"         1838    "tcp" []
  , makeService "talnet"         1838    "udp" []
  , makeService "netopia-vo1"    1839    "tcp" []
  , makeService "netopia-vo1"    1839    "udp" []
  , makeService "netopia-vo2"    1840    "tcp" []
  , makeService "netopia-vo2"    1840    "udp" []
  , makeService "netopia-vo3"    1841    "tcp" []
  , makeService "netopia-vo3"    1841    "udp" []
  , makeService "netopia-vo4"    1842    "tcp" []
  , makeService "netopia-vo4"    1842    "udp" []
  , makeService "netopia-vo5"    1843    "tcp" []
  , makeService "netopia-vo5"    1843    "udp" []
  , makeService "direcpc-dll"    1844    "tcp" []
  , makeService "direcpc-dll"    1844    "udp" []
  , makeService "altalink"       1845    "tcp" []
  , makeService "altalink"       1845    "udp" []
  , makeService "tunstall-pnc"   1846    "tcp" []
  , makeService "tunstall-pnc"   1846    "udp" []
  , makeService "slp-notify"     1847    "tcp" []
  , makeService "slp-notify"     1847    "udp" []
  , makeService "fjdocdist"      1848    "tcp" []
  , makeService "fjdocdist"      1848    "udp" []
  , makeService "alpha-sms"      1849    "tcp" []
  , makeService "alpha-sms"      1849    "udp" []
  , makeService "gsi"            1850    "tcp" []
  , makeService "gsi"            1850    "udp" []
  , makeService "ctcd"           1851    "tcp" []
  , makeService "ctcd"           1851    "udp" []
  , makeService "virtual-time"   1852    "tcp" []
  , makeService "virtual-time"   1852    "udp" []
  , makeService "vids-avtp"      1853    "tcp" []
  , makeService "vids-avtp"      1853    "udp" []
  , makeService "buddy-draw"     1854    "tcp" []
  , makeService "buddy-draw"     1854    "udp" []
  , makeService "fiorano-rtrsvc" 1855    "tcp" []
  , makeService "fiorano-rtrsvc" 1855    "udp" []
  , makeService "fiorano-msgsvc" 1856    "tcp" []
  , makeService "fiorano-msgsvc" 1856    "udp" []
  , makeService "datacaptor"     1857    "tcp" []
  , makeService "datacaptor"     1857    "udp" []
  , makeService "privateark"     1858    "tcp" []
  , makeService "privateark"     1858    "udp" []
  , makeService "gammafetchsvr"  1859    "tcp" []
  , makeService "gammafetchsvr"  1859    "udp" []
  , makeService "sunscalar-svc"  1860    "tcp" []
  , makeService "sunscalar-svc"  1860    "udp" []
  , makeService "lecroy-vicp"    1861    "tcp" []
  , makeService "lecroy-vicp"    1861    "udp" []
  , makeService "mysql-cm-agent" 1862    "tcp" []
  , makeService "mysql-cm-agent" 1862    "udp" []
  , makeService "msnp"           1863    "tcp" []
  , makeService "msnp"           1863    "udp" []
  , makeService "paradym-31port" 1864    "tcp" []
  , makeService "paradym-31port" 1864    "udp" []
  , makeService "entp"           1865    "tcp" []
  , makeService "entp"           1865    "udp" []
  , makeService "swrmi"          1866    "tcp" []
  , makeService "swrmi"          1866    "udp" []
  , makeService "udrive"         1867    "tcp" []
  , makeService "udrive"         1867    "udp" []
  , makeService "viziblebrowser" 1868    "tcp" []
  , makeService "viziblebrowser" 1868    "udp" []
  , makeService "transact"       1869    "tcp" []
  , makeService "transact"       1869    "udp" []
  , makeService "sunscalar-dns"  1870    "tcp" []
  , makeService "sunscalar-dns"  1870    "udp" []
  , makeService "canocentral0"   1871    "tcp" []
  , makeService "canocentral0"   1871    "udp" []
  , makeService "canocentral1"   1872    "tcp" []
  , makeService "canocentral1"   1872    "udp" []
  , makeService "fjmpjps"        1873    "tcp" []
  , makeService "fjmpjps"        1873    "udp" []
  , makeService "fjswapsnp"      1874    "tcp" []
  , makeService "fjswapsnp"      1874    "udp" []
  , makeService "westell-stats"  1875    "tcp" []
  , makeService "westell-stats"  1875    "udp" []
  , makeService "ewcappsrv"      1876    "tcp" []
  , makeService "ewcappsrv"      1876    "udp" []
  , makeService "hp-webqosdb"    1877    "tcp" []
  , makeService "hp-webqosdb"    1877    "udp" []
  , makeService "drmsmc"         1878    "tcp" []
  , makeService "drmsmc"         1878    "udp" []
  , makeService "nettgain-nms"   1879    "tcp" []
  , makeService "nettgain-nms"   1879    "udp" []
  , makeService "vsat-control"   1880    "tcp" []
  , makeService "vsat-control"   1880    "udp" []
  , makeService "ibm-mqseries2"  1881    "tcp" []
  , makeService "ibm-mqseries2"  1881    "udp" []
  , makeService "ecsqdmn"        1882    "tcp" []
  , makeService "ecsqdmn"        1882    "udp" []
  , makeService "ibm-mqisdp"     1883    "tcp" []
  , makeService "ibm-mqisdp"     1883    "udp" []
  , makeService "idmaps"         1884    "tcp" []
  , makeService "idmaps"         1884    "udp" []
  , makeService "vrtstrapserver" 1885    "tcp" []
  , makeService "vrtstrapserver" 1885    "udp" []
  , makeService "leoip"          1886    "tcp" []
  , makeService "leoip"          1886    "udp" []
  , makeService "filex-lport"    1887    "tcp" []
  , makeService "filex-lport"    1887    "udp" []
  , makeService "ncconfig"       1888    "tcp" []
  , makeService "ncconfig"       1888    "udp" []
  , makeService "unify-adapter"  1889    "tcp" []
  , makeService "unify-adapter"  1889    "udp" []
  , makeService "wilkenlistener" 1890    "tcp" []
  , makeService "wilkenlistener" 1890    "udp" []
  , makeService "childkey-notif" 1891    "tcp" []
  , makeService "childkey-notif" 1891    "udp" []
  , makeService "childkey-ctrl"  1892    "tcp" []
  , makeService "childkey-ctrl"  1892    "udp" []
  , makeService "elad"           1893    "tcp" []
  , makeService "elad"           1893    "udp" []
  , makeService "o2server-port"  1894    "tcp" []
  , makeService "o2server-port"  1894    "udp" []
  , makeService "b-novative-ls"  1896    "tcp" []
  , makeService "b-novative-ls"  1896    "udp" []
  , makeService "metaagent"      1897    "tcp" []
  , makeService "metaagent"      1897    "udp" []
  , makeService "cymtec-port"    1898    "tcp" []
  , makeService "cymtec-port"    1898    "udp" []
  , makeService "mc2studios"     1899    "tcp" []
  , makeService "mc2studios"     1899    "udp" []
  , makeService "ssdp"           1900    "tcp" []
  , makeService "ssdp"           1900    "udp" []
  , makeService "fjicl-tep-a"    1901    "tcp" []
  , makeService "fjicl-tep-a"    1901    "udp" []
  , makeService "fjicl-tep-b"    1902    "tcp" []
  , makeService "fjicl-tep-b"    1902    "udp" []
  , makeService "linkname"       1903    "tcp" []
  , makeService "linkname"       1903    "udp" []
  , makeService "fjicl-tep-c"    1904    "tcp" []
  , makeService "fjicl-tep-c"    1904    "udp" []
  , makeService "sugp"           1905    "tcp" []
  , makeService "sugp"           1905    "udp" []
  , makeService "tpmd"           1906    "tcp" []
  , makeService "tpmd"           1906    "udp" []
  , makeService "intrastar"      1907    "tcp" []
  , makeService "intrastar"      1907    "udp" []
  , makeService "dawn"           1908    "tcp" []
  , makeService "dawn"           1908    "udp" []
  , makeService "global-wlink"   1909    "tcp" []
  , makeService "global-wlink"   1909    "udp" []
  , makeService "ultrabac"       1910    "tcp" []
  , makeService "ultrabac"       1910    "udp" []
  , makeService "rhp-iibp"       1912    "tcp" []
  , makeService "rhp-iibp"       1912    "udp" []
  , makeService "armadp"         1913    "tcp" []
  , makeService "armadp"         1913    "udp" []
  , makeService "elm-momentum"   1914    "tcp" []
  , makeService "elm-momentum"   1914    "udp" []
  , makeService "facelink"       1915    "tcp" []
  , makeService "facelink"       1915    "udp" []
  , makeService "persona"        1916    "tcp" []
  , makeService "persona"        1916    "udp" []
  , makeService "noagent"        1917    "tcp" []
  , makeService "noagent"        1917    "udp" []
  , makeService "can-nds"        1918    "tcp" []
  , makeService "can-nds"        1918    "udp" []
  , makeService "can-dch"        1919    "tcp" []
  , makeService "can-dch"        1919    "udp" []
  , makeService "can-ferret"     1920    "tcp" []
  , makeService "can-ferret"     1920    "udp" []
  , makeService "noadmin"        1921    "tcp" []
  , makeService "noadmin"        1921    "udp" []
  , makeService "tapestry"       1922    "tcp" []
  , makeService "tapestry"       1922    "udp" []
  , makeService "spice"          1923    "tcp" []
  , makeService "spice"          1923    "udp" []
  , makeService "xiip"           1924    "tcp" []
  , makeService "xiip"           1924    "udp" []
  , makeService "discovery-port" 1925    "tcp" []
  , makeService "discovery-port" 1925    "udp" []
  , makeService "egs"            1926    "tcp" []
  , makeService "egs"            1926    "udp" []
  , makeService "videte-cipc"    1927    "tcp" []
  , makeService "videte-cipc"    1927    "udp" []
  , makeService "emsd-port"      1928    "tcp" []
  , makeService "emsd-port"      1928    "udp" []
  , makeService "bandwiz-system" 1929    "tcp" []
  , makeService "bandwiz-system" 1929    "udp" []
  , makeService "driveappserver" 1930    "tcp" []
  , makeService "driveappserver" 1930    "udp" []
  , makeService "amdsched"       1931    "tcp" []
  , makeService "amdsched"       1931    "udp" []
  , makeService "ctt-broker"     1932    "tcp" []
  , makeService "ctt-broker"     1932    "udp" []
  , makeService "xmapi"          1933    "tcp" []
  , makeService "xmapi"          1933    "udp" []
  , makeService "xaapi"          1934    "tcp" []
  , makeService "xaapi"          1934    "udp" []
  , makeService "macromedia-fcs" 1935    "tcp" []
  , makeService "macromedia-fcs" 1935    "udp" []
  , makeService "jetcmeserver"   1936    "tcp" []
  , makeService "jetcmeserver"   1936    "udp" []
  , makeService "jwserver"       1937    "tcp" []
  , makeService "jwserver"       1937    "udp" []
  , makeService "jwclient"       1938    "tcp" []
  , makeService "jwclient"       1938    "udp" []
  , makeService "jvserver"       1939    "tcp" []
  , makeService "jvserver"       1939    "udp" []
  , makeService "jvclient"       1940    "tcp" []
  , makeService "jvclient"       1940    "udp" []
  , makeService "dic-aida"       1941    "tcp" []
  , makeService "dic-aida"       1941    "udp" []
  , makeService "res"            1942    "tcp" []
  , makeService "res"            1942    "udp" []
  , makeService "beeyond-media"  1943    "tcp" []
  , makeService "beeyond-media"  1943    "udp" []
  , makeService "close-combat"   1944    "tcp" []
  , makeService "close-combat"   1944    "udp" []
  , makeService "dialogic-elmd"  1945    "tcp" []
  , makeService "dialogic-elmd"  1945    "udp" []
  , makeService "tekpls"         1946    "tcp" []
  , makeService "tekpls"         1946    "udp" []
  , makeService "sentinelsrm"    1947    "tcp" []
  , makeService "sentinelsrm"    1947    "udp" []
  , makeService "eye2eye"        1948    "tcp" []
  , makeService "eye2eye"        1948    "udp" []
  , makeService "ismaeasdaqlive" 1949    "tcp" []
  , makeService "ismaeasdaqlive" 1949    "udp" []
  , makeService "ismaeasdaqtest" 1950    "tcp" []
  , makeService "ismaeasdaqtest" 1950    "udp" []
  , makeService "bcs-lmserver"   1951    "tcp" []
  , makeService "bcs-lmserver"   1951    "udp" []
  , makeService "mpnjsc"         1952    "tcp" []
  , makeService "mpnjsc"         1952    "udp" []
  , makeService "rapidbase"      1953    "tcp" []
  , makeService "rapidbase"      1953    "udp" []
  , makeService "abr-api"        1954    "tcp" []
  , makeService "abr-api"        1954    "udp" []
  , makeService "abr-secure"     1955    "tcp" []
  , makeService "abr-secure"     1955    "udp" []
  , makeService "vrtl-vmf-ds"    1956    "tcp" []
  , makeService "vrtl-vmf-ds"    1956    "udp" []
  , makeService "unix-status"    1957    "tcp" []
  , makeService "unix-status"    1957    "udp" []
  , makeService "dxadmind"       1958    "tcp" []
  , makeService "dxadmind"       1958    "udp" []
  , makeService "simp-all"       1959    "tcp" []
  , makeService "simp-all"       1959    "udp" []
  , makeService "nasmanager"     1960    "tcp" []
  , makeService "nasmanager"     1960    "udp" []
  , makeService "bts-appserver"  1961    "tcp" []
  , makeService "bts-appserver"  1961    "udp" []
  , makeService "biap-mp"        1962    "tcp" []
  , makeService "biap-mp"        1962    "udp" []
  , makeService "webmachine"     1963    "tcp" []
  , makeService "webmachine"     1963    "udp" []
  , makeService "solid-e-engine" 1964    "tcp" []
  , makeService "solid-e-engine" 1964    "udp" []
  , makeService "tivoli-npm"     1965    "tcp" []
  , makeService "tivoli-npm"     1965    "udp" []
  , makeService "slush"          1966    "tcp" []
  , makeService "slush"          1966    "udp" []
  , makeService "sns-quote"      1967    "tcp" []
  , makeService "sns-quote"      1967    "udp" []
  , makeService "lipsinc"        1968    "tcp" []
  , makeService "lipsinc"        1968    "udp" []
  , makeService "lipsinc1"       1969    "tcp" []
  , makeService "lipsinc1"       1969    "udp" []
  , makeService "netop-rc"       1970    "tcp" []
  , makeService "netop-rc"       1970    "udp" []
  , makeService "netop-school"   1971    "tcp" []
  , makeService "netop-school"   1971    "udp" []
  , makeService "intersys-cache" 1972    "tcp" []
  , makeService "intersys-cache" 1972    "udp" []
  , makeService "dlsrap"         1973    "tcp" []
  , makeService "dlsrap"         1973    "udp" []
  , makeService "drp"            1974    "tcp" []
  , makeService "drp"            1974    "udp" []
  , makeService "tcoflashagent"  1975    "tcp" []
  , makeService "tcoflashagent"  1975    "udp" []
  , makeService "tcoregagent"    1976    "tcp" []
  , makeService "tcoregagent"    1976    "udp" []
  , makeService "tcoaddressbook" 1977    "tcp" []
  , makeService "tcoaddressbook" 1977    "udp" []
  , makeService "unisql"         1978    "tcp" []
  , makeService "unisql"         1978    "udp" []
  , makeService "unisql-java"    1979    "tcp" []
  , makeService "unisql-java"    1979    "udp" []
  , makeService "pearldoc-xact"  1980    "tcp" []
  , makeService "pearldoc-xact"  1980    "udp" []
  , makeService "p2pq"           1981    "tcp" []
  , makeService "p2pq"           1981    "udp" []
  , makeService "estamp"         1982    "tcp" []
  , makeService "estamp"         1982    "udp" []
  , makeService "lhtp"           1983    "tcp" []
  , makeService "lhtp"           1983    "udp" []
  , makeService "bb"             1984    "tcp" []
  , makeService "bb"             1984    "udp" []
  , makeService "tr-rsrb-p1"     1987    "tcp" []
  , makeService "tr-rsrb-p1"     1987    "udp" []
  , makeService "tr-rsrb-p2"     1988    "tcp" []
  , makeService "tr-rsrb-p2"     1988    "udp" []
  , makeService "tr-rsrb-p3"     1989    "tcp" []
  , makeService "tr-rsrb-p3"     1989    "udp" []
  , makeService "stun-p1"        1990    "tcp" []
  , makeService "stun-p1"        1990    "udp" []
  , makeService "stun-p2"        1991    "tcp" []
  , makeService "stun-p2"        1991    "udp" []
  , makeService "stun-p3"        1992    "tcp" []
  , makeService "stun-p3"        1992    "udp" []
  , makeService "snmp-tcp-port"  1993    "tcp" []
  , makeService "snmp-tcp-port"  1993    "udp" []
  , makeService "stun-port"      1994    "tcp" []
  , makeService "stun-port"      1994    "udp" []
  , makeService "perf-port"      1995    "tcp" []
  , makeService "perf-port"      1995    "udp" []
  , makeService "tr-rsrb-port"   1996    "tcp" []
  , makeService "tr-rsrb-port"   1996    "udp" []
  , makeService "x25-svc-port"   1998    "tcp" []
  , makeService "x25-svc-port"   1998    "udp" []
  , makeService "tcp-id-port"    1999    "tcp" []
  , makeService "tcp-id-port"    1999    "udp" []
  , makeService "dc"             2001    "tcp" []
  , makeService "wizard"         2001    "udp" []
  , makeService "globe"          2002    "tcp" []
  , makeService "globe"          2002    "udp" []
  , makeService "brutus"         2003    "udp" []
  , makeService "mailbox"        2004    "tcp" []
  , makeService "emce"           2004    "udp" []
  , makeService "berknet"        2005    "tcp" ["csync"]
  , makeService "oracle"         2005    "udp" ["csync"]
  , makeService "invokator"      2006    "tcp" []
  , makeService "raid-cd"        2006    "udp" []
  , makeService "dectalk"        2007    "tcp" []
  , makeService "raid-am"        2007    "udp" []
  , makeService "conf"           2008    "tcp" []
  , makeService "terminaldb"     2008    "udp" []
  , makeService "news"           2009    "tcp" []
  , makeService "whosockami"     2009    "udp" []
  , makeService "search"         2010    "tcp" []
  , makeService "pipe_server"    2010    "udp" ["pipe-server"]
  , makeService "raid-cc"        2011    "tcp" []
  , makeService "servserv"       2011    "udp" []
  , makeService "ttyinfo"        2012    "tcp" []
  , makeService "raid-ac"        2012    "udp" []
  , makeService "raid-am"        2013    "tcp" []
  , makeService "troff"          2014    "tcp" []
  , makeService "raid-sf"        2014    "udp" []
  , makeService "cypress"        2015    "tcp" []
  , makeService "raid-cs"        2015    "udp" []
  , makeService "bootserver"     2016    "tcp" []
  , makeService "bootserver"     2016    "udp" []
  , makeService "cypress-stat"   2017    "tcp" []
  , makeService "bootclient"     2017    "udp" []
  , makeService "terminaldb"     2018    "tcp" []
  , makeService "rellpack"       2018    "udp" []
  , makeService "whosockami"     2019    "tcp" []
  , makeService "about"          2019    "udp" []
  , makeService "xinupageserver" 2020    "tcp" []
  , makeService "xinupageserver" 2020    "udp" []
  , makeService "servexec"       2021    "tcp" []
  , makeService "xinuexpansion1" 2021    "udp" []
  , makeService "down"           2022    "tcp" []
  , makeService "xinuexpansion2" 2022    "udp" []
  , makeService "xinuexpansion3" 2023    "tcp" []
  , makeService "xinuexpansion3" 2023    "udp" []
  , makeService "xinuexpansion4" 2024    "tcp" []
  , makeService "xinuexpansion4" 2024    "udp" []
  , makeService "ellpack"        2025    "tcp" []
  , makeService "xribs"          2025    "udp" []
  , makeService "scrabble"       2026    "tcp" []
  , makeService "scrabble"       2026    "udp" []
  , makeService "shadowserver"   2027    "tcp" []
  , makeService "shadowserver"   2027    "udp" []
  , makeService "submitserver"   2028    "tcp" []
  , makeService "submitserver"   2028    "udp" []
  , makeService "hsrpv6"         2029    "tcp" []
  , makeService "hsrpv6"         2029    "udp" []
  , makeService "device2"        2030    "tcp" []
  , makeService "device2"        2030    "udp" []
  , makeService "mobrien-chat"   2031    "tcp" []
  , makeService "mobrien-chat"   2031    "udp" []
  , makeService "blackboard"     2032    "tcp" []
  , makeService "blackboard"     2032    "udp" []
  , makeService "glogger"        2033    "tcp" []
  , makeService "glogger"        2033    "udp" []
  , makeService "scoremgr"       2034    "tcp" []
  , makeService "scoremgr"       2034    "udp" []
  , makeService "imsldoc"        2035    "tcp" []
  , makeService "imsldoc"        2035    "udp" []
  , makeService "e-dpnet"        2036    "tcp" []
  , makeService "e-dpnet"        2036    "udp" []
  , makeService "applus"         2037    "tcp" []
  , makeService "applus"         2037    "udp" []
  , makeService "objectmanager"  2038    "tcp" []
  , makeService "objectmanager"  2038    "udp" []
  , makeService "prizma"         2039    "tcp" []
  , makeService "prizma"         2039    "udp" []
  , makeService "lam"            2040    "tcp" []
  , makeService "lam"            2040    "udp" []
  , makeService "interbase"      2041    "tcp" []
  , makeService "interbase"      2041    "udp" []
  , makeService "isis"           2042    "tcp" []
  , makeService "isis"           2042    "udp" []
  , makeService "isis-bcast"     2043    "tcp" []
  , makeService "isis-bcast"     2043    "udp" []
  , makeService "rimsl"          2044    "tcp" []
  , makeService "rimsl"          2044    "udp" []
  , makeService "cdfunc"         2045    "tcp" []
  , makeService "cdfunc"         2045    "udp" []
  , makeService "sdfunc"         2046    "tcp" []
  , makeService "sdfunc"         2046    "udp" []
  , makeService "dls-monitor"    2048    "tcp" []
  , makeService "dls-monitor"    2048    "udp" []
  , makeService "av-emb-config"  2050    "tcp" []
  , makeService "av-emb-config"  2050    "udp" []
  , makeService "epnsdp"         2051    "tcp" []
  , makeService "epnsdp"         2051    "udp" []
  , makeService "clearvisn"      2052    "tcp" []
  , makeService "clearvisn"      2052    "udp" []
  , makeService "lot105-ds-upd"  2053    "udp" []
  , makeService "weblogin"       2054    "tcp" []
  , makeService "weblogin"       2054    "udp" []
  , makeService "iop"            2055    "tcp" []
  , makeService "iop"            2055    "udp" []
  , makeService "omnisky"        2056    "tcp" []
  , makeService "omnisky"        2056    "udp" []
  , makeService "rich-cp"        2057    "tcp" []
  , makeService "rich-cp"        2057    "udp" []
  , makeService "newwavesearch"  2058    "tcp" []
  , makeService "newwavesearch"  2058    "udp" []
  , makeService "bmc-messaging"  2059    "tcp" []
  , makeService "bmc-messaging"  2059    "udp" []
  , makeService "teleniumdaemon" 2060    "tcp" []
  , makeService "teleniumdaemon" 2060    "udp" []
  , makeService "netmount"       2061    "tcp" []
  , makeService "netmount"       2061    "udp" []
  , makeService "icg-swp"        2062    "tcp" []
  , makeService "icg-swp"        2062    "udp" []
  , makeService "icg-bridge"     2063    "tcp" []
  , makeService "icg-bridge"     2063    "udp" []
  , makeService "icg-iprelay"    2064    "tcp" []
  , makeService "icg-iprelay"    2064    "udp" []
  , makeService "dlsrpn"         2065    "tcp" []
  , makeService "dlsrpn"         2065    "udp" []
  , makeService "aura"           2066    "tcp" []
  , makeService "aura"           2066    "udp" []
  , makeService "dlswpn"         2067    "tcp" []
  , makeService "dlswpn"         2067    "udp" []
  , makeService "avauthsrvprtcl" 2068    "tcp" []
  , makeService "avauthsrvprtcl" 2068    "udp" []
  , makeService "event-port"     2069    "tcp" []
  , makeService "event-port"     2069    "udp" []
  , makeService "ah-esp-encap"   2070    "tcp" []
  , makeService "ah-esp-encap"   2070    "udp" []
  , makeService "acp-port"       2071    "tcp" []
  , makeService "acp-port"       2071    "udp" []
  , makeService "msync"          2072    "tcp" []
  , makeService "msync"          2072    "udp" []
  , makeService "gxs-data-port"  2073    "tcp" []
  , makeService "gxs-data-port"  2073    "udp" []
  , makeService "vrtl-vmf-sa"    2074    "tcp" []
  , makeService "vrtl-vmf-sa"    2074    "udp" []
  , makeService "newlixengine"   2075    "tcp" []
  , makeService "newlixengine"   2075    "udp" []
  , makeService "newlixconfig"   2076    "tcp" []
  , makeService "newlixconfig"   2076    "udp" []
  , makeService "tsrmagt"        2077    "tcp" []
  , makeService "tsrmagt"        2077    "udp" []
  , makeService "tpcsrvr"        2078    "tcp" []
  , makeService "tpcsrvr"        2078    "udp" []
  , makeService "idware-router"  2079    "tcp" []
  , makeService "idware-router"  2079    "udp" []
  , makeService "autodesk-nlm"   2080    "tcp" []
  , makeService "autodesk-nlm"   2080    "udp" []
  , makeService "kme-trap-port"  2081    "tcp" []
  , makeService "kme-trap-port"  2081    "udp" []
  , makeService "infowave"       2082    "tcp" []
  , makeService "infowave"       2082    "udp" []
  , makeService "radsec"         2083    "tcp" []
  , makeService "radsec"         2083    "udp" []
  , makeService "sunclustergeo"  2084    "tcp" []
  , makeService "sunclustergeo"  2084    "udp" []
  , makeService "ada-cip"        2085    "tcp" []
  , makeService "ada-cip"        2085    "udp" []
  , makeService "gnunet"         2086    "tcp" []
  , makeService "gnunet"         2086    "udp" []
  , makeService "eli"            2087    "tcp" []
  , makeService "eli"            2087    "udp" []
  , makeService "ip-blf"         2088    "tcp" []
  , makeService "ip-blf"         2088    "udp" []
  , makeService "sep"            2089    "tcp" []
  , makeService "sep"            2089    "udp" []
  , makeService "lrp"            2090    "tcp" []
  , makeService "lrp"            2090    "udp" []
  , makeService "prp"            2091    "tcp" []
  , makeService "prp"            2091    "udp" []
  , makeService "descent3"       2092    "tcp" []
  , makeService "descent3"       2092    "udp" []
  , makeService "nbx-cc"         2093    "tcp" []
  , makeService "nbx-cc"         2093    "udp" []
  , makeService "nbx-au"         2094    "tcp" []
  , makeService "nbx-au"         2094    "udp" []
  , makeService "nbx-ser"        2095    "tcp" []
  , makeService "nbx-ser"        2095    "udp" []
  , makeService "nbx-dir"        2096    "tcp" []
  , makeService "nbx-dir"        2096    "udp" []
  , makeService "jetformpreview" 2097    "tcp" []
  , makeService "jetformpreview" 2097    "udp" []
  , makeService "dialog-port"    2098    "tcp" []
  , makeService "dialog-port"    2098    "udp" []
  , makeService "h2250-annex-g"  2099    "tcp" []
  , makeService "h2250-annex-g"  2099    "udp" []
  , makeService "amiganetfs"     2100    "tcp" []
  , makeService "amiganetfs"     2100    "udp" []
  , makeService "rtcm-sc104"     2101    "tcp" []
  , makeService "rtcm-sc104"     2101    "udp" []
  , makeService "minipay"        2105    "udp" []
  , makeService "mzap"           2106    "tcp" []
  , makeService "mzap"           2106    "udp" []
  , makeService "bintec-admin"   2107    "tcp" []
  , makeService "bintec-admin"   2107    "udp" []
  , makeService "comcam"         2108    "tcp" []
  , makeService "comcam"         2108    "udp" []
  , makeService "ergolight"      2109    "tcp" []
  , makeService "ergolight"      2109    "udp" []
  , makeService "umsp"           2110    "tcp" []
  , makeService "umsp"           2110    "udp" []
  , makeService "dsatp"          2111    "tcp" []
  , makeService "dsatp"          2111    "udp" []
  , makeService "idonix-metanet" 2112    "tcp" []
  , makeService "idonix-metanet" 2112    "udp" []
  , makeService "hsl-storm"      2113    "tcp" []
  , makeService "hsl-storm"      2113    "udp" []
  , makeService "newheights"     2114    "tcp" []
  , makeService "newheights"     2114    "udp" []
  , makeService "kdm"            2115    "tcp" []
  , makeService "kdm"            2115    "udp" []
  , makeService "ccowcmr"        2116    "tcp" []
  , makeService "ccowcmr"        2116    "udp" []
  , makeService "mentaclient"    2117    "tcp" []
  , makeService "mentaclient"    2117    "udp" []
  , makeService "mentaserver"    2118    "tcp" []
  , makeService "mentaserver"    2118    "udp" []
  , makeService "gsigatekeeper"  2119    "tcp" []
  , makeService "gsigatekeeper"  2119    "udp" []
  , makeService "qencp"          2120    "tcp" []
  , makeService "qencp"          2120    "udp" []
  , makeService "scientia-ssdb"  2121    "tcp" []
  , makeService "scientia-ssdb"  2121    "udp" []
  , makeService "caupc-remote"   2122    "tcp" []
  , makeService "caupc-remote"   2122    "udp" []
  , makeService "gtp-control"    2123    "tcp" []
  , makeService "gtp-control"    2123    "udp" []
  , makeService "elatelink"      2124    "tcp" []
  , makeService "elatelink"      2124    "udp" []
  , makeService "lockstep"       2125    "tcp" []
  , makeService "lockstep"       2125    "udp" []
  , makeService "pktcable-cops"  2126    "tcp" []
  , makeService "pktcable-cops"  2126    "udp" []
  , makeService "index-pc-wb"    2127    "tcp" []
  , makeService "index-pc-wb"    2127    "udp" []
  , makeService "net-steward"    2128    "tcp" []
  , makeService "net-steward"    2128    "udp" []
  , makeService "cs-live"        2129    "tcp" []
  , makeService "cs-live"        2129    "udp" []
  , makeService "xds"            2130    "tcp" []
  , makeService "xds"            2130    "udp" []
  , makeService "avantageb2b"    2131    "tcp" []
  , makeService "avantageb2b"    2131    "udp" []
  , makeService "solera-epmap"   2132    "tcp" []
  , makeService "solera-epmap"   2132    "udp" []
  , makeService "zymed-zpp"      2133    "tcp" []
  , makeService "zymed-zpp"      2133    "udp" []
  , makeService "avenue"         2134    "tcp" []
  , makeService "avenue"         2134    "udp" []
  , makeService "gris"           2135    "tcp" []
  , makeService "gris"           2135    "udp" []
  , makeService "appworxsrv"     2136    "tcp" []
  , makeService "appworxsrv"     2136    "udp" []
  , makeService "connect"        2137    "tcp" []
  , makeService "connect"        2137    "udp" []
  , makeService "unbind-cluster" 2138    "tcp" []
  , makeService "unbind-cluster" 2138    "udp" []
  , makeService "ias-auth"       2139    "tcp" []
  , makeService "ias-auth"       2139    "udp" []
  , makeService "ias-reg"        2140    "tcp" []
  , makeService "ias-reg"        2140    "udp" []
  , makeService "ias-admind"     2141    "tcp" []
  , makeService "ias-admind"     2141    "udp" []
  , makeService "tdmoip"         2142    "tcp" []
  , makeService "tdmoip"         2142    "udp" []
  , makeService "lv-jc"          2143    "tcp" []
  , makeService "lv-jc"          2143    "udp" []
  , makeService "lv-ffx"         2144    "tcp" []
  , makeService "lv-ffx"         2144    "udp" []
  , makeService "lv-pici"        2145    "tcp" []
  , makeService "lv-pici"        2145    "udp" []
  , makeService "lv-not"         2146    "tcp" []
  , makeService "lv-not"         2146    "udp" []
  , makeService "lv-auth"        2147    "tcp" []
  , makeService "lv-auth"        2147    "udp" []
  , makeService "veritas-ucl"    2148    "tcp" []
  , makeService "veritas-ucl"    2148    "udp" []
  , makeService "acptsys"        2149    "tcp" []
  , makeService "acptsys"        2149    "udp" []
  , makeService "docent"         2151    "tcp" []
  , makeService "docent"         2151    "udp" []
  , makeService "gtp-user"       2152    "tcp" []
  , makeService "gtp-user"       2152    "udp" []
  , makeService "ctlptc"         2153    "tcp" []
  , makeService "ctlptc"         2153    "udp" []
  , makeService "stdptc"         2154    "tcp" []
  , makeService "stdptc"         2154    "udp" []
  , makeService "brdptc"         2155    "tcp" []
  , makeService "brdptc"         2155    "udp" []
  , makeService "trp"            2156    "tcp" []
  , makeService "trp"            2156    "udp" []
  , makeService "xnds"           2157    "tcp" []
  , makeService "xnds"           2157    "udp" []
  , makeService "touchnetplus"   2158    "tcp" []
  , makeService "touchnetplus"   2158    "udp" []
  , makeService "gdbremote"      2159    "tcp" []
  , makeService "gdbremote"      2159    "udp" []
  , makeService "apc-2160"       2160    "tcp" []
  , makeService "apc-2160"       2160    "udp" []
  , makeService "apc-2161"       2161    "tcp" []
  , makeService "apc-2161"       2161    "udp" []
  , makeService "navisphere"     2162    "tcp" []
  , makeService "navisphere"     2162    "udp" []
  , makeService "navisphere-sec" 2163    "tcp" []
  , makeService "navisphere-sec" 2163    "udp" []
  , makeService "ddns-v3"        2164    "tcp" []
  , makeService "ddns-v3"        2164    "udp" []
  , makeService "x-bone-api"     2165    "tcp" []
  , makeService "x-bone-api"     2165    "udp" []
  , makeService "iwserver"       2166    "tcp" []
  , makeService "iwserver"       2166    "udp" []
  , makeService "raw-serial"     2167    "tcp" []
  , makeService "raw-serial"     2167    "udp" []
  , makeService "easy-soft-mux"  2168    "tcp" []
  , makeService "easy-soft-mux"  2168    "udp" []
  , makeService "brain"          2169    "tcp" []
  , makeService "brain"          2169    "udp" []
  , makeService "eyetv"          2170    "tcp" []
  , makeService "eyetv"          2170    "udp" []
  , makeService "msfw-storage"   2171    "tcp" []
  , makeService "msfw-storage"   2171    "udp" []
  , makeService "msfw-s-storage" 2172    "tcp" []
  , makeService "msfw-s-storage" 2172    "udp" []
  , makeService "msfw-replica"   2173    "tcp" []
  , makeService "msfw-replica"   2173    "udp" []
  , makeService "msfw-array"     2174    "tcp" []
  , makeService "msfw-array"     2174    "udp" []
  , makeService "airsync"        2175    "tcp" []
  , makeService "airsync"        2175    "udp" []
  , makeService "rapi"           2176    "tcp" []
  , makeService "rapi"           2176    "udp" []
  , makeService "qwave"          2177    "tcp" []
  , makeService "qwave"          2177    "udp" []
  , makeService "bitspeer"       2178    "tcp" []
  , makeService "bitspeer"       2178    "udp" []
  , makeService "vmrdp"          2179    "tcp" []
  , makeService "vmrdp"          2179    "udp" []
  , makeService "mc-gt-srv"      2180    "tcp" []
  , makeService "mc-gt-srv"      2180    "udp" []
  , makeService "eforward"       2181    "tcp" []
  , makeService "eforward"       2181    "udp" []
  , makeService "cgn-stat"       2182    "tcp" []
  , makeService "cgn-stat"       2182    "udp" []
  , makeService "cgn-config"     2183    "tcp" []
  , makeService "cgn-config"     2183    "udp" []
  , makeService "nvd"            2184    "tcp" []
  , makeService "nvd"            2184    "udp" []
  , makeService "onbase-dds"     2185    "tcp" []
  , makeService "onbase-dds"     2185    "udp" []
  , makeService "gtaua"          2186    "tcp" []
  , makeService "gtaua"          2186    "udp" []
  , makeService "ssmc"           2187    "tcp" []
  , makeService "ssmd"           2187    "udp" []
  , makeService "radware-rpm"    2188    "tcp" []
  , makeService "radware-rpm-s"  2189    "tcp" []
  , makeService "tivoconnect"    2190    "tcp" []
  , makeService "tivoconnect"    2190    "udp" []
  , makeService "tvbus"          2191    "tcp" []
  , makeService "tvbus"          2191    "udp" []
  , makeService "asdis"          2192    "tcp" []
  , makeService "asdis"          2192    "udp" []
  , makeService "drwcs"          2193    "tcp" []
  , makeService "drwcs"          2193    "udp" []
  , makeService "mnp-exchange"   2197    "tcp" []
  , makeService "mnp-exchange"   2197    "udp" []
  , makeService "onehome-remote" 2198    "tcp" []
  , makeService "onehome-remote" 2198    "udp" []
  , makeService "onehome-help"   2199    "tcp" []
  , makeService "onehome-help"   2199    "udp" []
  , makeService "ici"            2200    "tcp" []
  , makeService "ici"            2200    "udp" []
  , makeService "ats"            2201    "tcp" []
  , makeService "ats"            2201    "udp" []
  , makeService "imtc-map"       2202    "tcp" []
  , makeService "imtc-map"       2202    "udp" []
  , makeService "b2-runtime"     2203    "tcp" []
  , makeService "b2-runtime"     2203    "udp" []
  , makeService "b2-license"     2204    "tcp" []
  , makeService "b2-license"     2204    "udp" []
  , makeService "jps"            2205    "tcp" []
  , makeService "jps"            2205    "udp" []
  , makeService "hpocbus"        2206    "tcp" []
  , makeService "hpocbus"        2206    "udp" []
  , makeService "hpssd"          2207    "tcp" []
  , makeService "hpssd"          2207    "udp" []
  , makeService "hpiod"          2208    "tcp" []
  , makeService "hpiod"          2208    "udp" []
  , makeService "rimf-ps"        2209    "tcp" []
  , makeService "rimf-ps"        2209    "udp" []
  , makeService "noaaport"       2210    "tcp" []
  , makeService "noaaport"       2210    "udp" []
  , makeService "emwin"          2211    "tcp" []
  , makeService "emwin"          2211    "udp" []
  , makeService "leecoposserver" 2212    "tcp" []
  , makeService "leecoposserver" 2212    "udp" []
  , makeService "kali"           2213    "tcp" []
  , makeService "kali"           2213    "udp" []
  , makeService "rpi"            2214    "tcp" []
  , makeService "rpi"            2214    "udp" []
  , makeService "ipcore"         2215    "tcp" []
  , makeService "ipcore"         2215    "udp" []
  , makeService "vtu-comms"      2216    "tcp" []
  , makeService "vtu-comms"      2216    "udp" []
  , makeService "gotodevice"     2217    "tcp" []
  , makeService "gotodevice"     2217    "udp" []
  , makeService "bounzza"        2218    "tcp" []
  , makeService "bounzza"        2218    "udp" []
  , makeService "netiq-ncap"     2219    "tcp" []
  , makeService "netiq-ncap"     2219    "udp" []
  , makeService "netiq"          2220    "tcp" []
  , makeService "netiq"          2220    "udp" []
  , makeService "rockwell-csp1"  2221    "tcp" []
  , makeService "rockwell-csp1"  2221    "udp" []
  , makeService "EtherNet/IP-1"  2222    "tcp" ["EtherNet-IP-1"]
  , makeService "EtherNet/IP-1"  2222    "udp" ["EtherNet-IP-1"]
  , makeService "rockwell-csp2"  2223    "tcp" []
  , makeService "rockwell-csp2"  2223    "udp" []
  , makeService "efi-mg"         2224    "tcp" []
  , makeService "efi-mg"         2224    "udp" []
  , makeService "rcip-itu"       2225    "tcp" []
  , makeService "di-drm"         2226    "tcp" []
  , makeService "di-drm"         2226    "udp" []
  , makeService "di-msg"         2227    "tcp" []
  , makeService "di-msg"         2227    "udp" []
  , makeService "ehome-ms"       2228    "tcp" []
  , makeService "ehome-ms"       2228    "udp" []
  , makeService "datalens"       2229    "tcp" []
  , makeService "datalens"       2229    "udp" []
  , makeService "queueadm"       2230    "tcp" []
  , makeService "queueadm"       2230    "udp" []
  , makeService "wimaxasncp"     2231    "tcp" []
  , makeService "wimaxasncp"     2231    "udp" []
  , makeService "ivs-video"      2232    "tcp" []
  , makeService "ivs-video"      2232    "udp" []
  , makeService "infocrypt"      2233    "tcp" []
  , makeService "infocrypt"      2233    "udp" []
  , makeService "directplay"     2234    "tcp" []
  , makeService "directplay"     2234    "udp" []
  , makeService "sercomm-wlink"  2235    "tcp" []
  , makeService "sercomm-wlink"  2235    "udp" []
  , makeService "nani"           2236    "tcp" []
  , makeService "nani"           2236    "udp" []
  , makeService "optech-port1-lm"2237    "tcp" []
  , makeService "optech-port1-lm"2237    "udp" []
  , makeService "aviva-sna"      2238    "tcp" []
  , makeService "aviva-sna"      2238    "udp" []
  , makeService "imagequery"     2239    "tcp" []
  , makeService "imagequery"     2239    "udp" []
  , makeService "recipe"         2240    "tcp" []
  , makeService "recipe"         2240    "udp" []
  , makeService "ivsd"           2241    "tcp" []
  , makeService "ivsd"           2241    "udp" []
  , makeService "foliocorp"      2242    "tcp" []
  , makeService "foliocorp"      2242    "udp" []
  , makeService "magicom"        2243    "tcp" []
  , makeService "magicom"        2243    "udp" []
  , makeService "nmsserver"      2244    "tcp" []
  , makeService "nmsserver"      2244    "udp" []
  , makeService "hao"            2245    "tcp" []
  , makeService "hao"            2245    "udp" []
  , makeService "pc-mta-addrmap" 2246    "tcp" []
  , makeService "pc-mta-addrmap" 2246    "udp" []
  , makeService "antidotemgrsvr" 2247    "tcp" []
  , makeService "antidotemgrsvr" 2247    "udp" []
  , makeService "ums"            2248    "tcp" []
  , makeService "ums"            2248    "udp" []
  , makeService "rfmp"           2249    "tcp" []
  , makeService "rfmp"           2249    "udp" []
  , makeService "remote-collab"  2250    "tcp" []
  , makeService "remote-collab"  2250    "udp" []
  , makeService "dif-port"       2251    "tcp" []
  , makeService "dif-port"       2251    "udp" []
  , makeService "njenet-ssl"     2252    "tcp" []
  , makeService "njenet-ssl"     2252    "udp" []
  , makeService "dtv-chan-req"   2253    "tcp" []
  , makeService "dtv-chan-req"   2253    "udp" []
  , makeService "seispoc"        2254    "tcp" []
  , makeService "seispoc"        2254    "udp" []
  , makeService "vrtp"           2255    "tcp" []
  , makeService "vrtp"           2255    "udp" []
  , makeService "pcc-mfp"        2256    "tcp" []
  , makeService "pcc-mfp"        2256    "udp" []
  , makeService "simple-tx-rx"   2257    "tcp" []
  , makeService "simple-tx-rx"   2257    "udp" []
  , makeService "rcts"           2258    "tcp" []
  , makeService "rcts"           2258    "udp" []
  , makeService "apc-2260"       2260    "tcp" []
  , makeService "apc-2260"       2260    "udp" []
  , makeService "comotionmaster" 2261    "tcp" []
  , makeService "comotionmaster" 2261    "udp" []
  , makeService "comotionback"   2262    "tcp" []
  , makeService "comotionback"   2262    "udp" []
  , makeService "ecwcfg"         2263    "tcp" []
  , makeService "ecwcfg"         2263    "udp" []
  , makeService "apx500api-1"    2264    "tcp" []
  , makeService "apx500api-1"    2264    "udp" []
  , makeService "apx500api-2"    2265    "tcp" []
  , makeService "apx500api-2"    2265    "udp" []
  , makeService "mfserver"       2266    "tcp" []
  , makeService "mfserver"       2266    "udp" []
  , makeService "ontobroker"     2267    "tcp" []
  , makeService "ontobroker"     2267    "udp" []
  , makeService "amt"            2268    "tcp" []
  , makeService "amt"            2268    "udp" []
  , makeService "mikey"          2269    "tcp" []
  , makeService "mikey"          2269    "udp" []
  , makeService "starschool"     2270    "tcp" []
  , makeService "starschool"     2270    "udp" []
  , makeService "mmcals"         2271    "tcp" []
  , makeService "mmcals"         2271    "udp" []
  , makeService "mmcal"          2272    "tcp" []
  , makeService "mmcal"          2272    "udp" []
  , makeService "mysql-im"       2273    "tcp" []
  , makeService "mysql-im"       2273    "udp" []
  , makeService "pcttunnell"     2274    "tcp" []
  , makeService "pcttunnell"     2274    "udp" []
  , makeService "ibridge-data"   2275    "tcp" []
  , makeService "ibridge-data"   2275    "udp" []
  , makeService "ibridge-mgmt"   2276    "tcp" []
  , makeService "ibridge-mgmt"   2276    "udp" []
  , makeService "bluectrlproxy"  2277    "tcp" []
  , makeService "bluectrlproxy"  2277    "udp" []
  , makeService "s3db"           2278    "tcp" []
  , makeService "s3db"           2278    "udp" []
  , makeService "xmquery"        2279    "tcp" []
  , makeService "xmquery"        2279    "udp" []
  , makeService "lnvpoller"      2280    "tcp" []
  , makeService "lnvpoller"      2280    "udp" []
  , makeService "lnvconsole"     2281    "tcp" []
  , makeService "lnvconsole"     2281    "udp" []
  , makeService "lnvalarm"       2282    "tcp" []
  , makeService "lnvalarm"       2282    "udp" []
  , makeService "lnvstatus"      2283    "tcp" []
  , makeService "lnvstatus"      2283    "udp" []
  , makeService "lnvmaps"        2284    "tcp" []
  , makeService "lnvmaps"        2284    "udp" []
  , makeService "lnvmailmon"     2285    "tcp" []
  , makeService "lnvmailmon"     2285    "udp" []
  , makeService "nas-metering"   2286    "tcp" []
  , makeService "nas-metering"   2286    "udp" []
  , makeService "dna"            2287    "tcp" []
  , makeService "dna"            2287    "udp" []
  , makeService "netml"          2288    "tcp" []
  , makeService "netml"          2288    "udp" []
  , makeService "dict-lookup"    2289    "tcp" []
  , makeService "dict-lookup"    2289    "udp" []
  , makeService "sonus-logging"  2290    "tcp" []
  , makeService "sonus-logging"  2290    "udp" []
  , makeService "eapsp"          2291    "tcp" []
  , makeService "eapsp"          2291    "udp" []
  , makeService "mib-streaming"  2292    "tcp" []
  , makeService "mib-streaming"  2292    "udp" []
  , makeService "npdbgmngr"      2293    "tcp" []
  , makeService "npdbgmngr"      2293    "udp" []
  , makeService "konshus-lm"     2294    "tcp" []
  , makeService "konshus-lm"     2294    "udp" []
  , makeService "advant-lm"      2295    "tcp" []
  , makeService "advant-lm"      2295    "udp" []
  , makeService "theta-lm"       2296    "tcp" []
  , makeService "theta-lm"       2296    "udp" []
  , makeService "d2k-datamover1" 2297    "tcp" []
  , makeService "d2k-datamover1" 2297    "udp" []
  , makeService "d2k-datamover2" 2298    "tcp" []
  , makeService "d2k-datamover2" 2298    "udp" []
  , makeService "pc-telecommute" 2299    "tcp" []
  , makeService "pc-telecommute" 2299    "udp" []
  , makeService "cvmmon"         2300    "tcp" []
  , makeService "cvmmon"         2300    "udp" []
  , makeService "cpq-wbem"       2301    "tcp" []
  , makeService "cpq-wbem"       2301    "udp" []
  , makeService "binderysupport" 2302    "tcp" []
  , makeService "binderysupport" 2302    "udp" []
  , makeService "proxy-gateway"  2303    "tcp" []
  , makeService "proxy-gateway"  2303    "udp" []
  , makeService "attachmate-uts" 2304    "tcp" []
  , makeService "attachmate-uts" 2304    "udp" []
  , makeService "mt-scaleserver" 2305    "tcp" []
  , makeService "mt-scaleserver" 2305    "udp" []
  , makeService "tappi-boxnet"   2306    "tcp" []
  , makeService "tappi-boxnet"   2306    "udp" []
  , makeService "pehelp"         2307    "tcp" []
  , makeService "pehelp"         2307    "udp" []
  , makeService "sdhelp"         2308    "tcp" []
  , makeService "sdhelp"         2308    "udp" []
  , makeService "sdserver"       2309    "tcp" []
  , makeService "sdserver"       2309    "udp" []
  , makeService "sdclient"       2310    "tcp" []
  , makeService "sdclient"       2310    "udp" []
  , makeService "messageservice" 2311    "tcp" []
  , makeService "messageservice" 2311    "udp" []
  , makeService "wanscaler"      2312    "tcp" []
  , makeService "wanscaler"      2312    "udp" []
  , makeService "iapp"           2313    "tcp" []
  , makeService "iapp"           2313    "udp" []
  , makeService "cr-websystems"  2314    "tcp" []
  , makeService "cr-websystems"  2314    "udp" []
  , makeService "precise-sft"    2315    "tcp" []
  , makeService "precise-sft"    2315    "udp" []
  , makeService "sent-lm"        2316    "tcp" []
  , makeService "sent-lm"        2316    "udp" []
  , makeService "attachmate-g32" 2317    "tcp" []
  , makeService "attachmate-g32" 2317    "udp" []
  , makeService "cadencecontrol" 2318    "tcp" []
  , makeService "cadencecontrol" 2318    "udp" []
  , makeService "infolibria"     2319    "tcp" []
  , makeService "infolibria"     2319    "udp" []
  , makeService "siebel-ns"      2320    "tcp" []
  , makeService "siebel-ns"      2320    "udp" []
  , makeService "rdlap"          2321    "tcp" []
  , makeService "rdlap"          2321    "udp" []
  , makeService "ofsd"           2322    "tcp" []
  , makeService "ofsd"           2322    "udp" []
  , makeService "3d-nfsd"        2323    "tcp" []
  , makeService "3d-nfsd"        2323    "udp" []
  , makeService "cosmocall"      2324    "tcp" []
  , makeService "cosmocall"      2324    "udp" []
  , makeService "ansysli"        2325    "tcp" []
  , makeService "ansysli"        2325    "udp" []
  , makeService "idcp"           2326    "tcp" []
  , makeService "idcp"           2326    "udp" []
  , makeService "xingcsm"        2327    "tcp" []
  , makeService "xingcsm"        2327    "udp" []
  , makeService "netrix-sftm"    2328    "tcp" []
  , makeService "netrix-sftm"    2328    "udp" []
  , makeService "tscchat"        2330    "tcp" []
  , makeService "tscchat"        2330    "udp" []
  , makeService "agentview"      2331    "tcp" []
  , makeService "agentview"      2331    "udp" []
  , makeService "rcc-host"       2332    "tcp" []
  , makeService "rcc-host"       2332    "udp" []
  , makeService "snapp"          2333    "tcp" []
  , makeService "snapp"          2333    "udp" []
  , makeService "ace-client"     2334    "tcp" []
  , makeService "ace-client"     2334    "udp" []
  , makeService "ace-proxy"      2335    "tcp" []
  , makeService "ace-proxy"      2335    "udp" []
  , makeService "appleugcontrol" 2336    "tcp" []
  , makeService "appleugcontrol" 2336    "udp" []
  , makeService "ideesrv"        2337    "tcp" []
  , makeService "ideesrv"        2337    "udp" []
  , makeService "norton-lambert" 2338    "tcp" []
  , makeService "norton-lambert" 2338    "udp" []
  , makeService "3com-webview"   2339    "tcp" []
  , makeService "3com-webview"   2339    "udp" []
  , makeService "wrs_registry"   2340    "tcp" ["wrs-registry"]
  , makeService "wrs_registry"   2340    "udp" ["wrs-registry"]
  , makeService "xiostatus"      2341    "tcp" []
  , makeService "xiostatus"      2341    "udp" []
  , makeService "manage-exec"    2342    "tcp" []
  , makeService "manage-exec"    2342    "udp" []
  , makeService "nati-logos"     2343    "tcp" []
  , makeService "nati-logos"     2343    "udp" []
  , makeService "fcmsys"         2344    "tcp" []
  , makeService "fcmsys"         2344    "udp" []
  , makeService "dbm"            2345    "tcp" []
  , makeService "dbm"            2345    "udp" []
  , makeService "redstorm_join"  2346    "tcp" ["redstorm-join"]
  , makeService "redstorm_join"  2346    "udp" ["redstorm-join"]
  , makeService "redstorm_find"  2347    "tcp" ["redstorm-find"]
  , makeService "redstorm_find"  2347    "udp" ["redstorm-find"]
  , makeService "redstorm_info"  2348    "tcp" ["redstorm-info"]
  , makeService "redstorm_info"  2348    "udp" ["redstorm-info"]
  , makeService "redstorm_diag"  2349    "tcp" ["redstorm-diag"]
  , makeService "redstorm_diag"  2349    "udp" ["redstorm-diag"]
  , makeService "psbserver"      2350    "tcp" []
  , makeService "psbserver"      2350    "udp" []
  , makeService "psrserver"      2351    "tcp" []
  , makeService "psrserver"      2351    "udp" []
  , makeService "pslserver"      2352    "tcp" []
  , makeService "pslserver"      2352    "udp" []
  , makeService "pspserver"      2353    "tcp" []
  , makeService "pspserver"      2353    "udp" []
  , makeService "psprserver"     2354    "tcp" []
  , makeService "psprserver"     2354    "udp" []
  , makeService "psdbserver"     2355    "tcp" []
  , makeService "psdbserver"     2355    "udp" []
  , makeService "gxtelmd"        2356    "tcp" []
  , makeService "gxtelmd"        2356    "udp" []
  , makeService "unihub-server"  2357    "tcp" []
  , makeService "unihub-server"  2357    "udp" []
  , makeService "futrix"         2358    "tcp" []
  , makeService "futrix"         2358    "udp" []
  , makeService "flukeserver"    2359    "tcp" []
  , makeService "flukeserver"    2359    "udp" []
  , makeService "nexstorindltd"  2360    "tcp" []
  , makeService "nexstorindltd"  2360    "udp" []
  , makeService "tl1"            2361    "tcp" []
  , makeService "tl1"            2361    "udp" []
  , makeService "digiman"        2362    "tcp" []
  , makeService "digiman"        2362    "udp" []
  , makeService "mediacntrlnfsd" 2363    "tcp" []
  , makeService "mediacntrlnfsd" 2363    "udp" []
  , makeService "oi-2000"        2364    "tcp" []
  , makeService "oi-2000"        2364    "udp" []
  , makeService "dbref"          2365    "tcp" []
  , makeService "dbref"          2365    "udp" []
  , makeService "qip-login"      2366    "tcp" []
  , makeService "qip-login"      2366    "udp" []
  , makeService "service-ctrl"   2367    "tcp" []
  , makeService "service-ctrl"   2367    "udp" []
  , makeService "opentable"      2368    "tcp" []
  , makeService "opentable"      2368    "udp" []
  , makeService "l3-hbmon"       2370    "tcp" []
  , makeService "l3-hbmon"       2370    "udp" []
  , makeService "worldwire"      2371    "tcp" []
  , makeService "worldwire"      2371    "udp" []
  , makeService "lanmessenger"   2372    "tcp" []
  , makeService "lanmessenger"   2372    "udp" []
  , makeService "remographlm"    2373    "tcp" []
  , makeService "hydra"          2374    "tcp" []
  , makeService "compaq-https"   2381    "tcp" []
  , makeService "compaq-https"   2381    "udp" []
  , makeService "ms-olap3"       2382    "tcp" []
  , makeService "ms-olap3"       2382    "udp" []
  , makeService "ms-olap4"       2383    "tcp" []
  , makeService "ms-olap4"       2383    "udp" []
  , makeService "sd-request"     2384    "tcp" []
  , makeService "sd-capacity"    2384    "udp" []
  , makeService "sd-data"        2385    "tcp" []
  , makeService "sd-data"        2385    "udp" []
  , makeService "virtualtape"    2386    "tcp" []
  , makeService "virtualtape"    2386    "udp" []
  , makeService "vsamredirector" 2387    "tcp" []
  , makeService "vsamredirector" 2387    "udp" []
  , makeService "mynahautostart" 2388    "tcp" []
  , makeService "mynahautostart" 2388    "udp" []
  , makeService "ovsessionmgr"   2389    "tcp" []
  , makeService "ovsessionmgr"   2389    "udp" []
  , makeService "rsmtp"          2390    "tcp" []
  , makeService "rsmtp"          2390    "udp" []
  , makeService "3com-net-mgmt"  2391    "tcp" []
  , makeService "3com-net-mgmt"  2391    "udp" []
  , makeService "tacticalauth"   2392    "tcp" []
  , makeService "tacticalauth"   2392    "udp" []
  , makeService "ms-olap1"       2393    "tcp" []
  , makeService "ms-olap1"       2393    "udp" []
  , makeService "ms-olap2"       2394    "tcp" []
  , makeService "ms-olap2"       2394    "udp" []
  , makeService "lan900_remote"  2395    "tcp" ["lan900-remote"]
  , makeService "lan900_remote"  2395    "udp" ["lan900-remote"]
  , makeService "wusage"         2396    "tcp" []
  , makeService "wusage"         2396    "udp" []
  , makeService "ncl"            2397    "tcp" []
  , makeService "ncl"            2397    "udp" []
  , makeService "orbiter"        2398    "tcp" []
  , makeService "orbiter"        2398    "udp" []
  , makeService "fmpro-fdal"     2399    "tcp" []
  , makeService "fmpro-fdal"     2399    "udp" []
  , makeService "opequus-server" 2400    "tcp" []
  , makeService "opequus-server" 2400    "udp" []
  , makeService "taskmaster2000" 2402    "tcp" []
  , makeService "taskmaster2000" 2402    "udp" []
  , makeService "iec-104"        2404    "tcp" []
  , makeService "iec-104"        2404    "udp" []
  , makeService "trc-netpoll"    2405    "tcp" []
  , makeService "trc-netpoll"    2405    "udp" []
  , makeService "jediserver"     2406    "tcp" []
  , makeService "jediserver"     2406    "udp" []
  , makeService "orion"          2407    "tcp" []
  , makeService "orion"          2407    "udp" []
  , makeService "railgun-webaccl"2408    "tcp" []
  , makeService "sns-protocol"   2409    "tcp" []
  , makeService "sns-protocol"   2409    "udp" []
  , makeService "vrts-registry"  2410    "tcp" []
  , makeService "vrts-registry"  2410    "udp" []
  , makeService "netwave-ap-mgmt"2411    "tcp" []
  , makeService "netwave-ap-mgmt"2411    "udp" []
  , makeService "cdn"            2412    "tcp" []
  , makeService "cdn"            2412    "udp" []
  , makeService "orion-rmi-reg"  2413    "tcp" []
  , makeService "orion-rmi-reg"  2413    "udp" []
  , makeService "beeyond"        2414    "tcp" []
  , makeService "beeyond"        2414    "udp" []
  , makeService "codima-rtp"     2415    "tcp" []
  , makeService "codima-rtp"     2415    "udp" []
  , makeService "rmtserver"      2416    "tcp" []
  , makeService "rmtserver"      2416    "udp" []
  , makeService "composit-server"2417    "tcp" []
  , makeService "composit-server"2417    "udp" []
  , makeService "cas"            2418    "tcp" []
  , makeService "cas"            2418    "udp" []
  , makeService "attachmate-s2s" 2419    "tcp" []
  , makeService "attachmate-s2s" 2419    "udp" []
  , makeService "dslremote-mgmt" 2420    "tcp" []
  , makeService "dslremote-mgmt" 2420    "udp" []
  , makeService "g-talk"         2421    "tcp" []
  , makeService "g-talk"         2421    "udp" []
  , makeService "crmsbits"       2422    "tcp" []
  , makeService "crmsbits"       2422    "udp" []
  , makeService "rnrp"           2423    "tcp" []
  , makeService "rnrp"           2423    "udp" []
  , makeService "kofax-svr"      2424    "tcp" []
  , makeService "kofax-svr"      2424    "udp" []
  , makeService "fjitsuappmgr"   2425    "tcp" []
  , makeService "fjitsuappmgr"   2425    "udp" []
  , makeService "mgcp-gateway"   2427    "tcp" []
  , makeService "mgcp-gateway"   2427    "udp" []
  , makeService "ott"            2428    "tcp" []
  , makeService "ott"            2428    "udp" []
  , makeService "ft-role"        2429    "tcp" []
  , makeService "ft-role"        2429    "udp" []
  , makeService "pxc-epmap"      2434    "tcp" []
  , makeService "pxc-epmap"      2434    "udp" []
  , makeService "optilogic"      2435    "tcp" []
  , makeService "optilogic"      2435    "udp" []
  , makeService "topx"           2436    "tcp" []
  , makeService "topx"           2436    "udp" []
  , makeService "unicontrol"     2437    "tcp" []
  , makeService "unicontrol"     2437    "udp" []
  , makeService "sybasedbsynch"  2439    "tcp" []
  , makeService "sybasedbsynch"  2439    "udp" []
  , makeService "spearway"       2440    "tcp" []
  , makeService "spearway"       2440    "udp" []
  , makeService "pvsw-inet"      2441    "tcp" []
  , makeService "pvsw-inet"      2441    "udp" []
  , makeService "netangel"       2442    "tcp" []
  , makeService "netangel"       2442    "udp" []
  , makeService "powerclientcsf" 2443    "tcp" []
  , makeService "powerclientcsf" 2443    "udp" []
  , makeService "btpp2sectrans"  2444    "tcp" []
  , makeService "btpp2sectrans"  2444    "udp" []
  , makeService "dtn1"           2445    "tcp" []
  , makeService "dtn1"           2445    "udp" []
  , makeService "bues_service"   2446    "tcp" ["bues-service"]
  , makeService "bues_service"   2446    "udp" ["bues-service"]
  , makeService "ovwdb"          2447    "tcp" []
  , makeService "ovwdb"          2447    "udp" []
  , makeService "hpppssvr"       2448    "tcp" []
  , makeService "hpppssvr"       2448    "udp" []
  , makeService "ratl"           2449    "tcp" []
  , makeService "ratl"           2449    "udp" []
  , makeService "netadmin"       2450    "tcp" []
  , makeService "netadmin"       2450    "udp" []
  , makeService "netchat"        2451    "tcp" []
  , makeService "netchat"        2451    "udp" []
  , makeService "snifferclient"  2452    "tcp" []
  , makeService "snifferclient"  2452    "udp" []
  , makeService "madge-ltd"      2453    "tcp" []
  , makeService "madge-ltd"      2453    "udp" []
  , makeService "indx-dds"       2454    "tcp" []
  , makeService "indx-dds"       2454    "udp" []
  , makeService "wago-io-system" 2455    "tcp" []
  , makeService "wago-io-system" 2455    "udp" []
  , makeService "altav-remmgt"   2456    "tcp" []
  , makeService "altav-remmgt"   2456    "udp" []
  , makeService "rapido-ip"      2457    "tcp" []
  , makeService "rapido-ip"      2457    "udp" []
  , makeService "griffin"        2458    "tcp" []
  , makeService "griffin"        2458    "udp" []
  , makeService "community"      2459    "tcp" []
  , makeService "community"      2459    "udp" []
  , makeService "ms-theater"     2460    "tcp" []
  , makeService "ms-theater"     2460    "udp" []
  , makeService "qadmifoper"     2461    "tcp" []
  , makeService "qadmifoper"     2461    "udp" []
  , makeService "qadmifevent"    2462    "tcp" []
  , makeService "qadmifevent"    2462    "udp" []
  , makeService "lsi-raid-mgmt"  2463    "tcp" []
  , makeService "lsi-raid-mgmt"  2463    "udp" []
  , makeService "direcpc-si"     2464    "tcp" []
  , makeService "direcpc-si"     2464    "udp" []
  , makeService "lbm"            2465    "tcp" []
  , makeService "lbm"            2465    "udp" []
  , makeService "lbf"            2466    "tcp" []
  , makeService "lbf"            2466    "udp" []
  , makeService "high-criteria"  2467    "tcp" []
  , makeService "high-criteria"  2467    "udp" []
  , makeService "qip-msgd"       2468    "tcp" []
  , makeService "qip-msgd"       2468    "udp" []
  , makeService "mti-tcs-comm"   2469    "tcp" []
  , makeService "mti-tcs-comm"   2469    "udp" []
  , makeService "taskman-port"   2470    "tcp" []
  , makeService "taskman-port"   2470    "udp" []
  , makeService "seaodbc"        2471    "tcp" []
  , makeService "seaodbc"        2471    "udp" []
  , makeService "c3"             2472    "tcp" []
  , makeService "c3"             2472    "udp" []
  , makeService "aker-cdp"       2473    "tcp" []
  , makeService "aker-cdp"       2473    "udp" []
  , makeService "vitalanalysis"  2474    "tcp" []
  , makeService "vitalanalysis"  2474    "udp" []
  , makeService "ace-server"     2475    "tcp" []
  , makeService "ace-server"     2475    "udp" []
  , makeService "ace-svr-prop"   2476    "tcp" []
  , makeService "ace-svr-prop"   2476    "udp" []
  , makeService "ssm-cvs"        2477    "tcp" []
  , makeService "ssm-cvs"        2477    "udp" []
  , makeService "ssm-cssps"      2478    "tcp" []
  , makeService "ssm-cssps"      2478    "udp" []
  , makeService "ssm-els"        2479    "tcp" []
  , makeService "ssm-els"        2479    "udp" []
  , makeService "powerexchange"  2480    "tcp" []
  , makeService "powerexchange"  2480    "udp" []
  , makeService "giop"           2481    "tcp" []
  , makeService "giop"           2481    "udp" []
  , makeService "giop-ssl"       2482    "tcp" []
  , makeService "giop-ssl"       2482    "udp" []
  , makeService "ttc"            2483    "tcp" []
  , makeService "ttc"            2483    "udp" []
  , makeService "ttc-ssl"        2484    "tcp" []
  , makeService "ttc-ssl"        2484    "udp" []
  , makeService "netobjects1"    2485    "tcp" []
  , makeService "netobjects1"    2485    "udp" []
  , makeService "netobjects2"    2486    "tcp" []
  , makeService "netobjects2"    2486    "udp" []
  , makeService "pns"            2487    "tcp" []
  , makeService "pns"            2487    "udp" []
  , makeService "moy-corp"       2488    "tcp" []
  , makeService "moy-corp"       2488    "udp" []
  , makeService "tsilb"          2489    "tcp" []
  , makeService "tsilb"          2489    "udp" []
  , makeService "qip-qdhcp"      2490    "tcp" []
  , makeService "qip-qdhcp"      2490    "udp" []
  , makeService "conclave-cpp"   2491    "tcp" []
  , makeService "conclave-cpp"   2491    "udp" []
  , makeService "groove"         2492    "tcp" []
  , makeService "groove"         2492    "udp" []
  , makeService "talarian-mqs"   2493    "tcp" []
  , makeService "talarian-mqs"   2493    "udp" []
  , makeService "bmc-ar"         2494    "tcp" []
  , makeService "bmc-ar"         2494    "udp" []
  , makeService "fast-rem-serv"  2495    "tcp" []
  , makeService "fast-rem-serv"  2495    "udp" []
  , makeService "dirgis"         2496    "tcp" []
  , makeService "dirgis"         2496    "udp" []
  , makeService "quaddb"         2497    "tcp" []
  , makeService "quaddb"         2497    "udp" []
  , makeService "odn-castraq"    2498    "tcp" []
  , makeService "odn-castraq"    2498    "udp" []
  , makeService "rtsserv"        2500    "tcp" []
  , makeService "rtsserv"        2500    "udp" []
  , makeService "rtsclient"      2501    "tcp" []
  , makeService "rtsclient"      2501    "udp" []
  , makeService "kentrox-prot"   2502    "tcp" []
  , makeService "kentrox-prot"   2502    "udp" []
  , makeService "nms-dpnss"      2503    "tcp" []
  , makeService "nms-dpnss"      2503    "udp" []
  , makeService "wlbs"           2504    "tcp" []
  , makeService "wlbs"           2504    "udp" []
  , makeService "ppcontrol"      2505    "tcp" []
  , makeService "ppcontrol"      2505    "udp" []
  , makeService "jbroker"        2506    "tcp" []
  , makeService "jbroker"        2506    "udp" []
  , makeService "spock"          2507    "tcp" []
  , makeService "spock"          2507    "udp" []
  , makeService "jdatastore"     2508    "tcp" []
  , makeService "jdatastore"     2508    "udp" []
  , makeService "fjmpss"         2509    "tcp" []
  , makeService "fjmpss"         2509    "udp" []
  , makeService "fjappmgrbulk"   2510    "tcp" []
  , makeService "fjappmgrbulk"   2510    "udp" []
  , makeService "metastorm"      2511    "tcp" []
  , makeService "metastorm"      2511    "udp" []
  , makeService "citrixima"      2512    "tcp" []
  , makeService "citrixima"      2512    "udp" []
  , makeService "citrixadmin"    2513    "tcp" []
  , makeService "citrixadmin"    2513    "udp" []
  , makeService "facsys-ntp"     2514    "tcp" []
  , makeService "facsys-ntp"     2514    "udp" []
  , makeService "facsys-router"  2515    "tcp" []
  , makeService "facsys-router"  2515    "udp" []
  , makeService "maincontrol"    2516    "tcp" []
  , makeService "maincontrol"    2516    "udp" []
  , makeService "call-sig-trans" 2517    "tcp" []
  , makeService "call-sig-trans" 2517    "udp" []
  , makeService "willy"          2518    "tcp" []
  , makeService "willy"          2518    "udp" []
  , makeService "globmsgsvc"     2519    "tcp" []
  , makeService "globmsgsvc"     2519    "udp" []
  , makeService "pvsw"           2520    "tcp" []
  , makeService "pvsw"           2520    "udp" []
  , makeService "adaptecmgr"     2521    "tcp" []
  , makeService "adaptecmgr"     2521    "udp" []
  , makeService "windb"          2522    "tcp" []
  , makeService "windb"          2522    "udp" []
  , makeService "qke-llc-v3"     2523    "tcp" []
  , makeService "qke-llc-v3"     2523    "udp" []
  , makeService "optiwave-lm"    2524    "tcp" []
  , makeService "optiwave-lm"    2524    "udp" []
  , makeService "ms-v-worlds"    2525    "tcp" []
  , makeService "ms-v-worlds"    2525    "udp" []
  , makeService "ema-sent-lm"    2526    "tcp" []
  , makeService "ema-sent-lm"    2526    "udp" []
  , makeService "iqserver"       2527    "tcp" []
  , makeService "iqserver"       2527    "udp" []
  , makeService "ncr_ccl"        2528    "tcp" ["ncr-ccl"]
  , makeService "ncr_ccl"        2528    "udp" ["ncr-ccl"]
  , makeService "utsftp"         2529    "tcp" []
  , makeService "utsftp"         2529    "udp" []
  , makeService "vrcommerce"     2530    "tcp" []
  , makeService "vrcommerce"     2530    "udp" []
  , makeService "ito-e-gui"      2531    "tcp" []
  , makeService "ito-e-gui"      2531    "udp" []
  , makeService "ovtopmd"        2532    "tcp" []
  , makeService "ovtopmd"        2532    "udp" []
  , makeService "snifferserver"  2533    "tcp" []
  , makeService "snifferserver"  2533    "udp" []
  , makeService "combox-web-acc" 2534    "tcp" []
  , makeService "combox-web-acc" 2534    "udp" []
  , makeService "madcap"         2535    "tcp" []
  , makeService "madcap"         2535    "udp" []
  , makeService "btpp2audctr1"   2536    "tcp" []
  , makeService "btpp2audctr1"   2536    "udp" []
  , makeService "upgrade"        2537    "tcp" []
  , makeService "upgrade"        2537    "udp" []
  , makeService "vnwk-prapi"     2538    "tcp" []
  , makeService "vnwk-prapi"     2538    "udp" []
  , makeService "vsiadmin"       2539    "tcp" []
  , makeService "vsiadmin"       2539    "udp" []
  , makeService "lonworks"       2540    "tcp" []
  , makeService "lonworks"       2540    "udp" []
  , makeService "lonworks2"      2541    "tcp" []
  , makeService "lonworks2"      2541    "udp" []
  , makeService "udrawgraph"     2542    "tcp" []
  , makeService "udrawgraph"     2542    "udp" []
  , makeService "reftek"         2543    "tcp" []
  , makeService "reftek"         2543    "udp" []
  , makeService "novell-zen"     2544    "tcp" []
  , makeService "novell-zen"     2544    "udp" []
  , makeService "sis-emt"        2545    "tcp" []
  , makeService "sis-emt"        2545    "udp" []
  , makeService "vytalvaultbrtp" 2546    "tcp" []
  , makeService "vytalvaultbrtp" 2546    "udp" []
  , makeService "vytalvaultvsmp" 2547    "tcp" []
  , makeService "vytalvaultvsmp" 2547    "udp" []
  , makeService "vytalvaultpipe" 2548    "tcp" []
  , makeService "vytalvaultpipe" 2548    "udp" []
  , makeService "ipass"          2549    "tcp" []
  , makeService "ipass"          2549    "udp" []
  , makeService "ads"            2550    "tcp" []
  , makeService "ads"            2550    "udp" []
  , makeService "isg-uda-server" 2551    "tcp" []
  , makeService "isg-uda-server" 2551    "udp" []
  , makeService "call-logging"   2552    "tcp" []
  , makeService "call-logging"   2552    "udp" []
  , makeService "efidiningport"  2553    "tcp" []
  , makeService "efidiningport"  2553    "udp" []
  , makeService "vcnet-link-v10" 2554    "tcp" []
  , makeService "vcnet-link-v10" 2554    "udp" []
  , makeService "compaq-wcp"     2555    "tcp" []
  , makeService "compaq-wcp"     2555    "udp" []
  , makeService "nicetec-nmsvc"  2556    "tcp" []
  , makeService "nicetec-nmsvc"  2556    "udp" []
  , makeService "nicetec-mgmt"   2557    "tcp" []
  , makeService "nicetec-mgmt"   2557    "udp" []
  , makeService "pclemultimedia" 2558    "tcp" []
  , makeService "pclemultimedia" 2558    "udp" []
  , makeService "lstp"           2559    "tcp" []
  , makeService "lstp"           2559    "udp" []
  , makeService "labrat"         2560    "tcp" []
  , makeService "labrat"         2560    "udp" []
  , makeService "mosaixcc"       2561    "tcp" []
  , makeService "mosaixcc"       2561    "udp" []
  , makeService "delibo"         2562    "tcp" []
  , makeService "delibo"         2562    "udp" []
  , makeService "cti-redwood"    2563    "tcp" []
  , makeService "cti-redwood"    2563    "udp" []
  , makeService "hp-3000-telnet" 2564    "tcp" []
  , makeService "hp-3000-telnet" 2564    "udp" []
  , makeService "coord-svr"      2565    "tcp" []
  , makeService "coord-svr"      2565    "udp" []
  , makeService "pcs-pcw"        2566    "tcp" []
  , makeService "pcs-pcw"        2566    "udp" []
  , makeService "clp"            2567    "tcp" []
  , makeService "clp"            2567    "udp" []
  , makeService "spamtrap"       2568    "tcp" []
  , makeService "spamtrap"       2568    "udp" []
  , makeService "sonuscallsig"   2569    "tcp" []
  , makeService "sonuscallsig"   2569    "udp" []
  , makeService "hs-port"        2570    "tcp" []
  , makeService "hs-port"        2570    "udp" []
  , makeService "cecsvc"         2571    "tcp" []
  , makeService "cecsvc"         2571    "udp" []
  , makeService "ibp"            2572    "tcp" []
  , makeService "ibp"            2572    "udp" []
  , makeService "trustestablish" 2573    "tcp" []
  , makeService "trustestablish" 2573    "udp" []
  , makeService "blockade-bpsp"  2574    "tcp" []
  , makeService "blockade-bpsp"  2574    "udp" []
  , makeService "hl7"            2575    "tcp" []
  , makeService "hl7"            2575    "udp" []
  , makeService "tclprodebugger" 2576    "tcp" []
  , makeService "tclprodebugger" 2576    "udp" []
  , makeService "scipticslsrvr"  2577    "tcp" []
  , makeService "scipticslsrvr"  2577    "udp" []
  , makeService "rvs-isdn-dcp"   2578    "tcp" []
  , makeService "rvs-isdn-dcp"   2578    "udp" []
  , makeService "mpfoncl"        2579    "tcp" []
  , makeService "mpfoncl"        2579    "udp" []
  , makeService "tributary"      2580    "tcp" []
  , makeService "tributary"      2580    "udp" []
  , makeService "argis-te"       2581    "tcp" []
  , makeService "argis-te"       2581    "udp" []
  , makeService "argis-ds"       2582    "tcp" []
  , makeService "argis-ds"       2582    "udp" []
  , makeService "mon"            2583    "tcp" []
  , makeService "mon"            2583    "udp" []
  , makeService "cyaserv"        2584    "tcp" []
  , makeService "cyaserv"        2584    "udp" []
  , makeService "netx-server"    2585    "tcp" []
  , makeService "netx-server"    2585    "udp" []
  , makeService "netx-agent"     2586    "tcp" []
  , makeService "netx-agent"     2586    "udp" []
  , makeService "masc"           2587    "tcp" []
  , makeService "masc"           2587    "udp" []
  , makeService "privilege"      2588    "tcp" []
  , makeService "privilege"      2588    "udp" []
  , makeService "quartus-tcl"    2589    "tcp" []
  , makeService "quartus-tcl"    2589    "udp" []
  , makeService "idotdist"       2590    "tcp" []
  , makeService "idotdist"       2590    "udp" []
  , makeService "maytagshuffle"  2591    "tcp" []
  , makeService "maytagshuffle"  2591    "udp" []
  , makeService "netrek"         2592    "tcp" []
  , makeService "netrek"         2592    "udp" []
  , makeService "mns-mail"       2593    "tcp" []
  , makeService "mns-mail"       2593    "udp" []
  , makeService "dts"            2594    "tcp" []
  , makeService "dts"            2594    "udp" []
  , makeService "worldfusion1"   2595    "tcp" []
  , makeService "worldfusion1"   2595    "udp" []
  , makeService "worldfusion2"   2596    "tcp" []
  , makeService "worldfusion2"   2596    "udp" []
  , makeService "homesteadglory" 2597    "tcp" []
  , makeService "homesteadglory" 2597    "udp" []
  , makeService "citriximaclient"2598    "tcp" []
  , makeService "citriximaclient"2598    "udp" []
  , makeService "snapd"          2599    "tcp" []
  , makeService "snapd"          2599    "udp" []
  , makeService "connection"     2607    "tcp" []
  , makeService "connection"     2607    "udp" []
  , makeService "wag-service"    2608    "tcp" []
  , makeService "wag-service"    2608    "udp" []
  , makeService "system-monitor" 2609    "tcp" []
  , makeService "system-monitor" 2609    "udp" []
  , makeService "versa-tek"      2610    "tcp" []
  , makeService "versa-tek"      2610    "udp" []
  , makeService "lionhead"       2611    "tcp" []
  , makeService "lionhead"       2611    "udp" []
  , makeService "qpasa-agent"    2612    "tcp" []
  , makeService "qpasa-agent"    2612    "udp" []
  , makeService "smntubootstrap" 2613    "tcp" []
  , makeService "smntubootstrap" 2613    "udp" []
  , makeService "neveroffline"   2614    "tcp" []
  , makeService "neveroffline"   2614    "udp" []
  , makeService "firepower"      2615    "tcp" []
  , makeService "firepower"      2615    "udp" []
  , makeService "appswitch-emp"  2616    "tcp" []
  , makeService "appswitch-emp"  2616    "udp" []
  , makeService "cmadmin"        2617    "tcp" []
  , makeService "cmadmin"        2617    "udp" []
  , makeService "priority-e-com" 2618    "tcp" []
  , makeService "priority-e-com" 2618    "udp" []
  , makeService "bruce"          2619    "tcp" []
  , makeService "bruce"          2619    "udp" []
  , makeService "lpsrecommender" 2620    "tcp" []
  , makeService "lpsrecommender" 2620    "udp" []
  , makeService "miles-apart"    2621    "tcp" []
  , makeService "miles-apart"    2621    "udp" []
  , makeService "metricadbc"     2622    "tcp" []
  , makeService "metricadbc"     2622    "udp" []
  , makeService "lmdp"           2623    "tcp" []
  , makeService "lmdp"           2623    "udp" []
  , makeService "aria"           2624    "tcp" []
  , makeService "aria"           2624    "udp" []
  , makeService "blwnkl-port"    2625    "tcp" []
  , makeService "blwnkl-port"    2625    "udp" []
  , makeService "gbjd816"        2626    "tcp" []
  , makeService "gbjd816"        2626    "udp" []
  , makeService "moshebeeri"     2627    "tcp" []
  , makeService "moshebeeri"     2627    "udp" []
  , makeService "sitaraserver"   2629    "tcp" []
  , makeService "sitaraserver"   2629    "udp" []
  , makeService "sitaramgmt"     2630    "tcp" []
  , makeService "sitaramgmt"     2630    "udp" []
  , makeService "sitaradir"      2631    "tcp" []
  , makeService "sitaradir"      2631    "udp" []
  , makeService "irdg-post"      2632    "tcp" []
  , makeService "irdg-post"      2632    "udp" []
  , makeService "interintelli"   2633    "tcp" []
  , makeService "interintelli"   2633    "udp" []
  , makeService "pk-electronics" 2634    "tcp" []
  , makeService "pk-electronics" 2634    "udp" []
  , makeService "backburner"     2635    "tcp" []
  , makeService "backburner"     2635    "udp" []
  , makeService "solve"          2636    "tcp" []
  , makeService "solve"          2636    "udp" []
  , makeService "imdocsvc"       2637    "tcp" []
  , makeService "imdocsvc"       2637    "udp" []
  , makeService "sybaseanywhere" 2638    "tcp" []
  , makeService "sybaseanywhere" 2638    "udp" []
  , makeService "aminet"         2639    "tcp" []
  , makeService "aminet"         2639    "udp" []
  , makeService "sai_sentlm"     2640    "tcp" ["sai-sentlm"]
  , makeService "sai_sentlm"     2640    "udp" ["sai-sentlm"]
  , makeService "hdl-srv"        2641    "tcp" []
  , makeService "hdl-srv"        2641    "udp" []
  , makeService "tragic"         2642    "tcp" []
  , makeService "tragic"         2642    "udp" []
  , makeService "gte-samp"       2643    "tcp" []
  , makeService "gte-samp"       2643    "udp" []
  , makeService "travsoft-ipx-t" 2644    "tcp" []
  , makeService "travsoft-ipx-t" 2644    "udp" []
  , makeService "novell-ipx-cmd" 2645    "tcp" []
  , makeService "novell-ipx-cmd" 2645    "udp" []
  , makeService "and-lm"         2646    "tcp" []
  , makeService "and-lm"         2646    "udp" []
  , makeService "syncserver"     2647    "tcp" []
  , makeService "syncserver"     2647    "udp" []
  , makeService "upsnotifyprot"  2648    "tcp" []
  , makeService "upsnotifyprot"  2648    "udp" []
  , makeService "vpsipport"      2649    "tcp" []
  , makeService "vpsipport"      2649    "udp" []
  , makeService "eristwoguns"    2650    "tcp" []
  , makeService "eristwoguns"    2650    "udp" []
  , makeService "ebinsite"       2651    "tcp" []
  , makeService "ebinsite"       2651    "udp" []
  , makeService "interpathpanel" 2652    "tcp" []
  , makeService "interpathpanel" 2652    "udp" []
  , makeService "sonus"          2653    "tcp" []
  , makeService "sonus"          2653    "udp" []
  , makeService "corel_vncadmin" 2654    "tcp" ["corel-vncadmin"]
  , makeService "corel_vncadmin" 2654    "udp" ["corel-vncadmin"]
  , makeService "unglue"         2655    "tcp" []
  , makeService "unglue"         2655    "udp" []
  , makeService "kana"           2656    "tcp" []
  , makeService "kana"           2656    "udp" []
  , makeService "sns-dispatcher" 2657    "tcp" []
  , makeService "sns-dispatcher" 2657    "udp" []
  , makeService "sns-admin"      2658    "tcp" []
  , makeService "sns-admin"      2658    "udp" []
  , makeService "sns-query"      2659    "tcp" []
  , makeService "sns-query"      2659    "udp" []
  , makeService "gcmonitor"      2660    "tcp" []
  , makeService "gcmonitor"      2660    "udp" []
  , makeService "olhost"         2661    "tcp" []
  , makeService "olhost"         2661    "udp" []
  , makeService "bintec-capi"    2662    "tcp" []
  , makeService "bintec-capi"    2662    "udp" []
  , makeService "bintec-tapi"    2663    "tcp" []
  , makeService "bintec-tapi"    2663    "udp" []
  , makeService "patrol-mq-gm"   2664    "tcp" []
  , makeService "patrol-mq-gm"   2664    "udp" []
  , makeService "patrol-mq-nm"   2665    "tcp" []
  , makeService "patrol-mq-nm"   2665    "udp" []
  , makeService "extensis"       2666    "tcp" []
  , makeService "extensis"       2666    "udp" []
  , makeService "alarm-clock-s"  2667    "tcp" []
  , makeService "alarm-clock-s"  2667    "udp" []
  , makeService "alarm-clock-c"  2668    "tcp" []
  , makeService "alarm-clock-c"  2668    "udp" []
  , makeService "toad"           2669    "tcp" []
  , makeService "toad"           2669    "udp" []
  , makeService "tve-announce"   2670    "tcp" []
  , makeService "tve-announce"   2670    "udp" []
  , makeService "newlixreg"      2671    "tcp" []
  , makeService "newlixreg"      2671    "udp" []
  , makeService "nhserver"       2672    "tcp" []
  , makeService "nhserver"       2672    "udp" []
  , makeService "firstcall42"    2673    "tcp" []
  , makeService "firstcall42"    2673    "udp" []
  , makeService "ewnn"           2674    "tcp" []
  , makeService "ewnn"           2674    "udp" []
  , makeService "ttc-etap"       2675    "tcp" []
  , makeService "ttc-etap"       2675    "udp" []
  , makeService "simslink"       2676    "tcp" []
  , makeService "simslink"       2676    "udp" []
  , makeService "gadgetgate1way" 2677    "tcp" []
  , makeService "gadgetgate1way" 2677    "udp" []
  , makeService "gadgetgate2way" 2678    "tcp" []
  , makeService "gadgetgate2way" 2678    "udp" []
  , makeService "syncserverssl"  2679    "tcp" []
  , makeService "syncserverssl"  2679    "udp" []
  , makeService "pxc-sapxom"     2680    "tcp" []
  , makeService "pxc-sapxom"     2680    "udp" []
  , makeService "mpnjsomb"       2681    "tcp" []
  , makeService "mpnjsomb"       2681    "udp" []
  , makeService "ncdloadbalance" 2683    "tcp" []
  , makeService "ncdloadbalance" 2683    "udp" []
  , makeService "mpnjsosv"       2684    "tcp" []
  , makeService "mpnjsosv"       2684    "udp" []
  , makeService "mpnjsocl"       2685    "tcp" []
  , makeService "mpnjsocl"       2685    "udp" []
  , makeService "mpnjsomg"       2686    "tcp" []
  , makeService "mpnjsomg"       2686    "udp" []
  , makeService "pq-lic-mgmt"    2687    "tcp" []
  , makeService "pq-lic-mgmt"    2687    "udp" []
  , makeService "md-cg-http"     2688    "tcp" []
  , makeService "md-cg-http"     2688    "udp" []
  , makeService "fastlynx"       2689    "tcp" []
  , makeService "fastlynx"       2689    "udp" []
  , makeService "hp-nnm-data"    2690    "tcp" []
  , makeService "hp-nnm-data"    2690    "udp" []
  , makeService "itinternet"     2691    "tcp" []
  , makeService "itinternet"     2691    "udp" []
  , makeService "admins-lms"     2692    "tcp" []
  , makeService "admins-lms"     2692    "udp" []
  , makeService "pwrsevent"      2694    "tcp" []
  , makeService "pwrsevent"      2694    "udp" []
  , makeService "vspread"        2695    "tcp" []
  , makeService "vspread"        2695    "udp" []
  , makeService "unifyadmin"     2696    "tcp" []
  , makeService "unifyadmin"     2696    "udp" []
  , makeService "oce-snmp-trap"  2697    "tcp" []
  , makeService "oce-snmp-trap"  2697    "udp" []
  , makeService "mck-ivpip"      2698    "tcp" []
  , makeService "mck-ivpip"      2698    "udp" []
  , makeService "csoft-plusclnt" 2699    "tcp" []
  , makeService "csoft-plusclnt" 2699    "udp" []
  , makeService "tqdata"         2700    "tcp" []
  , makeService "tqdata"         2700    "udp" []
  , makeService "sms-rcinfo"     2701    "tcp" []
  , makeService "sms-rcinfo"     2701    "udp" []
  , makeService "sms-xfer"       2702    "tcp" []
  , makeService "sms-xfer"       2702    "udp" []
  , makeService "sms-chat"       2703    "tcp" []
  , makeService "sms-chat"       2703    "udp" []
  , makeService "sms-remctrl"    2704    "tcp" []
  , makeService "sms-remctrl"    2704    "udp" []
  , makeService "sds-admin"      2705    "tcp" []
  , makeService "sds-admin"      2705    "udp" []
  , makeService "ncdmirroring"   2706    "tcp" []
  , makeService "ncdmirroring"   2706    "udp" []
  , makeService "emcsymapiport"  2707    "tcp" []
  , makeService "emcsymapiport"  2707    "udp" []
  , makeService "banyan-net"     2708    "tcp" []
  , makeService "banyan-net"     2708    "udp" []
  , makeService "supermon"       2709    "tcp" []
  , makeService "supermon"       2709    "udp" []
  , makeService "sso-service"    2710    "tcp" []
  , makeService "sso-service"    2710    "udp" []
  , makeService "sso-control"    2711    "tcp" []
  , makeService "sso-control"    2711    "udp" []
  , makeService "aocp"           2712    "tcp" []
  , makeService "aocp"           2712    "udp" []
  , makeService "raventbs"       2713    "tcp" []
  , makeService "raventbs"       2713    "udp" []
  , makeService "raventdm"       2714    "tcp" []
  , makeService "raventdm"       2714    "udp" []
  , makeService "hpstgmgr2"      2715    "tcp" []
  , makeService "hpstgmgr2"      2715    "udp" []
  , makeService "inova-ip-disco" 2716    "tcp" []
  , makeService "inova-ip-disco" 2716    "udp" []
  , makeService "pn-requester"   2717    "tcp" []
  , makeService "pn-requester"   2717    "udp" []
  , makeService "pn-requester2"  2718    "tcp" []
  , makeService "pn-requester2"  2718    "udp" []
  , makeService "scan-change"    2719    "tcp" []
  , makeService "scan-change"    2719    "udp" []
  , makeService "wkars"          2720    "tcp" []
  , makeService "wkars"          2720    "udp" []
  , makeService "smart-diagnose" 2721    "tcp" []
  , makeService "smart-diagnose" 2721    "udp" []
  , makeService "proactivesrvr"  2722    "tcp" []
  , makeService "proactivesrvr"  2722    "udp" []
  , makeService "watchdog-nt"    2723    "tcp" []
  , makeService "watchdog-nt"    2723    "udp" []
  , makeService "qotps"          2724    "tcp" []
  , makeService "qotps"          2724    "udp" []
  , makeService "msolap-ptp2"    2725    "tcp" []
  , makeService "msolap-ptp2"    2725    "udp" []
  , makeService "tams"           2726    "tcp" []
  , makeService "tams"           2726    "udp" []
  , makeService "mgcp-callagent" 2727    "tcp" []
  , makeService "mgcp-callagent" 2727    "udp" []
  , makeService "sqdr"           2728    "tcp" []
  , makeService "sqdr"           2728    "udp" []
  , makeService "tcim-control"   2729    "tcp" []
  , makeService "tcim-control"   2729    "udp" []
  , makeService "nec-raidplus"   2730    "tcp" []
  , makeService "nec-raidplus"   2730    "udp" []
  , makeService "fyre-messanger" 2731    "tcp" []
  , makeService "fyre-messanger" 2731    "udp" []
  , makeService "g5m"            2732    "tcp" []
  , makeService "g5m"            2732    "udp" []
  , makeService "signet-ctf"     2733    "tcp" []
  , makeService "signet-ctf"     2733    "udp" []
  , makeService "ccs-software"   2734    "tcp" []
  , makeService "ccs-software"   2734    "udp" []
  , makeService "netiq-mc"       2735    "tcp" []
  , makeService "netiq-mc"       2735    "udp" []
  , makeService "radwiz-nms-srv" 2736    "tcp" []
  , makeService "radwiz-nms-srv" 2736    "udp" []
  , makeService "srp-feedback"   2737    "tcp" []
  , makeService "srp-feedback"   2737    "udp" []
  , makeService "ndl-tcp-ois-gw" 2738    "tcp" []
  , makeService "ndl-tcp-ois-gw" 2738    "udp" []
  , makeService "tn-timing"      2739    "tcp" []
  , makeService "tn-timing"      2739    "udp" []
  , makeService "alarm"          2740    "tcp" []
  , makeService "alarm"          2740    "udp" []
  , makeService "tsb"            2741    "tcp" []
  , makeService "tsb"            2741    "udp" []
  , makeService "tsb2"           2742    "tcp" []
  , makeService "tsb2"           2742    "udp" []
  , makeService "murx"           2743    "tcp" []
  , makeService "murx"           2743    "udp" []
  , makeService "honyaku"        2744    "tcp" []
  , makeService "honyaku"        2744    "udp" []
  , makeService "urbisnet"       2745    "tcp" []
  , makeService "urbisnet"       2745    "udp" []
  , makeService "cpudpencap"     2746    "tcp" []
  , makeService "cpudpencap"     2746    "udp" []
  , makeService "fjippol-swrly"  2747    "tcp" []
  , makeService "fjippol-swrly"  2747    "udp" []
  , makeService "fjippol-polsvr" 2748    "tcp" []
  , makeService "fjippol-polsvr" 2748    "udp" []
  , makeService "fjippol-cnsl"   2749    "tcp" []
  , makeService "fjippol-cnsl"   2749    "udp" []
  , makeService "fjippol-port1"  2750    "tcp" []
  , makeService "fjippol-port1"  2750    "udp" []
  , makeService "fjippol-port2"  2751    "tcp" []
  , makeService "fjippol-port2"  2751    "udp" []
  , makeService "rsisysaccess"   2752    "tcp" []
  , makeService "rsisysaccess"   2752    "udp" []
  , makeService "de-spot"        2753    "tcp" []
  , makeService "de-spot"        2753    "udp" []
  , makeService "apollo-cc"      2754    "tcp" []
  , makeService "apollo-cc"      2754    "udp" []
  , makeService "expresspay"     2755    "tcp" []
  , makeService "expresspay"     2755    "udp" []
  , makeService "simplement-tie" 2756    "tcp" []
  , makeService "simplement-tie" 2756    "udp" []
  , makeService "cnrp"           2757    "tcp" []
  , makeService "cnrp"           2757    "udp" []
  , makeService "apollo-status"  2758    "tcp" []
  , makeService "apollo-status"  2758    "udp" []
  , makeService "apollo-gms"     2759    "tcp" []
  , makeService "apollo-gms"     2759    "udp" []
  , makeService "sabams"         2760    "tcp" []
  , makeService "sabams"         2760    "udp" []
  , makeService "dicom-iscl"     2761    "tcp" []
  , makeService "dicom-iscl"     2761    "udp" []
  , makeService "dicom-tls"      2762    "tcp" []
  , makeService "dicom-tls"      2762    "udp" []
  , makeService "desktop-dna"    2763    "tcp" []
  , makeService "desktop-dna"    2763    "udp" []
  , makeService "data-insurance" 2764    "tcp" []
  , makeService "data-insurance" 2764    "udp" []
  , makeService "qip-audup"      2765    "tcp" []
  , makeService "qip-audup"      2765    "udp" []
  , makeService "compaq-scp"     2766    "tcp" []
  , makeService "compaq-scp"     2766    "udp" []
  , makeService "uadtc"          2767    "tcp" []
  , makeService "uadtc"          2767    "udp" []
  , makeService "uacs"           2768    "tcp" []
  , makeService "uacs"           2768    "udp" []
  , makeService "exce"           2769    "tcp" []
  , makeService "exce"           2769    "udp" []
  , makeService "veronica"       2770    "tcp" []
  , makeService "veronica"       2770    "udp" []
  , makeService "vergencecm"     2771    "tcp" []
  , makeService "vergencecm"     2771    "udp" []
  , makeService "auris"          2772    "tcp" []
  , makeService "auris"          2772    "udp" []
  , makeService "rbakcup1"       2773    "tcp" []
  , makeService "rbakcup1"       2773    "udp" []
  , makeService "rbakcup2"       2774    "tcp" []
  , makeService "rbakcup2"       2774    "udp" []
  , makeService "smpp"           2775    "tcp" []
  , makeService "smpp"           2775    "udp" []
  , makeService "ridgeway1"      2776    "tcp" []
  , makeService "ridgeway1"      2776    "udp" []
  , makeService "ridgeway2"      2777    "tcp" []
  , makeService "ridgeway2"      2777    "udp" []
  , makeService "gwen-sonya"     2778    "tcp" []
  , makeService "gwen-sonya"     2778    "udp" []
  , makeService "lbc-sync"       2779    "tcp" []
  , makeService "lbc-sync"       2779    "udp" []
  , makeService "lbc-control"    2780    "tcp" []
  , makeService "lbc-control"    2780    "udp" []
  , makeService "whosells"       2781    "tcp" []
  , makeService "whosells"       2781    "udp" []
  , makeService "everydayrc"     2782    "tcp" []
  , makeService "everydayrc"     2782    "udp" []
  , makeService "aises"          2783    "tcp" []
  , makeService "aises"          2783    "udp" []
  , makeService "www-dev"        2784    "tcp" []
  , makeService "www-dev"        2784    "udp" []
  , makeService "aic-np"         2785    "tcp" []
  , makeService "aic-np"         2785    "udp" []
  , makeService "aic-oncrpc"     2786    "tcp" []
  , makeService "aic-oncrpc"     2786    "udp" []
  , makeService "piccolo"        2787    "tcp" []
  , makeService "piccolo"        2787    "udp" []
  , makeService "fryeserv"       2788    "tcp" []
  , makeService "fryeserv"       2788    "udp" []
  , makeService "media-agent"    2789    "tcp" []
  , makeService "media-agent"    2789    "udp" []
  , makeService "plgproxy"       2790    "tcp" []
  , makeService "plgproxy"       2790    "udp" []
  , makeService "mtport-regist"  2791    "tcp" []
  , makeService "mtport-regist"  2791    "udp" []
  , makeService "f5-globalsite"  2792    "tcp" []
  , makeService "f5-globalsite"  2792    "udp" []
  , makeService "initlsmsad"     2793    "tcp" []
  , makeService "initlsmsad"     2793    "udp" []
  , makeService "livestats"      2795    "tcp" []
  , makeService "livestats"      2795    "udp" []
  , makeService "ac-tech"        2796    "tcp" []
  , makeService "ac-tech"        2796    "udp" []
  , makeService "esp-encap"      2797    "tcp" []
  , makeService "esp-encap"      2797    "udp" []
  , makeService "tmesis-upshot"  2798    "tcp" []
  , makeService "tmesis-upshot"  2798    "udp" []
  , makeService "icon-discover"  2799    "tcp" []
  , makeService "icon-discover"  2799    "udp" []
  , makeService "acc-raid"       2800    "tcp" []
  , makeService "acc-raid"       2800    "udp" []
  , makeService "igcp"           2801    "tcp" []
  , makeService "igcp"           2801    "udp" []
  , makeService "veritas-tcp1"   2802    "tcp" []
  , makeService "veritas-udp1"   2802    "udp" []
  , makeService "btprjctrl"      2803    "tcp" []
  , makeService "btprjctrl"      2803    "udp" []
  , makeService "dvr-esm"        2804    "tcp" []
  , makeService "dvr-esm"        2804    "udp" []
  , makeService "wta-wsp-s"      2805    "tcp" []
  , makeService "wta-wsp-s"      2805    "udp" []
  , makeService "cspuni"         2806    "tcp" []
  , makeService "cspuni"         2806    "udp" []
  , makeService "cspmulti"       2807    "tcp" []
  , makeService "cspmulti"       2807    "udp" []
  , makeService "j-lan-p"        2808    "tcp" []
  , makeService "j-lan-p"        2808    "udp" []
  , makeService "corbaloc"       2809    "udp" []
  , makeService "netsteward"     2810    "tcp" []
  , makeService "netsteward"     2810    "udp" []
  , makeService "gsiftp"         2811    "tcp" []
  , makeService "gsiftp"         2811    "udp" []
  , makeService "atmtcp"         2812    "tcp" []
  , makeService "atmtcp"         2812    "udp" []
  , makeService "llm-pass"       2813    "tcp" []
  , makeService "llm-pass"       2813    "udp" []
  , makeService "llm-csv"        2814    "tcp" []
  , makeService "llm-csv"        2814    "udp" []
  , makeService "lbc-measure"    2815    "tcp" []
  , makeService "lbc-measure"    2815    "udp" []
  , makeService "lbc-watchdog"   2816    "tcp" []
  , makeService "lbc-watchdog"   2816    "udp" []
  , makeService "nmsigport"      2817    "tcp" []
  , makeService "nmsigport"      2817    "udp" []
  , makeService "rmlnk"          2818    "tcp" []
  , makeService "rmlnk"          2818    "udp" []
  , makeService "fc-faultnotify" 2819    "tcp" []
  , makeService "fc-faultnotify" 2819    "udp" []
  , makeService "univision"      2820    "tcp" []
  , makeService "univision"      2820    "udp" []
  , makeService "vrts-at-port"   2821    "tcp" []
  , makeService "vrts-at-port"   2821    "udp" []
  , makeService "ka0wuc"         2822    "tcp" []
  , makeService "ka0wuc"         2822    "udp" []
  , makeService "cqg-netlan"     2823    "tcp" []
  , makeService "cqg-netlan"     2823    "udp" []
  , makeService "cqg-netlan-1"   2824    "tcp" []
  , makeService "cqg-netlan-1"   2824    "udp" []
  , makeService "slc-systemlog"  2826    "tcp" []
  , makeService "slc-systemlog"  2826    "udp" []
  , makeService "slc-ctrlrloops" 2827    "tcp" []
  , makeService "slc-ctrlrloops" 2827    "udp" []
  , makeService "itm-lm"         2828    "tcp" []
  , makeService "itm-lm"         2828    "udp" []
  , makeService "silkp1"         2829    "tcp" []
  , makeService "silkp1"         2829    "udp" []
  , makeService "silkp2"         2830    "tcp" []
  , makeService "silkp2"         2830    "udp" []
  , makeService "silkp3"         2831    "tcp" []
  , makeService "silkp3"         2831    "udp" []
  , makeService "silkp4"         2832    "tcp" []
  , makeService "silkp4"         2832    "udp" []
  , makeService "glishd"         2833    "tcp" []
  , makeService "glishd"         2833    "udp" []
  , makeService "evtp"           2834    "tcp" []
  , makeService "evtp"           2834    "udp" []
  , makeService "evtp-data"      2835    "tcp" []
  , makeService "evtp-data"      2835    "udp" []
  , makeService "catalyst"       2836    "tcp" []
  , makeService "catalyst"       2836    "udp" []
  , makeService "repliweb"       2837    "tcp" []
  , makeService "repliweb"       2837    "udp" []
  , makeService "starbot"        2838    "tcp" []
  , makeService "starbot"        2838    "udp" []
  , makeService "l3-exprt"       2840    "tcp" []
  , makeService "l3-exprt"       2840    "udp" []
  , makeService "l3-ranger"      2841    "tcp" []
  , makeService "l3-ranger"      2841    "udp" []
  , makeService "l3-hawk"        2842    "tcp" []
  , makeService "l3-hawk"        2842    "udp" []
  , makeService "pdnet"          2843    "tcp" []
  , makeService "pdnet"          2843    "udp" []
  , makeService "bpcp-poll"      2844    "tcp" []
  , makeService "bpcp-poll"      2844    "udp" []
  , makeService "bpcp-trap"      2845    "tcp" []
  , makeService "bpcp-trap"      2845    "udp" []
  , makeService "aimpp-hello"    2846    "tcp" []
  , makeService "aimpp-hello"    2846    "udp" []
  , makeService "aimpp-port-req" 2847    "tcp" []
  , makeService "aimpp-port-req" 2847    "udp" []
  , makeService "amt-blc-port"   2848    "tcp" []
  , makeService "amt-blc-port"   2848    "udp" []
  , makeService "metaconsole"    2850    "tcp" []
  , makeService "metaconsole"    2850    "udp" []
  , makeService "webemshttp"     2851    "tcp" []
  , makeService "webemshttp"     2851    "udp" []
  , makeService "bears-01"       2852    "tcp" []
  , makeService "bears-01"       2852    "udp" []
  , makeService "ispipes"        2853    "tcp" []
  , makeService "ispipes"        2853    "udp" []
  , makeService "infomover"      2854    "tcp" []
  , makeService "infomover"      2854    "udp" []
  , makeService "msrp"           2855    "tcp" []
  , makeService "msrp"           2855    "udp" []
  , makeService "cesdinv"        2856    "tcp" []
  , makeService "cesdinv"        2856    "udp" []
  , makeService "simctlp"        2857    "tcp" []
  , makeService "simctlp"        2857    "udp" []
  , makeService "ecnp"           2858    "tcp" []
  , makeService "ecnp"           2858    "udp" []
  , makeService "activememory"   2859    "tcp" []
  , makeService "activememory"   2859    "udp" []
  , makeService "dialpad-voice1" 2860    "tcp" []
  , makeService "dialpad-voice1" 2860    "udp" []
  , makeService "dialpad-voice2" 2861    "tcp" []
  , makeService "dialpad-voice2" 2861    "udp" []
  , makeService "ttg-protocol"   2862    "tcp" []
  , makeService "ttg-protocol"   2862    "udp" []
  , makeService "sonardata"      2863    "tcp" []
  , makeService "sonardata"      2863    "udp" []
  , makeService "astromed-main"  2864    "tcp" []
  , makeService "astromed-main"  2864    "udp" []
  , makeService "pit-vpn"        2865    "tcp" []
  , makeService "pit-vpn"        2865    "udp" []
  , makeService "iwlistener"     2866    "tcp" []
  , makeService "iwlistener"     2866    "udp" []
  , makeService "esps-portal"    2867    "tcp" []
  , makeService "esps-portal"    2867    "udp" []
  , makeService "npep-messaging" 2868    "tcp" []
  , makeService "npep-messaging" 2868    "udp" []
  , makeService "icslap"         2869    "tcp" []
  , makeService "icslap"         2869    "udp" []
  , makeService "daishi"         2870    "tcp" []
  , makeService "daishi"         2870    "udp" []
  , makeService "msi-selectplay" 2871    "tcp" []
  , makeService "msi-selectplay" 2871    "udp" []
  , makeService "radix"          2872    "tcp" []
  , makeService "radix"          2872    "udp" []
  , makeService "dxmessagebase1" 2874    "tcp" []
  , makeService "dxmessagebase1" 2874    "udp" []
  , makeService "dxmessagebase2" 2875    "tcp" []
  , makeService "dxmessagebase2" 2875    "udp" []
  , makeService "sps-tunnel"     2876    "tcp" []
  , makeService "sps-tunnel"     2876    "udp" []
  , makeService "bluelance"      2877    "tcp" []
  , makeService "bluelance"      2877    "udp" []
  , makeService "aap"            2878    "tcp" []
  , makeService "aap"            2878    "udp" []
  , makeService "ucentric-ds"    2879    "tcp" []
  , makeService "ucentric-ds"    2879    "udp" []
  , makeService "synapse"        2880    "tcp" []
  , makeService "synapse"        2880    "udp" []
  , makeService "ndsp"           2881    "tcp" []
  , makeService "ndsp"           2881    "udp" []
  , makeService "ndtp"           2882    "tcp" []
  , makeService "ndtp"           2882    "udp" []
  , makeService "ndnp"           2883    "tcp" []
  , makeService "ndnp"           2883    "udp" []
  , makeService "flashmsg"       2884    "tcp" []
  , makeService "flashmsg"       2884    "udp" []
  , makeService "topflow"        2885    "tcp" []
  , makeService "topflow"        2885    "udp" []
  , makeService "responselogic"  2886    "tcp" []
  , makeService "responselogic"  2886    "udp" []
  , makeService "aironetddp"     2887    "tcp" []
  , makeService "aironetddp"     2887    "udp" []
  , makeService "spcsdlobby"     2888    "tcp" []
  , makeService "spcsdlobby"     2888    "udp" []
  , makeService "rsom"           2889    "tcp" []
  , makeService "rsom"           2889    "udp" []
  , makeService "cspclmulti"     2890    "tcp" []
  , makeService "cspclmulti"     2890    "udp" []
  , makeService "cinegrfx-elmd"  2891    "tcp" []
  , makeService "cinegrfx-elmd"  2891    "udp" []
  , makeService "snifferdata"    2892    "tcp" []
  , makeService "snifferdata"    2892    "udp" []
  , makeService "vseconnector"   2893    "tcp" []
  , makeService "vseconnector"   2893    "udp" []
  , makeService "abacus-remote"  2894    "tcp" []
  , makeService "abacus-remote"  2894    "udp" []
  , makeService "natuslink"      2895    "tcp" []
  , makeService "natuslink"      2895    "udp" []
  , makeService "ecovisiong6-1"  2896    "tcp" []
  , makeService "ecovisiong6-1"  2896    "udp" []
  , makeService "citrix-rtmp"    2897    "tcp" []
  , makeService "citrix-rtmp"    2897    "udp" []
  , makeService "appliance-cfg"  2898    "tcp" []
  , makeService "appliance-cfg"  2898    "udp" []
  , makeService "powergemplus"   2899    "tcp" []
  , makeService "powergemplus"   2899    "udp" []
  , makeService "quicksuite"     2900    "tcp" []
  , makeService "quicksuite"     2900    "udp" []
  , makeService "allstorcns"     2901    "tcp" []
  , makeService "allstorcns"     2901    "udp" []
  , makeService "netaspi"        2902    "tcp" []
  , makeService "netaspi"        2902    "udp" []
  , makeService "suitcase"       2903    "tcp" []
  , makeService "suitcase"       2903    "udp" []
  , makeService "m2ua"           2904    "tcp" []
  , makeService "m2ua"           2904    "udp" []
  , makeService "m3ua"           2905    "tcp" []
  , makeService "caller9"        2906    "tcp" []
  , makeService "caller9"        2906    "udp" []
  , makeService "webmethods-b2b" 2907    "tcp" []
  , makeService "webmethods-b2b" 2907    "udp" []
  , makeService "mao"            2908    "tcp" []
  , makeService "mao"            2908    "udp" []
  , makeService "funk-dialout"   2909    "tcp" []
  , makeService "funk-dialout"   2909    "udp" []
  , makeService "tdaccess"       2910    "tcp" []
  , makeService "tdaccess"       2910    "udp" []
  , makeService "blockade"       2911    "tcp" []
  , makeService "blockade"       2911    "udp" []
  , makeService "epicon"         2912    "tcp" []
  , makeService "epicon"         2912    "udp" []
  , makeService "boosterware"    2913    "tcp" []
  , makeService "boosterware"    2913    "udp" []
  , makeService "gamelobby"      2914    "tcp" []
  , makeService "gamelobby"      2914    "udp" []
  , makeService "tksocket"       2915    "tcp" []
  , makeService "tksocket"       2915    "udp" []
  , makeService "elvin_server"   2916    "tcp" ["elvin-server"]
  , makeService "elvin_server"   2916    "udp" ["elvin-server"]
  , makeService "elvin_client"   2917    "tcp" ["elvin-client"]
  , makeService "elvin_client"   2917    "udp" ["elvin-client"]
  , makeService "kastenchasepad" 2918    "tcp" []
  , makeService "kastenchasepad" 2918    "udp" []
  , makeService "roboer"         2919    "tcp" []
  , makeService "roboer"         2919    "udp" []
  , makeService "roboeda"        2920    "tcp" []
  , makeService "roboeda"        2920    "udp" []
  , makeService "cesdcdman"      2921    "tcp" []
  , makeService "cesdcdman"      2921    "udp" []
  , makeService "cesdcdtrn"      2922    "tcp" []
  , makeService "cesdcdtrn"      2922    "udp" []
  , makeService "wta-wsp-wtp-s"  2923    "tcp" []
  , makeService "wta-wsp-wtp-s"  2923    "udp" []
  , makeService "precise-vip"    2924    "tcp" []
  , makeService "precise-vip"    2924    "udp" []
  , makeService "mobile-file-dl" 2926    "tcp" []
  , makeService "mobile-file-dl" 2926    "udp" []
  , makeService "unimobilectrl"  2927    "tcp" []
  , makeService "unimobilectrl"  2927    "udp" []
  , makeService "redstone-cpss"  2928    "tcp" []
  , makeService "redstone-cpss"  2928    "udp" []
  , makeService "amx-webadmin"   2929    "tcp" []
  , makeService "amx-webadmin"   2929    "udp" []
  , makeService "amx-weblinx"    2930    "tcp" []
  , makeService "amx-weblinx"    2930    "udp" []
  , makeService "circle-x"       2931    "tcp" []
  , makeService "circle-x"       2931    "udp" []
  , makeService "incp"           2932    "tcp" []
  , makeService "incp"           2932    "udp" []
  , makeService "4-tieropmgw"    2933    "tcp" []
  , makeService "4-tieropmgw"    2933    "udp" []
  , makeService "4-tieropmcli"   2934    "tcp" []
  , makeService "4-tieropmcli"   2934    "udp" []
  , makeService "qtp"            2935    "tcp" []
  , makeService "qtp"            2935    "udp" []
  , makeService "otpatch"        2936    "tcp" []
  , makeService "otpatch"        2936    "udp" []
  , makeService "pnaconsult-lm"  2937    "tcp" []
  , makeService "pnaconsult-lm"  2937    "udp" []
  , makeService "sm-pas-1"       2938    "tcp" []
  , makeService "sm-pas-1"       2938    "udp" []
  , makeService "sm-pas-2"       2939    "tcp" []
  , makeService "sm-pas-2"       2939    "udp" []
  , makeService "sm-pas-3"       2940    "tcp" []
  , makeService "sm-pas-3"       2940    "udp" []
  , makeService "sm-pas-4"       2941    "tcp" []
  , makeService "sm-pas-4"       2941    "udp" []
  , makeService "sm-pas-5"       2942    "tcp" []
  , makeService "sm-pas-5"       2942    "udp" []
  , makeService "ttnrepository"  2943    "tcp" []
  , makeService "ttnrepository"  2943    "udp" []
  , makeService "megaco-h248"    2944    "tcp" []
  , makeService "megaco-h248"    2944    "udp" []
  , makeService "h248-binary"    2945    "tcp" []
  , makeService "h248-binary"    2945    "udp" []
  , makeService "fjsvmpor"       2946    "tcp" []
  , makeService "fjsvmpor"       2946    "udp" []
  , makeService "gpsd"           2947    "tcp" []
  , makeService "gpsd"           2947    "udp" []
  , makeService "wap-push"       2948    "tcp" []
  , makeService "wap-push"       2948    "udp" []
  , makeService "wap-pushsecure" 2949    "tcp" []
  , makeService "wap-pushsecure" 2949    "udp" []
  , makeService "esip"           2950    "tcp" []
  , makeService "esip"           2950    "udp" []
  , makeService "ottp"           2951    "tcp" []
  , makeService "ottp"           2951    "udp" []
  , makeService "mpfwsas"        2952    "tcp" []
  , makeService "mpfwsas"        2952    "udp" []
  , makeService "ovalarmsrv"     2953    "tcp" []
  , makeService "ovalarmsrv"     2953    "udp" []
  , makeService "ovalarmsrv-cmd" 2954    "tcp" []
  , makeService "ovalarmsrv-cmd" 2954    "udp" []
  , makeService "csnotify"       2955    "tcp" []
  , makeService "csnotify"       2955    "udp" []
  , makeService "ovrimosdbman"   2956    "tcp" []
  , makeService "ovrimosdbman"   2956    "udp" []
  , makeService "jmact5"         2957    "tcp" []
  , makeService "jmact5"         2957    "udp" []
  , makeService "jmact6"         2958    "tcp" []
  , makeService "jmact6"         2958    "udp" []
  , makeService "rmopagt"        2959    "tcp" []
  , makeService "rmopagt"        2959    "udp" []
  , makeService "dfoxserver"     2960    "tcp" []
  , makeService "dfoxserver"     2960    "udp" []
  , makeService "boldsoft-lm"    2961    "tcp" []
  , makeService "boldsoft-lm"    2961    "udp" []
  , makeService "iph-policy-cli" 2962    "tcp" []
  , makeService "iph-policy-cli" 2962    "udp" []
  , makeService "iph-policy-adm" 2963    "tcp" []
  , makeService "iph-policy-adm" 2963    "udp" []
  , makeService "bullant-srap"   2964    "tcp" []
  , makeService "bullant-srap"   2964    "udp" []
  , makeService "bullant-rap"    2965    "tcp" []
  , makeService "bullant-rap"    2965    "udp" []
  , makeService "idp-infotrieve" 2966    "tcp" []
  , makeService "idp-infotrieve" 2966    "udp" []
  , makeService "ssc-agent"      2967    "tcp" []
  , makeService "ssc-agent"      2967    "udp" []
  , makeService "enpp"           2968    "tcp" []
  , makeService "enpp"           2968    "udp" []
  , makeService "essp"           2969    "tcp" []
  , makeService "essp"           2969    "udp" []
  , makeService "index-net"      2970    "tcp" []
  , makeService "index-net"      2970    "udp" []
  , makeService "netclip"        2971    "tcp" []
  , makeService "netclip"        2971    "udp" []
  , makeService "pmsm-webrctl"   2972    "tcp" []
  , makeService "pmsm-webrctl"   2972    "udp" []
  , makeService "svnetworks"     2973    "tcp" []
  , makeService "svnetworks"     2973    "udp" []
  , makeService "signal"         2974    "tcp" []
  , makeService "signal"         2974    "udp" []
  , makeService "fjmpcm"         2975    "tcp" []
  , makeService "fjmpcm"         2975    "udp" []
  , makeService "cns-srv-port"   2976    "tcp" []
  , makeService "cns-srv-port"   2976    "udp" []
  , makeService "ttc-etap-ns"    2977    "tcp" []
  , makeService "ttc-etap-ns"    2977    "udp" []
  , makeService "ttc-etap-ds"    2978    "tcp" []
  , makeService "ttc-etap-ds"    2978    "udp" []
  , makeService "h263-video"     2979    "tcp" []
  , makeService "h263-video"     2979    "udp" []
  , makeService "wimd"           2980    "tcp" []
  , makeService "wimd"           2980    "udp" []
  , makeService "mylxamport"     2981    "tcp" []
  , makeService "mylxamport"     2981    "udp" []
  , makeService "iwb-whiteboard" 2982    "tcp" []
  , makeService "iwb-whiteboard" 2982    "udp" []
  , makeService "netplan"        2983    "tcp" []
  , makeService "netplan"        2983    "udp" []
  , makeService "hpidsadmin"     2984    "tcp" []
  , makeService "hpidsadmin"     2984    "udp" []
  , makeService "hpidsagent"     2985    "tcp" []
  , makeService "hpidsagent"     2985    "udp" []
  , makeService "stonefalls"     2986    "tcp" []
  , makeService "stonefalls"     2986    "udp" []
  , makeService "identify"       2987    "tcp" []
  , makeService "identify"       2987    "udp" []
  , makeService "zarkov"         2989    "tcp" []
  , makeService "zarkov"         2989    "udp" []
  , makeService "boscap"         2990    "tcp" []
  , makeService "boscap"         2990    "udp" []
  , makeService "wkstn-mon"      2991    "tcp" []
  , makeService "wkstn-mon"      2991    "udp" []
  , makeService "avenyo"         2992    "tcp" []
  , makeService "avenyo"         2992    "udp" []
  , makeService "veritas-vis1"   2993    "tcp" []
  , makeService "veritas-vis1"   2993    "udp" []
  , makeService "veritas-vis2"   2994    "tcp" []
  , makeService "veritas-vis2"   2994    "udp" []
  , makeService "idrs"           2995    "tcp" []
  , makeService "idrs"           2995    "udp" []
  , makeService "vsixml"         2996    "tcp" []
  , makeService "vsixml"         2996    "udp" []
  , makeService "rebol"          2997    "tcp" []
  , makeService "rebol"          2997    "udp" []
  , makeService "realsecure"     2998    "tcp" []
  , makeService "realsecure"     2998    "udp" []
  , makeService "remoteware-un"  2999    "tcp" []
  , makeService "remoteware-un"  2999    "udp" []
  , makeService "hbci"           3000    "tcp" []
  , makeService "hbci"           3000    "udp" []
  , makeService "origo-native"   3001    "tcp" []
  , makeService "exlm-agent"     3002    "tcp" []
  , makeService "exlm-agent"     3002    "udp" []
  , makeService "cgms"           3003    "tcp" []
  , makeService "cgms"           3003    "udp" []
  , makeService "csoftragent"    3004    "tcp" []
  , makeService "csoftragent"    3004    "udp" []
  , makeService "geniuslm"       3005    "tcp" []
  , makeService "geniuslm"       3005    "udp" []
  , makeService "ii-admin"       3006    "tcp" []
  , makeService "ii-admin"       3006    "udp" []
  , makeService "lotusmtap"      3007    "tcp" []
  , makeService "lotusmtap"      3007    "udp" []
  , makeService "midnight-tech"  3008    "tcp" []
  , makeService "midnight-tech"  3008    "udp" []
  , makeService "pxc-ntfy"       3009    "tcp" []
  , makeService "pxc-ntfy"       3009    "udp" []
  , makeService "gw"             3010    "tcp" []
  , makeService "ping-pong"      3010    "udp" []
  , makeService "trusted-web"    3011    "tcp" []
  , makeService "trusted-web"    3011    "udp" []
  , makeService "twsdss"         3012    "tcp" []
  , makeService "twsdss"         3012    "udp" []
  , makeService "gilatskysurfer" 3013    "tcp" []
  , makeService "gilatskysurfer" 3013    "udp" []
  , makeService "broker_service" 3014    "tcp" ["broker-service"]
  , makeService "broker_service" 3014    "udp" ["broker-service"]
  , makeService "nati-dstp"      3015    "tcp" []
  , makeService "nati-dstp"      3015    "udp" []
  , makeService "notify_srvr"    3016    "tcp" ["notify-srvr"]
  , makeService "notify_srvr"    3016    "udp" ["notify-srvr"]
  , makeService "event_listener" 3017    "tcp" ["event-listener"]
  , makeService "event_listener" 3017    "udp" ["event-listener"]
  , makeService "srvc_registry"  3018    "tcp" ["srvc-registry"]
  , makeService "srvc_registry"  3018    "udp" ["srvc-registry"]
  , makeService "resource_mgr"   3019    "tcp" ["resource-mgr"]
  , makeService "resource_mgr"   3019    "udp" ["resource-mgr"]
  , makeService "cifs"           3020    "tcp" []
  , makeService "cifs"           3020    "udp" []
  , makeService "agriserver"     3021    "tcp" []
  , makeService "agriserver"     3021    "udp" []
  , makeService "csregagent"     3022    "tcp" []
  , makeService "csregagent"     3022    "udp" []
  , makeService "magicnotes"     3023    "tcp" []
  , makeService "magicnotes"     3023    "udp" []
  , makeService "nds_sso"        3024    "tcp" ["nds-sso"]
  , makeService "nds_sso"        3024    "udp" ["nds-sso"]
  , makeService "arepa-raft"     3025    "tcp" []
  , makeService "arepa-raft"     3025    "udp" []
  , makeService "agri-gateway"   3026    "tcp" []
  , makeService "agri-gateway"   3026    "udp" []
  , makeService "LiebDevMgmt_C"  3027    "tcp" ["LiebDevMgmt-C"]
  , makeService "LiebDevMgmt_C"  3027    "udp" ["LiebDevMgmt-C"]
  , makeService "LiebDevMgmt_DM" 3028    "tcp" ["LiebDevMgmt-DM"]
  , makeService "LiebDevMgmt_DM" 3028    "udp" ["LiebDevMgmt-DM"]
  , makeService "LiebDevMgmt_A"  3029    "tcp" ["LiebDevMgmt-A"]
  , makeService "LiebDevMgmt_A"  3029    "udp" ["LiebDevMgmt-A"]
  , makeService "arepa-cas"      3030    "tcp" []
  , makeService "arepa-cas"      3030    "udp" []
  , makeService "eppc"           3031    "tcp" []
  , makeService "eppc"           3031    "udp" []
  , makeService "redwood-chat"   3032    "tcp" []
  , makeService "redwood-chat"   3032    "udp" []
  , makeService "pdb"            3033    "tcp" []
  , makeService "pdb"            3033    "udp" []
  , makeService "osmosis-aeea"   3034    "tcp" []
  , makeService "osmosis-aeea"   3034    "udp" []
  , makeService "fjsv-gssagt"    3035    "tcp" []
  , makeService "fjsv-gssagt"    3035    "udp" []
  , makeService "hagel-dump"     3036    "tcp" []
  , makeService "hagel-dump"     3036    "udp" []
  , makeService "hp-san-mgmt"    3037    "tcp" []
  , makeService "hp-san-mgmt"    3037    "udp" []
  , makeService "santak-ups"     3038    "tcp" []
  , makeService "santak-ups"     3038    "udp" []
  , makeService "cogitate"       3039    "tcp" []
  , makeService "cogitate"       3039    "udp" []
  , makeService "tomato-springs" 3040    "tcp" []
  , makeService "tomato-springs" 3040    "udp" []
  , makeService "di-traceware"   3041    "tcp" []
  , makeService "di-traceware"   3041    "udp" []
  , makeService "journee"        3042    "tcp" []
  , makeService "journee"        3042    "udp" []
  , makeService "brp"            3043    "tcp" []
  , makeService "brp"            3043    "udp" []
  , makeService "responsenet"    3045    "tcp" []
  , makeService "responsenet"    3045    "udp" []
  , makeService "di-ase"         3046    "tcp" []
  , makeService "di-ase"         3046    "udp" []
  , makeService "hlserver"       3047    "tcp" []
  , makeService "hlserver"       3047    "udp" []
  , makeService "pctrader"       3048    "tcp" []
  , makeService "pctrader"       3048    "udp" []
  , makeService "nsws"           3049    "tcp" []
  , makeService "nsws"           3049    "udp" []
  , makeService "gds_db"         3050    "tcp" ["gds-db"]
  , makeService "gds_db"         3050    "udp" ["gds-db"]
  , makeService "galaxy-server"  3051    "tcp" []
  , makeService "galaxy-server"  3051    "udp" []
  , makeService "apc-3052"       3052    "tcp" []
  , makeService "apc-3052"       3052    "udp" []
  , makeService "dsom-server"    3053    "tcp" []
  , makeService "dsom-server"    3053    "udp" []
  , makeService "amt-cnf-prot"   3054    "tcp" []
  , makeService "amt-cnf-prot"   3054    "udp" []
  , makeService "policyserver"   3055    "tcp" []
  , makeService "policyserver"   3055    "udp" []
  , makeService "cdl-server"     3056    "tcp" []
  , makeService "cdl-server"     3056    "udp" []
  , makeService "goahead-fldup"  3057    "tcp" []
  , makeService "goahead-fldup"  3057    "udp" []
  , makeService "videobeans"     3058    "tcp" []
  , makeService "videobeans"     3058    "udp" []
  , makeService "qsoft"          3059    "tcp" []
  , makeService "qsoft"          3059    "udp" []
  , makeService "interserver"    3060    "tcp" []
  , makeService "interserver"    3060    "udp" []
  , makeService "cautcpd"        3061    "tcp" []
  , makeService "cautcpd"        3061    "udp" []
  , makeService "ncacn-ip-tcp"   3062    "tcp" []
  , makeService "ncacn-ip-tcp"   3062    "udp" []
  , makeService "ncadg-ip-udp"   3063    "tcp" []
  , makeService "ncadg-ip-udp"   3063    "udp" []
  , makeService "rprt"           3064    "tcp" []
  , makeService "rprt"           3064    "udp" []
  , makeService "slinterbase"    3065    "tcp" []
  , makeService "slinterbase"    3065    "udp" []
  , makeService "netattachsdmp"  3066    "tcp" []
  , makeService "netattachsdmp"  3066    "udp" []
  , makeService "fjhpjp"         3067    "tcp" []
  , makeService "fjhpjp"         3067    "udp" []
  , makeService "ls3bcast"       3068    "tcp" []
  , makeService "ls3bcast"       3068    "udp" []
  , makeService "ls3"            3069    "tcp" []
  , makeService "ls3"            3069    "udp" []
  , makeService "mgxswitch"      3070    "tcp" []
  , makeService "mgxswitch"      3070    "udp" []
  , makeService "csd-mgmt-port"  3071    "tcp" []
  , makeService "csd-mgmt-port"  3071    "udp" []
  , makeService "csd-monitor"    3072    "tcp" []
  , makeService "csd-monitor"    3072    "udp" []
  , makeService "vcrp"           3073    "tcp" []
  , makeService "vcrp"           3073    "udp" []
  , makeService "xbox"           3074    "tcp" []
  , makeService "xbox"           3074    "udp" []
  , makeService "orbix-locator"  3075    "tcp" []
  , makeService "orbix-locator"  3075    "udp" []
  , makeService "orbix-config"   3076    "tcp" []
  , makeService "orbix-config"   3076    "udp" []
  , makeService "orbix-loc-ssl"  3077    "tcp" []
  , makeService "orbix-loc-ssl"  3077    "udp" []
  , makeService "orbix-cfg-ssl"  3078    "tcp" []
  , makeService "orbix-cfg-ssl"  3078    "udp" []
  , makeService "lv-frontpanel"  3079    "tcp" []
  , makeService "lv-frontpanel"  3079    "udp" []
  , makeService "stm_pproc"      3080    "tcp" ["stm-pproc"]
  , makeService "stm_pproc"      3080    "udp" ["stm-pproc"]
  , makeService "tl1-lv"         3081    "tcp" []
  , makeService "tl1-lv"         3081    "udp" []
  , makeService "tl1-raw"        3082    "tcp" []
  , makeService "tl1-raw"        3082    "udp" []
  , makeService "tl1-telnet"     3083    "tcp" []
  , makeService "tl1-telnet"     3083    "udp" []
  , makeService "itm-mccs"       3084    "tcp" []
  , makeService "itm-mccs"       3084    "udp" []
  , makeService "pcihreq"        3085    "tcp" []
  , makeService "pcihreq"        3085    "udp" []
  , makeService "jdl-dbkitchen"  3086    "tcp" []
  , makeService "jdl-dbkitchen"  3086    "udp" []
  , makeService "asoki-sma"      3087    "tcp" []
  , makeService "asoki-sma"      3087    "udp" []
  , makeService "xdtp"           3088    "tcp" []
  , makeService "xdtp"           3088    "udp" []
  , makeService "ptk-alink"      3089    "tcp" []
  , makeService "ptk-alink"      3089    "udp" []
  , makeService "stss"           3090    "tcp" []
  , makeService "stss"           3090    "udp" []
  , makeService "1ci-smcs"       3091    "tcp" []
  , makeService "1ci-smcs"       3091    "udp" []
  , makeService "rapidmq-center" 3093    "tcp" []
  , makeService "rapidmq-center" 3093    "udp" []
  , makeService "rapidmq-reg"    3094    "tcp" []
  , makeService "rapidmq-reg"    3094    "udp" []
  , makeService "panasas"        3095    "tcp" []
  , makeService "panasas"        3095    "udp" []
  , makeService "ndl-aps"        3096    "tcp" []
  , makeService "ndl-aps"        3096    "udp" []
  , makeService "umm-port"       3098    "tcp" []
  , makeService "umm-port"       3098    "udp" []
  , makeService "chmd"           3099    "tcp" []
  , makeService "chmd"           3099    "udp" []
  , makeService "opcon-xps"      3100    "tcp" []
  , makeService "opcon-xps"      3100    "udp" []
  , makeService "hp-pxpib"       3101    "tcp" []
  , makeService "hp-pxpib"       3101    "udp" []
  , makeService "slslavemon"     3102    "tcp" []
  , makeService "slslavemon"     3102    "udp" []
  , makeService "autocuesmi"     3103    "tcp" []
  , makeService "autocuesmi"     3103    "udp" []
  , makeService "autocuelog"     3104    "tcp" []
  , makeService "autocuetime"    3104    "udp" []
  , makeService "cardbox"        3105    "tcp" []
  , makeService "cardbox"        3105    "udp" []
  , makeService "cardbox-http"   3106    "tcp" []
  , makeService "cardbox-http"   3106    "udp" []
  , makeService "business"       3107    "tcp" []
  , makeService "business"       3107    "udp" []
  , makeService "geolocate"      3108    "tcp" []
  , makeService "geolocate"      3108    "udp" []
  , makeService "personnel"      3109    "tcp" []
  , makeService "personnel"      3109    "udp" []
  , makeService "sim-control"    3110    "tcp" []
  , makeService "sim-control"    3110    "udp" []
  , makeService "wsynch"         3111    "tcp" []
  , makeService "wsynch"         3111    "udp" []
  , makeService "ksysguard"      3112    "tcp" []
  , makeService "ksysguard"      3112    "udp" []
  , makeService "cs-auth-svr"    3113    "tcp" []
  , makeService "cs-auth-svr"    3113    "udp" []
  , makeService "ccmad"          3114    "tcp" []
  , makeService "ccmad"          3114    "udp" []
  , makeService "mctet-master"   3115    "tcp" []
  , makeService "mctet-master"   3115    "udp" []
  , makeService "mctet-gateway"  3116    "tcp" []
  , makeService "mctet-gateway"  3116    "udp" []
  , makeService "mctet-jserv"    3117    "tcp" []
  , makeService "mctet-jserv"    3117    "udp" []
  , makeService "pkagent"        3118    "tcp" []
  , makeService "pkagent"        3118    "udp" []
  , makeService "d2000kernel"    3119    "tcp" []
  , makeService "d2000kernel"    3119    "udp" []
  , makeService "d2000webserver" 3120    "tcp" []
  , makeService "d2000webserver" 3120    "udp" []
  , makeService "pcmk-remote"    3121    "tcp" []
  , makeService "vtr-emulator"   3122    "tcp" []
  , makeService "vtr-emulator"   3122    "udp" []
  , makeService "edix"           3123    "tcp" []
  , makeService "edix"           3123    "udp" []
  , makeService "beacon-port"    3124    "tcp" []
  , makeService "beacon-port"    3124    "udp" []
  , makeService "a13-an"         3125    "tcp" []
  , makeService "a13-an"         3125    "udp" []
  , makeService "ctx-bridge"     3127    "tcp" []
  , makeService "ctx-bridge"     3127    "udp" []
  , makeService "ndl-aas"        3128    "udp" []
  , makeService "netport-id"     3129    "tcp" []
  , makeService "netport-id"     3129    "udp" []
  , makeService "netbookmark"    3131    "tcp" []
  , makeService "netbookmark"    3131    "udp" []
  , makeService "ms-rule-engine" 3132    "tcp" []
  , makeService "ms-rule-engine" 3132    "udp" []
  , makeService "prism-deploy"   3133    "tcp" []
  , makeService "prism-deploy"   3133    "udp" []
  , makeService "ecp"            3134    "tcp" []
  , makeService "ecp"            3134    "udp" []
  , makeService "peerbook-port"  3135    "tcp" []
  , makeService "peerbook-port"  3135    "udp" []
  , makeService "grubd"          3136    "tcp" []
  , makeService "grubd"          3136    "udp" []
  , makeService "rtnt-1"         3137    "tcp" []
  , makeService "rtnt-1"         3137    "udp" []
  , makeService "rtnt-2"         3138    "tcp" []
  , makeService "rtnt-2"         3138    "udp" []
  , makeService "incognitorv"    3139    "tcp" []
  , makeService "incognitorv"    3139    "udp" []
  , makeService "ariliamulti"    3140    "tcp" []
  , makeService "ariliamulti"    3140    "udp" []
  , makeService "vmodem"         3141    "tcp" []
  , makeService "vmodem"         3141    "udp" []
  , makeService "rdc-wh-eos"     3142    "tcp" []
  , makeService "rdc-wh-eos"     3142    "udp" []
  , makeService "seaview"        3143    "tcp" []
  , makeService "seaview"        3143    "udp" []
  , makeService "tarantella"     3144    "tcp" []
  , makeService "tarantella"     3144    "udp" []
  , makeService "csi-lfap"       3145    "tcp" []
  , makeService "csi-lfap"       3145    "udp" []
  , makeService "bears-02"       3146    "tcp" []
  , makeService "bears-02"       3146    "udp" []
  , makeService "rfio"           3147    "tcp" []
  , makeService "rfio"           3147    "udp" []
  , makeService "nm-game-admin"  3148    "tcp" []
  , makeService "nm-game-admin"  3148    "udp" []
  , makeService "nm-game-server" 3149    "tcp" []
  , makeService "nm-game-server" 3149    "udp" []
  , makeService "nm-asses-admin" 3150    "tcp" []
  , makeService "nm-asses-admin" 3150    "udp" []
  , makeService "nm-assessor"    3151    "tcp" []
  , makeService "nm-assessor"    3151    "udp" []
  , makeService "feitianrockey"  3152    "tcp" []
  , makeService "feitianrockey"  3152    "udp" []
  , makeService "s8-client-port" 3153    "tcp" []
  , makeService "s8-client-port" 3153    "udp" []
  , makeService "ccmrmi"         3154    "tcp" []
  , makeService "ccmrmi"         3154    "udp" []
  , makeService "jpegmpeg"       3155    "tcp" []
  , makeService "jpegmpeg"       3155    "udp" []
  , makeService "indura"         3156    "tcp" []
  , makeService "indura"         3156    "udp" []
  , makeService "e3consultants"  3157    "tcp" []
  , makeService "e3consultants"  3157    "udp" []
  , makeService "stvp"           3158    "tcp" []
  , makeService "stvp"           3158    "udp" []
  , makeService "navegaweb-port" 3159    "tcp" []
  , makeService "navegaweb-port" 3159    "udp" []
  , makeService "tip-app-server" 3160    "tcp" []
  , makeService "tip-app-server" 3160    "udp" []
  , makeService "doc1lm"         3161    "tcp" []
  , makeService "doc1lm"         3161    "udp" []
  , makeService "sflm"           3162    "tcp" []
  , makeService "sflm"           3162    "udp" []
  , makeService "res-sap"        3163    "tcp" []
  , makeService "res-sap"        3163    "udp" []
  , makeService "imprs"          3164    "tcp" []
  , makeService "imprs"          3164    "udp" []
  , makeService "newgenpay"      3165    "tcp" []
  , makeService "newgenpay"      3165    "udp" []
  , makeService "sossecollector" 3166    "tcp" []
  , makeService "sossecollector" 3166    "udp" []
  , makeService "nowcontact"     3167    "tcp" []
  , makeService "nowcontact"     3167    "udp" []
  , makeService "poweronnud"     3168    "tcp" []
  , makeService "poweronnud"     3168    "udp" []
  , makeService "serverview-as"  3169    "tcp" []
  , makeService "serverview-as"  3169    "udp" []
  , makeService "serverview-asn" 3170    "tcp" []
  , makeService "serverview-asn" 3170    "udp" []
  , makeService "serverview-gf"  3171    "tcp" []
  , makeService "serverview-gf"  3171    "udp" []
  , makeService "serverview-rm"  3172    "tcp" []
  , makeService "serverview-rm"  3172    "udp" []
  , makeService "serverview-icc" 3173    "tcp" []
  , makeService "serverview-icc" 3173    "udp" []
  , makeService "armi-server"    3174    "tcp" []
  , makeService "armi-server"    3174    "udp" []
  , makeService "t1-e1-over-ip"  3175    "tcp" []
  , makeService "t1-e1-over-ip"  3175    "udp" []
  , makeService "ars-master"     3176    "tcp" []
  , makeService "ars-master"     3176    "udp" []
  , makeService "phonex-port"    3177    "tcp" []
  , makeService "phonex-port"    3177    "udp" []
  , makeService "radclientport"  3178    "tcp" []
  , makeService "radclientport"  3178    "udp" []
  , makeService "h2gf-w-2m"      3179    "tcp" []
  , makeService "h2gf-w-2m"      3179    "udp" []
  , makeService "mc-brk-srv"     3180    "tcp" []
  , makeService "mc-brk-srv"     3180    "udp" []
  , makeService "bmcpatrolagent" 3181    "tcp" []
  , makeService "bmcpatrolagent" 3181    "udp" []
  , makeService "bmcpatrolrnvu"  3182    "tcp" []
  , makeService "bmcpatrolrnvu"  3182    "udp" []
  , makeService "cops-tls"       3183    "tcp" []
  , makeService "cops-tls"       3183    "udp" []
  , makeService "apogeex-port"   3184    "tcp" []
  , makeService "apogeex-port"   3184    "udp" []
  , makeService "smpppd"         3185    "tcp" []
  , makeService "smpppd"         3185    "udp" []
  , makeService "iiw-port"       3186    "tcp" []
  , makeService "iiw-port"       3186    "udp" []
  , makeService "odi-port"       3187    "tcp" []
  , makeService "odi-port"       3187    "udp" []
  , makeService "brcm-comm-port" 3188    "tcp" []
  , makeService "brcm-comm-port" 3188    "udp" []
  , makeService "pcle-infex"     3189    "tcp" []
  , makeService "pcle-infex"     3189    "udp" []
  , makeService "csvr-proxy"     3190    "tcp" []
  , makeService "csvr-proxy"     3190    "udp" []
  , makeService "csvr-sslproxy"  3191    "tcp" []
  , makeService "csvr-sslproxy"  3191    "udp" []
  , makeService "firemonrcc"     3192    "tcp" []
  , makeService "firemonrcc"     3192    "udp" []
  , makeService "spandataport"   3193    "tcp" []
  , makeService "spandataport"   3193    "udp" []
  , makeService "magbind"        3194    "tcp" []
  , makeService "magbind"        3194    "udp" []
  , makeService "ncu-1"          3195    "tcp" []
  , makeService "ncu-1"          3195    "udp" []
  , makeService "ncu-2"          3196    "tcp" []
  , makeService "ncu-2"          3196    "udp" []
  , makeService "embrace-dp-s"   3197    "tcp" []
  , makeService "embrace-dp-s"   3197    "udp" []
  , makeService "embrace-dp-c"   3198    "tcp" []
  , makeService "embrace-dp-c"   3198    "udp" []
  , makeService "dmod-workspace" 3199    "tcp" []
  , makeService "dmod-workspace" 3199    "udp" []
  , makeService "tick-port"      3200    "tcp" []
  , makeService "tick-port"      3200    "udp" []
  , makeService "cpq-tasksmart"  3201    "tcp" []
  , makeService "cpq-tasksmart"  3201    "udp" []
  , makeService "intraintra"     3202    "tcp" []
  , makeService "intraintra"     3202    "udp" []
  , makeService "netwatcher-mon" 3203    "tcp" []
  , makeService "netwatcher-mon" 3203    "udp" []
  , makeService "netwatcher-db"  3204    "tcp" []
  , makeService "netwatcher-db"  3204    "udp" []
  , makeService "isns"           3205    "tcp" []
  , makeService "isns"           3205    "udp" []
  , makeService "ironmail"       3206    "tcp" []
  , makeService "ironmail"       3206    "udp" []
  , makeService "vx-auth-port"   3207    "tcp" []
  , makeService "vx-auth-port"   3207    "udp" []
  , makeService "pfu-prcallback" 3208    "tcp" []
  , makeService "pfu-prcallback" 3208    "udp" []
  , makeService "netwkpathengine"3209    "tcp" []
  , makeService "netwkpathengine"3209    "udp" []
  , makeService "flamenco-proxy" 3210    "tcp" []
  , makeService "flamenco-proxy" 3210    "udp" []
  , makeService "avsecuremgmt"   3211    "tcp" []
  , makeService "avsecuremgmt"   3211    "udp" []
  , makeService "surveyinst"     3212    "tcp" []
  , makeService "surveyinst"     3212    "udp" []
  , makeService "neon24x7"       3213    "tcp" []
  , makeService "neon24x7"       3213    "udp" []
  , makeService "jmq-daemon-1"   3214    "tcp" []
  , makeService "jmq-daemon-1"   3214    "udp" []
  , makeService "jmq-daemon-2"   3215    "tcp" []
  , makeService "jmq-daemon-2"   3215    "udp" []
  , makeService "ferrari-foam"   3216    "tcp" []
  , makeService "ferrari-foam"   3216    "udp" []
  , makeService "unite"          3217    "tcp" []
  , makeService "unite"          3217    "udp" []
  , makeService "smartpackets"   3218    "tcp" []
  , makeService "smartpackets"   3218    "udp" []
  , makeService "wms-messenger"  3219    "tcp" []
  , makeService "wms-messenger"  3219    "udp" []
  , makeService "xnm-ssl"        3220    "tcp" []
  , makeService "xnm-ssl"        3220    "udp" []
  , makeService "xnm-clear-text" 3221    "tcp" []
  , makeService "xnm-clear-text" 3221    "udp" []
  , makeService "glbp"           3222    "tcp" []
  , makeService "glbp"           3222    "udp" []
  , makeService "digivote"       3223    "tcp" []
  , makeService "digivote"       3223    "udp" []
  , makeService "aes-discovery"  3224    "tcp" []
  , makeService "aes-discovery"  3224    "udp" []
  , makeService "fcip-port"      3225    "tcp" []
  , makeService "fcip-port"      3225    "udp" []
  , makeService "isi-irp"        3226    "tcp" []
  , makeService "isi-irp"        3226    "udp" []
  , makeService "dwnmshttp"      3227    "tcp" []
  , makeService "dwnmshttp"      3227    "udp" []
  , makeService "dwmsgserver"    3228    "tcp" []
  , makeService "dwmsgserver"    3228    "udp" []
  , makeService "global-cd-port" 3229    "tcp" []
  , makeService "global-cd-port" 3229    "udp" []
  , makeService "sftdst-port"    3230    "tcp" []
  , makeService "sftdst-port"    3230    "udp" []
  , makeService "vidigo"         3231    "tcp" []
  , makeService "vidigo"         3231    "udp" []
  , makeService "mdtp"           3232    "tcp" []
  , makeService "mdtp"           3232    "udp" []
  , makeService "whisker"        3233    "tcp" []
  , makeService "whisker"        3233    "udp" []
  , makeService "alchemy"        3234    "tcp" []
  , makeService "alchemy"        3234    "udp" []
  , makeService "mdap-port"      3235    "tcp" []
  , makeService "mdap-port"      3235    "udp" []
  , makeService "apparenet-ts"   3236    "tcp" []
  , makeService "apparenet-ts"   3236    "udp" []
  , makeService "apparenet-tps"  3237    "tcp" []
  , makeService "apparenet-tps"  3237    "udp" []
  , makeService "apparenet-as"   3238    "tcp" []
  , makeService "apparenet-as"   3238    "udp" []
  , makeService "apparenet-ui"   3239    "tcp" []
  , makeService "apparenet-ui"   3239    "udp" []
  , makeService "triomotion"     3240    "tcp" []
  , makeService "triomotion"     3240    "udp" []
  , makeService "sysorb"         3241    "tcp" []
  , makeService "sysorb"         3241    "udp" []
  , makeService "sdp-id-port"    3242    "tcp" []
  , makeService "sdp-id-port"    3242    "udp" []
  , makeService "timelot"        3243    "tcp" []
  , makeService "timelot"        3243    "udp" []
  , makeService "onesaf"         3244    "tcp" []
  , makeService "onesaf"         3244    "udp" []
  , makeService "vieo-fe"        3245    "tcp" []
  , makeService "vieo-fe"        3245    "udp" []
  , makeService "dvt-system"     3246    "tcp" []
  , makeService "dvt-system"     3246    "udp" []
  , makeService "dvt-data"       3247    "tcp" []
  , makeService "dvt-data"       3247    "udp" []
  , makeService "procos-lm"      3248    "tcp" []
  , makeService "procos-lm"      3248    "udp" []
  , makeService "ssp"            3249    "tcp" []
  , makeService "ssp"            3249    "udp" []
  , makeService "hicp"           3250    "tcp" []
  , makeService "hicp"           3250    "udp" []
  , makeService "sysscanner"     3251    "tcp" []
  , makeService "sysscanner"     3251    "udp" []
  , makeService "dhe"            3252    "tcp" []
  , makeService "dhe"            3252    "udp" []
  , makeService "pda-data"       3253    "tcp" []
  , makeService "pda-data"       3253    "udp" []
  , makeService "pda-sys"        3254    "tcp" []
  , makeService "pda-sys"        3254    "udp" []
  , makeService "semaphore"      3255    "tcp" []
  , makeService "semaphore"      3255    "udp" []
  , makeService "cpqrpm-agent"   3256    "tcp" []
  , makeService "cpqrpm-agent"   3256    "udp" []
  , makeService "cpqrpm-server"  3257    "tcp" []
  , makeService "cpqrpm-server"  3257    "udp" []
  , makeService "ivecon-port"    3258    "tcp" []
  , makeService "ivecon-port"    3258    "udp" []
  , makeService "epncdp2"        3259    "tcp" []
  , makeService "epncdp2"        3259    "udp" []
  , makeService "iscsi-target"   3260    "tcp" []
  , makeService "iscsi-target"   3260    "udp" []
  , makeService "winshadow"      3261    "tcp" []
  , makeService "winshadow"      3261    "udp" []
  , makeService "necp"           3262    "tcp" []
  , makeService "necp"           3262    "udp" []
  , makeService "ecolor-imager"  3263    "tcp" []
  , makeService "ecolor-imager"  3263    "udp" []
  , makeService "ccmail"         3264    "tcp" []
  , makeService "ccmail"         3264    "udp" []
  , makeService "altav-tunnel"   3265    "tcp" []
  , makeService "altav-tunnel"   3265    "udp" []
  , makeService "ns-cfg-server"  3266    "tcp" []
  , makeService "ns-cfg-server"  3266    "udp" []
  , makeService "ibm-dial-out"   3267    "tcp" []
  , makeService "ibm-dial-out"   3267    "udp" []
  , makeService "msft-gc"        3268    "tcp" []
  , makeService "msft-gc"        3268    "udp" []
  , makeService "msft-gc-ssl"    3269    "tcp" []
  , makeService "msft-gc-ssl"    3269    "udp" []
  , makeService "verismart"      3270    "tcp" []
  , makeService "verismart"      3270    "udp" []
  , makeService "csoft-prev"     3271    "tcp" []
  , makeService "csoft-prev"     3271    "udp" []
  , makeService "user-manager"   3272    "tcp" []
  , makeService "user-manager"   3272    "udp" []
  , makeService "sxmp"           3273    "tcp" []
  , makeService "sxmp"           3273    "udp" []
  , makeService "ordinox-server" 3274    "tcp" []
  , makeService "ordinox-server" 3274    "udp" []
  , makeService "samd"           3275    "tcp" []
  , makeService "samd"           3275    "udp" []
  , makeService "maxim-asics"    3276    "tcp" []
  , makeService "maxim-asics"    3276    "udp" []
  , makeService "awg-proxy"      3277    "tcp" []
  , makeService "awg-proxy"      3277    "udp" []
  , makeService "lkcmserver"     3278    "tcp" []
  , makeService "lkcmserver"     3278    "udp" []
  , makeService "admind"         3279    "tcp" []
  , makeService "admind"         3279    "udp" []
  , makeService "vs-server"      3280    "tcp" []
  , makeService "vs-server"      3280    "udp" []
  , makeService "sysopt"         3281    "tcp" []
  , makeService "sysopt"         3281    "udp" []
  , makeService "datusorb"       3282    "tcp" []
  , makeService "datusorb"       3282    "udp" []
  , makeService "net-assistant"  3283    "tcp" []
  , makeService "net-assistant"  3283    "udp" []
  , makeService "4talk"          3284    "tcp" []
  , makeService "4talk"          3284    "udp" []
  , makeService "plato"          3285    "tcp" []
  , makeService "plato"          3285    "udp" []
  , makeService "e-net"          3286    "tcp" []
  , makeService "e-net"          3286    "udp" []
  , makeService "directvdata"    3287    "tcp" []
  , makeService "directvdata"    3287    "udp" []
  , makeService "cops"           3288    "tcp" []
  , makeService "cops"           3288    "udp" []
  , makeService "enpc"           3289    "tcp" []
  , makeService "enpc"           3289    "udp" []
  , makeService "caps-lm"        3290    "tcp" []
  , makeService "caps-lm"        3290    "udp" []
  , makeService "sah-lm"         3291    "tcp" []
  , makeService "sah-lm"         3291    "udp" []
  , makeService "cart-o-rama"    3292    "tcp" []
  , makeService "cart-o-rama"    3292    "udp" []
  , makeService "fg-fps"         3293    "tcp" []
  , makeService "fg-fps"         3293    "udp" []
  , makeService "fg-gip"         3294    "tcp" []
  , makeService "fg-gip"         3294    "udp" []
  , makeService "dyniplookup"    3295    "tcp" []
  , makeService "dyniplookup"    3295    "udp" []
  , makeService "rib-slm"        3296    "tcp" []
  , makeService "rib-slm"        3296    "udp" []
  , makeService "cytel-lm"       3297    "tcp" []
  , makeService "cytel-lm"       3297    "udp" []
  , makeService "deskview"       3298    "tcp" []
  , makeService "deskview"       3298    "udp" []
  , makeService "pdrncs"         3299    "tcp" []
  , makeService "pdrncs"         3299    "udp" []
  , makeService "mcs-fastmail"   3302    "tcp" []
  , makeService "mcs-fastmail"   3302    "udp" []
  , makeService "opsession-clnt" 3303    "tcp" []
  , makeService "opsession-clnt" 3303    "udp" []
  , makeService "opsession-srvr" 3304    "tcp" []
  , makeService "opsession-srvr" 3304    "udp" []
  , makeService "odette-ftp"     3305    "tcp" []
  , makeService "odette-ftp"     3305    "udp" []
  , makeService "opsession-prxy" 3307    "tcp" []
  , makeService "opsession-prxy" 3307    "udp" []
  , makeService "tns-server"     3308    "tcp" []
  , makeService "tns-server"     3308    "udp" []
  , makeService "tns-adv"        3309    "tcp" []
  , makeService "tns-adv"        3309    "udp" []
  , makeService "dyna-access"    3310    "tcp" []
  , makeService "dyna-access"    3310    "udp" []
  , makeService "mcns-tel-ret"   3311    "tcp" []
  , makeService "mcns-tel-ret"   3311    "udp" []
  , makeService "appman-server"  3312    "tcp" []
  , makeService "appman-server"  3312    "udp" []
  , makeService "uorb"           3313    "tcp" []
  , makeService "uorb"           3313    "udp" []
  , makeService "uohost"         3314    "tcp" []
  , makeService "uohost"         3314    "udp" []
  , makeService "cdid"           3315    "tcp" []
  , makeService "cdid"           3315    "udp" []
  , makeService "aicc-cmi"       3316    "tcp" []
  , makeService "aicc-cmi"       3316    "udp" []
  , makeService "vsaiport"       3317    "tcp" []
  , makeService "vsaiport"       3317    "udp" []
  , makeService "ssrip"          3318    "tcp" []
  , makeService "ssrip"          3318    "udp" []
  , makeService "sdt-lmd"        3319    "tcp" []
  , makeService "sdt-lmd"        3319    "udp" []
  , makeService "officelink2000" 3320    "tcp" []
  , makeService "officelink2000" 3320    "udp" []
  , makeService "vnsstr"         3321    "tcp" []
  , makeService "vnsstr"         3321    "udp" []
  , makeService "sftu"           3326    "tcp" []
  , makeService "sftu"           3326    "udp" []
  , makeService "bbars"          3327    "tcp" []
  , makeService "bbars"          3327    "udp" []
  , makeService "egptlm"         3328    "tcp" []
  , makeService "egptlm"         3328    "udp" []
  , makeService "hp-device-disc" 3329    "tcp" []
  , makeService "hp-device-disc" 3329    "udp" []
  , makeService "mcs-calypsoicf" 3330    "tcp" []
  , makeService "mcs-calypsoicf" 3330    "udp" []
  , makeService "mcs-messaging"  3331    "tcp" []
  , makeService "mcs-messaging"  3331    "udp" []
  , makeService "mcs-mailsvr"    3332    "tcp" []
  , makeService "mcs-mailsvr"    3332    "udp" []
  , makeService "dec-notes"      3333    "tcp" []
  , makeService "dec-notes"      3333    "udp" []
  , makeService "directv-web"    3334    "tcp" []
  , makeService "directv-web"    3334    "udp" []
  , makeService "directv-soft"   3335    "tcp" []
  , makeService "directv-soft"   3335    "udp" []
  , makeService "directv-tick"   3336    "tcp" []
  , makeService "directv-tick"   3336    "udp" []
  , makeService "directv-catlg"  3337    "tcp" []
  , makeService "directv-catlg"  3337    "udp" []
  , makeService "anet-b"         3338    "tcp" []
  , makeService "anet-b"         3338    "udp" []
  , makeService "anet-l"         3339    "tcp" []
  , makeService "anet-l"         3339    "udp" []
  , makeService "anet-m"         3340    "tcp" []
  , makeService "anet-m"         3340    "udp" []
  , makeService "anet-h"         3341    "tcp" []
  , makeService "anet-h"         3341    "udp" []
  , makeService "webtie"         3342    "tcp" []
  , makeService "webtie"         3342    "udp" []
  , makeService "ms-cluster-net" 3343    "tcp" []
  , makeService "ms-cluster-net" 3343    "udp" []
  , makeService "bnt-manager"    3344    "tcp" []
  , makeService "bnt-manager"    3344    "udp" []
  , makeService "influence"      3345    "tcp" []
  , makeService "influence"      3345    "udp" []
  , makeService "phoenix-rpc"    3347    "tcp" []
  , makeService "phoenix-rpc"    3347    "udp" []
  , makeService "pangolin-laser" 3348    "tcp" []
  , makeService "pangolin-laser" 3348    "udp" []
  , makeService "chevinservices" 3349    "tcp" []
  , makeService "chevinservices" 3349    "udp" []
  , makeService "findviatv"      3350    "tcp" []
  , makeService "findviatv"      3350    "udp" []
  , makeService "btrieve"        3351    "tcp" []
  , makeService "btrieve"        3351    "udp" []
  , makeService "ssql"           3352    "tcp" []
  , makeService "ssql"           3352    "udp" []
  , makeService "fatpipe"        3353    "tcp" []
  , makeService "fatpipe"        3353    "udp" []
  , makeService "suitjd"         3354    "tcp" []
  , makeService "suitjd"         3354    "udp" []
  , makeService "ordinox-dbase"  3355    "tcp" []
  , makeService "ordinox-dbase"  3355    "udp" []
  , makeService "upnotifyps"     3356    "tcp" []
  , makeService "upnotifyps"     3356    "udp" []
  , makeService "adtech-test"    3357    "tcp" []
  , makeService "adtech-test"    3357    "udp" []
  , makeService "mpsysrmsvr"     3358    "tcp" []
  , makeService "mpsysrmsvr"     3358    "udp" []
  , makeService "wg-netforce"    3359    "tcp" []
  , makeService "wg-netforce"    3359    "udp" []
  , makeService "kv-server"      3360    "tcp" []
  , makeService "kv-server"      3360    "udp" []
  , makeService "kv-agent"       3361    "tcp" []
  , makeService "kv-agent"       3361    "udp" []
  , makeService "dj-ilm"         3362    "tcp" []
  , makeService "dj-ilm"         3362    "udp" []
  , makeService "nati-vi-server" 3363    "tcp" []
  , makeService "nati-vi-server" 3363    "udp" []
  , makeService "tip2"           3372    "tcp" []
  , makeService "tip2"           3372    "udp" []
  , makeService "lavenir-lm"     3373    "tcp" []
  , makeService "lavenir-lm"     3373    "udp" []
  , makeService "cluster-disc"   3374    "tcp" []
  , makeService "cluster-disc"   3374    "udp" []
  , makeService "vsnm-agent"     3375    "tcp" []
  , makeService "vsnm-agent"     3375    "udp" []
  , makeService "cdbroker"       3376    "tcp" []
  , makeService "cdbroker"       3376    "udp" []
  , makeService "cogsys-lm"      3377    "tcp" []
  , makeService "cogsys-lm"      3377    "udp" []
  , makeService "wsicopy"        3378    "tcp" []
  , makeService "wsicopy"        3378    "udp" []
  , makeService "socorfs"        3379    "tcp" []
  , makeService "socorfs"        3379    "udp" []
  , makeService "sns-channels"   3380    "tcp" []
  , makeService "sns-channels"   3380    "udp" []
  , makeService "geneous"        3381    "tcp" []
  , makeService "geneous"        3381    "udp" []
  , makeService "fujitsu-neat"   3382    "tcp" []
  , makeService "fujitsu-neat"   3382    "udp" []
  , makeService "esp-lm"         3383    "tcp" []
  , makeService "esp-lm"         3383    "udp" []
  , makeService "hp-clic"        3384    "tcp" []
  , makeService "hp-clic"        3384    "udp" []
  , makeService "qnxnetman"      3385    "tcp" []
  , makeService "qnxnetman"      3385    "udp" []
  , makeService "gprs-data"      3386    "tcp" []
  , makeService "gprs-sig"       3386    "udp" []
  , makeService "backroomnet"    3387    "tcp" []
  , makeService "backroomnet"    3387    "udp" []
  , makeService "cbserver"       3388    "tcp" []
  , makeService "cbserver"       3388    "udp" []
  , makeService "ms-wbt-server"  3389    "tcp" []
  , makeService "ms-wbt-server"  3389    "udp" []
  , makeService "dsc"            3390    "tcp" []
  , makeService "dsc"            3390    "udp" []
  , makeService "savant"         3391    "tcp" []
  , makeService "savant"         3391    "udp" []
  , makeService "efi-lm"         3392    "tcp" []
  , makeService "efi-lm"         3392    "udp" []
  , makeService "d2k-tapestry1"  3393    "tcp" []
  , makeService "d2k-tapestry1"  3393    "udp" []
  , makeService "d2k-tapestry2"  3394    "tcp" []
  , makeService "d2k-tapestry2"  3394    "udp" []
  , makeService "dyna-lm"        3395    "tcp" []
  , makeService "dyna-lm"        3395    "udp" []
  , makeService "printer_agent"  3396    "tcp" ["printer-agent"]
  , makeService "printer_agent"  3396    "udp" ["printer-agent"]
  , makeService "cloanto-lm"     3397    "tcp" []
  , makeService "cloanto-lm"     3397    "udp" []
  , makeService "mercantile"     3398    "tcp" []
  , makeService "mercantile"     3398    "udp" []
  , makeService "csms"           3399    "tcp" []
  , makeService "csms"           3399    "udp" []
  , makeService "csms2"          3400    "tcp" []
  , makeService "csms2"          3400    "udp" []
  , makeService "filecast"       3401    "tcp" []
  , makeService "filecast"       3401    "udp" []
  , makeService "fxaengine-net"  3402    "tcp" []
  , makeService "fxaengine-net"  3402    "udp" []
  , makeService "nokia-ann-ch1"  3405    "tcp" []
  , makeService "nokia-ann-ch1"  3405    "udp" []
  , makeService "nokia-ann-ch2"  3406    "tcp" []
  , makeService "nokia-ann-ch2"  3406    "udp" []
  , makeService "ldap-admin"     3407    "tcp" []
  , makeService "ldap-admin"     3407    "udp" []
  , makeService "BESApi"         3408    "tcp" []
  , makeService "BESApi"         3408    "udp" []
  , makeService "networklens"    3409    "tcp" []
  , makeService "networklens"    3409    "udp" []
  , makeService "networklenss"   3410    "tcp" []
  , makeService "networklenss"   3410    "udp" []
  , makeService "biolink-auth"   3411    "tcp" []
  , makeService "biolink-auth"   3411    "udp" []
  , makeService "xmlblaster"     3412    "tcp" []
  , makeService "xmlblaster"     3412    "udp" []
  , makeService "svnet"          3413    "tcp" []
  , makeService "svnet"          3413    "udp" []
  , makeService "wip-port"       3414    "tcp" []
  , makeService "wip-port"       3414    "udp" []
  , makeService "bcinameservice" 3415    "tcp" []
  , makeService "bcinameservice" 3415    "udp" []
  , makeService "commandport"    3416    "tcp" []
  , makeService "commandport"    3416    "udp" []
  , makeService "csvr"           3417    "tcp" []
  , makeService "csvr"           3417    "udp" []
  , makeService "rnmap"          3418    "tcp" []
  , makeService "rnmap"          3418    "udp" []
  , makeService "softaudit"      3419    "tcp" []
  , makeService "softaudit"      3419    "udp" []
  , makeService "ifcp-port"      3420    "tcp" []
  , makeService "ifcp-port"      3420    "udp" []
  , makeService "bmap"           3421    "tcp" []
  , makeService "bmap"           3421    "udp" []
  , makeService "rusb-sys-port"  3422    "tcp" []
  , makeService "rusb-sys-port"  3422    "udp" []
  , makeService "xtrm"           3423    "tcp" []
  , makeService "xtrm"           3423    "udp" []
  , makeService "xtrms"          3424    "tcp" []
  , makeService "xtrms"          3424    "udp" []
  , makeService "agps-port"      3425    "tcp" []
  , makeService "agps-port"      3425    "udp" []
  , makeService "arkivio"        3426    "tcp" []
  , makeService "arkivio"        3426    "udp" []
  , makeService "websphere-snmp" 3427    "tcp" []
  , makeService "websphere-snmp" 3427    "udp" []
  , makeService "twcss"          3428    "tcp" []
  , makeService "twcss"          3428    "udp" []
  , makeService "gcsp"           3429    "tcp" []
  , makeService "gcsp"           3429    "udp" []
  , makeService "ssdispatch"     3430    "tcp" []
  , makeService "ssdispatch"     3430    "udp" []
  , makeService "ndl-als"        3431    "tcp" []
  , makeService "ndl-als"        3431    "udp" []
  , makeService "osdcp"          3432    "tcp" []
  , makeService "osdcp"          3432    "udp" []
  , makeService "opnet-smp"      3433    "tcp" []
  , makeService "opnet-smp"      3433    "udp" []
  , makeService "opencm"         3434    "tcp" []
  , makeService "opencm"         3434    "udp" []
  , makeService "pacom"          3435    "tcp" []
  , makeService "pacom"          3435    "udp" []
  , makeService "gc-config"      3436    "tcp" []
  , makeService "gc-config"      3436    "udp" []
  , makeService "autocueds"      3437    "tcp" []
  , makeService "autocueds"      3437    "udp" []
  , makeService "spiral-admin"   3438    "tcp" []
  , makeService "spiral-admin"   3438    "udp" []
  , makeService "hri-port"       3439    "tcp" []
  , makeService "hri-port"       3439    "udp" []
  , makeService "ans-console"    3440    "tcp" []
  , makeService "ans-console"    3440    "udp" []
  , makeService "connect-client" 3441    "tcp" []
  , makeService "connect-client" 3441    "udp" []
  , makeService "connect-server" 3442    "tcp" []
  , makeService "connect-server" 3442    "udp" []
  , makeService "ov-nnm-websrv"  3443    "tcp" []
  , makeService "ov-nnm-websrv"  3443    "udp" []
  , makeService "denali-server"  3444    "tcp" []
  , makeService "denali-server"  3444    "udp" []
  , makeService "monp"           3445    "tcp" []
  , makeService "monp"           3445    "udp" []
  , makeService "3comfaxrpc"     3446    "tcp" []
  , makeService "3comfaxrpc"     3446    "udp" []
  , makeService "directnet"      3447    "tcp" []
  , makeService "directnet"      3447    "udp" []
  , makeService "dnc-port"       3448    "tcp" []
  , makeService "dnc-port"       3448    "udp" []
  , makeService "hotu-chat"      3449    "tcp" []
  , makeService "hotu-chat"      3449    "udp" []
  , makeService "castorproxy"    3450    "tcp" []
  , makeService "castorproxy"    3450    "udp" []
  , makeService "asam"           3451    "tcp" []
  , makeService "asam"           3451    "udp" []
  , makeService "sabp-signal"    3452    "tcp" []
  , makeService "sabp-signal"    3452    "udp" []
  , makeService "pscupd"         3453    "tcp" []
  , makeService "pscupd"         3453    "udp" []
  , makeService "mira"           3454    "tcp" []
  , makeService "mira"           3454    "udp" []
  , makeService "vat"            3456    "tcp" []
  , makeService "vat"            3456    "udp" []
  , makeService "vat-control"    3457    "tcp" []
  , makeService "vat-control"    3457    "udp" []
  , makeService "d3winosfi"      3458    "tcp" []
  , makeService "d3winosfi"      3458    "udp" []
  , makeService "integral"       3459    "tcp" []
  , makeService "integral"       3459    "udp" []
  , makeService "edm-manager"    3460    "tcp" []
  , makeService "edm-manager"    3460    "udp" []
  , makeService "edm-stager"     3461    "tcp" []
  , makeService "edm-stager"     3461    "udp" []
  , makeService "edm-std-notify" 3462    "tcp" []
  , makeService "edm-std-notify" 3462    "udp" []
  , makeService "edm-adm-notify" 3463    "tcp" []
  , makeService "edm-adm-notify" 3463    "udp" []
  , makeService "edm-mgr-sync"   3464    "tcp" []
  , makeService "edm-mgr-sync"   3464    "udp" []
  , makeService "edm-mgr-cntrl"  3465    "tcp" []
  , makeService "edm-mgr-cntrl"  3465    "udp" []
  , makeService "workflow"       3466    "tcp" []
  , makeService "workflow"       3466    "udp" []
  , makeService "rcst"           3467    "tcp" []
  , makeService "rcst"           3467    "udp" []
  , makeService "ttcmremotectrl" 3468    "tcp" []
  , makeService "ttcmremotectrl" 3468    "udp" []
  , makeService "pluribus"       3469    "tcp" []
  , makeService "pluribus"       3469    "udp" []
  , makeService "jt400"          3470    "tcp" []
  , makeService "jt400"          3470    "udp" []
  , makeService "jt400-ssl"      3471    "tcp" []
  , makeService "jt400-ssl"      3471    "udp" []
  , makeService "jaugsremotec-1" 3472    "tcp" []
  , makeService "jaugsremotec-1" 3472    "udp" []
  , makeService "jaugsremotec-2" 3473    "tcp" []
  , makeService "jaugsremotec-2" 3473    "udp" []
  , makeService "ttntspauto"     3474    "tcp" []
  , makeService "ttntspauto"     3474    "udp" []
  , makeService "genisar-port"   3475    "tcp" []
  , makeService "genisar-port"   3475    "udp" []
  , makeService "nppmp"          3476    "tcp" []
  , makeService "nppmp"          3476    "udp" []
  , makeService "ecomm"          3477    "tcp" []
  , makeService "ecomm"          3477    "udp" []
  , makeService "stun"           3478    "tcp" ["turn"]
  , makeService "stun"           3478    "udp" ["turn"]
  , makeService "twrpc"          3479    "tcp" []
  , makeService "twrpc"          3479    "udp" []
  , makeService "plethora"       3480    "tcp" []
  , makeService "plethora"       3480    "udp" []
  , makeService "cleanerliverc"  3481    "tcp" []
  , makeService "cleanerliverc"  3481    "udp" []
  , makeService "vulture"        3482    "tcp" []
  , makeService "vulture"        3482    "udp" []
  , makeService "slim-devices"   3483    "tcp" []
  , makeService "slim-devices"   3483    "udp" []
  , makeService "gbs-stp"        3484    "tcp" []
  , makeService "gbs-stp"        3484    "udp" []
  , makeService "celatalk"       3485    "tcp" []
  , makeService "celatalk"       3485    "udp" []
  , makeService "ifsf-hb-port"   3486    "tcp" []
  , makeService "ifsf-hb-port"   3486    "udp" []
  , makeService "ltctcp"         3487    "tcp" []
  , makeService "ltcudp"         3487    "udp" []
  , makeService "fs-rh-srv"      3488    "tcp" []
  , makeService "fs-rh-srv"      3488    "udp" []
  , makeService "dtp-dia"        3489    "tcp" []
  , makeService "dtp-dia"        3489    "udp" []
  , makeService "colubris"       3490    "tcp" []
  , makeService "colubris"       3490    "udp" []
  , makeService "swr-port"       3491    "tcp" []
  , makeService "swr-port"       3491    "udp" []
  , makeService "tvdumtray-port" 3492    "tcp" []
  , makeService "tvdumtray-port" 3492    "udp" []
  , makeService "nut"            3493    "tcp" []
  , makeService "nut"            3493    "udp" []
  , makeService "ibm3494"        3494    "tcp" []
  , makeService "ibm3494"        3494    "udp" []
  , makeService "seclayer-tcp"   3495    "tcp" []
  , makeService "seclayer-tcp"   3495    "udp" []
  , makeService "seclayer-tls"   3496    "tcp" []
  , makeService "seclayer-tls"   3496    "udp" []
  , makeService "ipether232port" 3497    "tcp" []
  , makeService "ipether232port" 3497    "udp" []
  , makeService "dashpas-port"   3498    "tcp" []
  , makeService "dashpas-port"   3498    "udp" []
  , makeService "sccip-media"    3499    "tcp" []
  , makeService "sccip-media"    3499    "udp" []
  , makeService "rtmp-port"      3500    "tcp" []
  , makeService "rtmp-port"      3500    "udp" []
  , makeService "isoft-p2p"      3501    "tcp" []
  , makeService "isoft-p2p"      3501    "udp" []
  , makeService "avinstalldisc"  3502    "tcp" []
  , makeService "avinstalldisc"  3502    "udp" []
  , makeService "lsp-ping"       3503    "tcp" []
  , makeService "lsp-ping"       3503    "udp" []
  , makeService "ironstorm"      3504    "tcp" []
  , makeService "ironstorm"      3504    "udp" []
  , makeService "ccmcomm"        3505    "tcp" []
  , makeService "ccmcomm"        3505    "udp" []
  , makeService "apc-3506"       3506    "tcp" []
  , makeService "apc-3506"       3506    "udp" []
  , makeService "nesh-broker"    3507    "tcp" []
  , makeService "nesh-broker"    3507    "udp" []
  , makeService "interactionweb" 3508    "tcp" []
  , makeService "interactionweb" 3508    "udp" []
  , makeService "vt-ssl"         3509    "tcp" []
  , makeService "vt-ssl"         3509    "udp" []
  , makeService "xss-port"       3510    "tcp" []
  , makeService "xss-port"       3510    "udp" []
  , makeService "webmail-2"      3511    "tcp" []
  , makeService "webmail-2"      3511    "udp" []
  , makeService "aztec"          3512    "tcp" []
  , makeService "aztec"          3512    "udp" []
  , makeService "arcpd"          3513    "tcp" []
  , makeService "arcpd"          3513    "udp" []
  , makeService "must-p2p"       3514    "tcp" []
  , makeService "must-p2p"       3514    "udp" []
  , makeService "must-backplane" 3515    "tcp" []
  , makeService "must-backplane" 3515    "udp" []
  , makeService "smartcard-port" 3516    "tcp" []
  , makeService "smartcard-port" 3516    "udp" []
  , makeService "802-11-iapp"    3517    "tcp" []
  , makeService "802-11-iapp"    3517    "udp" []
  , makeService "artifact-msg"   3518    "tcp" []
  , makeService "artifact-msg"   3518    "udp" []
  , makeService "nvmsgd"         3519    "tcp" []
  , makeService "galileo"        3519    "udp" []
  , makeService "galileolog"     3520    "tcp" []
  , makeService "galileolog"     3520    "udp" []
  , makeService "mc3ss"          3521    "tcp" []
  , makeService "mc3ss"          3521    "udp" []
  , makeService "nssocketport"   3522    "tcp" []
  , makeService "nssocketport"   3522    "udp" []
  , makeService "odeumservlink"  3523    "tcp" []
  , makeService "odeumservlink"  3523    "udp" []
  , makeService "ecmport"        3524    "tcp" []
  , makeService "ecmport"        3524    "udp" []
  , makeService "eisport"        3525    "tcp" []
  , makeService "eisport"        3525    "udp" []
  , makeService "starquiz-port"  3526    "tcp" []
  , makeService "starquiz-port"  3526    "udp" []
  , makeService "beserver-msg-q" 3527    "tcp" []
  , makeService "beserver-msg-q" 3527    "udp" []
  , makeService "jboss-iiop"     3528    "tcp" []
  , makeService "jboss-iiop"     3528    "udp" []
  , makeService "jboss-iiop-ssl" 3529    "tcp" []
  , makeService "jboss-iiop-ssl" 3529    "udp" []
  , makeService "gf"             3530    "tcp" []
  , makeService "gf"             3530    "udp" []
  , makeService "joltid"         3531    "tcp" []
  , makeService "joltid"         3531    "udp" []
  , makeService "raven-rmp"      3532    "tcp" []
  , makeService "raven-rmp"      3532    "udp" []
  , makeService "raven-rdp"      3533    "tcp" []
  , makeService "raven-rdp"      3533    "udp" []
  , makeService "urld-port"      3534    "tcp" []
  , makeService "urld-port"      3534    "udp" []
  , makeService "ms-la"          3535    "tcp" []
  , makeService "ms-la"          3535    "udp" []
  , makeService "snac"           3536    "tcp" []
  , makeService "snac"           3536    "udp" []
  , makeService "ni-visa-remote" 3537    "tcp" []
  , makeService "ni-visa-remote" 3537    "udp" []
  , makeService "ibm-diradm"     3538    "tcp" []
  , makeService "ibm-diradm"     3538    "udp" []
  , makeService "ibm-diradm-ssl" 3539    "tcp" []
  , makeService "ibm-diradm-ssl" 3539    "udp" []
  , makeService "pnrp-port"      3540    "tcp" []
  , makeService "pnrp-port"      3540    "udp" []
  , makeService "voispeed-port"  3541    "tcp" []
  , makeService "voispeed-port"  3541    "udp" []
  , makeService "hacl-monitor"   3542    "tcp" []
  , makeService "hacl-monitor"   3542    "udp" []
  , makeService "qftest-lookup"  3543    "tcp" []
  , makeService "qftest-lookup"  3543    "udp" []
  , makeService "teredo"         3544    "tcp" []
  , makeService "teredo"         3544    "udp" []
  , makeService "camac"          3545    "tcp" []
  , makeService "camac"          3545    "udp" []
  , makeService "symantec-sim"   3547    "tcp" []
  , makeService "symantec-sim"   3547    "udp" []
  , makeService "interworld"     3548    "tcp" []
  , makeService "interworld"     3548    "udp" []
  , makeService "tellumat-nms"   3549    "tcp" []
  , makeService "tellumat-nms"   3549    "udp" []
  , makeService "ssmpp"          3550    "tcp" []
  , makeService "ssmpp"          3550    "udp" []
  , makeService "apcupsd"        3551    "tcp" []
  , makeService "apcupsd"        3551    "udp" []
  , makeService "taserver"       3552    "tcp" []
  , makeService "taserver"       3552    "udp" []
  , makeService "rbr-discovery"  3553    "tcp" []
  , makeService "rbr-discovery"  3553    "udp" []
  , makeService "questnotify"    3554    "tcp" []
  , makeService "questnotify"    3554    "udp" []
  , makeService "razor"          3555    "tcp" []
  , makeService "razor"          3555    "udp" []
  , makeService "sky-transport"  3556    "tcp" []
  , makeService "sky-transport"  3556    "udp" []
  , makeService "personalos-001" 3557    "tcp" []
  , makeService "personalos-001" 3557    "udp" []
  , makeService "mcp-port"       3558    "tcp" []
  , makeService "mcp-port"       3558    "udp" []
  , makeService "cctv-port"      3559    "tcp" []
  , makeService "cctv-port"      3559    "udp" []
  , makeService "iniserve-port"  3560    "tcp" []
  , makeService "iniserve-port"  3560    "udp" []
  , makeService "bmc-onekey"     3561    "tcp" []
  , makeService "bmc-onekey"     3561    "udp" []
  , makeService "sdbproxy"       3562    "tcp" []
  , makeService "sdbproxy"       3562    "udp" []
  , makeService "watcomdebug"    3563    "tcp" []
  , makeService "watcomdebug"    3563    "udp" []
  , makeService "esimport"       3564    "tcp" []
  , makeService "esimport"       3564    "udp" []
  , makeService "m2pa"           3565    "tcp" []
  , makeService "quest-data-hub" 3566    "tcp" []
  , makeService "enc-eps"        3567    "tcp" []
  , makeService "enc-eps"        3567    "udp" []
  , makeService "enc-tunel-sec"  3568    "tcp" []
  , makeService "enc-tunel-sec"  3568    "udp" []
  , makeService "mbg-ctrl"       3569    "tcp" []
  , makeService "mbg-ctrl"       3569    "udp" []
  , makeService "mccwebsvr-port" 3570    "tcp" []
  , makeService "mccwebsvr-port" 3570    "udp" []
  , makeService "megardsvr-port" 3571    "tcp" []
  , makeService "megardsvr-port" 3571    "udp" []
  , makeService "megaregsvrport" 3572    "tcp" []
  , makeService "megaregsvrport" 3572    "udp" []
  , makeService "tag-ups-1"      3573    "tcp" []
  , makeService "tag-ups-1"      3573    "udp" []
  , makeService "dmaf-server"    3574    "tcp" []
  , makeService "dmaf-caster"    3574    "udp" []
  , makeService "ccm-port"       3575    "tcp" []
  , makeService "ccm-port"       3575    "udp" []
  , makeService "cmc-port"       3576    "tcp" []
  , makeService "cmc-port"       3576    "udp" []
  , makeService "config-port"    3577    "tcp" []
  , makeService "config-port"    3577    "udp" []
  , makeService "data-port"      3578    "tcp" []
  , makeService "data-port"      3578    "udp" []
  , makeService "ttat3lb"        3579    "tcp" []
  , makeService "ttat3lb"        3579    "udp" []
  , makeService "nati-svrloc"    3580    "tcp" []
  , makeService "nati-svrloc"    3580    "udp" []
  , makeService "kfxaclicensing" 3581    "tcp" []
  , makeService "kfxaclicensing" 3581    "udp" []
  , makeService "press"          3582    "tcp" []
  , makeService "press"          3582    "udp" []
  , makeService "canex-watch"    3583    "tcp" []
  , makeService "canex-watch"    3583    "udp" []
  , makeService "u-dbap"         3584    "tcp" []
  , makeService "u-dbap"         3584    "udp" []
  , makeService "emprise-lls"    3585    "tcp" []
  , makeService "emprise-lls"    3585    "udp" []
  , makeService "emprise-lsc"    3586    "tcp" []
  , makeService "emprise-lsc"    3586    "udp" []
  , makeService "p2pgroup"       3587    "tcp" []
  , makeService "p2pgroup"       3587    "udp" []
  , makeService "sentinel"       3588    "tcp" []
  , makeService "sentinel"       3588    "udp" []
  , makeService "isomair"        3589    "tcp" []
  , makeService "isomair"        3589    "udp" []
  , makeService "wv-csp-sms"     3590    "tcp" []
  , makeService "wv-csp-sms"     3590    "udp" []
  , makeService "gtrack-server"  3591    "tcp" []
  , makeService "gtrack-server"  3591    "udp" []
  , makeService "gtrack-ne"      3592    "tcp" []
  , makeService "gtrack-ne"      3592    "udp" []
  , makeService "bpmd"           3593    "tcp" []
  , makeService "bpmd"           3593    "udp" []
  , makeService "mediaspace"     3594    "tcp" []
  , makeService "mediaspace"     3594    "udp" []
  , makeService "shareapp"       3595    "tcp" []
  , makeService "shareapp"       3595    "udp" []
  , makeService "iw-mmogame"     3596    "tcp" []
  , makeService "iw-mmogame"     3596    "udp" []
  , makeService "a14"            3597    "tcp" []
  , makeService "a14"            3597    "udp" []
  , makeService "a15"            3598    "tcp" []
  , makeService "a15"            3598    "udp" []
  , makeService "quasar-server"  3599    "tcp" []
  , makeService "quasar-server"  3599    "udp" []
  , makeService "trap-daemon"    3600    "tcp" []
  , makeService "trap-daemon"    3600    "udp" []
  , makeService "visinet-gui"    3601    "tcp" []
  , makeService "visinet-gui"    3601    "udp" []
  , makeService "infiniswitchcl" 3602    "tcp" []
  , makeService "infiniswitchcl" 3602    "udp" []
  , makeService "int-rcv-cntrl"  3603    "tcp" []
  , makeService "int-rcv-cntrl"  3603    "udp" []
  , makeService "bmc-jmx-port"   3604    "tcp" []
  , makeService "bmc-jmx-port"   3604    "udp" []
  , makeService "comcam-io"      3605    "tcp" []
  , makeService "comcam-io"      3605    "udp" []
  , makeService "splitlock"      3606    "tcp" []
  , makeService "splitlock"      3606    "udp" []
  , makeService "precise-i3"     3607    "tcp" []
  , makeService "precise-i3"     3607    "udp" []
  , makeService "trendchip-dcp"  3608    "tcp" []
  , makeService "trendchip-dcp"  3608    "udp" []
  , makeService "cpdi-pidas-cm"  3609    "tcp" []
  , makeService "cpdi-pidas-cm"  3609    "udp" []
  , makeService "echonet"        3610    "tcp" []
  , makeService "echonet"        3610    "udp" []
  , makeService "six-degrees"    3611    "tcp" []
  , makeService "six-degrees"    3611    "udp" []
  , makeService "hp-dataprotect" 3612    "tcp" []
  , makeService "hp-dataprotect" 3612    "udp" []
  , makeService "alaris-disc"    3613    "tcp" []
  , makeService "alaris-disc"    3613    "udp" []
  , makeService "sigma-port"     3614    "tcp" []
  , makeService "sigma-port"     3614    "udp" []
  , makeService "start-network"  3615    "tcp" []
  , makeService "start-network"  3615    "udp" []
  , makeService "cd3o-protocol"  3616    "tcp" []
  , makeService "cd3o-protocol"  3616    "udp" []
  , makeService "sharp-server"   3617    "tcp" []
  , makeService "sharp-server"   3617    "udp" []
  , makeService "aairnet-1"      3618    "tcp" []
  , makeService "aairnet-1"      3618    "udp" []
  , makeService "aairnet-2"      3619    "tcp" []
  , makeService "aairnet-2"      3619    "udp" []
  , makeService "ep-pcp"         3620    "tcp" []
  , makeService "ep-pcp"         3620    "udp" []
  , makeService "ep-nsp"         3621    "tcp" []
  , makeService "ep-nsp"         3621    "udp" []
  , makeService "ff-lr-port"     3622    "tcp" []
  , makeService "ff-lr-port"     3622    "udp" []
  , makeService "haipe-discover" 3623    "tcp" []
  , makeService "haipe-discover" 3623    "udp" []
  , makeService "dist-upgrade"   3624    "tcp" []
  , makeService "dist-upgrade"   3624    "udp" []
  , makeService "volley"         3625    "tcp" []
  , makeService "volley"         3625    "udp" []
  , makeService "bvcdaemon-port" 3626    "tcp" []
  , makeService "bvcdaemon-port" 3626    "udp" []
  , makeService "jamserverport"  3627    "tcp" []
  , makeService "jamserverport"  3627    "udp" []
  , makeService "ept-machine"    3628    "tcp" []
  , makeService "ept-machine"    3628    "udp" []
  , makeService "escvpnet"       3629    "tcp" []
  , makeService "escvpnet"       3629    "udp" []
  , makeService "cs-remote-db"   3630    "tcp" []
  , makeService "cs-remote-db"   3630    "udp" []
  , makeService "cs-services"    3631    "tcp" []
  , makeService "cs-services"    3631    "udp" []
  , makeService "distcc"         3632    "udp" []
  , makeService "wacp"           3633    "tcp" []
  , makeService "wacp"           3633    "udp" []
  , makeService "hlibmgr"        3634    "tcp" []
  , makeService "hlibmgr"        3634    "udp" []
  , makeService "sdo"            3635    "tcp" []
  , makeService "sdo"            3635    "udp" []
  , makeService "servistaitsm"   3636    "tcp" []
  , makeService "servistaitsm"   3636    "udp" []
  , makeService "scservp"        3637    "tcp" []
  , makeService "scservp"        3637    "udp" []
  , makeService "ehp-backup"     3638    "tcp" []
  , makeService "ehp-backup"     3638    "udp" []
  , makeService "xap-ha"         3639    "tcp" []
  , makeService "xap-ha"         3639    "udp" []
  , makeService "netplay-port1"  3640    "tcp" []
  , makeService "netplay-port1"  3640    "udp" []
  , makeService "netplay-port2"  3641    "tcp" []
  , makeService "netplay-port2"  3641    "udp" []
  , makeService "juxml-port"     3642    "tcp" []
  , makeService "juxml-port"     3642    "udp" []
  , makeService "audiojuggler"   3643    "tcp" []
  , makeService "audiojuggler"   3643    "udp" []
  , makeService "ssowatch"       3644    "tcp" []
  , makeService "ssowatch"       3644    "udp" []
  , makeService "cyc"            3645    "tcp" []
  , makeService "cyc"            3645    "udp" []
  , makeService "xss-srv-port"   3646    "tcp" []
  , makeService "xss-srv-port"   3646    "udp" []
  , makeService "splitlock-gw"   3647    "tcp" []
  , makeService "splitlock-gw"   3647    "udp" []
  , makeService "fjcp"           3648    "tcp" []
  , makeService "fjcp"           3648    "udp" []
  , makeService "nmmp"           3649    "tcp" []
  , makeService "nmmp"           3649    "udp" []
  , makeService "prismiq-plugin" 3650    "tcp" []
  , makeService "prismiq-plugin" 3650    "udp" []
  , makeService "xrpc-registry"  3651    "tcp" []
  , makeService "xrpc-registry"  3651    "udp" []
  , makeService "vxcrnbuport"    3652    "tcp" []
  , makeService "vxcrnbuport"    3652    "udp" []
  , makeService "tsp"            3653    "tcp" []
  , makeService "tsp"            3653    "udp" []
  , makeService "vaprtm"         3654    "tcp" []
  , makeService "vaprtm"         3654    "udp" []
  , makeService "abatemgr"       3655    "tcp" []
  , makeService "abatemgr"       3655    "udp" []
  , makeService "abatjss"        3656    "tcp" []
  , makeService "abatjss"        3656    "udp" []
  , makeService "immedianet-bcn" 3657    "tcp" []
  , makeService "immedianet-bcn" 3657    "udp" []
  , makeService "ps-ams"         3658    "tcp" []
  , makeService "ps-ams"         3658    "udp" []
  , makeService "apple-sasl"     3659    "tcp" []
  , makeService "apple-sasl"     3659    "udp" []
  , makeService "can-nds-ssl"    3660    "tcp" []
  , makeService "can-nds-ssl"    3660    "udp" []
  , makeService "can-ferret-ssl" 3661    "tcp" []
  , makeService "can-ferret-ssl" 3661    "udp" []
  , makeService "pserver"        3662    "tcp" []
  , makeService "pserver"        3662    "udp" []
  , makeService "dtp"            3663    "tcp" []
  , makeService "dtp"            3663    "udp" []
  , makeService "ups-engine"     3664    "tcp" []
  , makeService "ups-engine"     3664    "udp" []
  , makeService "ent-engine"     3665    "tcp" []
  , makeService "ent-engine"     3665    "udp" []
  , makeService "eserver-pap"    3666    "tcp" []
  , makeService "eserver-pap"    3666    "udp" []
  , makeService "infoexch"       3667    "tcp" []
  , makeService "infoexch"       3667    "udp" []
  , makeService "dell-rm-port"   3668    "tcp" []
  , makeService "dell-rm-port"   3668    "udp" []
  , makeService "casanswmgmt"    3669    "tcp" []
  , makeService "casanswmgmt"    3669    "udp" []
  , makeService "smile"          3670    "tcp" []
  , makeService "smile"          3670    "udp" []
  , makeService "efcp"           3671    "tcp" []
  , makeService "efcp"           3671    "udp" []
  , makeService "lispworks-orb"  3672    "tcp" []
  , makeService "lispworks-orb"  3672    "udp" []
  , makeService "mediavault-gui" 3673    "tcp" []
  , makeService "mediavault-gui" 3673    "udp" []
  , makeService "wininstall-ipc" 3674    "tcp" []
  , makeService "wininstall-ipc" 3674    "udp" []
  , makeService "calltrax"       3675    "tcp" []
  , makeService "calltrax"       3675    "udp" []
  , makeService "va-pacbase"     3676    "tcp" []
  , makeService "va-pacbase"     3676    "udp" []
  , makeService "roverlog"       3677    "tcp" []
  , makeService "roverlog"       3677    "udp" []
  , makeService "ipr-dglt"       3678    "tcp" []
  , makeService "ipr-dglt"       3678    "udp" []
  , makeService "newton-dock"    3679    "tcp" []
  , makeService "newton-dock"    3679    "udp" []
  , makeService "npds-tracker"   3680    "tcp" []
  , makeService "npds-tracker"   3680    "udp" []
  , makeService "bts-x73"        3681    "tcp" []
  , makeService "bts-x73"        3681    "udp" []
  , makeService "cas-mapi"       3682    "tcp" []
  , makeService "cas-mapi"       3682    "udp" []
  , makeService "bmc-ea"         3683    "tcp" []
  , makeService "bmc-ea"         3683    "udp" []
  , makeService "faxstfx-port"   3684    "tcp" []
  , makeService "faxstfx-port"   3684    "udp" []
  , makeService "dsx-agent"      3685    "tcp" []
  , makeService "dsx-agent"      3685    "udp" []
  , makeService "tnmpv2"         3686    "tcp" []
  , makeService "tnmpv2"         3686    "udp" []
  , makeService "simple-push"    3687    "tcp" []
  , makeService "simple-push"    3687    "udp" []
  , makeService "simple-push-s"  3688    "tcp" []
  , makeService "simple-push-s"  3688    "udp" []
  , makeService "daap"           3689    "tcp" []
  , makeService "daap"           3689    "udp" []
  , makeService "magaya-network" 3691    "tcp" []
  , makeService "magaya-network" 3691    "udp" []
  , makeService "intelsync"      3692    "tcp" []
  , makeService "intelsync"      3692    "udp" []
  , makeService "bmc-data-coll"  3695    "tcp" []
  , makeService "bmc-data-coll"  3695    "udp" []
  , makeService "telnetcpcd"     3696    "tcp" []
  , makeService "telnetcpcd"     3696    "udp" []
  , makeService "nw-license"     3697    "tcp" []
  , makeService "nw-license"     3697    "udp" []
  , makeService "sagectlpanel"   3698    "tcp" []
  , makeService "sagectlpanel"   3698    "udp" []
  , makeService "kpn-icw"        3699    "tcp" []
  , makeService "kpn-icw"        3699    "udp" []
  , makeService "lrs-paging"     3700    "tcp" []
  , makeService "lrs-paging"     3700    "udp" []
  , makeService "netcelera"      3701    "tcp" []
  , makeService "netcelera"      3701    "udp" []
  , makeService "ws-discovery"   3702    "tcp" []
  , makeService "ws-discovery"   3702    "udp" []
  , makeService "adobeserver-3"  3703    "tcp" []
  , makeService "adobeserver-3"  3703    "udp" []
  , makeService "adobeserver-4"  3704    "tcp" []
  , makeService "adobeserver-4"  3704    "udp" []
  , makeService "adobeserver-5"  3705    "tcp" []
  , makeService "adobeserver-5"  3705    "udp" []
  , makeService "rt-event"       3706    "tcp" []
  , makeService "rt-event"       3706    "udp" []
  , makeService "rt-event-s"     3707    "tcp" []
  , makeService "rt-event-s"     3707    "udp" []
  , makeService "sun-as-iiops"   3708    "tcp" []
  , makeService "sun-as-iiops"   3708    "udp" []
  , makeService "ca-idms"        3709    "tcp" []
  , makeService "ca-idms"        3709    "udp" []
  , makeService "portgate-auth"  3710    "tcp" []
  , makeService "portgate-auth"  3710    "udp" []
  , makeService "edb-server2"    3711    "tcp" []
  , makeService "edb-server2"    3711    "udp" []
  , makeService "sentinel-ent"   3712    "tcp" []
  , makeService "sentinel-ent"   3712    "udp" []
  , makeService "tftps"          3713    "tcp" []
  , makeService "tftps"          3713    "udp" []
  , makeService "delos-dms"      3714    "tcp" []
  , makeService "delos-dms"      3714    "udp" []
  , makeService "anoto-rendezv"  3715    "tcp" []
  , makeService "anoto-rendezv"  3715    "udp" []
  , makeService "wv-csp-sms-cir" 3716    "tcp" []
  , makeService "wv-csp-sms-cir" 3716    "udp" []
  , makeService "wv-csp-udp-cir" 3717    "tcp" []
  , makeService "wv-csp-udp-cir" 3717    "udp" []
  , makeService "opus-services"  3718    "tcp" []
  , makeService "opus-services"  3718    "udp" []
  , makeService "itelserverport" 3719    "tcp" []
  , makeService "itelserverport" 3719    "udp" []
  , makeService "ufastro-instr"  3720    "tcp" []
  , makeService "ufastro-instr"  3720    "udp" []
  , makeService "xsync"          3721    "tcp" []
  , makeService "xsync"          3721    "udp" []
  , makeService "xserveraid"     3722    "tcp" []
  , makeService "xserveraid"     3722    "udp" []
  , makeService "sychrond"       3723    "tcp" []
  , makeService "sychrond"       3723    "udp" []
  , makeService "blizwow"        3724    "tcp" []
  , makeService "blizwow"        3724    "udp" []
  , makeService "na-er-tip"      3725    "tcp" []
  , makeService "na-er-tip"      3725    "udp" []
  , makeService "array-manager"  3726    "tcp" []
  , makeService "array-manager"  3726    "udp" []
  , makeService "e-mdu"          3727    "tcp" []
  , makeService "e-mdu"          3727    "udp" []
  , makeService "e-woa"          3728    "tcp" []
  , makeService "e-woa"          3728    "udp" []
  , makeService "fksp-audit"     3729    "tcp" []
  , makeService "fksp-audit"     3729    "udp" []
  , makeService "client-ctrl"    3730    "tcp" []
  , makeService "client-ctrl"    3730    "udp" []
  , makeService "smap"           3731    "tcp" []
  , makeService "smap"           3731    "udp" []
  , makeService "m-wnn"          3732    "tcp" []
  , makeService "m-wnn"          3732    "udp" []
  , makeService "multip-msg"     3733    "tcp" []
  , makeService "multip-msg"     3733    "udp" []
  , makeService "synel-data"     3734    "tcp" []
  , makeService "synel-data"     3734    "udp" []
  , makeService "pwdis"          3735    "tcp" []
  , makeService "pwdis"          3735    "udp" []
  , makeService "rs-rmi"         3736    "tcp" []
  , makeService "rs-rmi"         3736    "udp" []
  , makeService "xpanel"         3737    "tcp" []
  , makeService "versatalk"      3738    "tcp" []
  , makeService "versatalk"      3738    "udp" []
  , makeService "launchbird-lm"  3739    "tcp" []
  , makeService "launchbird-lm"  3739    "udp" []
  , makeService "heartbeat"      3740    "tcp" []
  , makeService "heartbeat"      3740    "udp" []
  , makeService "wysdma"         3741    "tcp" []
  , makeService "wysdma"         3741    "udp" []
  , makeService "cst-port"       3742    "tcp" []
  , makeService "cst-port"       3742    "udp" []
  , makeService "ipcs-command"   3743    "tcp" []
  , makeService "ipcs-command"   3743    "udp" []
  , makeService "sasg"           3744    "tcp" []
  , makeService "sasg"           3744    "udp" []
  , makeService "gw-call-port"   3745    "tcp" []
  , makeService "gw-call-port"   3745    "udp" []
  , makeService "linktest"       3746    "tcp" []
  , makeService "linktest"       3746    "udp" []
  , makeService "linktest-s"     3747    "tcp" []
  , makeService "linktest-s"     3747    "udp" []
  , makeService "webdata"        3748    "tcp" []
  , makeService "webdata"        3748    "udp" []
  , makeService "cimtrak"        3749    "tcp" []
  , makeService "cimtrak"        3749    "udp" []
  , makeService "cbos-ip-port"   3750    "tcp" []
  , makeService "cbos-ip-port"   3750    "udp" []
  , makeService "gprs-cube"      3751    "tcp" []
  , makeService "gprs-cube"      3751    "udp" []
  , makeService "vipremoteagent" 3752    "tcp" []
  , makeService "vipremoteagent" 3752    "udp" []
  , makeService "nattyserver"    3753    "tcp" []
  , makeService "nattyserver"    3753    "udp" []
  , makeService "timestenbroker" 3754    "tcp" []
  , makeService "timestenbroker" 3754    "udp" []
  , makeService "sas-remote-hlp" 3755    "tcp" []
  , makeService "sas-remote-hlp" 3755    "udp" []
  , makeService "canon-capt"     3756    "tcp" []
  , makeService "canon-capt"     3756    "udp" []
  , makeService "grf-port"       3757    "tcp" []
  , makeService "grf-port"       3757    "udp" []
  , makeService "apw-registry"   3758    "tcp" []
  , makeService "apw-registry"   3758    "udp" []
  , makeService "exapt-lmgr"     3759    "tcp" []
  , makeService "exapt-lmgr"     3759    "udp" []
  , makeService "adtempusclient" 3760    "tcp" []
  , makeService "adtempusclient" 3760    "udp" []
  , makeService "gsakmp"         3761    "tcp" []
  , makeService "gsakmp"         3761    "udp" []
  , makeService "gbs-smp"        3762    "tcp" []
  , makeService "gbs-smp"        3762    "udp" []
  , makeService "xo-wave"        3763    "tcp" []
  , makeService "xo-wave"        3763    "udp" []
  , makeService "mni-prot-rout"  3764    "tcp" []
  , makeService "mni-prot-rout"  3764    "udp" []
  , makeService "rtraceroute"    3765    "tcp" []
  , makeService "rtraceroute"    3765    "udp" []
  , makeService "listmgr-port"   3767    "tcp" []
  , makeService "listmgr-port"   3767    "udp" []
  , makeService "rblcheckd"      3768    "tcp" []
  , makeService "rblcheckd"      3768    "udp" []
  , makeService "haipe-otnk"     3769    "tcp" []
  , makeService "haipe-otnk"     3769    "udp" []
  , makeService "cindycollab"    3770    "tcp" []
  , makeService "cindycollab"    3770    "udp" []
  , makeService "paging-port"    3771    "tcp" []
  , makeService "paging-port"    3771    "udp" []
  , makeService "ctp"            3772    "tcp" []
  , makeService "ctp"            3772    "udp" []
  , makeService "ctdhercules"    3773    "tcp" []
  , makeService "ctdhercules"    3773    "udp" []
  , makeService "zicom"          3774    "tcp" []
  , makeService "zicom"          3774    "udp" []
  , makeService "ispmmgr"        3775    "tcp" []
  , makeService "ispmmgr"        3775    "udp" []
  , makeService "dvcprov-port"   3776    "tcp" []
  , makeService "dvcprov-port"   3776    "udp" []
  , makeService "jibe-eb"        3777    "tcp" []
  , makeService "jibe-eb"        3777    "udp" []
  , makeService "c-h-it-port"    3778    "tcp" []
  , makeService "c-h-it-port"    3778    "udp" []
  , makeService "cognima"        3779    "tcp" []
  , makeService "cognima"        3779    "udp" []
  , makeService "nnp"            3780    "tcp" []
  , makeService "nnp"            3780    "udp" []
  , makeService "abcvoice-port"  3781    "tcp" []
  , makeService "abcvoice-port"  3781    "udp" []
  , makeService "iso-tp0s"       3782    "tcp" []
  , makeService "iso-tp0s"       3782    "udp" []
  , makeService "bim-pem"        3783    "tcp" []
  , makeService "bim-pem"        3783    "udp" []
  , makeService "bfd-control"    3784    "tcp" []
  , makeService "bfd-control"    3784    "udp" []
  , makeService "bfd-echo"       3785    "tcp" []
  , makeService "bfd-echo"       3785    "udp" []
  , makeService "upstriggervsw"  3786    "tcp" []
  , makeService "upstriggervsw"  3786    "udp" []
  , makeService "fintrx"         3787    "tcp" []
  , makeService "fintrx"         3787    "udp" []
  , makeService "isrp-port"      3788    "tcp" []
  , makeService "isrp-port"      3788    "udp" []
  , makeService "remotedeploy"   3789    "tcp" []
  , makeService "remotedeploy"   3789    "udp" []
  , makeService "quickbooksrds"  3790    "tcp" []
  , makeService "quickbooksrds"  3790    "udp" []
  , makeService "tvnetworkvideo" 3791    "tcp" []
  , makeService "tvnetworkvideo" 3791    "udp" []
  , makeService "sitewatch"      3792    "tcp" []
  , makeService "sitewatch"      3792    "udp" []
  , makeService "dcsoftware"     3793    "tcp" []
  , makeService "dcsoftware"     3793    "udp" []
  , makeService "jaus"           3794    "tcp" []
  , makeService "jaus"           3794    "udp" []
  , makeService "myblast"        3795    "tcp" []
  , makeService "myblast"        3795    "udp" []
  , makeService "spw-dialer"     3796    "tcp" []
  , makeService "spw-dialer"     3796    "udp" []
  , makeService "idps"           3797    "tcp" []
  , makeService "idps"           3797    "udp" []
  , makeService "minilock"       3798    "tcp" []
  , makeService "minilock"       3798    "udp" []
  , makeService "radius-dynauth" 3799    "tcp" []
  , makeService "radius-dynauth" 3799    "udp" []
  , makeService "pwgpsi"         3800    "tcp" []
  , makeService "pwgpsi"         3800    "udp" []
  , makeService "ibm-mgr"        3801    "tcp" []
  , makeService "ibm-mgr"        3801    "udp" []
  , makeService "vhd"            3802    "tcp" []
  , makeService "vhd"            3802    "udp" []
  , makeService "soniqsync"      3803    "tcp" []
  , makeService "soniqsync"      3803    "udp" []
  , makeService "iqnet-port"     3804    "tcp" []
  , makeService "iqnet-port"     3804    "udp" []
  , makeService "tcpdataserver"  3805    "tcp" []
  , makeService "tcpdataserver"  3805    "udp" []
  , makeService "wsmlb"          3806    "tcp" []
  , makeService "wsmlb"          3806    "udp" []
  , makeService "spugna"         3807    "tcp" []
  , makeService "spugna"         3807    "udp" []
  , makeService "sun-as-iiops-ca"3808    "tcp" []
  , makeService "sun-as-iiops-ca"3808    "udp" []
  , makeService "apocd"          3809    "tcp" []
  , makeService "apocd"          3809    "udp" []
  , makeService "wlanauth"       3810    "tcp" []
  , makeService "wlanauth"       3810    "udp" []
  , makeService "amp"            3811    "tcp" []
  , makeService "amp"            3811    "udp" []
  , makeService "neto-wol-server"3812    "tcp" []
  , makeService "neto-wol-server"3812    "udp" []
  , makeService "rap-ip"         3813    "tcp" []
  , makeService "rap-ip"         3813    "udp" []
  , makeService "neto-dcs"       3814    "tcp" []
  , makeService "neto-dcs"       3814    "udp" []
  , makeService "lansurveyorxml" 3815    "tcp" []
  , makeService "lansurveyorxml" 3815    "udp" []
  , makeService "sunlps-http"    3816    "tcp" []
  , makeService "sunlps-http"    3816    "udp" []
  , makeService "tapeware"       3817    "tcp" []
  , makeService "tapeware"       3817    "udp" []
  , makeService "crinis-hb"      3818    "tcp" []
  , makeService "crinis-hb"      3818    "udp" []
  , makeService "epl-slp"        3819    "tcp" []
  , makeService "epl-slp"        3819    "udp" []
  , makeService "scp"            3820    "tcp" []
  , makeService "scp"            3820    "udp" []
  , makeService "pmcp"           3821    "tcp" []
  , makeService "pmcp"           3821    "udp" []
  , makeService "acp-discovery"  3822    "tcp" []
  , makeService "acp-discovery"  3822    "udp" []
  , makeService "acp-conduit"    3823    "tcp" []
  , makeService "acp-conduit"    3823    "udp" []
  , makeService "acp-policy"     3824    "tcp" []
  , makeService "acp-policy"     3824    "udp" []
  , makeService "ffserver"       3825    "tcp" []
  , makeService "ffserver"       3825    "udp" []
  , makeService "warmux"         3826    "tcp" []
  , makeService "warmux"         3826    "udp" []
  , makeService "netmpi"         3827    "tcp" []
  , makeService "netmpi"         3827    "udp" []
  , makeService "neteh"          3828    "tcp" []
  , makeService "neteh"          3828    "udp" []
  , makeService "neteh-ext"      3829    "tcp" []
  , makeService "neteh-ext"      3829    "udp" []
  , makeService "cernsysmgmtagt" 3830    "tcp" []
  , makeService "cernsysmgmtagt" 3830    "udp" []
  , makeService "dvapps"         3831    "tcp" []
  , makeService "dvapps"         3831    "udp" []
  , makeService "xxnetserver"    3832    "tcp" []
  , makeService "xxnetserver"    3832    "udp" []
  , makeService "aipn-auth"      3833    "tcp" []
  , makeService "aipn-auth"      3833    "udp" []
  , makeService "spectardata"    3834    "tcp" []
  , makeService "spectardata"    3834    "udp" []
  , makeService "spectardb"      3835    "tcp" []
  , makeService "spectardb"      3835    "udp" []
  , makeService "markem-dcp"     3836    "tcp" []
  , makeService "markem-dcp"     3836    "udp" []
  , makeService "mkm-discovery"  3837    "tcp" []
  , makeService "mkm-discovery"  3837    "udp" []
  , makeService "sos"            3838    "tcp" []
  , makeService "sos"            3838    "udp" []
  , makeService "amx-rms"        3839    "tcp" []
  , makeService "amx-rms"        3839    "udp" []
  , makeService "flirtmitmir"    3840    "tcp" []
  , makeService "flirtmitmir"    3840    "udp" []
  , makeService "zfirm-shiprush3"3841    "tcp" []
  , makeService "zfirm-shiprush3"3841    "udp" []
  , makeService "nhci"           3842    "tcp" []
  , makeService "nhci"           3842    "udp" []
  , makeService "quest-agent"    3843    "tcp" []
  , makeService "quest-agent"    3843    "udp" []
  , makeService "rnm"            3844    "tcp" []
  , makeService "rnm"            3844    "udp" []
  , makeService "v-one-spp"      3845    "tcp" []
  , makeService "v-one-spp"      3845    "udp" []
  , makeService "an-pcp"         3846    "tcp" []
  , makeService "an-pcp"         3846    "udp" []
  , makeService "msfw-control"   3847    "tcp" []
  , makeService "msfw-control"   3847    "udp" []
  , makeService "item"           3848    "tcp" []
  , makeService "item"           3848    "udp" []
  , makeService "spw-dnspreload" 3849    "tcp" []
  , makeService "spw-dnspreload" 3849    "udp" []
  , makeService "qtms-bootstrap" 3850    "tcp" []
  , makeService "qtms-bootstrap" 3850    "udp" []
  , makeService "spectraport"    3851    "tcp" []
  , makeService "spectraport"    3851    "udp" []
  , makeService "sse-app-config" 3852    "tcp" []
  , makeService "sse-app-config" 3852    "udp" []
  , makeService "sscan"          3853    "tcp" []
  , makeService "sscan"          3853    "udp" []
  , makeService "stryker-com"    3854    "tcp" []
  , makeService "stryker-com"    3854    "udp" []
  , makeService "opentrac"       3855    "tcp" []
  , makeService "opentrac"       3855    "udp" []
  , makeService "informer"       3856    "tcp" []
  , makeService "informer"       3856    "udp" []
  , makeService "trap-port"      3857    "tcp" []
  , makeService "trap-port"      3857    "udp" []
  , makeService "trap-port-mom"  3858    "tcp" []
  , makeService "trap-port-mom"  3858    "udp" []
  , makeService "nav-port"       3859    "tcp" []
  , makeService "nav-port"       3859    "udp" []
  , makeService "sasp"           3860    "tcp" []
  , makeService "sasp"           3860    "udp" []
  , makeService "winshadow-hd"   3861    "tcp" []
  , makeService "winshadow-hd"   3861    "udp" []
  , makeService "giga-pocket"    3862    "tcp" []
  , makeService "giga-pocket"    3862    "udp" []
  , makeService "asap-tcp"       3863    "tcp" []
  , makeService "asap-udp"       3863    "udp" []
  , makeService "asap-tcp-tls"   3864    "tcp" []
  , makeService "xpl"            3865    "tcp" []
  , makeService "xpl"            3865    "udp" []
  , makeService "dzdaemon"       3866    "tcp" []
  , makeService "dzdaemon"       3866    "udp" []
  , makeService "dzoglserver"    3867    "tcp" []
  , makeService "dzoglserver"    3867    "udp" []
  , makeService "diameter"       3868    "tcp" []
  , makeService "ovsam-mgmt"     3869    "tcp" []
  , makeService "ovsam-mgmt"     3869    "udp" []
  , makeService "ovsam-d-agent"  3870    "tcp" []
  , makeService "ovsam-d-agent"  3870    "udp" []
  , makeService "avocent-adsap"  3871    "tcp" []
  , makeService "avocent-adsap"  3871    "udp" []
  , makeService "oem-agent"      3872    "tcp" []
  , makeService "oem-agent"      3872    "udp" []
  , makeService "fagordnc"       3873    "tcp" []
  , makeService "fagordnc"       3873    "udp" []
  , makeService "sixxsconfig"    3874    "tcp" []
  , makeService "sixxsconfig"    3874    "udp" []
  , makeService "pnbscada"       3875    "tcp" []
  , makeService "pnbscada"       3875    "udp" []
  , makeService "dl_agent"       3876    "tcp" ["dl-agent"]
  , makeService "dl_agent"       3876    "udp" ["dl-agent"]
  , makeService "xmpcr-interface"3877    "tcp" []
  , makeService "xmpcr-interface"3877    "udp" []
  , makeService "fotogcad"       3878    "tcp" []
  , makeService "fotogcad"       3878    "udp" []
  , makeService "appss-lm"       3879    "tcp" []
  , makeService "appss-lm"       3879    "udp" []
  , makeService "igrs"           3880    "tcp" []
  , makeService "igrs"           3880    "udp" []
  , makeService "idac"           3881    "tcp" []
  , makeService "idac"           3881    "udp" []
  , makeService "msdts1"         3882    "tcp" []
  , makeService "msdts1"         3882    "udp" []
  , makeService "vrpn"           3883    "tcp" []
  , makeService "vrpn"           3883    "udp" []
  , makeService "softrack-meter" 3884    "tcp" []
  , makeService "softrack-meter" 3884    "udp" []
  , makeService "topflow-ssl"    3885    "tcp" []
  , makeService "topflow-ssl"    3885    "udp" []
  , makeService "nei-management" 3886    "tcp" []
  , makeService "nei-management" 3886    "udp" []
  , makeService "ciphire-data"   3887    "tcp" []
  , makeService "ciphire-data"   3887    "udp" []
  , makeService "ciphire-serv"   3888    "tcp" []
  , makeService "ciphire-serv"   3888    "udp" []
  , makeService "dandv-tester"   3889    "tcp" []
  , makeService "dandv-tester"   3889    "udp" []
  , makeService "ndsconnect"     3890    "tcp" []
  , makeService "ndsconnect"     3890    "udp" []
  , makeService "rtc-pm-port"    3891    "tcp" []
  , makeService "rtc-pm-port"    3891    "udp" []
  , makeService "pcc-image-port" 3892    "tcp" []
  , makeService "pcc-image-port" 3892    "udp" []
  , makeService "cgi-starapi"    3893    "tcp" []
  , makeService "cgi-starapi"    3893    "udp" []
  , makeService "syam-agent"     3894    "tcp" []
  , makeService "syam-agent"     3894    "udp" []
  , makeService "syam-smc"       3895    "tcp" []
  , makeService "syam-smc"       3895    "udp" []
  , makeService "sdo-tls"        3896    "tcp" []
  , makeService "sdo-tls"        3896    "udp" []
  , makeService "sdo-ssh"        3897    "tcp" []
  , makeService "sdo-ssh"        3897    "udp" []
  , makeService "senip"          3898    "tcp" []
  , makeService "senip"          3898    "udp" []
  , makeService "itv-control"    3899    "tcp" []
  , makeService "itv-control"    3899    "udp" []
  , makeService "nimsh"          3901    "tcp" []
  , makeService "nimsh"          3901    "udp" []
  , makeService "nimaux"         3902    "tcp" []
  , makeService "nimaux"         3902    "udp" []
  , makeService "charsetmgr"     3903    "tcp" []
  , makeService "charsetmgr"     3903    "udp" []
  , makeService "omnilink-port"  3904    "tcp" []
  , makeService "omnilink-port"  3904    "udp" []
  , makeService "mupdate"        3905    "tcp" []
  , makeService "mupdate"        3905    "udp" []
  , makeService "topovista-data" 3906    "tcp" []
  , makeService "topovista-data" 3906    "udp" []
  , makeService "imoguia-port"   3907    "tcp" []
  , makeService "imoguia-port"   3907    "udp" []
  , makeService "hppronetman"    3908    "tcp" []
  , makeService "hppronetman"    3908    "udp" []
  , makeService "surfcontrolcpa" 3909    "tcp" []
  , makeService "surfcontrolcpa" 3909    "udp" []
  , makeService "prnrequest"     3910    "tcp" []
  , makeService "prnrequest"     3910    "udp" []
  , makeService "prnstatus"      3911    "tcp" []
  , makeService "prnstatus"      3911    "udp" []
  , makeService "gbmt-stars"     3912    "tcp" []
  , makeService "gbmt-stars"     3912    "udp" []
  , makeService "listcrt-port"   3913    "tcp" []
  , makeService "listcrt-port"   3913    "udp" []
  , makeService "listcrt-port-2" 3914    "tcp" []
  , makeService "listcrt-port-2" 3914    "udp" []
  , makeService "agcat"          3915    "tcp" []
  , makeService "agcat"          3915    "udp" []
  , makeService "wysdmc"         3916    "tcp" []
  , makeService "wysdmc"         3916    "udp" []
  , makeService "aftmux"         3917    "tcp" []
  , makeService "aftmux"         3917    "udp" []
  , makeService "pktcablemmcops" 3918    "tcp" []
  , makeService "pktcablemmcops" 3918    "udp" []
  , makeService "hyperip"        3919    "tcp" []
  , makeService "hyperip"        3919    "udp" []
  , makeService "exasoftport1"   3920    "tcp" []
  , makeService "exasoftport1"   3920    "udp" []
  , makeService "herodotus-net"  3921    "tcp" []
  , makeService "herodotus-net"  3921    "udp" []
  , makeService "sor-update"     3922    "tcp" []
  , makeService "sor-update"     3922    "udp" []
  , makeService "symb-sb-port"   3923    "tcp" []
  , makeService "symb-sb-port"   3923    "udp" []
  , makeService "mpl-gprs-port"  3924    "tcp" []
  , makeService "mpl-gprs-port"  3924    "udp" []
  , makeService "zmp"            3925    "tcp" []
  , makeService "zmp"            3925    "udp" []
  , makeService "winport"        3926    "tcp" []
  , makeService "winport"        3926    "udp" []
  , makeService "natdataservice" 3927    "tcp" []
  , makeService "natdataservice" 3927    "udp" []
  , makeService "netboot-pxe"    3928    "tcp" []
  , makeService "netboot-pxe"    3928    "udp" []
  , makeService "smauth-port"    3929    "tcp" []
  , makeService "smauth-port"    3929    "udp" []
  , makeService "syam-webserver" 3930    "tcp" []
  , makeService "syam-webserver" 3930    "udp" []
  , makeService "msr-plugin-port"3931    "tcp" []
  , makeService "msr-plugin-port"3931    "udp" []
  , makeService "dyn-site"       3932    "tcp" []
  , makeService "dyn-site"       3932    "udp" []
  , makeService "plbserve-port"  3933    "tcp" []
  , makeService "plbserve-port"  3933    "udp" []
  , makeService "sunfm-port"     3934    "tcp" []
  , makeService "sunfm-port"     3934    "udp" []
  , makeService "sdp-portmapper" 3935    "tcp" []
  , makeService "sdp-portmapper" 3935    "udp" []
  , makeService "mailprox"       3936    "tcp" []
  , makeService "mailprox"       3936    "udp" []
  , makeService "dvbservdsc"     3937    "tcp" []
  , makeService "dvbservdsc"     3937    "udp" []
  , makeService "dbcontrol_agent"3938    "tcp" ["dbcontrol-agent"]
  , makeService "dbcontrol_agent"3938    "udp" ["dbcontrol-agent"]
  , makeService "aamp"           3939    "tcp" []
  , makeService "aamp"           3939    "udp" []
  , makeService "xecp-node"      3940    "tcp" []
  , makeService "xecp-node"      3940    "udp" []
  , makeService "homeportal-web" 3941    "tcp" []
  , makeService "homeportal-web" 3941    "udp" []
  , makeService "srdp"           3942    "tcp" []
  , makeService "srdp"           3942    "udp" []
  , makeService "tig"            3943    "tcp" []
  , makeService "tig"            3943    "udp" []
  , makeService "sops"           3944    "tcp" []
  , makeService "sops"           3944    "udp" []
  , makeService "emcads"         3945    "tcp" []
  , makeService "emcads"         3945    "udp" []
  , makeService "backupedge"     3946    "tcp" []
  , makeService "backupedge"     3946    "udp" []
  , makeService "ccp"            3947    "tcp" []
  , makeService "ccp"            3947    "udp" []
  , makeService "apdap"          3948    "tcp" []
  , makeService "apdap"          3948    "udp" []
  , makeService "drip"           3949    "tcp" []
  , makeService "drip"           3949    "udp" []
  , makeService "namemunge"      3950    "tcp" []
  , makeService "namemunge"      3950    "udp" []
  , makeService "pwgippfax"      3951    "tcp" []
  , makeService "pwgippfax"      3951    "udp" []
  , makeService "i3-sessionmgr"  3952    "tcp" []
  , makeService "i3-sessionmgr"  3952    "udp" []
  , makeService "xmlink-connect" 3953    "tcp" []
  , makeService "xmlink-connect" 3953    "udp" []
  , makeService "adrep"          3954    "tcp" []
  , makeService "adrep"          3954    "udp" []
  , makeService "p2pcommunity"   3955    "tcp" []
  , makeService "p2pcommunity"   3955    "udp" []
  , makeService "gvcp"           3956    "tcp" []
  , makeService "gvcp"           3956    "udp" []
  , makeService "mqe-broker"     3957    "tcp" []
  , makeService "mqe-broker"     3957    "udp" []
  , makeService "mqe-agent"      3958    "tcp" []
  , makeService "mqe-agent"      3958    "udp" []
  , makeService "treehopper"     3959    "tcp" []
  , makeService "treehopper"     3959    "udp" []
  , makeService "bess"           3960    "tcp" []
  , makeService "bess"           3960    "udp" []
  , makeService "proaxess"       3961    "tcp" []
  , makeService "proaxess"       3961    "udp" []
  , makeService "sbi-agent"      3962    "tcp" []
  , makeService "sbi-agent"      3962    "udp" []
  , makeService "thrp"           3963    "tcp" []
  , makeService "thrp"           3963    "udp" []
  , makeService "sasggprs"       3964    "tcp" []
  , makeService "sasggprs"       3964    "udp" []
  , makeService "ati-ip-to-ncpe" 3965    "tcp" []
  , makeService "ati-ip-to-ncpe" 3965    "udp" []
  , makeService "bflckmgr"       3966    "tcp" []
  , makeService "bflckmgr"       3966    "udp" []
  , makeService "ppsms"          3967    "tcp" []
  , makeService "ppsms"          3967    "udp" []
  , makeService "ianywhere-dbns" 3968    "tcp" []
  , makeService "ianywhere-dbns" 3968    "udp" []
  , makeService "landmarks"      3969    "tcp" []
  , makeService "landmarks"      3969    "udp" []
  , makeService "lanrevagent"    3970    "tcp" []
  , makeService "lanrevagent"    3970    "udp" []
  , makeService "lanrevserver"   3971    "tcp" []
  , makeService "lanrevserver"   3971    "udp" []
  , makeService "iconp"          3972    "tcp" []
  , makeService "iconp"          3972    "udp" []
  , makeService "progistics"     3973    "tcp" []
  , makeService "progistics"     3973    "udp" []
  , makeService "citysearch"     3974    "tcp" []
  , makeService "citysearch"     3974    "udp" []
  , makeService "airshot"        3975    "tcp" []
  , makeService "airshot"        3975    "udp" []
  , makeService "opswagent"      3976    "tcp" []
  , makeService "opswagent"      3976    "udp" []
  , makeService "opswmanager"    3977    "tcp" []
  , makeService "opswmanager"    3977    "udp" []
  , makeService "secure-cfg-svr" 3978    "tcp" []
  , makeService "secure-cfg-svr" 3978    "udp" []
  , makeService "smwan"          3979    "tcp" []
  , makeService "smwan"          3979    "udp" []
  , makeService "acms"           3980    "tcp" []
  , makeService "acms"           3980    "udp" []
  , makeService "starfish"       3981    "tcp" []
  , makeService "starfish"       3981    "udp" []
  , makeService "eis"            3982    "tcp" []
  , makeService "eis"            3982    "udp" []
  , makeService "eisp"           3983    "tcp" []
  , makeService "eisp"           3983    "udp" []
  , makeService "mapper-nodemgr" 3984    "tcp" []
  , makeService "mapper-nodemgr" 3984    "udp" []
  , makeService "mapper-mapethd" 3985    "tcp" []
  , makeService "mapper-mapethd" 3985    "udp" []
  , makeService "mapper-ws_ethd" 3986    "tcp" ["mapper-ws-ethd"]
  , makeService "mapper-ws_ethd" 3986    "udp" ["mapper-ws-ethd"]
  , makeService "centerline"     3987    "tcp" []
  , makeService "centerline"     3987    "udp" []
  , makeService "dcs-config"     3988    "tcp" []
  , makeService "dcs-config"     3988    "udp" []
  , makeService "bv-queryengine" 3989    "tcp" []
  , makeService "bv-queryengine" 3989    "udp" []
  , makeService "bv-is"          3990    "tcp" []
  , makeService "bv-is"          3990    "udp" []
  , makeService "bv-smcsrv"      3991    "tcp" []
  , makeService "bv-smcsrv"      3991    "udp" []
  , makeService "bv-ds"          3992    "tcp" []
  , makeService "bv-ds"          3992    "udp" []
  , makeService "bv-agent"       3993    "tcp" []
  , makeService "bv-agent"       3993    "udp" []
  , makeService "iss-mgmt-ssl"   3995    "tcp" []
  , makeService "iss-mgmt-ssl"   3995    "udp" []
  , makeService "abcsoftware"    3996    "tcp" []
  , makeService "abcsoftware"    3996    "udp" []
  , makeService "agentsease-db"  3997    "tcp" []
  , makeService "agentsease-db"  3997    "udp" []
  , makeService "dnx"            3998    "tcp" []
  , makeService "dnx"            3998    "udp" []
  , makeService "nvcnet"         3999    "tcp" []
  , makeService "nvcnet"         3999    "udp" []
  , makeService "terabase"       4000    "tcp" []
  , makeService "terabase"       4000    "udp" []
  , makeService "newoak"         4001    "tcp" []
  , makeService "newoak"         4001    "udp" []
  , makeService "pxc-spvr-ft"    4002    "tcp" []
  , makeService "pxc-spvr-ft"    4002    "udp" []
  , makeService "pxc-splr-ft"    4003    "tcp" []
  , makeService "pxc-splr-ft"    4003    "udp" []
  , makeService "pxc-roid"       4004    "tcp" []
  , makeService "pxc-roid"       4004    "udp" []
  , makeService "pxc-pin"        4005    "tcp" []
  , makeService "pxc-pin"        4005    "udp" []
  , makeService "pxc-spvr"       4006    "tcp" []
  , makeService "pxc-spvr"       4006    "udp" []
  , makeService "pxc-splr"       4007    "tcp" []
  , makeService "pxc-splr"       4007    "udp" []
  , makeService "netcheque"      4008    "tcp" []
  , makeService "netcheque"      4008    "udp" []
  , makeService "chimera-hwm"    4009    "tcp" []
  , makeService "chimera-hwm"    4009    "udp" []
  , makeService "samsung-unidex" 4010    "tcp" []
  , makeService "samsung-unidex" 4010    "udp" []
  , makeService "altserviceboot" 4011    "tcp" []
  , makeService "pda-gate"       4012    "tcp" []
  , makeService "pda-gate"       4012    "udp" []
  , makeService "acl-manager"    4013    "tcp" []
  , makeService "acl-manager"    4013    "udp" []
  , makeService "taiclock"       4014    "tcp" []
  , makeService "taiclock"       4014    "udp" []
  , makeService "talarian-mcast1"4015    "tcp" []
  , makeService "talarian-mcast1"4015    "udp" []
  , makeService "talarian-mcast2"4016    "tcp" []
  , makeService "talarian-mcast2"4016    "udp" []
  , makeService "talarian-mcast3"4017    "tcp" []
  , makeService "talarian-mcast3"4017    "udp" []
  , makeService "talarian-mcast4"4018    "tcp" []
  , makeService "talarian-mcast4"4018    "udp" []
  , makeService "talarian-mcast5"4019    "tcp" []
  , makeService "talarian-mcast5"4019    "udp" []
  , makeService "trap"           4020    "tcp" []
  , makeService "trap"           4020    "udp" []
  , makeService "nexus-portal"   4021    "tcp" []
  , makeService "nexus-portal"   4021    "udp" []
  , makeService "dnox"           4022    "tcp" []
  , makeService "dnox"           4022    "udp" []
  , makeService "esnm-zoning"    4023    "tcp" []
  , makeService "esnm-zoning"    4023    "udp" []
  , makeService "tnp1-port"      4024    "tcp" []
  , makeService "tnp1-port"      4024    "udp" []
  , makeService "partimage"      4025    "tcp" []
  , makeService "partimage"      4025    "udp" []
  , makeService "as-debug"       4026    "tcp" []
  , makeService "as-debug"       4026    "udp" []
  , makeService "bxp"            4027    "tcp" []
  , makeService "bxp"            4027    "udp" []
  , makeService "dtserver-port"  4028    "tcp" []
  , makeService "dtserver-port"  4028    "udp" []
  , makeService "ip-qsig"        4029    "tcp" []
  , makeService "ip-qsig"        4029    "udp" []
  , makeService "jdmn-port"      4030    "tcp" []
  , makeService "jdmn-port"      4030    "udp" []
  , makeService "suucp"          4031    "tcp" []
  , makeService "suucp"          4031    "udp" []
  , makeService "vrts-auth-port" 4032    "tcp" []
  , makeService "vrts-auth-port" 4032    "udp" []
  , makeService "sanavigator"    4033    "tcp" []
  , makeService "sanavigator"    4033    "udp" []
  , makeService "ubxd"           4034    "tcp" []
  , makeService "ubxd"           4034    "udp" []
  , makeService "wap-push-http"  4035    "tcp" []
  , makeService "wap-push-http"  4035    "udp" []
  , makeService "wap-push-https" 4036    "tcp" []
  , makeService "wap-push-https" 4036    "udp" []
  , makeService "ravehd"         4037    "tcp" []
  , makeService "ravehd"         4037    "udp" []
  , makeService "fazzt-ptp"      4038    "tcp" []
  , makeService "fazzt-ptp"      4038    "udp" []
  , makeService "fazzt-admin"    4039    "tcp" []
  , makeService "fazzt-admin"    4039    "udp" []
  , makeService "yo-main"        4040    "tcp" []
  , makeService "yo-main"        4040    "udp" []
  , makeService "houston"        4041    "tcp" []
  , makeService "houston"        4041    "udp" []
  , makeService "ldxp"           4042    "tcp" []
  , makeService "ldxp"           4042    "udp" []
  , makeService "nirp"           4043    "tcp" []
  , makeService "nirp"           4043    "udp" []
  , makeService "ltp"            4044    "tcp" []
  , makeService "ltp"            4044    "udp" []
  , makeService "acp-proto"      4046    "tcp" []
  , makeService "acp-proto"      4046    "udp" []
  , makeService "ctp-state"      4047    "tcp" []
  , makeService "ctp-state"      4047    "udp" []
  , makeService "wafs"           4049    "tcp" []
  , makeService "wafs"           4049    "udp" []
  , makeService "cisco-wafs"     4050    "tcp" []
  , makeService "cisco-wafs"     4050    "udp" []
  , makeService "cppdp"          4051    "tcp" []
  , makeService "cppdp"          4051    "udp" []
  , makeService "interact"       4052    "tcp" []
  , makeService "interact"       4052    "udp" []
  , makeService "ccu-comm-1"     4053    "tcp" []
  , makeService "ccu-comm-1"     4053    "udp" []
  , makeService "ccu-comm-2"     4054    "tcp" []
  , makeService "ccu-comm-2"     4054    "udp" []
  , makeService "ccu-comm-3"     4055    "tcp" []
  , makeService "ccu-comm-3"     4055    "udp" []
  , makeService "lms"            4056    "tcp" []
  , makeService "lms"            4056    "udp" []
  , makeService "wfm"            4057    "tcp" []
  , makeService "wfm"            4057    "udp" []
  , makeService "kingfisher"     4058    "tcp" []
  , makeService "kingfisher"     4058    "udp" []
  , makeService "dlms-cosem"     4059    "tcp" []
  , makeService "dlms-cosem"     4059    "udp" []
  , makeService "dsmeter_iatc"   4060    "tcp" ["dsmeter-iatc"]
  , makeService "dsmeter_iatc"   4060    "udp" ["dsmeter-iatc"]
  , makeService "ice-location"   4061    "tcp" []
  , makeService "ice-location"   4061    "udp" []
  , makeService "ice-slocation"  4062    "tcp" []
  , makeService "ice-slocation"  4062    "udp" []
  , makeService "ice-router"     4063    "tcp" []
  , makeService "ice-router"     4063    "udp" []
  , makeService "ice-srouter"    4064    "tcp" []
  , makeService "ice-srouter"    4064    "udp" []
  , makeService "avanti_cdp"     4065    "tcp" ["avanti-cdp"]
  , makeService "avanti_cdp"     4065    "udp" ["avanti-cdp"]
  , makeService "pmas"           4066    "tcp" []
  , makeService "pmas"           4066    "udp" []
  , makeService "idp"            4067    "tcp" []
  , makeService "idp"            4067    "udp" []
  , makeService "ipfltbcst"      4068    "tcp" []
  , makeService "ipfltbcst"      4068    "udp" []
  , makeService "minger"         4069    "tcp" []
  , makeService "minger"         4069    "udp" []
  , makeService "tripe"          4070    "tcp" []
  , makeService "tripe"          4070    "udp" []
  , makeService "aibkup"         4071    "tcp" []
  , makeService "aibkup"         4071    "udp" []
  , makeService "zieto-sock"     4072    "tcp" []
  , makeService "zieto-sock"     4072    "udp" []
  , makeService "iRAPP"          4073    "tcp" []
  , makeService "iRAPP"          4073    "udp" []
  , makeService "cequint-cityid" 4074    "tcp" []
  , makeService "cequint-cityid" 4074    "udp" []
  , makeService "perimlan"       4075    "tcp" []
  , makeService "perimlan"       4075    "udp" []
  , makeService "seraph"         4076    "tcp" []
  , makeService "seraph"         4076    "udp" []
  , makeService "ascomalarm"     4077    "udp" []
  , makeService "cssp"           4078    "tcp" []
  , makeService "lorica-in"      4080    "tcp" []
  , makeService "lorica-in"      4080    "udp" []
  , makeService "lorica-in-sec"  4081    "tcp" []
  , makeService "lorica-in-sec"  4081    "udp" []
  , makeService "lorica-out"     4082    "tcp" []
  , makeService "lorica-out"     4082    "udp" []
  , makeService "lorica-out-sec" 4083    "tcp" []
  , makeService "lorica-out-sec" 4083    "udp" []
  , makeService "fortisphere-vm" 4084    "udp" []
  , makeService "ezmessagesrv"   4085    "tcp" []
  , makeService "ftsync"         4086    "udp" []
  , makeService "applusservice"  4087    "tcp" []
  , makeService "npsp"           4088    "tcp" []
  , makeService "opencore"       4089    "tcp" []
  , makeService "opencore"       4089    "udp" []
  , makeService "omasgport"      4090    "tcp" []
  , makeService "omasgport"      4090    "udp" []
  , makeService "ewinstaller"    4091    "tcp" []
  , makeService "ewinstaller"    4091    "udp" []
  , makeService "ewdgs"          4092    "tcp" []
  , makeService "ewdgs"          4092    "udp" []
  , makeService "pvxpluscs"      4093    "tcp" []
  , makeService "pvxpluscs"      4093    "udp" []
  , makeService "sysrqd"         4094    "tcp" []
  , makeService "sysrqd"         4094    "udp" []
  , makeService "xtgui"          4095    "tcp" []
  , makeService "xtgui"          4095    "udp" []
  , makeService "bre"            4096    "tcp" []
  , makeService "bre"            4096    "udp" []
  , makeService "patrolview"     4097    "tcp" []
  , makeService "patrolview"     4097    "udp" []
  , makeService "drmsfsd"        4098    "tcp" []
  , makeService "drmsfsd"        4098    "udp" []
  , makeService "dpcp"           4099    "tcp" []
  , makeService "dpcp"           4099    "udp" []
  , makeService "igo-incognito"  4100    "tcp" []
  , makeService "igo-incognito"  4100    "udp" []
  , makeService "brlp-0"         4101    "tcp" []
  , makeService "brlp-0"         4101    "udp" []
  , makeService "brlp-1"         4102    "tcp" []
  , makeService "brlp-1"         4102    "udp" []
  , makeService "brlp-2"         4103    "tcp" []
  , makeService "brlp-2"         4103    "udp" []
  , makeService "brlp-3"         4104    "tcp" []
  , makeService "brlp-3"         4104    "udp" []
  , makeService "shofar"         4105    "tcp" []
  , makeService "shofar"         4105    "udp" []
  , makeService "synchronite"    4106    "tcp" []
  , makeService "synchronite"    4106    "udp" []
  , makeService "j-ac"           4107    "tcp" []
  , makeService "j-ac"           4107    "udp" []
  , makeService "accel"          4108    "tcp" []
  , makeService "accel"          4108    "udp" []
  , makeService "izm"            4109    "tcp" []
  , makeService "izm"            4109    "udp" []
  , makeService "g2tag"          4110    "tcp" []
  , makeService "g2tag"          4110    "udp" []
  , makeService "xgrid"          4111    "tcp" []
  , makeService "xgrid"          4111    "udp" []
  , makeService "apple-vpns-rp"  4112    "tcp" []
  , makeService "apple-vpns-rp"  4112    "udp" []
  , makeService "aipn-reg"       4113    "tcp" []
  , makeService "aipn-reg"       4113    "udp" []
  , makeService "jomamqmonitor"  4114    "tcp" []
  , makeService "jomamqmonitor"  4114    "udp" []
  , makeService "cds"            4115    "tcp" []
  , makeService "cds"            4115    "udp" []
  , makeService "smartcard-tls"  4116    "tcp" []
  , makeService "smartcard-tls"  4116    "udp" []
  , makeService "hillrserv"      4117    "tcp" []
  , makeService "hillrserv"      4117    "udp" []
  , makeService "netscript"      4118    "tcp" []
  , makeService "netscript"      4118    "udp" []
  , makeService "assuria-slm"    4119    "tcp" []
  , makeService "assuria-slm"    4119    "udp" []
  , makeService "e-builder"      4121    "tcp" []
  , makeService "e-builder"      4121    "udp" []
  , makeService "fprams"         4122    "tcp" []
  , makeService "fprams"         4122    "udp" []
  , makeService "z-wave"         4123    "tcp" []
  , makeService "z-wave"         4123    "udp" []
  , makeService "tigv2"          4124    "tcp" []
  , makeService "tigv2"          4124    "udp" []
  , makeService "opsview-envoy"  4125    "tcp" []
  , makeService "opsview-envoy"  4125    "udp" []
  , makeService "ddrepl"         4126    "tcp" []
  , makeService "ddrepl"         4126    "udp" []
  , makeService "unikeypro"      4127    "tcp" []
  , makeService "unikeypro"      4127    "udp" []
  , makeService "nufw"           4128    "tcp" []
  , makeService "nufw"           4128    "udp" []
  , makeService "nuauth"         4129    "tcp" []
  , makeService "nuauth"         4129    "udp" []
  , makeService "fronet"         4130    "tcp" []
  , makeService "fronet"         4130    "udp" []
  , makeService "stars"          4131    "tcp" []
  , makeService "stars"          4131    "udp" []
  , makeService "nuts_dem"       4132    "tcp" ["nuts-dem"]
  , makeService "nuts_dem"       4132    "udp" ["nuts-dem"]
  , makeService "nuts_bootp"     4133    "tcp" ["nuts-bootp"]
  , makeService "nuts_bootp"     4133    "udp" ["nuts-bootp"]
  , makeService "nifty-hmi"      4134    "tcp" []
  , makeService "nifty-hmi"      4134    "udp" []
  , makeService "cl-db-attach"   4135    "tcp" []
  , makeService "cl-db-attach"   4135    "udp" []
  , makeService "cl-db-request"  4136    "tcp" []
  , makeService "cl-db-request"  4136    "udp" []
  , makeService "cl-db-remote"   4137    "tcp" []
  , makeService "cl-db-remote"   4137    "udp" []
  , makeService "nettest"        4138    "tcp" []
  , makeService "nettest"        4138    "udp" []
  , makeService "thrtx"          4139    "tcp" []
  , makeService "thrtx"          4139    "udp" []
  , makeService "cedros_fds"     4140    "tcp" ["cedros-fds"]
  , makeService "cedros_fds"     4140    "udp" ["cedros-fds"]
  , makeService "oirtgsvc"       4141    "tcp" []
  , makeService "oirtgsvc"       4141    "udp" []
  , makeService "oidocsvc"       4142    "tcp" []
  , makeService "oidocsvc"       4142    "udp" []
  , makeService "oidsr"          4143    "tcp" []
  , makeService "oidsr"          4143    "udp" []
  , makeService "vvr-control"    4145    "tcp" []
  , makeService "vvr-control"    4145    "udp" []
  , makeService "tgcconnect"     4146    "tcp" []
  , makeService "tgcconnect"     4146    "udp" []
  , makeService "vrxpservman"    4147    "tcp" []
  , makeService "vrxpservman"    4147    "udp" []
  , makeService "hhb-handheld"   4148    "tcp" []
  , makeService "hhb-handheld"   4148    "udp" []
  , makeService "agslb"          4149    "tcp" []
  , makeService "agslb"          4149    "udp" []
  , makeService "PowerAlert-nsa" 4150    "tcp" []
  , makeService "PowerAlert-nsa" 4150    "udp" []
  , makeService "menandmice_noh" 4151    "tcp" ["menandmice-noh"]
  , makeService "menandmice_noh" 4151    "udp" ["menandmice-noh"]
  , makeService "idig_mux"       4152    "tcp" ["idig-mux"]
  , makeService "idig_mux"       4152    "udp" ["idig-mux"]
  , makeService "mbl-battd"      4153    "tcp" []
  , makeService "mbl-battd"      4153    "udp" []
  , makeService "atlinks"        4154    "tcp" []
  , makeService "atlinks"        4154    "udp" []
  , makeService "bzr"            4155    "tcp" []
  , makeService "bzr"            4155    "udp" []
  , makeService "stat-results"   4156    "tcp" []
  , makeService "stat-results"   4156    "udp" []
  , makeService "stat-scanner"   4157    "tcp" []
  , makeService "stat-scanner"   4157    "udp" []
  , makeService "stat-cc"        4158    "tcp" []
  , makeService "stat-cc"        4158    "udp" []
  , makeService "nss"            4159    "tcp" []
  , makeService "nss"            4159    "udp" []
  , makeService "jini-discovery" 4160    "tcp" []
  , makeService "jini-discovery" 4160    "udp" []
  , makeService "omscontact"     4161    "tcp" []
  , makeService "omscontact"     4161    "udp" []
  , makeService "omstopology"    4162    "tcp" []
  , makeService "omstopology"    4162    "udp" []
  , makeService "silverpeakpeer" 4163    "tcp" []
  , makeService "silverpeakpeer" 4163    "udp" []
  , makeService "silverpeakcomm" 4164    "tcp" []
  , makeService "silverpeakcomm" 4164    "udp" []
  , makeService "altcp"          4165    "tcp" []
  , makeService "altcp"          4165    "udp" []
  , makeService "joost"          4166    "tcp" []
  , makeService "joost"          4166    "udp" []
  , makeService "ddgn"           4167    "tcp" []
  , makeService "ddgn"           4167    "udp" []
  , makeService "pslicser"       4168    "tcp" []
  , makeService "pslicser"       4168    "udp" []
  , makeService "iadt"           4169    "tcp" []
  , makeService "iadt-disc"      4169    "udp" []
  , makeService "d-cinema-csp"   4170    "tcp" []
  , makeService "ml-svnet"       4171    "tcp" []
  , makeService "pcoip"          4172    "tcp" []
  , makeService "pcoip"          4172    "udp" []
  , makeService "mma-discovery"  4173    "udp" []
  , makeService "smcluster"      4174    "tcp" []
  , makeService "sm-disc"        4174    "udp" []
  , makeService "bccp"           4175    "tcp" []
  , makeService "tl-ipcproxy"    4176    "tcp" []
  , makeService "wello"          4177    "tcp" []
  , makeService "wello"          4177    "udp" []
  , makeService "storman"        4178    "tcp" []
  , makeService "storman"        4178    "udp" []
  , makeService "MaxumSP"        4179    "tcp" []
  , makeService "MaxumSP"        4179    "udp" []
  , makeService "httpx"          4180    "tcp" []
  , makeService "httpx"          4180    "udp" []
  , makeService "macbak"         4181    "tcp" []
  , makeService "macbak"         4181    "udp" []
  , makeService "pcptcpservice"  4182    "tcp" []
  , makeService "pcptcpservice"  4182    "udp" []
  , makeService "gmmp"           4183    "tcp" []
  , makeService "gmmp"           4183    "udp" []
  , makeService "universe_suite" 4184    "tcp" ["universe-suite"]
  , makeService "universe_suite" 4184    "udp" ["universe-suite"]
  , makeService "wcpp"           4185    "tcp" []
  , makeService "wcpp"           4185    "udp" []
  , makeService "boxbackupstore" 4186    "tcp" []
  , makeService "csc_proxy"      4187    "tcp" ["csc-proxy"]
  , makeService "vatata"         4188    "tcp" []
  , makeService "vatata"         4188    "udp" []
  , makeService "pcep"           4189    "tcp" []
  , makeService "sieve"          4190    "tcp" []
  , makeService "dsmipv6"        4191    "udp" []
  , makeService "azeti"          4192    "tcp" []
  , makeService "azeti-bd"       4192    "udp" []
  , makeService "pvxplusio"      4193    "tcp" []
  , makeService "eims-admin"     4199    "tcp" []
  , makeService "eims-admin"     4199    "udp" []
  , makeService "corelccam"      4300    "tcp" []
  , makeService "corelccam"      4300    "udp" []
  , makeService "d-data"         4301    "tcp" []
  , makeService "d-data"         4301    "udp" []
  , makeService "d-data-control" 4302    "tcp" []
  , makeService "d-data-control" 4302    "udp" []
  , makeService "srcp"           4303    "tcp" []
  , makeService "srcp"           4303    "udp" []
  , makeService "owserver"       4304    "tcp" []
  , makeService "owserver"       4304    "udp" []
  , makeService "batman"         4305    "tcp" []
  , makeService "batman"         4305    "udp" []
  , makeService "pinghgl"        4306    "tcp" []
  , makeService "pinghgl"        4306    "udp" []
  , makeService "visicron-vs"    4307    "tcp" []
  , makeService "visicron-vs"    4307    "udp" []
  , makeService "compx-lockview" 4308    "tcp" []
  , makeService "compx-lockview" 4308    "udp" []
  , makeService "dserver"        4309    "tcp" []
  , makeService "dserver"        4309    "udp" []
  , makeService "mirrtex"        4310    "tcp" []
  , makeService "mirrtex"        4310    "udp" []
  , makeService "p6ssmc"         4311    "tcp" []
  , makeService "pscl-mgt"       4312    "tcp" []
  , makeService "perrla"         4313    "tcp" []
  , makeService "choiceview-agt" 4314    "tcp" []
  , makeService "choiceview-clt" 4316    "tcp" []
  , makeService "fdt-rcatp"      4320    "tcp" []
  , makeService "fdt-rcatp"      4320    "udp" []
  , makeService "trim-event"     4322    "tcp" []
  , makeService "trim-event"     4322    "udp" []
  , makeService "trim-ice"       4323    "tcp" []
  , makeService "trim-ice"       4323    "udp" []
  , makeService "balour"         4324    "tcp" []
  , makeService "balour"         4324    "udp" []
  , makeService "geognosisman"   4325    "tcp" []
  , makeService "geognosisman"   4325    "udp" []
  , makeService "geognosis"      4326    "tcp" []
  , makeService "geognosis"      4326    "udp" []
  , makeService "jaxer-web"      4327    "tcp" []
  , makeService "jaxer-web"      4327    "udp" []
  , makeService "jaxer-manager"  4328    "tcp" []
  , makeService "jaxer-manager"  4328    "udp" []
  , makeService "publiqare-sync" 4329    "tcp" []
  , makeService "dey-sapi"       4330    "tcp" []
  , makeService "gaia"           4340    "tcp" []
  , makeService "gaia"           4340    "udp" []
  , makeService "lisp-data"      4341    "tcp" []
  , makeService "lisp-data"      4341    "udp" []
  , makeService "lisp-cons"      4342    "tcp" []
  , makeService "lisp-control"   4342    "udp" []
  , makeService "unicall"        4343    "tcp" []
  , makeService "unicall"        4343    "udp" []
  , makeService "vinainstall"    4344    "tcp" []
  , makeService "vinainstall"    4344    "udp" []
  , makeService "m4-network-as"  4345    "tcp" []
  , makeService "m4-network-as"  4345    "udp" []
  , makeService "elanlm"         4346    "tcp" []
  , makeService "elanlm"         4346    "udp" []
  , makeService "lansurveyor"    4347    "tcp" []
  , makeService "lansurveyor"    4347    "udp" []
  , makeService "itose"          4348    "tcp" []
  , makeService "itose"          4348    "udp" []
  , makeService "fsportmap"      4349    "tcp" []
  , makeService "fsportmap"      4349    "udp" []
  , makeService "net-device"     4350    "tcp" []
  , makeService "net-device"     4350    "udp" []
  , makeService "plcy-net-svcs"  4351    "tcp" []
  , makeService "plcy-net-svcs"  4351    "udp" []
  , makeService "pjlink"         4352    "tcp" []
  , makeService "pjlink"         4352    "udp" []
  , makeService "f5-iquery"      4353    "tcp" []
  , makeService "f5-iquery"      4353    "udp" []
  , makeService "qsnet-trans"    4354    "tcp" []
  , makeService "qsnet-trans"    4354    "udp" []
  , makeService "qsnet-workst"   4355    "tcp" []
  , makeService "qsnet-workst"   4355    "udp" []
  , makeService "qsnet-assist"   4356    "tcp" []
  , makeService "qsnet-assist"   4356    "udp" []
  , makeService "qsnet-cond"     4357    "tcp" []
  , makeService "qsnet-cond"     4357    "udp" []
  , makeService "qsnet-nucl"     4358    "tcp" []
  , makeService "qsnet-nucl"     4358    "udp" []
  , makeService "omabcastltkm"   4359    "tcp" []
  , makeService "omabcastltkm"   4359    "udp" []
  , makeService "matrix_vnet"    4360    "tcp" ["matrix-vnet"]
  , makeService "nacnl"          4361    "udp" []
  , makeService "afore-vdp-disc" 4362    "udp" []
  , makeService "wxbrief"        4368    "tcp" []
  , makeService "wxbrief"        4368    "udp" []
  , makeService "epmd"           4369    "tcp" []
  , makeService "epmd"           4369    "udp" []
  , makeService "elpro_tunnel"   4370    "tcp" ["elpro-tunnel"]
  , makeService "elpro_tunnel"   4370    "udp" ["elpro-tunnel"]
  , makeService "l2c-control"    4371    "tcp" []
  , makeService "l2c-disc"       4371    "udp" []
  , makeService "l2c-data"       4372    "tcp" []
  , makeService "l2c-data"       4372    "udp" []
  , makeService "remctl"         4373    "tcp" []
  , makeService "remctl"         4373    "udp" []
  , makeService "psi-ptt"        4374    "tcp" []
  , makeService "tolteces"       4375    "tcp" []
  , makeService "tolteces"       4375    "udp" []
  , makeService "bip"            4376    "tcp" []
  , makeService "bip"            4376    "udp" []
  , makeService "cp-spxsvr"      4377    "tcp" []
  , makeService "cp-spxsvr"      4377    "udp" []
  , makeService "cp-spxdpy"      4378    "tcp" []
  , makeService "cp-spxdpy"      4378    "udp" []
  , makeService "ctdb"           4379    "tcp" []
  , makeService "ctdb"           4379    "udp" []
  , makeService "xandros-cms"    4389    "tcp" []
  , makeService "xandros-cms"    4389    "udp" []
  , makeService "wiegand"        4390    "tcp" []
  , makeService "wiegand"        4390    "udp" []
  , makeService "apwi-imserver"  4391    "tcp" []
  , makeService "apwi-rxserver"  4392    "tcp" []
  , makeService "apwi-rxspooler" 4393    "tcp" []
  , makeService "apwi-disc"      4394    "udp" []
  , makeService "omnivisionesx"  4395    "tcp" []
  , makeService "omnivisionesx"  4395    "udp" []
  , makeService "fly"            4396    "tcp" []
  , makeService "ds-srv"         4400    "tcp" []
  , makeService "ds-srv"         4400    "udp" []
  , makeService "ds-srvr"        4401    "tcp" []
  , makeService "ds-srvr"        4401    "udp" []
  , makeService "ds-clnt"        4402    "tcp" []
  , makeService "ds-clnt"        4402    "udp" []
  , makeService "ds-user"        4403    "tcp" []
  , makeService "ds-user"        4403    "udp" []
  , makeService "ds-admin"       4404    "tcp" []
  , makeService "ds-admin"       4404    "udp" []
  , makeService "ds-mail"        4405    "tcp" []
  , makeService "ds-mail"        4405    "udp" []
  , makeService "ds-slp"         4406    "tcp" []
  , makeService "ds-slp"         4406    "udp" []
  , makeService "nacagent"       4407    "tcp" []
  , makeService "slscc"          4408    "tcp" []
  , makeService "netcabinet-com" 4409    "tcp" []
  , makeService "itwo-server"    4410    "tcp" []
  , makeService "found"          4411    "tcp" []
  , makeService "netrockey6"     4425    "tcp" []
  , makeService "netrockey6"     4425    "udp" []
  , makeService "beacon-port-2"  4426    "tcp" []
  , makeService "beacon-port-2"  4426    "udp" []
  , makeService "drizzle"        4427    "tcp" []
  , makeService "omviserver"     4428    "tcp" []
  , makeService "omviagent"      4429    "tcp" []
  , makeService "sqlserver"      4430    "tcp" []
  , makeService "rsqlserver"     4430    "udp" []
  , makeService "wspipe"         4431    "tcp" []
  , makeService "l-acoustics"    4432    "tcp" []
  , makeService "l-acoustics"    4432    "udp" []
  , makeService "vop"            4433    "tcp" []
  , makeService "netblox"        4441    "udp" []
  , makeService "saris"          4442    "tcp" []
  , makeService "saris"          4442    "udp" []
  , makeService "pharos"         4443    "tcp" []
  , makeService "pharos"         4443    "udp" []
  , makeService "upnotifyp"      4445    "tcp" []
  , makeService "upnotifyp"      4445    "udp" []
  , makeService "n1-fwp"         4446    "tcp" []
  , makeService "n1-fwp"         4446    "udp" []
  , makeService "n1-rmgmt"       4447    "tcp" []
  , makeService "n1-rmgmt"       4447    "udp" []
  , makeService "asc-slmd"       4448    "tcp" []
  , makeService "asc-slmd"       4448    "udp" []
  , makeService "privatewire"    4449    "tcp" []
  , makeService "privatewire"    4449    "udp" []
  , makeService "camp"           4450    "tcp" []
  , makeService "camp"           4450    "udp" []
  , makeService "ctisystemmsg"   4451    "tcp" []
  , makeService "ctisystemmsg"   4451    "udp" []
  , makeService "ctiprogramload" 4452    "tcp" []
  , makeService "ctiprogramload" 4452    "udp" []
  , makeService "nssalertmgr"    4453    "tcp" []
  , makeService "nssalertmgr"    4453    "udp" []
  , makeService "nssagentmgr"    4454    "tcp" []
  , makeService "nssagentmgr"    4454    "udp" []
  , makeService "prchat-user"    4455    "tcp" []
  , makeService "prchat-user"    4455    "udp" []
  , makeService "prchat-server"  4456    "tcp" []
  , makeService "prchat-server"  4456    "udp" []
  , makeService "prRegister"     4457    "tcp" []
  , makeService "prRegister"     4457    "udp" []
  , makeService "mcp"            4458    "tcp" []
  , makeService "mcp"            4458    "udp" []
  , makeService "hpssmgmt"       4484    "tcp" []
  , makeService "hpssmgmt"       4484    "udp" []
  , makeService "assyst-dr"      4485    "tcp" []
  , makeService "icms"           4486    "tcp" []
  , makeService "icms"           4486    "udp" []
  , makeService "prex-tcp"       4487    "tcp" []
  , makeService "awacs-ice"      4488    "tcp" []
  , makeService "awacs-ice"      4488    "udp" []
  , makeService "ipsec-nat-t"    4500    "tcp" []
  , makeService "ipsec-nat-t"    4500    "udp" []
  , makeService "armagetronad"   4534    "udp" []
  , makeService "ehs"            4535    "tcp" []
  , makeService "ehs"            4535    "udp" []
  , makeService "ehs-ssl"        4536    "tcp" []
  , makeService "ehs-ssl"        4536    "udp" []
  , makeService "wssauthsvc"     4537    "tcp" []
  , makeService "wssauthsvc"     4537    "udp" []
  , makeService "swx-gate"       4538    "tcp" []
  , makeService "swx-gate"       4538    "udp" []
  , makeService "worldscores"    4545    "tcp" []
  , makeService "worldscores"    4545    "udp" []
  , makeService "sf-lm"          4546    "tcp" []
  , makeService "sf-lm"          4546    "udp" []
  , makeService "lanner-lm"      4547    "tcp" []
  , makeService "lanner-lm"      4547    "udp" []
  , makeService "synchromesh"    4548    "tcp" []
  , makeService "synchromesh"    4548    "udp" []
  , makeService "aegate"         4549    "tcp" []
  , makeService "aegate"         4549    "udp" []
  , makeService "gds-adppiw-db"  4550    "tcp" []
  , makeService "gds-adppiw-db"  4550    "udp" []
  , makeService "ieee-mih"       4551    "tcp" []
  , makeService "ieee-mih"       4551    "udp" []
  , makeService "menandmice-mon" 4552    "tcp" []
  , makeService "menandmice-mon" 4552    "udp" []
  , makeService "icshostsvc"     4553    "tcp" []
  , makeService "msfrs"          4554    "tcp" []
  , makeService "msfrs"          4554    "udp" []
  , makeService "rsip"           4555    "tcp" []
  , makeService "rsip"           4555    "udp" []
  , makeService "dtn-bundle-tcp" 4556    "tcp" []
  , makeService "dtn-bundle-udp" 4556    "udp" []
  , makeService "mtcevrunqss"    4557    "udp" []
  , makeService "mtcevrunqman"   4558    "udp" []
  , makeService "hylafax"        4559    "udp" []
  , makeService "kwtc"           4566    "tcp" []
  , makeService "kwtc"           4566    "udp" []
  , makeService "tram"           4567    "tcp" []
  , makeService "tram"           4567    "udp" []
  , makeService "bmc-reporting"  4568    "tcp" []
  , makeService "bmc-reporting"  4568    "udp" []
  , makeService "iax"            4569    "tcp" []
  , makeService "iax"            4569    "udp" []
  , makeService "rid"            4590    "tcp" []
  , makeService "l3t-at-an"      4591    "tcp" []
  , makeService "l3t-at-an"      4591    "udp" []
  , makeService "hrpd-ith-at-an" 4592    "udp" []
  , makeService "ipt-anri-anri"  4593    "tcp" []
  , makeService "ipt-anri-anri"  4593    "udp" []
  , makeService "ias-session"    4594    "tcp" []
  , makeService "ias-session"    4594    "udp" []
  , makeService "ias-paging"     4595    "tcp" []
  , makeService "ias-paging"     4595    "udp" []
  , makeService "ias-neighbor"   4596    "tcp" []
  , makeService "ias-neighbor"   4596    "udp" []
  , makeService "a21-an-1xbs"    4597    "tcp" []
  , makeService "a21-an-1xbs"    4597    "udp" []
  , makeService "a16-an-an"      4598    "tcp" []
  , makeService "a16-an-an"      4598    "udp" []
  , makeService "a17-an-an"      4599    "tcp" []
  , makeService "a17-an-an"      4599    "udp" []
  , makeService "piranha1"       4600    "tcp" []
  , makeService "piranha1"       4600    "udp" []
  , makeService "piranha2"       4601    "tcp" []
  , makeService "piranha2"       4601    "udp" []
  , makeService "mtsserver"      4602    "tcp" []
  , makeService "menandmice-upg" 4603    "tcp" []
  , makeService "playsta2-app"   4658    "tcp" []
  , makeService "playsta2-app"   4658    "udp" []
  , makeService "playsta2-lob"   4659    "tcp" []
  , makeService "playsta2-lob"   4659    "udp" []
  , makeService "smaclmgr"       4660    "tcp" []
  , makeService "smaclmgr"       4660    "udp" []
  , makeService "kar2ouche"      4661    "tcp" []
  , makeService "kar2ouche"      4661    "udp" []
  , makeService "oms"            4662    "tcp" []
  , makeService "oms"            4662    "udp" []
  , makeService "noteit"         4663    "tcp" []
  , makeService "noteit"         4663    "udp" []
  , makeService "ems"            4664    "tcp" []
  , makeService "ems"            4664    "udp" []
  , makeService "contclientms"   4665    "tcp" []
  , makeService "contclientms"   4665    "udp" []
  , makeService "eportcomm"      4666    "tcp" []
  , makeService "eportcomm"      4666    "udp" []
  , makeService "mmacomm"        4667    "tcp" []
  , makeService "mmacomm"        4667    "udp" []
  , makeService "mmaeds"         4668    "tcp" []
  , makeService "mmaeds"         4668    "udp" []
  , makeService "eportcommdata"  4669    "tcp" []
  , makeService "eportcommdata"  4669    "udp" []
  , makeService "light"          4670    "tcp" []
  , makeService "light"          4670    "udp" []
  , makeService "acter"          4671    "tcp" []
  , makeService "acter"          4671    "udp" []
  , makeService "rfa"            4672    "tcp" []
  , makeService "rfa"            4672    "udp" []
  , makeService "cxws"           4673    "tcp" []
  , makeService "cxws"           4673    "udp" []
  , makeService "appiq-mgmt"     4674    "tcp" []
  , makeService "appiq-mgmt"     4674    "udp" []
  , makeService "dhct-status"    4675    "tcp" []
  , makeService "dhct-status"    4675    "udp" []
  , makeService "dhct-alerts"    4676    "tcp" []
  , makeService "dhct-alerts"    4676    "udp" []
  , makeService "bcs"            4677    "tcp" []
  , makeService "bcs"            4677    "udp" []
  , makeService "traversal"      4678    "tcp" []
  , makeService "traversal"      4678    "udp" []
  , makeService "mgesupervision" 4679    "tcp" []
  , makeService "mgesupervision" 4679    "udp" []
  , makeService "mgemanagement"  4680    "tcp" []
  , makeService "mgemanagement"  4680    "udp" []
  , makeService "parliant"       4681    "tcp" []
  , makeService "parliant"       4681    "udp" []
  , makeService "finisar"        4682    "tcp" []
  , makeService "finisar"        4682    "udp" []
  , makeService "spike"          4683    "tcp" []
  , makeService "spike"          4683    "udp" []
  , makeService "rfid-rp1"       4684    "tcp" []
  , makeService "rfid-rp1"       4684    "udp" []
  , makeService "autopac"        4685    "tcp" []
  , makeService "autopac"        4685    "udp" []
  , makeService "msp-os"         4686    "tcp" []
  , makeService "msp-os"         4686    "udp" []
  , makeService "nst"            4687    "tcp" []
  , makeService "nst"            4687    "udp" []
  , makeService "mobile-p2p"     4688    "tcp" []
  , makeService "mobile-p2p"     4688    "udp" []
  , makeService "altovacentral"  4689    "tcp" []
  , makeService "altovacentral"  4689    "udp" []
  , makeService "prelude"        4690    "tcp" []
  , makeService "prelude"        4690    "udp" []
  , makeService "mtn"            4691    "tcp" []
  , makeService "mtn"            4691    "udp" []
  , makeService "conspiracy"     4692    "tcp" []
  , makeService "conspiracy"     4692    "udp" []
  , makeService "netxms-agent"   4700    "tcp" []
  , makeService "netxms-agent"   4700    "udp" []
  , makeService "netxms-mgmt"    4701    "tcp" []
  , makeService "netxms-mgmt"    4701    "udp" []
  , makeService "netxms-sync"    4702    "tcp" []
  , makeService "netxms-sync"    4702    "udp" []
  , makeService "npqes-test"     4703    "tcp" []
  , makeService "assuria-ins"    4704    "tcp" []
  , makeService "pulseaudio"     4713    "tcp" []
  , makeService "truckstar"      4725    "tcp" []
  , makeService "truckstar"      4725    "udp" []
  , makeService "a26-fap-fgw"    4726    "udp" []
  , makeService "fcis"           4727    "tcp" []
  , makeService "fcis-disc"      4727    "udp" []
  , makeService "capmux"         4728    "tcp" []
  , makeService "capmux"         4728    "udp" []
  , makeService "gsmtap"         4729    "udp" []
  , makeService "gearman"        4730    "tcp" []
  , makeService "gearman"        4730    "udp" []
  , makeService "remcap"         4731    "tcp" []
  , makeService "ohmtrigger"     4732    "udp" []
  , makeService "resorcs"        4733    "tcp" []
  , makeService "ipdr-sp"        4737    "tcp" []
  , makeService "ipdr-sp"        4737    "udp" []
  , makeService "solera-lpn"     4738    "tcp" []
  , makeService "solera-lpn"     4738    "udp" []
  , makeService "ipfix"          4739    "tcp" []
  , makeService "ipfix"          4739    "udp" []
  , makeService "ipfixs"         4740    "tcp" []
  , makeService "ipfixs"         4740    "udp" []
  , makeService "lumimgrd"       4741    "tcp" []
  , makeService "lumimgrd"       4741    "udp" []
  , makeService "sicct"          4742    "tcp" []
  , makeService "sicct-sdp"      4742    "udp" []
  , makeService "openhpid"       4743    "tcp" []
  , makeService "openhpid"       4743    "udp" []
  , makeService "ifsp"           4744    "tcp" []
  , makeService "ifsp"           4744    "udp" []
  , makeService "fmp"            4745    "tcp" []
  , makeService "fmp"            4745    "udp" []
  , makeService "buschtrommel"   4747    "udp" []
  , makeService "profilemac"     4749    "tcp" []
  , makeService "profilemac"     4749    "udp" []
  , makeService "ssad"           4750    "tcp" []
  , makeService "ssad"           4750    "udp" []
  , makeService "spocp"          4751    "tcp" []
  , makeService "spocp"          4751    "udp" []
  , makeService "snap"           4752    "tcp" []
  , makeService "snap"           4752    "udp" []
  , makeService "simon"          4753    "tcp" []
  , makeService "simon-disc"     4753    "udp" []
  , makeService "bfd-multi-ctl"  4784    "tcp" []
  , makeService "bfd-multi-ctl"  4784    "udp" []
  , makeService "cncp"           4785    "udp" []
  , makeService "smart-install"  4786    "tcp" []
  , makeService "sia-ctrl-plane" 4787    "tcp" []
  , makeService "xmcp"           4788    "tcp" []
  , makeService "iims"           4800    "tcp" []
  , makeService "iims"           4800    "udp" []
  , makeService "iwec"           4801    "tcp" []
  , makeService "iwec"           4801    "udp" []
  , makeService "ilss"           4802    "tcp" []
  , makeService "ilss"           4802    "udp" []
  , makeService "notateit"       4803    "tcp" []
  , makeService "notateit-disc"  4803    "udp" []
  , makeService "aja-ntv4-disc"  4804    "udp" []
  , makeService "htcp"           4827    "tcp" []
  , makeService "htcp"           4827    "udp" []
  , makeService "varadero-0"     4837    "tcp" []
  , makeService "varadero-0"     4837    "udp" []
  , makeService "varadero-1"     4838    "tcp" []
  , makeService "varadero-1"     4838    "udp" []
  , makeService "varadero-2"     4839    "tcp" []
  , makeService "varadero-2"     4839    "udp" []
  , makeService "opcua-tcp"      4840    "tcp" []
  , makeService "opcua-udp"      4840    "udp" []
  , makeService "quosa"          4841    "tcp" []
  , makeService "quosa"          4841    "udp" []
  , makeService "gw-asv"         4842    "tcp" []
  , makeService "gw-asv"         4842    "udp" []
  , makeService "opcua-tls"      4843    "tcp" []
  , makeService "opcua-tls"      4843    "udp" []
  , makeService "gw-log"         4844    "tcp" []
  , makeService "gw-log"         4844    "udp" []
  , makeService "wcr-remlib"     4845    "tcp" []
  , makeService "wcr-remlib"     4845    "udp" []
  , makeService "contamac_icm"   4846    "tcp" ["contamac-icm"]
  , makeService "contamac_icm"   4846    "udp" ["contamac-icm"]
  , makeService "wfc"            4847    "tcp" []
  , makeService "wfc"            4847    "udp" []
  , makeService "appserv-http"   4848    "tcp" []
  , makeService "appserv-http"   4848    "udp" []
  , makeService "appserv-https"  4849    "tcp" []
  , makeService "appserv-https"  4849    "udp" []
  , makeService "sun-as-nodeagt" 4850    "tcp" []
  , makeService "sun-as-nodeagt" 4850    "udp" []
  , makeService "derby-repli"    4851    "tcp" []
  , makeService "derby-repli"    4851    "udp" []
  , makeService "unify-debug"    4867    "tcp" []
  , makeService "unify-debug"    4867    "udp" []
  , makeService "phrelay"        4868    "tcp" []
  , makeService "phrelay"        4868    "udp" []
  , makeService "phrelaydbg"     4869    "tcp" []
  , makeService "phrelaydbg"     4869    "udp" []
  , makeService "cc-tracking"    4870    "tcp" []
  , makeService "cc-tracking"    4870    "udp" []
  , makeService "wired"          4871    "tcp" []
  , makeService "wired"          4871    "udp" []
  , makeService "tritium-can"    4876    "tcp" []
  , makeService "tritium-can"    4876    "udp" []
  , makeService "lmcs"           4877    "tcp" []
  , makeService "lmcs"           4877    "udp" []
  , makeService "inst-discovery" 4878    "udp" []
  , makeService "wsdl-event"     4879    "tcp" []
  , makeService "hislip"         4880    "tcp" []
  , makeService "socp-t"         4881    "udp" []
  , makeService "socp-c"         4882    "udp" []
  , makeService "wmlserver"      4883    "tcp" []
  , makeService "hivestor"       4884    "tcp" []
  , makeService "hivestor"       4884    "udp" []
  , makeService "abbs"           4885    "tcp" []
  , makeService "abbs"           4885    "udp" []
  , makeService "lyskom"         4894    "tcp" []
  , makeService "lyskom"         4894    "udp" []
  , makeService "radmin-port"    4899    "tcp" []
  , makeService "radmin-port"    4899    "udp" []
  , makeService "hfcs"           4900    "tcp" []
  , makeService "hfcs"           4900    "udp" []
  , makeService "flr_agent"      4901    "tcp" ["flr-agent"]
  , makeService "magiccontrol"   4902    "tcp" []
  , makeService "lutap"          4912    "tcp" []
  , makeService "lutcp"          4913    "tcp" []
  , makeService "bones"          4914    "tcp" []
  , makeService "bones"          4914    "udp" []
  , makeService "frcs"           4915    "tcp" []
  , makeService "atsc-mh-ssc"    4937    "udp" []
  , makeService "eq-office-4940" 4940    "tcp" []
  , makeService "eq-office-4940" 4940    "udp" []
  , makeService "eq-office-4941" 4941    "tcp" []
  , makeService "eq-office-4941" 4941    "udp" []
  , makeService "eq-office-4942" 4942    "tcp" []
  , makeService "eq-office-4942" 4942    "udp" []
  , makeService "munin"          4949    "tcp" []
  , makeService "munin"          4949    "udp" []
  , makeService "sybasesrvmon"   4950    "tcp" []
  , makeService "sybasesrvmon"   4950    "udp" []
  , makeService "pwgwims"        4951    "tcp" []
  , makeService "pwgwims"        4951    "udp" []
  , makeService "sagxtsds"       4952    "tcp" []
  , makeService "sagxtsds"       4952    "udp" []
  , makeService "dbsyncarbiter"  4953    "tcp" []
  , makeService "ccss-qmm"       4969    "tcp" []
  , makeService "ccss-qmm"       4969    "udp" []
  , makeService "ccss-qsm"       4970    "tcp" []
  , makeService "ccss-qsm"       4970    "udp" []
  , makeService "webyast"        4984    "tcp" []
  , makeService "gerhcs"         4985    "tcp" []
  , makeService "mrip"           4986    "tcp" []
  , makeService "mrip"           4986    "udp" []
  , makeService "smar-se-port1"  4987    "tcp" []
  , makeService "smar-se-port1"  4987    "udp" []
  , makeService "smar-se-port2"  4988    "tcp" []
  , makeService "smar-se-port2"  4988    "udp" []
  , makeService "parallel"       4989    "tcp" []
  , makeService "parallel"       4989    "udp" []
  , makeService "busycal"        4990    "tcp" []
  , makeService "busycal"        4990    "udp" []
  , makeService "vrt"            4991    "tcp" []
  , makeService "vrt"            4991    "udp" []
  , makeService "hfcs-manager"   4999    "tcp" []
  , makeService "hfcs-manager"   4999    "udp" []
  , makeService "commplex-main"  5000    "tcp" []
  , makeService "commplex-main"  5000    "udp" []
  , makeService "commplex-link"  5001    "tcp" []
  , makeService "commplex-link"  5001    "udp" []
  , makeService "fmpro-internal" 5003    "tcp" []
  , makeService "fmpro-internal" 5003    "udp" []
  , makeService "avt-profile-1"  5004    "tcp" []
  , makeService "avt-profile-1"  5004    "udp" []
  , makeService "avt-profile-2"  5005    "tcp" []
  , makeService "avt-profile-2"  5005    "udp" []
  , makeService "wsm-server"     5006    "tcp" []
  , makeService "wsm-server"     5006    "udp" []
  , makeService "wsm-server-ssl" 5007    "tcp" []
  , makeService "wsm-server-ssl" 5007    "udp" []
  , makeService "synapsis-edge"  5008    "tcp" []
  , makeService "synapsis-edge"  5008    "udp" []
  , makeService "winfs"          5009    "tcp" []
  , makeService "winfs"          5009    "udp" []
  , makeService "telelpathstart" 5010    "tcp" []
  , makeService "telelpathstart" 5010    "udp" []
  , makeService "telelpathattack"5011    "tcp" []
  , makeService "telelpathattack"5011    "udp" []
  , makeService "nsp"            5012    "tcp" []
  , makeService "nsp"            5012    "udp" []
  , makeService "fmpro-v6"       5013    "tcp" []
  , makeService "fmpro-v6"       5013    "udp" []
  , makeService "onpsocket"      5014    "udp" []
  , makeService "fmwp"           5015    "tcp" []
  , makeService "zenginkyo-1"    5020    "tcp" []
  , makeService "zenginkyo-1"    5020    "udp" []
  , makeService "zenginkyo-2"    5021    "tcp" []
  , makeService "zenginkyo-2"    5021    "udp" []
  , makeService "mice"           5022    "tcp" []
  , makeService "mice"           5022    "udp" []
  , makeService "htuilsrv"       5023    "tcp" []
  , makeService "htuilsrv"       5023    "udp" []
  , makeService "scpi-telnet"    5024    "tcp" []
  , makeService "scpi-telnet"    5024    "udp" []
  , makeService "scpi-raw"       5025    "tcp" []
  , makeService "scpi-raw"       5025    "udp" []
  , makeService "strexec-d"      5026    "tcp" []
  , makeService "strexec-d"      5026    "udp" []
  , makeService "strexec-s"      5027    "tcp" []
  , makeService "strexec-s"      5027    "udp" []
  , makeService "qvr"            5028    "tcp" []
  , makeService "infobright"     5029    "tcp" []
  , makeService "infobright"     5029    "udp" []
  , makeService "surfpass"       5030    "tcp" []
  , makeService "surfpass"       5030    "udp" []
  , makeService "dmp"            5031    "udp" []
  , makeService "signacert-agent"5032    "tcp" []
  , makeService "asnaacceler8db" 5042    "tcp" []
  , makeService "asnaacceler8db" 5042    "udp" []
  , makeService "swxadmin"       5043    "tcp" []
  , makeService "swxadmin"       5043    "udp" []
  , makeService "lxi-evntsvc"    5044    "tcp" []
  , makeService "lxi-evntsvc"    5044    "udp" []
  , makeService "osp"            5045    "tcp" []
  , makeService "vpm-udp"        5046    "udp" []
  , makeService "iscape"         5047    "udp" []
  , makeService "texai"          5048    "tcp" []
  , makeService "ivocalize"      5049    "tcp" []
  , makeService "ivocalize"      5049    "udp" []
  , makeService "mmcc"           5050    "tcp" []
  , makeService "mmcc"           5050    "udp" []
  , makeService "ita-agent"      5051    "tcp" []
  , makeService "ita-agent"      5051    "udp" []
  , makeService "ita-manager"    5052    "tcp" []
  , makeService "ita-manager"    5052    "udp" []
  , makeService "rlm"            5053    "tcp" []
  , makeService "rlm-disc"       5053    "udp" []
  , makeService "rlm-admin"      5054    "tcp" []
  , makeService "unot"           5055    "tcp" []
  , makeService "unot"           5055    "udp" []
  , makeService "intecom-ps1"    5056    "tcp" []
  , makeService "intecom-ps1"    5056    "udp" []
  , makeService "intecom-ps2"    5057    "tcp" []
  , makeService "intecom-ps2"    5057    "udp" []
  , makeService "locus-disc"     5058    "udp" []
  , makeService "sds"            5059    "tcp" []
  , makeService "sds"            5059    "udp" []
  , makeService "sip"            5060    "tcp" []
  , makeService "sip"            5060    "udp" []
  , makeService "sip-tls"        5061    "tcp" []
  , makeService "sip-tls"        5061    "udp" []
  , makeService "na-localise"    5062    "tcp" []
  , makeService "na-localise"    5062    "udp" []
  , makeService "csrpc"          5063    "tcp" []
  , makeService "ca-1"           5064    "tcp" []
  , makeService "ca-1"           5064    "udp" []
  , makeService "ca-2"           5065    "tcp" []
  , makeService "ca-2"           5065    "udp" []
  , makeService "stanag-5066"    5066    "tcp" []
  , makeService "stanag-5066"    5066    "udp" []
  , makeService "authentx"       5067    "tcp" []
  , makeService "authentx"       5067    "udp" []
  , makeService "bitforestsrv"   5068    "tcp" []
  , makeService "i-net-2000-npr" 5069    "tcp" []
  , makeService "i-net-2000-npr" 5069    "udp" []
  , makeService "vtsas"          5070    "tcp" []
  , makeService "vtsas"          5070    "udp" []
  , makeService "powerschool"    5071    "tcp" []
  , makeService "powerschool"    5071    "udp" []
  , makeService "ayiya"          5072    "tcp" []
  , makeService "ayiya"          5072    "udp" []
  , makeService "tag-pm"         5073    "tcp" []
  , makeService "tag-pm"         5073    "udp" []
  , makeService "alesquery"      5074    "tcp" []
  , makeService "alesquery"      5074    "udp" []
  , makeService "pvaccess"       5075    "tcp" []
  , makeService "cp-spxrpts"     5079    "udp" []
  , makeService "onscreen"       5080    "tcp" []
  , makeService "onscreen"       5080    "udp" []
  , makeService "sdl-ets"        5081    "tcp" []
  , makeService "sdl-ets"        5081    "udp" []
  , makeService "qcp"            5082    "tcp" []
  , makeService "qcp"            5082    "udp" []
  , makeService "qfp"            5083    "tcp" []
  , makeService "qfp"            5083    "udp" []
  , makeService "llrp"           5084    "tcp" []
  , makeService "llrp"           5084    "udp" []
  , makeService "encrypted-llrp" 5085    "tcp" []
  , makeService "encrypted-llrp" 5085    "udp" []
  , makeService "aprigo-cs"      5086    "tcp" []
  , makeService "magpie"         5092    "udp" []
  , makeService "sentinel-lm"    5093    "tcp" []
  , makeService "sentinel-lm"    5093    "udp" []
  , makeService "hart-ip"        5094    "tcp" []
  , makeService "hart-ip"        5094    "udp" []
  , makeService "sentlm-srv2srv" 5099    "tcp" []
  , makeService "sentlm-srv2srv" 5099    "udp" []
  , makeService "socalia"        5100    "tcp" []
  , makeService "socalia"        5100    "udp" []
  , makeService "talarian-tcp"   5101    "tcp" []
  , makeService "talarian-udp"   5101    "udp" []
  , makeService "oms-nonsecure"  5102    "tcp" []
  , makeService "oms-nonsecure"  5102    "udp" []
  , makeService "actifio-c2c"    5103    "tcp" []
  , makeService "tinymessage"    5104    "udp" []
  , makeService "hughes-ap"      5105    "udp" []
  , makeService "taep-as-svc"    5111    "tcp" []
  , makeService "taep-as-svc"    5111    "udp" []
  , makeService "pm-cmdsvr"      5112    "tcp" []
  , makeService "pm-cmdsvr"      5112    "udp" []
  , makeService "ev-services"    5114    "tcp" []
  , makeService "autobuild"      5115    "tcp" []
  , makeService "emb-proj-cmd"   5116    "udp" []
  , makeService "gradecam"       5117    "tcp" []
  , makeService "barracuda-bbs"  5120    "tcp" []
  , makeService "barracuda-bbs"  5120    "udp" []
  , makeService "nbt-pc"         5133    "tcp" []
  , makeService "nbt-pc"         5133    "udp" []
  , makeService "ppactivation"   5134    "tcp" []
  , makeService "erp-scale"      5135    "tcp" []
  , makeService "minotaur-sa"    5136    "udp" []
  , makeService "ctsd"           5137    "tcp" []
  , makeService "ctsd"           5137    "udp" []
  , makeService "rmonitor_secure"5145    "tcp" ["rmonitor-secure"]
  , makeService "rmonitor_secure"5145    "udp" ["rmonitor-secure"]
  , makeService "social-alarm"   5146    "tcp" []
  , makeService "atmp"           5150    "tcp" []
  , makeService "atmp"           5150    "udp" []
  , makeService "esri_sde"       5151    "tcp" ["esri-sde"]
  , makeService "esri_sde"       5151    "udp" ["esri-sde"]
  , makeService "sde-discovery"  5152    "tcp" []
  , makeService "sde-discovery"  5152    "udp" []
  , makeService "toruxserver"    5153    "tcp" []
  , makeService "bzflag"         5154    "tcp" []
  , makeService "bzflag"         5154    "udp" []
  , makeService "asctrl-agent"   5155    "tcp" []
  , makeService "asctrl-agent"   5155    "udp" []
  , makeService "rugameonline"   5156    "tcp" []
  , makeService "mediat"         5157    "tcp" []
  , makeService "snmpssh"        5161    "tcp" []
  , makeService "snmpssh-trap"   5162    "tcp" []
  , makeService "sbackup"        5163    "tcp" []
  , makeService "vpa"            5164    "tcp" []
  , makeService "vpa-disc"       5164    "udp" []
  , makeService "ife_icorp"      5165    "tcp" ["ife-icorp"]
  , makeService "ife_icorp"      5165    "udp" ["ife-icorp"]
  , makeService "winpcs"         5166    "tcp" []
  , makeService "winpcs"         5166    "udp" []
  , makeService "scte104"        5167    "tcp" []
  , makeService "scte104"        5167    "udp" []
  , makeService "scte30"         5168    "tcp" []
  , makeService "scte30"         5168    "udp" []
  , makeService "aol"            5190    "tcp" []
  , makeService "aol"            5190    "udp" []
  , makeService "aol-1"          5191    "tcp" []
  , makeService "aol-1"          5191    "udp" []
  , makeService "aol-2"          5192    "tcp" []
  , makeService "aol-2"          5192    "udp" []
  , makeService "aol-3"          5193    "tcp" []
  , makeService "aol-3"          5193    "udp" []
  , makeService "cpscomm"        5194    "tcp" []
  , makeService "ampl-lic"       5195    "tcp" []
  , makeService "ampl-tableproxy"5196    "tcp" []
  , makeService "targus-getdata" 5200    "tcp" []
  , makeService "targus-getdata" 5200    "udp" []
  , makeService "targus-getdata1"5201    "tcp" []
  , makeService "targus-getdata1"5201    "udp" []
  , makeService "targus-getdata2"5202    "tcp" []
  , makeService "targus-getdata2"5202    "udp" []
  , makeService "targus-getdata3"5203    "tcp" []
  , makeService "targus-getdata3"5203    "udp" []
  , makeService "nomad"          5209    "tcp" []
  , makeService "3exmp"          5221    "tcp" []
  , makeService "xmpp-client"    5222    "tcp" []
  , makeService "hpvirtgrp"      5223    "tcp" []
  , makeService "hpvirtgrp"      5223    "udp" []
  , makeService "hpvirtctrl"     5224    "tcp" []
  , makeService "hpvirtctrl"     5224    "udp" []
  , makeService "hp-server"      5225    "tcp" []
  , makeService "hp-server"      5225    "udp" []
  , makeService "hp-status"      5226    "tcp" []
  , makeService "hp-status"      5226    "udp" []
  , makeService "perfd"          5227    "tcp" []
  , makeService "perfd"          5227    "udp" []
  , makeService "hpvroom"        5228    "tcp" []
  , makeService "enfs"           5233    "tcp" []
  , makeService "eenet"          5234    "tcp" []
  , makeService "eenet"          5234    "udp" []
  , makeService "galaxy-network" 5235    "tcp" []
  , makeService "galaxy-network" 5235    "udp" []
  , makeService "padl2sim"       5236    "tcp" []
  , makeService "padl2sim"       5236    "udp" []
  , makeService "mnet-discovery" 5237    "tcp" []
  , makeService "mnet-discovery" 5237    "udp" []
  , makeService "downtools"      5245    "tcp" []
  , makeService "downtools-disc" 5245    "udp" []
  , makeService "capwap-control" 5246    "udp" []
  , makeService "capwap-data"    5247    "udp" []
  , makeService "caacws"         5248    "tcp" []
  , makeService "caacws"         5248    "udp" []
  , makeService "caaclang2"      5249    "tcp" []
  , makeService "caaclang2"      5249    "udp" []
  , makeService "soagateway"     5250    "tcp" []
  , makeService "soagateway"     5250    "udp" []
  , makeService "caevms"         5251    "tcp" []
  , makeService "caevms"         5251    "udp" []
  , makeService "movaz-ssc"      5252    "tcp" []
  , makeService "movaz-ssc"      5252    "udp" []
  , makeService "kpdp"           5253    "tcp" []
  , makeService "3com-njack-1"   5264    "tcp" []
  , makeService "3com-njack-1"   5264    "udp" []
  , makeService "3com-njack-2"   5265    "tcp" []
  , makeService "3com-njack-2"   5265    "udp" []
  , makeService "xmpp-server"    5269    "tcp" []
  , makeService "cartographerxmp"5270    "tcp" []
  , makeService "cartographerxmp"5270    "udp" []
  , makeService "cuelink"        5271    "tcp" []
  , makeService "cuelink-disc"   5271    "udp" []
  , makeService "pk"             5272    "tcp" []
  , makeService "pk"             5272    "udp" []
  , makeService "xmpp-bosh"      5280    "tcp" []
  , makeService "undo-lm"        5281    "tcp" []
  , makeService "transmit-port"  5282    "tcp" []
  , makeService "transmit-port"  5282    "udp" []
  , makeService "presence"       5298    "tcp" []
  , makeService "presence"       5298    "udp" []
  , makeService "nlg-data"       5299    "tcp" []
  , makeService "nlg-data"       5299    "udp" []
  , makeService "hacl-hb"        5300    "tcp" []
  , makeService "hacl-hb"        5300    "udp" []
  , makeService "hacl-gs"        5301    "tcp" []
  , makeService "hacl-gs"        5301    "udp" []
  , makeService "hacl-cfg"       5302    "tcp" []
  , makeService "hacl-cfg"       5302    "udp" []
  , makeService "hacl-probe"     5303    "tcp" []
  , makeService "hacl-probe"     5303    "udp" []
  , makeService "hacl-local"     5304    "tcp" []
  , makeService "hacl-local"     5304    "udp" []
  , makeService "hacl-test"      5305    "tcp" []
  , makeService "hacl-test"      5305    "udp" []
  , makeService "sun-mc-grp"     5306    "tcp" []
  , makeService "sun-mc-grp"     5306    "udp" []
  , makeService "sco-aip"        5307    "tcp" []
  , makeService "sco-aip"        5307    "udp" []
  , makeService "jprinter"       5309    "tcp" []
  , makeService "jprinter"       5309    "udp" []
  , makeService "outlaws"        5310    "tcp" []
  , makeService "outlaws"        5310    "udp" []
  , makeService "permabit-cs"    5312    "tcp" []
  , makeService "permabit-cs"    5312    "udp" []
  , makeService "rrdp"           5313    "tcp" []
  , makeService "rrdp"           5313    "udp" []
  , makeService "opalis-rbt-ipc" 5314    "tcp" []
  , makeService "opalis-rbt-ipc" 5314    "udp" []
  , makeService "hacl-poll"      5315    "tcp" []
  , makeService "hacl-poll"      5315    "udp" []
  , makeService "hpbladems"      5316    "tcp" []
  , makeService "hpdevms"        5317    "tcp" []
  , makeService "pkix-cmc"       5318    "tcp" []
  , makeService "bsfserver-zn"   5320    "tcp" []
  , makeService "bsfsvr-zn-ssl"  5321    "tcp" []
  , makeService "kfserver"       5343    "tcp" []
  , makeService "kfserver"       5343    "udp" []
  , makeService "xkotodrcp"      5344    "tcp" []
  , makeService "xkotodrcp"      5344    "udp" []
  , makeService "stuns"          5349    "tcp" ["turns"]
  , makeService "stuns"          5349    "udp" ["turns"]
  , makeService "pcp-multicast"  5350    "tcp" []
  , makeService "pcp"            5350    "udp" []
  , makeService "dns-llq"        5352    "tcp" []
  , makeService "dns-llq"        5352    "udp" []
  , makeService "mdns"           5353    "tcp" []
  , makeService "mdns"           5353    "udp" []
  , makeService "mdnsresponder"  5354    "tcp" ["noclog"]
  , makeService "mdnsresponder"  5354    "udp" ["noclog"]
  , makeService "ms-smlbiz"      5356    "tcp" []
  , makeService "ms-smlbiz"      5356    "udp" []
  , makeService "wsdapi"         5357    "tcp" []
  , makeService "wsdapi"         5357    "udp" []
  , makeService "wsdapi-s"       5358    "tcp" []
  , makeService "wsdapi-s"       5358    "udp" []
  , makeService "ms-alerter"     5359    "tcp" []
  , makeService "ms-alerter"     5359    "udp" []
  , makeService "ms-sideshow"    5360    "tcp" []
  , makeService "ms-sideshow"    5360    "udp" []
  , makeService "ms-s-sideshow"  5361    "tcp" []
  , makeService "ms-s-sideshow"  5361    "udp" []
  , makeService "serverwsd2"     5362    "tcp" []
  , makeService "serverwsd2"     5362    "udp" []
  , makeService "net-projection" 5363    "tcp" []
  , makeService "net-projection" 5363    "udp" []
  , makeService "stresstester"   5397    "tcp" []
  , makeService "stresstester"   5397    "udp" []
  , makeService "elektron-admin" 5398    "tcp" []
  , makeService "elektron-admin" 5398    "udp" []
  , makeService "securitychase"  5399    "tcp" []
  , makeService "securitychase"  5399    "udp" []
  , makeService "excerpt"        5400    "tcp" []
  , makeService "excerpt"        5400    "udp" []
  , makeService "excerpts"       5401    "tcp" []
  , makeService "excerpts"       5401    "udp" []
  , makeService "hpoms-ci-lstn"  5403    "tcp" []
  , makeService "hpoms-ci-lstn"  5403    "udp" []
  , makeService "hpoms-dps-lstn" 5404    "tcp" []
  , makeService "hpoms-dps-lstn" 5404    "udp" []
  , makeService "netsupport"     5405    "tcp" []
  , makeService "netsupport"     5405    "udp" []
  , makeService "systemics-sox"  5406    "tcp" []
  , makeService "systemics-sox"  5406    "udp" []
  , makeService "foresyte-clear" 5407    "tcp" []
  , makeService "foresyte-clear" 5407    "udp" []
  , makeService "foresyte-sec"   5408    "tcp" []
  , makeService "foresyte-sec"   5408    "udp" []
  , makeService "salient-dtasrv" 5409    "tcp" []
  , makeService "salient-dtasrv" 5409    "udp" []
  , makeService "salient-usrmgr" 5410    "tcp" []
  , makeService "salient-usrmgr" 5410    "udp" []
  , makeService "actnet"         5411    "tcp" []
  , makeService "actnet"         5411    "udp" []
  , makeService "continuus"      5412    "tcp" []
  , makeService "continuus"      5412    "udp" []
  , makeService "wwiotalk"       5413    "tcp" []
  , makeService "wwiotalk"       5413    "udp" []
  , makeService "statusd"        5414    "tcp" []
  , makeService "statusd"        5414    "udp" []
  , makeService "ns-server"      5415    "tcp" []
  , makeService "ns-server"      5415    "udp" []
  , makeService "sns-gateway"    5416    "tcp" []
  , makeService "sns-gateway"    5416    "udp" []
  , makeService "sns-agent"      5417    "tcp" []
  , makeService "sns-agent"      5417    "udp" []
  , makeService "mcntp"          5418    "tcp" []
  , makeService "mcntp"          5418    "udp" []
  , makeService "dj-ice"         5419    "tcp" []
  , makeService "dj-ice"         5419    "udp" []
  , makeService "cylink-c"       5420    "tcp" []
  , makeService "cylink-c"       5420    "udp" []
  , makeService "netsupport2"    5421    "tcp" []
  , makeService "netsupport2"    5421    "udp" []
  , makeService "salient-mux"    5422    "tcp" []
  , makeService "salient-mux"    5422    "udp" []
  , makeService "virtualuser"    5423    "tcp" []
  , makeService "virtualuser"    5423    "udp" []
  , makeService "beyond-remote"  5424    "tcp" []
  , makeService "beyond-remote"  5424    "udp" []
  , makeService "br-channel"     5425    "tcp" []
  , makeService "br-channel"     5425    "udp" []
  , makeService "devbasic"       5426    "tcp" []
  , makeService "devbasic"       5426    "udp" []
  , makeService "sco-peer-tta"   5427    "tcp" []
  , makeService "sco-peer-tta"   5427    "udp" []
  , makeService "telaconsole"    5428    "tcp" []
  , makeService "telaconsole"    5428    "udp" []
  , makeService "base"           5429    "tcp" []
  , makeService "base"           5429    "udp" []
  , makeService "radec-corp"     5430    "tcp" []
  , makeService "radec-corp"     5430    "udp" []
  , makeService "park-agent"     5431    "tcp" []
  , makeService "park-agent"     5431    "udp" []
  , makeService "pyrrho"         5433    "tcp" []
  , makeService "pyrrho"         5433    "udp" []
  , makeService "sgi-arrayd"     5434    "tcp" []
  , makeService "sgi-arrayd"     5434    "udp" []
  , makeService "sceanics"       5435    "tcp" []
  , makeService "sceanics"       5435    "udp" []
  , makeService "pmip6-cntl"     5436    "udp" []
  , makeService "pmip6-data"     5437    "udp" []
  , makeService "spss"           5443    "tcp" []
  , makeService "spss"           5443    "udp" []
  , makeService "smbdirect"      5445    "tcp" []
  , makeService "surebox"        5453    "tcp" []
  , makeService "surebox"        5453    "udp" []
  , makeService "apc-5454"       5454    "tcp" []
  , makeService "apc-5454"       5454    "udp" []
  , makeService "apc-5455"       5455    "tcp" []
  , makeService "apc-5455"       5455    "udp" []
  , makeService "apc-5456"       5456    "tcp" []
  , makeService "apc-5456"       5456    "udp" []
  , makeService "silkmeter"      5461    "tcp" []
  , makeService "silkmeter"      5461    "udp" []
  , makeService "ttl-publisher"  5462    "tcp" []
  , makeService "ttl-publisher"  5462    "udp" []
  , makeService "ttlpriceproxy"  5463    "tcp" []
  , makeService "ttlpriceproxy"  5463    "udp" []
  , makeService "quailnet"       5464    "tcp" []
  , makeService "quailnet"       5464    "udp" []
  , makeService "netops-broker"  5465    "tcp" []
  , makeService "netops-broker"  5465    "udp" []
  , makeService "fcp-addr-srvr1" 5500    "tcp" []
  , makeService "fcp-addr-srvr1" 5500    "udp" []
  , makeService "fcp-addr-srvr2" 5501    "tcp" []
  , makeService "fcp-addr-srvr2" 5501    "udp" []
  , makeService "fcp-srvr-inst1" 5502    "tcp" []
  , makeService "fcp-srvr-inst1" 5502    "udp" []
  , makeService "fcp-srvr-inst2" 5503    "tcp" []
  , makeService "fcp-srvr-inst2" 5503    "udp" []
  , makeService "fcp-cics-gw1"   5504    "tcp" []
  , makeService "fcp-cics-gw1"   5504    "udp" []
  , makeService "checkoutdb"     5505    "tcp" []
  , makeService "checkoutdb"     5505    "udp" []
  , makeService "amc"            5506    "tcp" []
  , makeService "amc"            5506    "udp" []
  , makeService "sgi-eventmond"  5553    "tcp" []
  , makeService "sgi-eventmond"  5553    "udp" []
  , makeService "sgi-esphttp"    5554    "tcp" []
  , makeService "sgi-esphttp"    5554    "udp" []
  , makeService "personal-agent" 5555    "tcp" []
  , makeService "personal-agent" 5555    "udp" []
  , makeService "freeciv"        5556    "tcp" []
  , makeService "freeciv"        5556    "udp" []
  , makeService "farenet"        5557    "tcp" []
  , makeService "westec-connect" 5566    "tcp" []
  , makeService "enc-eps-mc-sec" 5567    "tcp" []
  , makeService "enc-eps-mc-sec" 5567    "udp" []
  , makeService "sdt"            5568    "tcp" []
  , makeService "sdt"            5568    "udp" []
  , makeService "rdmnet-ctrl"    5569    "tcp" []
  , makeService "rdmnet-device"  5569    "udp" []
  , makeService "sdmmp"          5573    "tcp" []
  , makeService "sdmmp"          5573    "udp" []
  , makeService "lsi-bobcat"     5574    "tcp" []
  , makeService "ora-oap"        5575    "tcp" []
  , makeService "fdtracks"       5579    "tcp" []
  , makeService "tmosms0"        5580    "tcp" []
  , makeService "tmosms0"        5580    "udp" []
  , makeService "tmosms1"        5581    "tcp" []
  , makeService "tmosms1"        5581    "udp" []
  , makeService "fac-restore"    5582    "tcp" []
  , makeService "fac-restore"    5582    "udp" []
  , makeService "tmo-icon-sync"  5583    "tcp" []
  , makeService "tmo-icon-sync"  5583    "udp" []
  , makeService "bis-web"        5584    "tcp" []
  , makeService "bis-web"        5584    "udp" []
  , makeService "bis-sync"       5585    "tcp" []
  , makeService "bis-sync"       5585    "udp" []
  , makeService "ininmessaging"  5597    "tcp" []
  , makeService "ininmessaging"  5597    "udp" []
  , makeService "mctfeed"        5598    "tcp" []
  , makeService "mctfeed"        5598    "udp" []
  , makeService "esinstall"      5599    "tcp" []
  , makeService "esinstall"      5599    "udp" []
  , makeService "esmmanager"     5600    "tcp" []
  , makeService "esmmanager"     5600    "udp" []
  , makeService "esmagent"       5601    "tcp" []
  , makeService "esmagent"       5601    "udp" []
  , makeService "a1-msc"         5602    "tcp" []
  , makeService "a1-msc"         5602    "udp" []
  , makeService "a1-bs"          5603    "tcp" []
  , makeService "a1-bs"          5603    "udp" []
  , makeService "a3-sdunode"     5604    "tcp" []
  , makeService "a3-sdunode"     5604    "udp" []
  , makeService "a4-sdunode"     5605    "tcp" []
  , makeService "a4-sdunode"     5605    "udp" []
  , makeService "ninaf"          5627    "tcp" []
  , makeService "ninaf"          5627    "udp" []
  , makeService "htrust"         5628    "tcp" []
  , makeService "htrust"         5628    "udp" []
  , makeService "symantec-sfdb"  5629    "tcp" []
  , makeService "symantec-sfdb"  5629    "udp" []
  , makeService "precise-comm"   5630    "tcp" []
  , makeService "precise-comm"   5630    "udp" []
  , makeService "pcanywheredata" 5631    "tcp" []
  , makeService "pcanywheredata" 5631    "udp" []
  , makeService "pcanywherestat" 5632    "tcp" []
  , makeService "pcanywherestat" 5632    "udp" []
  , makeService "beorl"          5633    "tcp" []
  , makeService "beorl"          5633    "udp" []
  , makeService "xprtld"         5634    "tcp" []
  , makeService "xprtld"         5634    "udp" []
  , makeService "sfmsso"         5635    "tcp" []
  , makeService "sfm-db-server"  5636    "tcp" []
  , makeService "cssc"           5637    "tcp" []
  , makeService "flcrs"          5638    "tcp" []
  , makeService "ics"            5639    "tcp" []
  , makeService "vfmobile"       5646    "tcp" []
  , makeService "filemq"         5670    "tcp" []
  , makeService "zre-disc"       5670    "udp" []
  , makeService "amqps"          5671    "tcp" []
  , makeService "amqps"          5671    "udp" []
  , makeService "amqp"           5672    "tcp" []
  , makeService "amqp"           5672    "udp" []
  , makeService "jms"            5673    "tcp" []
  , makeService "jms"            5673    "udp" []
  , makeService "hyperscsi-port" 5674    "tcp" []
  , makeService "hyperscsi-port" 5674    "udp" []
  , makeService "v5ua"           5675    "tcp" []
  , makeService "v5ua"           5675    "udp" []
  , makeService "raadmin"        5676    "tcp" []
  , makeService "raadmin"        5676    "udp" []
  , makeService "questdb2-lnchr" 5677    "tcp" []
  , makeService "questdb2-lnchr" 5677    "udp" []
  , makeService "rrac"           5678    "tcp" []
  , makeService "rrac"           5678    "udp" []
  , makeService "dccm"           5679    "tcp" []
  , makeService "dccm"           5679    "udp" []
  , makeService "auriga-router"  5680    "udp" []
  , makeService "ncxcp"          5681    "tcp" []
  , makeService "ncxcp"          5681    "udp" []
  , makeService "brightcore"     5682    "udp" []
  , makeService "coap"           5683    "udp" []
  , makeService "ggz"            5688    "tcp" []
  , makeService "ggz"            5688    "udp" []
  , makeService "qmvideo"        5689    "tcp" []
  , makeService "qmvideo"        5689    "udp" []
  , makeService "rbsystem"       5693    "tcp" []
  , makeService "kmip"           5696    "tcp" []
  , makeService "proshareaudio"  5713    "tcp" []
  , makeService "proshareaudio"  5713    "udp" []
  , makeService "prosharevideo"  5714    "tcp" []
  , makeService "prosharevideo"  5714    "udp" []
  , makeService "prosharedata"   5715    "tcp" []
  , makeService "prosharedata"   5715    "udp" []
  , makeService "prosharerequest"5716    "tcp" []
  , makeService "prosharerequest"5716    "udp" []
  , makeService "prosharenotify" 5717    "tcp" []
  , makeService "prosharenotify" 5717    "udp" []
  , makeService "dpm"            5718    "tcp" []
  , makeService "dpm"            5718    "udp" []
  , makeService "dpm-agent"      5719    "tcp" []
  , makeService "dpm-agent"      5719    "udp" []
  , makeService "ms-licensing"   5720    "tcp" []
  , makeService "ms-licensing"   5720    "udp" []
  , makeService "dtpt"           5721    "tcp" []
  , makeService "dtpt"           5721    "udp" []
  , makeService "msdfsr"         5722    "tcp" []
  , makeService "msdfsr"         5722    "udp" []
  , makeService "omhs"           5723    "tcp" []
  , makeService "omhs"           5723    "udp" []
  , makeService "omsdk"          5724    "tcp" []
  , makeService "omsdk"          5724    "udp" []
  , makeService "ms-ilm"         5725    "tcp" []
  , makeService "ms-ilm-sts"     5726    "tcp" []
  , makeService "asgenf"         5727    "tcp" []
  , makeService "io-dist-data"   5728    "tcp" []
  , makeService "io-dist-group"  5728    "udp" []
  , makeService "openmail"       5729    "tcp" []
  , makeService "openmail"       5729    "udp" []
  , makeService "unieng"         5730    "tcp" []
  , makeService "unieng"         5730    "udp" []
  , makeService "ida-discover1"  5741    "tcp" []
  , makeService "ida-discover1"  5741    "udp" []
  , makeService "ida-discover2"  5742    "tcp" []
  , makeService "ida-discover2"  5742    "udp" []
  , makeService "watchdoc-pod"   5743    "tcp" []
  , makeService "watchdoc-pod"   5743    "udp" []
  , makeService "watchdoc"       5744    "tcp" []
  , makeService "watchdoc"       5744    "udp" []
  , makeService "fcopy-server"   5745    "tcp" []
  , makeService "fcopy-server"   5745    "udp" []
  , makeService "fcopys-server"  5746    "tcp" []
  , makeService "fcopys-server"  5746    "udp" []
  , makeService "tunatic"        5747    "tcp" []
  , makeService "tunatic"        5747    "udp" []
  , makeService "tunalyzer"      5748    "tcp" []
  , makeService "tunalyzer"      5748    "udp" []
  , makeService "rscd"           5750    "tcp" []
  , makeService "rscd"           5750    "udp" []
  , makeService "openmailg"      5755    "tcp" []
  , makeService "openmailg"      5755    "udp" []
  , makeService "x500ms"         5757    "tcp" []
  , makeService "x500ms"         5757    "udp" []
  , makeService "openmailns"     5766    "tcp" []
  , makeService "openmailns"     5766    "udp" []
  , makeService "s-openmail"     5767    "tcp" []
  , makeService "s-openmail"     5767    "udp" []
  , makeService "openmailpxy"    5768    "tcp" []
  , makeService "openmailpxy"    5768    "udp" []
  , makeService "spramsca"       5769    "tcp" []
  , makeService "spramsca"       5769    "udp" []
  , makeService "spramsd"        5770    "tcp" []
  , makeService "spramsd"        5770    "udp" []
  , makeService "netagent"       5771    "tcp" []
  , makeService "netagent"       5771    "udp" []
  , makeService "dali-port"      5777    "tcp" []
  , makeService "dali-port"      5777    "udp" []
  , makeService "vts-rpc"        5780    "tcp" []
  , makeService "3par-evts"      5781    "tcp" []
  , makeService "3par-evts"      5781    "udp" []
  , makeService "3par-mgmt"      5782    "tcp" []
  , makeService "3par-mgmt"      5782    "udp" []
  , makeService "3par-mgmt-ssl"  5783    "tcp" []
  , makeService "3par-mgmt-ssl"  5783    "udp" []
  , makeService "ibar"           5784    "udp" []
  , makeService "3par-rcopy"     5785    "tcp" []
  , makeService "3par-rcopy"     5785    "udp" []
  , makeService "cisco-redu"     5786    "udp" []
  , makeService "waascluster"    5787    "udp" []
  , makeService "xtreamx"        5793    "tcp" []
  , makeService "xtreamx"        5793    "udp" []
  , makeService "spdp"           5794    "udp" []
  , makeService "icmpd"          5813    "tcp" []
  , makeService "icmpd"          5813    "udp" []
  , makeService "spt-automation" 5814    "tcp" []
  , makeService "spt-automation" 5814    "udp" []
  , makeService "reversion"      5842    "tcp" []
  , makeService "wherehoo"       5859    "tcp" []
  , makeService "wherehoo"       5859    "udp" []
  , makeService "ppsuitemsg"     5863    "tcp" []
  , makeService "ppsuitemsg"     5863    "udp" []
  , makeService "diameters"      5868    "tcp" []
  , makeService "jute"           5883    "tcp" []
  , makeService "rfb"            5900    "tcp" []
  , makeService "rfb"            5900    "udp" []
  , makeService "cm"             5910    "tcp" []
  , makeService "cm"             5910    "udp" []
  , makeService "cpdlc"          5911    "tcp" []
  , makeService "cpdlc"          5911    "udp" []
  , makeService "fis"            5912    "tcp" []
  , makeService "fis"            5912    "udp" []
  , makeService "ads-c"          5913    "tcp" []
  , makeService "ads-c"          5913    "udp" []
  , makeService "indy"           5963    "tcp" []
  , makeService "indy"           5963    "udp" []
  , makeService "mppolicy-v5"    5968    "tcp" []
  , makeService "mppolicy-v5"    5968    "udp" []
  , makeService "mppolicy-mgr"   5969    "tcp" []
  , makeService "mppolicy-mgr"   5969    "udp" []
  , makeService "couchdb"        5984    "tcp" []
  , makeService "couchdb"        5984    "udp" []
  , makeService "wsman"          5985    "tcp" []
  , makeService "wsman"          5985    "udp" []
  , makeService "wsmans"         5986    "tcp" []
  , makeService "wsmans"         5986    "udp" []
  , makeService "wbem-rmi"       5987    "tcp" []
  , makeService "wbem-rmi"       5987    "udp" []
  , makeService "wbem-http"      5988    "tcp" []
  , makeService "wbem-http"      5988    "udp" []
  , makeService "wbem-https"     5989    "tcp" []
  , makeService "wbem-https"     5989    "udp" []
  , makeService "wbem-exp-https" 5990    "tcp" []
  , makeService "wbem-exp-https" 5990    "udp" []
  , makeService "nuxsl"          5991    "tcp" []
  , makeService "nuxsl"          5991    "udp" []
  , makeService "consul-insight" 5992    "tcp" []
  , makeService "consul-insight" 5992    "udp" []
  , makeService "ndl-ahp-svc"    6064    "tcp" []
  , makeService "ndl-ahp-svc"    6064    "udp" []
  , makeService "winpharaoh"     6065    "tcp" []
  , makeService "winpharaoh"     6065    "udp" []
  , makeService "ewctsp"         6066    "tcp" []
  , makeService "ewctsp"         6066    "udp" []
  , makeService "gsmp-ancp"      6068    "tcp" []
  , makeService "trip"           6069    "tcp" []
  , makeService "trip"           6069    "udp" []
  , makeService "messageasap"    6070    "tcp" []
  , makeService "messageasap"    6070    "udp" []
  , makeService "ssdtp"          6071    "tcp" []
  , makeService "ssdtp"          6071    "udp" []
  , makeService "diagnose-proc"  6072    "tcp" []
  , makeService "diagnose-proc"  6072    "udp" []
  , makeService "directplay8"    6073    "tcp" []
  , makeService "directplay8"    6073    "udp" []
  , makeService "max"            6074    "tcp" []
  , makeService "max"            6074    "udp" []
  , makeService "dpm-acm"        6075    "tcp" []
  , makeService "msft-dpm-cert"  6076    "tcp" []
  , makeService "iconstructsrv"  6077    "tcp" []
  , makeService "p25cai"         6082    "udp" []
  , makeService "miami-bcast"    6083    "udp" []
  , makeService "reload-config"  6084    "tcp" []
  , makeService "konspire2b"     6085    "tcp" []
  , makeService "konspire2b"     6085    "udp" []
  , makeService "pdtp"           6086    "tcp" []
  , makeService "pdtp"           6086    "udp" []
  , makeService "ldss"           6087    "tcp" []
  , makeService "ldss"           6087    "udp" []
  , makeService "doglms"         6088    "tcp" []
  , makeService "doglms-notify"  6088    "udp" []
  , makeService "raxa-mgmt"      6099    "tcp" []
  , makeService "synchronet-db"  6100    "tcp" []
  , makeService "synchronet-db"  6100    "udp" []
  , makeService "synchronet-rtc" 6101    "tcp" []
  , makeService "synchronet-rtc" 6101    "udp" []
  , makeService "synchronet-upd" 6102    "tcp" []
  , makeService "synchronet-upd" 6102    "udp" []
  , makeService "rets"           6103    "tcp" []
  , makeService "rets"           6103    "udp" []
  , makeService "dbdb"           6104    "tcp" []
  , makeService "dbdb"           6104    "udp" []
  , makeService "primaserver"    6105    "tcp" []
  , makeService "primaserver"    6105    "udp" []
  , makeService "mpsserver"      6106    "tcp" []
  , makeService "mpsserver"      6106    "udp" []
  , makeService "etc-control"    6107    "tcp" []
  , makeService "etc-control"    6107    "udp" []
  , makeService "sercomm-scadmin"6108    "tcp" []
  , makeService "sercomm-scadmin"6108    "udp" []
  , makeService "globecast-id"   6109    "tcp" []
  , makeService "globecast-id"   6109    "udp" []
  , makeService "softcm"         6110    "tcp" []
  , makeService "softcm"         6110    "udp" []
  , makeService "spc"            6111    "tcp" []
  , makeService "spc"            6111    "udp" []
  , makeService "dtspcd"         6112    "tcp" []
  , makeService "dtspcd"         6112    "udp" []
  , makeService "dayliteserver"  6113    "tcp" []
  , makeService "wrspice"        6114    "tcp" []
  , makeService "xic"            6115    "tcp" []
  , makeService "xtlserv"        6116    "tcp" []
  , makeService "daylitetouch"   6117    "tcp" []
  , makeService "tipc"           6118    "udp" []
  , makeService "spdy"           6121    "tcp" []
  , makeService "bex-webadmin"   6122    "tcp" []
  , makeService "bex-webadmin"   6122    "udp" []
  , makeService "backup-express" 6123    "tcp" []
  , makeService "backup-express" 6123    "udp" []
  , makeService "pnbs"           6124    "tcp" []
  , makeService "pnbs"           6124    "udp" []
  , makeService "nbt-wol"        6133    "tcp" []
  , makeService "nbt-wol"        6133    "udp" []
  , makeService "pulsonixnls"    6140    "tcp" []
  , makeService "pulsonixnls"    6140    "udp" []
  , makeService "meta-corp"      6141    "tcp" []
  , makeService "meta-corp"      6141    "udp" []
  , makeService "aspentec-lm"    6142    "tcp" []
  , makeService "aspentec-lm"    6142    "udp" []
  , makeService "watershed-lm"   6143    "tcp" []
  , makeService "watershed-lm"   6143    "udp" []
  , makeService "statsci1-lm"    6144    "tcp" []
  , makeService "statsci1-lm"    6144    "udp" []
  , makeService "statsci2-lm"    6145    "tcp" []
  , makeService "statsci2-lm"    6145    "udp" []
  , makeService "lonewolf-lm"    6146    "tcp" []
  , makeService "lonewolf-lm"    6146    "udp" []
  , makeService "montage-lm"     6147    "tcp" []
  , makeService "montage-lm"     6147    "udp" []
  , makeService "tal-pod"        6149    "tcp" []
  , makeService "tal-pod"        6149    "udp" []
  , makeService "efb-aci"        6159    "tcp" []
  , makeService "ecmp"           6160    "tcp" []
  , makeService "ecmp-data"      6160    "udp" []
  , makeService "patrol-ism"     6161    "tcp" []
  , makeService "patrol-ism"     6161    "udp" []
  , makeService "patrol-coll"    6162    "tcp" []
  , makeService "patrol-coll"    6162    "udp" []
  , makeService "pscribe"        6163    "tcp" []
  , makeService "pscribe"        6163    "udp" []
  , makeService "lm-x"           6200    "tcp" []
  , makeService "lm-x"           6200    "udp" []
  , makeService "thermo-calc"    6201    "udp" []
  , makeService "radmind"        6222    "tcp" []
  , makeService "radmind"        6222    "udp" []
  , makeService "jeol-nsdtp-1"   6241    "tcp" []
  , makeService "jeol-nsddp-1"   6241    "udp" []
  , makeService "jeol-nsdtp-2"   6242    "tcp" []
  , makeService "jeol-nsddp-2"   6242    "udp" []
  , makeService "jeol-nsdtp-3"   6243    "tcp" []
  , makeService "jeol-nsddp-3"   6243    "udp" []
  , makeService "jeol-nsdtp-4"   6244    "tcp" []
  , makeService "jeol-nsddp-4"   6244    "udp" []
  , makeService "tl1-raw-ssl"    6251    "tcp" []
  , makeService "tl1-raw-ssl"    6251    "udp" []
  , makeService "tl1-ssh"        6252    "tcp" []
  , makeService "tl1-ssh"        6252    "udp" []
  , makeService "crip"           6253    "tcp" []
  , makeService "crip"           6253    "udp" []
  , makeService "gld"            6267    "tcp" []
  , makeService "grid"           6268    "tcp" []
  , makeService "grid"           6268    "udp" []
  , makeService "grid-alt"       6269    "tcp" []
  , makeService "grid-alt"       6269    "udp" []
  , makeService "bmc-grx"        6300    "tcp" []
  , makeService "bmc-grx"        6300    "udp" []
  , makeService "bmc_ctd_ldap"   6301    "tcp" ["bmc-ctd-ldap"]
  , makeService "bmc_ctd_ldap"   6301    "udp" ["bmc-ctd-ldap"]
  , makeService "ufmp"           6306    "tcp" []
  , makeService "ufmp"           6306    "udp" []
  , makeService "scup"           6315    "tcp" []
  , makeService "scup-disc"      6315    "udp" []
  , makeService "abb-escp"       6316    "tcp" []
  , makeService "abb-escp"       6316    "udp" []
  , makeService "nav-data-cmd"   6317    "tcp" []
  , makeService "nav-data"       6317    "udp" []
  , makeService "repsvc"         6320    "tcp" []
  , makeService "repsvc"         6320    "udp" []
  , makeService "emp-server1"    6321    "tcp" []
  , makeService "emp-server1"    6321    "udp" []
  , makeService "emp-server2"    6322    "tcp" []
  , makeService "emp-server2"    6322    "udp" []
  , makeService "hrd-ncs"        6324    "tcp" []
  , makeService "hrd-ns-disc"    6324    "udp" []
  , makeService "dt-mgmtsvc"     6325    "tcp" []
  , makeService "dt-vra"         6326    "tcp" []
  , makeService "sflow"          6343    "tcp" []
  , makeService "sflow"          6343    "udp" []
  , makeService "gnutella-svc"   6346    "tcp" []
  , makeService "gnutella-svc"   6346    "udp" []
  , makeService "gnutella-rtr"   6347    "tcp" []
  , makeService "gnutella-rtr"   6347    "udp" []
  , makeService "adap"           6350    "tcp" []
  , makeService "adap"           6350    "udp" []
  , makeService "pmcs"           6355    "tcp" []
  , makeService "pmcs"           6355    "udp" []
  , makeService "metaedit-mu"    6360    "tcp" []
  , makeService "metaedit-mu"    6360    "udp" []
  , makeService "metaedit-se"    6370    "tcp" []
  , makeService "metaedit-se"    6370    "udp" []
  , makeService "metatude-mds"   6382    "tcp" []
  , makeService "metatude-mds"   6382    "udp" []
  , makeService "clariion-evr01" 6389    "tcp" []
  , makeService "clariion-evr01" 6389    "udp" []
  , makeService "metaedit-ws"    6390    "tcp" []
  , makeService "metaedit-ws"    6390    "udp" []
  , makeService "boe-cms"        6400    "tcp" []
  , makeService "boe-cms"        6400    "udp" []
  , makeService "boe-was"        6401    "tcp" []
  , makeService "boe-was"        6401    "udp" []
  , makeService "boe-eventsrv"   6402    "tcp" []
  , makeService "boe-eventsrv"   6402    "udp" []
  , makeService "boe-cachesvr"   6403    "tcp" []
  , makeService "boe-cachesvr"   6403    "udp" []
  , makeService "boe-filesvr"    6404    "tcp" []
  , makeService "boe-filesvr"    6404    "udp" []
  , makeService "boe-pagesvr"    6405    "tcp" []
  , makeService "boe-pagesvr"    6405    "udp" []
  , makeService "boe-processsvr" 6406    "tcp" []
  , makeService "boe-processsvr" 6406    "udp" []
  , makeService "boe-resssvr1"   6407    "tcp" []
  , makeService "boe-resssvr1"   6407    "udp" []
  , makeService "boe-resssvr2"   6408    "tcp" []
  , makeService "boe-resssvr2"   6408    "udp" []
  , makeService "boe-resssvr3"   6409    "tcp" []
  , makeService "boe-resssvr3"   6409    "udp" []
  , makeService "boe-resssvr4"   6410    "tcp" []
  , makeService "boe-resssvr4"   6410    "udp" []
  , makeService "faxcomservice"  6417    "tcp" []
  , makeService "faxcomservice"  6417    "udp" []
  , makeService "syserverremote" 6418    "tcp" []
  , makeService "svdrp"          6419    "tcp" []
  , makeService "nim-vdrshell"   6420    "tcp" []
  , makeService "nim-vdrshell"   6420    "udp" []
  , makeService "nim-wan"        6421    "tcp" []
  , makeService "nim-wan"        6421    "udp" []
  , makeService "pgbouncer"      6432    "tcp" []
  , makeService "sun-sr-https"   6443    "tcp" []
  , makeService "sun-sr-https"   6443    "udp" []
  , makeService "sge_qmaster"    6444    "tcp" ["sge-qmaster"]
  , makeService "sge_qmaster"    6444    "udp" ["sge-qmaster"]
  , makeService "sge_execd"      6445    "tcp" ["sge-execd"]
  , makeService "sge_execd"      6445    "udp" ["sge-execd"]
  , makeService "mysql-proxy"    6446    "tcp" []
  , makeService "mysql-proxy"    6446    "udp" []
  , makeService "skip-cert-recv" 6455    "tcp" []
  , makeService "skip-cert-recv" 6455    "udp" []
  , makeService "skip-cert-send" 6456    "tcp" []
  , makeService "skip-cert-send" 6456    "udp" []
  , makeService "lvision-lm"     6471    "tcp" []
  , makeService "lvision-lm"     6471    "udp" []
  , makeService "sun-sr-http"    6480    "tcp" []
  , makeService "sun-sr-http"    6480    "udp" []
  , makeService "servicetags"    6481    "tcp" []
  , makeService "servicetags"    6481    "udp" []
  , makeService "ldoms-mgmt"     6482    "tcp" []
  , makeService "ldoms-mgmt"     6482    "udp" []
  , makeService "SunVTS-RMI"     6483    "tcp" []
  , makeService "SunVTS-RMI"     6483    "udp" []
  , makeService "sun-sr-jms"     6484    "tcp" []
  , makeService "sun-sr-jms"     6484    "udp" []
  , makeService "sun-sr-iiop"    6485    "tcp" []
  , makeService "sun-sr-iiop"    6485    "udp" []
  , makeService "sun-sr-iiops"   6486    "tcp" []
  , makeService "sun-sr-iiops"   6486    "udp" []
  , makeService "sun-sr-iiop-aut"6487    "tcp" []
  , makeService "sun-sr-iiop-aut"6487    "udp" []
  , makeService "sun-sr-jmx"     6488    "tcp" []
  , makeService "sun-sr-jmx"     6488    "udp" []
  , makeService "sun-sr-admin"   6489    "tcp" []
  , makeService "sun-sr-admin"   6489    "udp" []
  , makeService "boks"           6500    "tcp" []
  , makeService "boks"           6500    "udp" []
  , makeService "boks_servc"     6501    "tcp" ["boks-servc"]
  , makeService "boks_servc"     6501    "udp" ["boks-servc"]
  , makeService "boks_servm"     6502    "tcp" ["boks-servm"]
  , makeService "boks_servm"     6502    "udp" ["boks-servm"]
  , makeService "boks_clntd"     6503    "tcp" ["boks-clntd"]
  , makeService "boks_clntd"     6503    "udp" ["boks-clntd"]
  , makeService "badm_priv"      6505    "tcp" ["badm-priv"]
  , makeService "badm_priv"      6505    "udp" ["badm-priv"]
  , makeService "badm_pub"       6506    "tcp" ["badm-pub"]
  , makeService "badm_pub"       6506    "udp" ["badm-pub"]
  , makeService "bdir_priv"      6507    "tcp" ["bdir-priv"]
  , makeService "bdir_priv"      6507    "udp" ["bdir-priv"]
  , makeService "bdir_pub"       6508    "tcp" ["bdir-pub"]
  , makeService "bdir_pub"       6508    "udp" ["bdir-pub"]
  , makeService "mgcs-mfp-port"  6509    "tcp" []
  , makeService "mgcs-mfp-port"  6509    "udp" []
  , makeService "mcer-port"      6510    "tcp" []
  , makeService "mcer-port"      6510    "udp" []
  , makeService "dccp-udp"       6511    "udp" []
  , makeService "netconf-tls"    6513    "tcp" []
  , makeService "syslog-tls"     6514    "tcp" []
  , makeService "syslog-tls"     6514    "udp" []
  , makeService "elipse-rec"     6515    "tcp" []
  , makeService "elipse-rec"     6515    "udp" []
  , makeService "lds-distrib"    6543    "tcp" []
  , makeService "lds-distrib"    6543    "udp" []
  , makeService "lds-dump"       6544    "tcp" []
  , makeService "lds-dump"       6544    "udp" []
  , makeService "apc-6547"       6547    "tcp" []
  , makeService "apc-6547"       6547    "udp" []
  , makeService "apc-6548"       6548    "tcp" []
  , makeService "apc-6548"       6548    "udp" []
  , makeService "apc-6549"       6549    "tcp" []
  , makeService "apc-6549"       6549    "udp" []
  , makeService "fg-sysupdate"   6550    "tcp" []
  , makeService "fg-sysupdate"   6550    "udp" []
  , makeService "sum"            6551    "tcp" []
  , makeService "sum"            6551    "udp" []
  , makeService "xdsxdm"         6558    "tcp" []
  , makeService "xdsxdm"         6558    "udp" []
  , makeService "sane-port"      6566    "tcp" []
  , makeService "sane-port"      6566    "udp" []
  , makeService "canit_store"    6568    "tcp" ["canit-store"]
  , makeService "rp-reputation"  6568    "udp" []
  , makeService "affiliate"      6579    "tcp" []
  , makeService "affiliate"      6579    "udp" []
  , makeService "parsec-master"  6580    "tcp" []
  , makeService "parsec-master"  6580    "udp" []
  , makeService "parsec-peer"    6581    "tcp" []
  , makeService "parsec-peer"    6581    "udp" []
  , makeService "parsec-game"    6582    "tcp" []
  , makeService "parsec-game"    6582    "udp" []
  , makeService "joaJewelSuite"  6583    "tcp" []
  , makeService "joaJewelSuite"  6583    "udp" []
  , makeService "mshvlm"         6600    "tcp" []
  , makeService "mstmg-sstp"     6601    "tcp" []
  , makeService "wsscomfrmwk"    6602    "tcp" []
  , makeService "odette-ftps"    6619    "tcp" []
  , makeService "odette-ftps"    6619    "udp" []
  , makeService "kftp-data"      6620    "tcp" []
  , makeService "kftp-data"      6620    "udp" []
  , makeService "kftp"           6621    "tcp" []
  , makeService "kftp"           6621    "udp" []
  , makeService "mcftp"          6622    "tcp" []
  , makeService "mcftp"          6622    "udp" []
  , makeService "ktelnet"        6623    "tcp" []
  , makeService "ktelnet"        6623    "udp" []
  , makeService "datascaler-db"  6624    "tcp" []
  , makeService "datascaler-ctl" 6625    "tcp" []
  , makeService "wago-service"   6626    "tcp" []
  , makeService "wago-service"   6626    "udp" []
  , makeService "nexgen"         6627    "tcp" []
  , makeService "nexgen"         6627    "udp" []
  , makeService "afesc-mc"       6628    "tcp" []
  , makeService "afesc-mc"       6628    "udp" []
  , makeService "mxodbc-connect" 6632    "tcp" []
  , makeService "cisco-vpath-tun"6633    "udp" []
  , makeService "pcs-sf-ui-man"  6655    "tcp" []
  , makeService "emgmsg"         6656    "tcp" []
  , makeService "palcom-disc"    6657    "udp" []
  , makeService "ircu"           6665    "tcp" []
  , makeService "ircu"           6665    "udp" []
  , makeService "ircu-2"         6666    "tcp" []
  , makeService "ircu-2"         6666    "udp" []
  , makeService "ircu-3"         6667    "tcp" ["ircd"]
  , makeService "ircu-3"         6667    "udp" ["ircd"]
  , makeService "ircu-4"         6668    "tcp" []
  , makeService "ircu-4"         6668    "udp" []
  , makeService "ircu-5"         6669    "tcp" []
  , makeService "ircu-5"         6669    "udp" []
  , makeService "vocaltec-gold"  6670    "tcp" []
  , makeService "vocaltec-gold"  6670    "udp" []
  , makeService "p4p-portal"     6671    "tcp" []
  , makeService "p4p-portal"     6671    "udp" []
  , makeService "vision_server"  6672    "tcp" ["vision-server"]
  , makeService "vision_server"  6672    "udp" ["vision-server"]
  , makeService "vision_elmd"    6673    "tcp" ["vision-elmd"]
  , makeService "vision_elmd"    6673    "udp" ["vision-elmd"]
  , makeService "vfbp"           6678    "tcp" []
  , makeService "vfbp-disc"      6678    "udp" []
  , makeService "osaut"          6679    "tcp" []
  , makeService "osaut"          6679    "udp" []
  , makeService "clever-ctrace"  6687    "tcp" []
  , makeService "clever-tcpip"   6688    "tcp" []
  , makeService "tsa"            6689    "tcp" []
  , makeService "tsa"            6689    "udp" []
  , makeService "babel"          6696    "udp" []
  , makeService "kti-icad-srvr"  6701    "tcp" []
  , makeService "kti-icad-srvr"  6701    "udp" []
  , makeService "e-design-net"   6702    "tcp" []
  , makeService "e-design-net"   6702    "udp" []
  , makeService "e-design-web"   6703    "tcp" []
  , makeService "e-design-web"   6703    "udp" []
  , makeService "ibprotocol"     6714    "tcp" []
  , makeService "ibprotocol"     6714    "udp" []
  , makeService "fibotrader-com" 6715    "tcp" []
  , makeService "fibotrader-com" 6715    "udp" []
  , makeService "bmc-perf-agent" 6767    "tcp" []
  , makeService "bmc-perf-agent" 6767    "udp" []
  , makeService "bmc-perf-mgrd"  6768    "tcp" []
  , makeService "bmc-perf-mgrd"  6768    "udp" []
  , makeService "adi-gxp-srvprt" 6769    "tcp" []
  , makeService "adi-gxp-srvprt" 6769    "udp" []
  , makeService "plysrv-http"    6770    "tcp" []
  , makeService "plysrv-http"    6770    "udp" []
  , makeService "plysrv-https"   6771    "tcp" []
  , makeService "plysrv-https"   6771    "udp" []
  , makeService "bfd-lag"        6784    "udp" []
  , makeService "dgpf-exchg"     6785    "tcp" []
  , makeService "dgpf-exchg"     6785    "udp" []
  , makeService "smc-jmx"        6786    "tcp" []
  , makeService "smc-jmx"        6786    "udp" []
  , makeService "smc-admin"      6787    "tcp" []
  , makeService "smc-admin"      6787    "udp" []
  , makeService "smc-http"       6788    "tcp" []
  , makeService "smc-http"       6788    "udp" []
  , makeService "smc-https"      6789    "tcp" []
  , makeService "smc-https"      6789    "udp" []
  , makeService "hnmp"           6790    "tcp" []
  , makeService "hnmp"           6790    "udp" []
  , makeService "hnm"            6791    "tcp" []
  , makeService "hnm"            6791    "udp" []
  , makeService "acnet"          6801    "tcp" []
  , makeService "acnet"          6801    "udp" []
  , makeService "pentbox-sim"    6817    "tcp" []
  , makeService "ambit-lm"       6831    "tcp" []
  , makeService "ambit-lm"       6831    "udp" []
  , makeService "netmo-default"  6841    "tcp" []
  , makeService "netmo-default"  6841    "udp" []
  , makeService "netmo-http"     6842    "tcp" []
  , makeService "netmo-http"     6842    "udp" []
  , makeService "iccrushmore"    6850    "tcp" []
  , makeService "iccrushmore"    6850    "udp" []
  , makeService "acctopus-cc"    6868    "tcp" []
  , makeService "acctopus-st"    6868    "udp" []
  , makeService "muse"           6888    "tcp" []
  , makeService "muse"           6888    "udp" []
  , makeService "jetstream"      6901    "tcp" []
  , makeService "ethoscan"       6935    "tcp" []
  , makeService "ethoscan"       6935    "udp" []
  , makeService "xsmsvc"         6936    "tcp" []
  , makeService "xsmsvc"         6936    "udp" []
  , makeService "bioserver"      6946    "tcp" []
  , makeService "bioserver"      6946    "udp" []
  , makeService "otlp"           6951    "tcp" []
  , makeService "otlp"           6951    "udp" []
  , makeService "jmact3"         6961    "tcp" []
  , makeService "jmact3"         6961    "udp" []
  , makeService "jmevt2"         6962    "tcp" []
  , makeService "jmevt2"         6962    "udp" []
  , makeService "swismgr1"       6963    "tcp" []
  , makeService "swismgr1"       6963    "udp" []
  , makeService "swismgr2"       6964    "tcp" []
  , makeService "swismgr2"       6964    "udp" []
  , makeService "swistrap"       6965    "tcp" []
  , makeService "swistrap"       6965    "udp" []
  , makeService "swispol"        6966    "tcp" []
  , makeService "swispol"        6966    "udp" []
  , makeService "acmsoda"        6969    "tcp" []
  , makeService "acmsoda"        6969    "udp" []
  , makeService "MobilitySrv"    6997    "tcp" []
  , makeService "MobilitySrv"    6997    "udp" []
  , makeService "iatp-highpri"   6998    "tcp" []
  , makeService "iatp-highpri"   6998    "udp" []
  , makeService "iatp-normalpri" 6999    "tcp" []
  , makeService "iatp-normalpri" 6999    "udp" []
  , makeService "ups-onlinet"    7010    "tcp" []
  , makeService "ups-onlinet"    7010    "udp" []
  , makeService "talon-disc"     7011    "tcp" []
  , makeService "talon-disc"     7011    "udp" []
  , makeService "talon-engine"   7012    "tcp" []
  , makeService "talon-engine"   7012    "udp" []
  , makeService "microtalon-dis" 7013    "tcp" []
  , makeService "microtalon-dis" 7013    "udp" []
  , makeService "microtalon-com" 7014    "tcp" []
  , makeService "microtalon-com" 7014    "udp" []
  , makeService "talon-webserver"7015    "tcp" []
  , makeService "talon-webserver"7015    "udp" []
  , makeService "fisa-svc"       7018    "tcp" []
  , makeService "doceri-ctl"     7019    "tcp" []
  , makeService "doceri-view"    7019    "udp" []
  , makeService "dpserve"        7020    "tcp" []
  , makeService "dpserve"        7020    "udp" []
  , makeService "dpserveadmin"   7021    "tcp" []
  , makeService "dpserveadmin"   7021    "udp" []
  , makeService "ctdp"           7022    "tcp" []
  , makeService "ctdp"           7022    "udp" []
  , makeService "ct2nmcs"        7023    "tcp" []
  , makeService "ct2nmcs"        7023    "udp" []
  , makeService "vmsvc"          7024    "tcp" []
  , makeService "vmsvc"          7024    "udp" []
  , makeService "vmsvc-2"        7025    "tcp" []
  , makeService "vmsvc-2"        7025    "udp" []
  , makeService "op-probe"       7030    "tcp" []
  , makeService "op-probe"       7030    "udp" []
  , makeService "iposplanet"     7031    "tcp" []
  , makeService "quest-disc"     7040    "udp" []
  , makeService "arcp"           7070    "tcp" []
  , makeService "arcp"           7070    "udp" []
  , makeService "iwg1"           7071    "tcp" []
  , makeService "iwg1"           7071    "udp" []
  , makeService "empowerid"      7080    "tcp" []
  , makeService "empowerid"      7080    "udp" []
  , makeService "jdp-disc"       7095    "udp" []
  , makeService "lazy-ptop"      7099    "tcp" []
  , makeService "lazy-ptop"      7099    "udp" []
  , makeService "font-service"   7100    "udp" []
  , makeService "elcn"           7101    "tcp" []
  , makeService "elcn"           7101    "udp" []
  , makeService "aes-x170"       7107    "udp" []
  , makeService "virprot-lm"     7121    "tcp" []
  , makeService "virprot-lm"     7121    "udp" []
  , makeService "scenidm"        7128    "tcp" []
  , makeService "scenidm"        7128    "udp" []
  , makeService "scenccs"        7129    "tcp" []
  , makeService "scenccs"        7129    "udp" []
  , makeService "cabsm-comm"     7161    "tcp" []
  , makeService "cabsm-comm"     7161    "udp" []
  , makeService "caistoragemgr"  7162    "tcp" []
  , makeService "caistoragemgr"  7162    "udp" []
  , makeService "cacsambroker"   7163    "tcp" []
  , makeService "cacsambroker"   7163    "udp" []
  , makeService "fsr"            7164    "tcp" []
  , makeService "fsr"            7164    "udp" []
  , makeService "doc-server"     7165    "tcp" []
  , makeService "doc-server"     7165    "udp" []
  , makeService "aruba-server"   7166    "tcp" []
  , makeService "aruba-server"   7166    "udp" []
  , makeService "casrmagent"     7167    "tcp" []
  , makeService "cnckadserver"   7168    "tcp" []
  , makeService "ccag-pib"       7169    "tcp" []
  , makeService "ccag-pib"       7169    "udp" []
  , makeService "nsrp"           7170    "tcp" []
  , makeService "nsrp"           7170    "udp" []
  , makeService "drm-production" 7171    "tcp" []
  , makeService "drm-production" 7171    "udp" []
  , makeService "metalbend"      7172    "tcp" []
  , makeService "zsecure"        7173    "tcp" []
  , makeService "clutild"        7174    "tcp" []
  , makeService "clutild"        7174    "udp" []
  , makeService "fodms"          7200    "tcp" []
  , makeService "fodms"          7200    "udp" []
  , makeService "dlip"           7201    "tcp" []
  , makeService "dlip"           7201    "udp" []
  , makeService "ramp"           7227    "tcp" []
  , makeService "ramp"           7227    "udp" []
  , makeService "citrixupp"      7228    "tcp" []
  , makeService "citrixuppg"     7229    "tcp" []
  , makeService "display"        7236    "tcp" []
  , makeService "pads"           7237    "tcp" []
  , makeService "cnap"           7262    "tcp" []
  , makeService "cnap"           7262    "udp" []
  , makeService "watchme-7272"   7272    "tcp" []
  , makeService "watchme-7272"   7272    "udp" []
  , makeService "oma-rlp"        7273    "tcp" []
  , makeService "oma-rlp"        7273    "udp" []
  , makeService "oma-rlp-s"      7274    "tcp" []
  , makeService "oma-rlp-s"      7274    "udp" []
  , makeService "oma-ulp"        7275    "tcp" []
  , makeService "oma-ulp"        7275    "udp" []
  , makeService "oma-ilp"        7276    "tcp" []
  , makeService "oma-ilp"        7276    "udp" []
  , makeService "oma-ilp-s"      7277    "tcp" []
  , makeService "oma-ilp-s"      7277    "udp" []
  , makeService "oma-dcdocbs"    7278    "tcp" []
  , makeService "oma-dcdocbs"    7278    "udp" []
  , makeService "ctxlic"         7279    "tcp" []
  , makeService "ctxlic"         7279    "udp" []
  , makeService "itactionserver1"7280    "tcp" []
  , makeService "itactionserver1"7280    "udp" []
  , makeService "itactionserver2"7281    "tcp" []
  , makeService "itactionserver2"7281    "udp" []
  , makeService "mzca-action"    7282    "tcp" []
  , makeService "mzca-alert"     7282    "udp" []
  , makeService "genstat"        7283    "tcp" []
  , makeService "lcm-server"     7365    "tcp" []
  , makeService "lcm-server"     7365    "udp" []
  , makeService "mindfilesys"    7391    "tcp" []
  , makeService "mindfilesys"    7391    "udp" []
  , makeService "mrssrendezvous" 7392    "tcp" []
  , makeService "mrssrendezvous" 7392    "udp" []
  , makeService "nfoldman"       7393    "tcp" []
  , makeService "nfoldman"       7393    "udp" []
  , makeService "fse"            7394    "tcp" []
  , makeService "fse"            7394    "udp" []
  , makeService "winqedit"       7395    "tcp" []
  , makeService "winqedit"       7395    "udp" []
  , makeService "hexarc"         7397    "tcp" []
  , makeService "hexarc"         7397    "udp" []
  , makeService "rtps-discovery" 7400    "tcp" []
  , makeService "rtps-discovery" 7400    "udp" []
  , makeService "rtps-dd-ut"     7401    "tcp" []
  , makeService "rtps-dd-ut"     7401    "udp" []
  , makeService "rtps-dd-mt"     7402    "tcp" []
  , makeService "rtps-dd-mt"     7402    "udp" []
  , makeService "ionixnetmon"    7410    "tcp" []
  , makeService "ionixnetmon"    7410    "udp" []
  , makeService "daqstream"      7411    "tcp" []
  , makeService "daqstream"      7411    "udp" []
  , makeService "mtportmon"      7421    "tcp" []
  , makeService "mtportmon"      7421    "udp" []
  , makeService "pmdmgr"         7426    "tcp" []
  , makeService "pmdmgr"         7426    "udp" []
  , makeService "oveadmgr"       7427    "tcp" []
  , makeService "oveadmgr"       7427    "udp" []
  , makeService "ovladmgr"       7428    "tcp" []
  , makeService "ovladmgr"       7428    "udp" []
  , makeService "opi-sock"       7429    "tcp" []
  , makeService "opi-sock"       7429    "udp" []
  , makeService "xmpv7"          7430    "tcp" []
  , makeService "xmpv7"          7430    "udp" []
  , makeService "pmd"            7431    "tcp" []
  , makeService "pmd"            7431    "udp" []
  , makeService "faximum"        7437    "tcp" []
  , makeService "faximum"        7437    "udp" []
  , makeService "oracleas-https" 7443    "tcp" []
  , makeService "oracleas-https" 7443    "udp" []
  , makeService "rise"           7473    "tcp" []
  , makeService "rise"           7473    "udp" []
  , makeService "neo4j"          7474    "tcp" []
  , makeService "telops-lmd"     7491    "tcp" []
  , makeService "telops-lmd"     7491    "udp" []
  , makeService "silhouette"     7500    "tcp" []
  , makeService "silhouette"     7500    "udp" []
  , makeService "ovbus"          7501    "tcp" []
  , makeService "ovbus"          7501    "udp" []
  , makeService "adcp"           7508    "tcp" []
  , makeService "acplt"          7509    "tcp" []
  , makeService "ovhpas"         7510    "tcp" []
  , makeService "ovhpas"         7510    "udp" []
  , makeService "pafec-lm"       7511    "tcp" []
  , makeService "pafec-lm"       7511    "udp" []
  , makeService "saratoga"       7542    "tcp" []
  , makeService "saratoga"       7542    "udp" []
  , makeService "atul"           7543    "tcp" []
  , makeService "atul"           7543    "udp" []
  , makeService "nta-ds"         7544    "tcp" []
  , makeService "nta-ds"         7544    "udp" []
  , makeService "nta-us"         7545    "tcp" []
  , makeService "nta-us"         7545    "udp" []
  , makeService "cfs"            7546    "tcp" []
  , makeService "cfs"            7546    "udp" []
  , makeService "cwmp"           7547    "tcp" []
  , makeService "cwmp"           7547    "udp" []
  , makeService "tidp"           7548    "tcp" []
  , makeService "tidp"           7548    "udp" []
  , makeService "nls-tl"         7549    "tcp" []
  , makeService "nls-tl"         7549    "udp" []
  , makeService "cloudsignaling" 7550    "udp" []
  , makeService "sncp"           7560    "tcp" []
  , makeService "sncp"           7560    "udp" []
  , makeService "cfw"            7563    "tcp" []
  , makeService "vsi-omega"      7566    "tcp" []
  , makeService "vsi-omega"      7566    "udp" []
  , makeService "dell-eql-asm"   7569    "tcp" []
  , makeService "aries-kfinder"  7570    "tcp" []
  , makeService "aries-kfinder"  7570    "udp" []
  , makeService "sun-lm"         7588    "tcp" []
  , makeService "sun-lm"         7588    "udp" []
  , makeService "indi"           7624    "tcp" []
  , makeService "indi"           7624    "udp" []
  , makeService "simco"          7626    "tcp" []
  , makeService "soap-http"      7627    "tcp" []
  , makeService "soap-http"      7627    "udp" []
  , makeService "zen-pawn"       7628    "tcp" []
  , makeService "zen-pawn"       7628    "udp" []
  , makeService "xdas"           7629    "tcp" []
  , makeService "xdas"           7629    "udp" []
  , makeService "hawk"           7630    "tcp" []
  , makeService "tesla-sys-msg"  7631    "tcp" []
  , makeService "pmdfmgt"        7633    "tcp" []
  , makeService "pmdfmgt"        7633    "udp" []
  , makeService "cuseeme"        7648    "tcp" []
  , makeService "cuseeme"        7648    "udp" []
  , makeService "imqstomp"       7672    "tcp" []
  , makeService "imqstomps"      7673    "tcp" []
  , makeService "imqtunnels"     7674    "tcp" []
  , makeService "imqtunnels"     7674    "udp" []
  , makeService "imqtunnel"      7675    "tcp" []
  , makeService "imqtunnel"      7675    "udp" []
  , makeService "imqbrokerd"     7676    "tcp" []
  , makeService "imqbrokerd"     7676    "udp" []
  , makeService "sun-user-https" 7677    "tcp" []
  , makeService "sun-user-https" 7677    "udp" []
  , makeService "pando-pub"      7680    "tcp" []
  , makeService "pando-pub"      7680    "udp" []
  , makeService "collaber"       7689    "tcp" []
  , makeService "collaber"       7689    "udp" []
  , makeService "klio"           7697    "tcp" []
  , makeService "klio"           7697    "udp" []
  , makeService "em7-secom"      7700    "tcp" []
  , makeService "sync-em7"       7707    "tcp" []
  , makeService "sync-em7"       7707    "udp" []
  , makeService "scinet"         7708    "tcp" []
  , makeService "scinet"         7708    "udp" []
  , makeService "medimageportal" 7720    "tcp" []
  , makeService "medimageportal" 7720    "udp" []
  , makeService "nsdeepfreezectl"7724    "tcp" []
  , makeService "nsdeepfreezectl"7724    "udp" []
  , makeService "nitrogen"       7725    "tcp" []
  , makeService "nitrogen"       7725    "udp" []
  , makeService "freezexservice" 7726    "tcp" []
  , makeService "freezexservice" 7726    "udp" []
  , makeService "trident-data"   7727    "tcp" []
  , makeService "trident-data"   7727    "udp" []
  , makeService "smip"           7734    "tcp" []
  , makeService "smip"           7734    "udp" []
  , makeService "aiagent"        7738    "tcp" []
  , makeService "aiagent"        7738    "udp" []
  , makeService "scriptview"     7741    "tcp" []
  , makeService "scriptview"     7741    "udp" []
  , makeService "msss"           7742    "tcp" []
  , makeService "sstp-1"         7743    "tcp" []
  , makeService "sstp-1"         7743    "udp" []
  , makeService "raqmon-pdu"     7744    "tcp" []
  , makeService "raqmon-pdu"     7744    "udp" []
  , makeService "prgp"           7747    "tcp" []
  , makeService "prgp"           7747    "udp" []
  , makeService "cbt"            7777    "tcp" []
  , makeService "cbt"            7777    "udp" []
  , makeService "interwise"      7778    "tcp" []
  , makeService "interwise"      7778    "udp" []
  , makeService "vstat"          7779    "tcp" []
  , makeService "vstat"          7779    "udp" []
  , makeService "accu-lmgr"      7781    "tcp" []
  , makeService "accu-lmgr"      7781    "udp" []
  , makeService "minivend"       7786    "tcp" []
  , makeService "minivend"       7786    "udp" []
  , makeService "popup-reminders"7787    "tcp" []
  , makeService "popup-reminders"7787    "udp" []
  , makeService "office-tools"   7789    "tcp" []
  , makeService "office-tools"   7789    "udp" []
  , makeService "q3ade"          7794    "tcp" []
  , makeService "q3ade"          7794    "udp" []
  , makeService "pnet-conn"      7797    "tcp" []
  , makeService "pnet-conn"      7797    "udp" []
  , makeService "pnet-enc"       7798    "tcp" []
  , makeService "pnet-enc"       7798    "udp" []
  , makeService "altbsdp"        7799    "tcp" []
  , makeService "altbsdp"        7799    "udp" []
  , makeService "asr"            7800    "tcp" []
  , makeService "asr"            7800    "udp" []
  , makeService "ssp-client"     7801    "tcp" []
  , makeService "ssp-client"     7801    "udp" []
  , makeService "vns-tp"         7802    "udp" []
  , makeService "rbt-wanopt"     7810    "tcp" []
  , makeService "rbt-wanopt"     7810    "udp" []
  , makeService "apc-7845"       7845    "tcp" []
  , makeService "apc-7845"       7845    "udp" []
  , makeService "apc-7846"       7846    "tcp" []
  , makeService "apc-7846"       7846    "udp" []
  , makeService "mobileanalyzer" 7869    "tcp" []
  , makeService "rbt-smc"        7870    "tcp" []
  , makeService "mdm"            7871    "tcp" []
  , makeService "mipv6tls"       7872    "udp" []
  , makeService "pss"            7880    "tcp" []
  , makeService "pss"            7880    "udp" []
  , makeService "ubroker"        7887    "tcp" []
  , makeService "ubroker"        7887    "udp" []
  , makeService "mevent"         7900    "tcp" []
  , makeService "mevent"         7900    "udp" []
  , makeService "tnos-sp"        7901    "tcp" []
  , makeService "tnos-sp"        7901    "udp" []
  , makeService "tnos-dp"        7902    "tcp" []
  , makeService "tnos-dp"        7902    "udp" []
  , makeService "tnos-dps"       7903    "tcp" []
  , makeService "tnos-dps"       7903    "udp" []
  , makeService "qo-secure"      7913    "tcp" []
  , makeService "qo-secure"      7913    "udp" []
  , makeService "t2-drm"         7932    "tcp" []
  , makeService "t2-drm"         7932    "udp" []
  , makeService "t2-brm"         7933    "tcp" []
  , makeService "t2-brm"         7933    "udp" []
  , makeService "supercell"      7967    "tcp" []
  , makeService "supercell"      7967    "udp" []
  , makeService "micromuse-ncps" 7979    "tcp" []
  , makeService "micromuse-ncps" 7979    "udp" []
  , makeService "quest-vista"    7980    "tcp" []
  , makeService "quest-vista"    7980    "udp" []
  , makeService "sossd-collect"  7981    "tcp" []
  , makeService "sossd-agent"    7982    "tcp" []
  , makeService "sossd-disc"     7982    "udp" []
  , makeService "pushns"         7997    "tcp" []
  , makeService "usicontentpush" 7998    "udp" []
  , makeService "irdmi2"         7999    "tcp" []
  , makeService "irdmi2"         7999    "udp" []
  , makeService "irdmi"          8000    "tcp" []
  , makeService "irdmi"          8000    "udp" []
  , makeService "vcom-tunnel"    8001    "tcp" []
  , makeService "vcom-tunnel"    8001    "udp" []
  , makeService "teradataordbms" 8002    "tcp" []
  , makeService "teradataordbms" 8002    "udp" []
  , makeService "mcreport"       8003    "tcp" []
  , makeService "mcreport"       8003    "udp" []
  , makeService "mxi"            8005    "tcp" []
  , makeService "mxi"            8005    "udp" []
  , makeService "qbdb"           8019    "tcp" []
  , makeService "qbdb"           8019    "udp" []
  , makeService "intu-ec-svcdisc"8020    "tcp" []
  , makeService "intu-ec-svcdisc"8020    "udp" []
  , makeService "intu-ec-client" 8021    "tcp" []
  , makeService "intu-ec-client" 8021    "udp" []
  , makeService "oa-system"      8022    "tcp" []
  , makeService "oa-system"      8022    "udp" []
  , makeService "ca-audit-da"    8025    "tcp" []
  , makeService "ca-audit-da"    8025    "udp" []
  , makeService "ca-audit-ds"    8026    "tcp" []
  , makeService "ca-audit-ds"    8026    "udp" []
  , makeService "pro-ed"         8032    "tcp" []
  , makeService "pro-ed"         8032    "udp" []
  , makeService "mindprint"      8033    "tcp" []
  , makeService "mindprint"      8033    "udp" []
  , makeService "vantronix-mgmt" 8034    "tcp" []
  , makeService "vantronix-mgmt" 8034    "udp" []
  , makeService "ampify"         8040    "tcp" []
  , makeService "ampify"         8040    "udp" []
  , makeService "fs-agent"       8042    "tcp" []
  , makeService "fs-server"      8043    "tcp" []
  , makeService "fs-mgmt"        8044    "tcp" []
  , makeService "rocrail"        8051    "tcp" []
  , makeService "senomix01"      8052    "tcp" []
  , makeService "senomix01"      8052    "udp" []
  , makeService "senomix02"      8053    "tcp" []
  , makeService "senomix02"      8053    "udp" []
  , makeService "senomix03"      8054    "tcp" []
  , makeService "senomix03"      8054    "udp" []
  , makeService "senomix04"      8055    "tcp" []
  , makeService "senomix04"      8055    "udp" []
  , makeService "senomix05"      8056    "tcp" []
  , makeService "senomix05"      8056    "udp" []
  , makeService "senomix06"      8057    "tcp" []
  , makeService "senomix06"      8057    "udp" []
  , makeService "senomix07"      8058    "tcp" []
  , makeService "senomix07"      8058    "udp" []
  , makeService "senomix08"      8059    "tcp" []
  , makeService "senomix08"      8059    "udp" []
  , makeService "aero"           8060    "udp" []
  , makeService "gadugadu"       8074    "tcp" []
  , makeService "gadugadu"       8074    "udp" []
  , makeService "us-cli"         8082    "tcp" []
  , makeService "us-cli"         8082    "udp" []
  , makeService "us-srv"         8083    "tcp" []
  , makeService "us-srv"         8083    "udp" []
  , makeService "d-s-n"          8086    "tcp" []
  , makeService "d-s-n"          8086    "udp" []
  , makeService "simplifymedia"  8087    "tcp" []
  , makeService "simplifymedia"  8087    "udp" []
  , makeService "radan-http"     8088    "tcp" []
  , makeService "radan-http"     8088    "udp" []
  , makeService "jamlink"        8091    "tcp" []
  , makeService "sac"            8097    "tcp" []
  , makeService "sac"            8097    "udp" []
  , makeService "xprint-server"  8100    "tcp" []
  , makeService "xprint-server"  8100    "udp" []
  , makeService "ldoms-migr"     8101    "tcp" []
  , makeService "mtl8000-matrix" 8115    "tcp" []
  , makeService "mtl8000-matrix" 8115    "udp" []
  , makeService "cp-cluster"     8116    "tcp" []
  , makeService "cp-cluster"     8116    "udp" []
  , makeService "privoxy"        8118    "tcp" []
  , makeService "privoxy"        8118    "udp" []
  , makeService "apollo-data"    8121    "tcp" []
  , makeService "apollo-data"    8121    "udp" []
  , makeService "apollo-admin"   8122    "tcp" []
  , makeService "apollo-admin"   8122    "udp" []
  , makeService "paycash-online" 8128    "tcp" []
  , makeService "paycash-online" 8128    "udp" []
  , makeService "paycash-wbp"    8129    "tcp" []
  , makeService "paycash-wbp"    8129    "udp" []
  , makeService "indigo-vrmi"    8130    "tcp" []
  , makeService "indigo-vrmi"    8130    "udp" []
  , makeService "indigo-vbcp"    8131    "tcp" []
  , makeService "indigo-vbcp"    8131    "udp" []
  , makeService "dbabble"        8132    "tcp" []
  , makeService "dbabble"        8132    "udp" []
  , makeService "isdd"           8148    "tcp" []
  , makeService "isdd"           8148    "udp" []
  , makeService "eor-game"       8149    "udp" []
  , makeService "quantastor"     8153    "tcp" []
  , makeService "patrol"         8160    "tcp" []
  , makeService "patrol"         8160    "udp" []
  , makeService "patrol-snmp"    8161    "tcp" []
  , makeService "patrol-snmp"    8161    "udp" []
  , makeService "intermapper"    8181    "tcp" []
  , makeService "vmware-fdm"     8182    "tcp" []
  , makeService "vmware-fdm"     8182    "udp" []
  , makeService "proremote"      8183    "tcp" []
  , makeService "itach"          8184    "tcp" []
  , makeService "itach"          8184    "udp" []
  , makeService "spytechphone"   8192    "tcp" []
  , makeService "spytechphone"   8192    "udp" []
  , makeService "blp1"           8194    "tcp" []
  , makeService "blp1"           8194    "udp" []
  , makeService "blp2"           8195    "tcp" []
  , makeService "blp2"           8195    "udp" []
  , makeService "vvr-data"       8199    "tcp" []
  , makeService "vvr-data"       8199    "udp" []
  , makeService "trivnet1"       8200    "tcp" []
  , makeService "trivnet1"       8200    "udp" []
  , makeService "trivnet2"       8201    "tcp" []
  , makeService "trivnet2"       8201    "udp" []
  , makeService "aesop"          8202    "udp" []
  , makeService "lm-perfworks"   8204    "tcp" []
  , makeService "lm-perfworks"   8204    "udp" []
  , makeService "lm-instmgr"     8205    "tcp" []
  , makeService "lm-instmgr"     8205    "udp" []
  , makeService "lm-dta"         8206    "tcp" []
  , makeService "lm-dta"         8206    "udp" []
  , makeService "lm-sserver"     8207    "tcp" []
  , makeService "lm-sserver"     8207    "udp" []
  , makeService "lm-webwatcher"  8208    "tcp" []
  , makeService "lm-webwatcher"  8208    "udp" []
  , makeService "rexecj"         8230    "tcp" []
  , makeService "rexecj"         8230    "udp" []
  , makeService "synapse-nhttps" 8243    "tcp" []
  , makeService "synapse-nhttps" 8243    "udp" []
  , makeService "pando-sec"      8276    "tcp" []
  , makeService "pando-sec"      8276    "udp" []
  , makeService "synapse-nhttp"  8280    "tcp" []
  , makeService "synapse-nhttp"  8280    "udp" []
  , makeService "blp3"           8292    "tcp" []
  , makeService "blp3"           8292    "udp" []
  , makeService "blp4"           8294    "tcp" []
  , makeService "blp4"           8294    "udp" []
  , makeService "hiperscan-id"   8293    "tcp" []
  , makeService "tmi"            8300    "tcp" []
  , makeService "tmi"            8300    "udp" []
  , makeService "amberon"        8301    "tcp" []
  , makeService "amberon"        8301    "udp" []
  , makeService "hub-open-net"   8313    "tcp" []
  , makeService "tnp-discover"   8320    "tcp" []
  , makeService "tnp-discover"   8320    "udp" []
  , makeService "tnp"            8321    "tcp" []
  , makeService "tnp"            8321    "udp" []
  , makeService "server-find"    8351    "tcp" []
  , makeService "server-find"    8351    "udp" []
  , makeService "cruise-enum"    8376    "tcp" []
  , makeService "cruise-enum"    8376    "udp" []
  , makeService "cruise-swroute" 8377    "tcp" []
  , makeService "cruise-swroute" 8377    "udp" []
  , makeService "cruise-config"  8378    "tcp" []
  , makeService "cruise-config"  8378    "udp" []
  , makeService "cruise-diags"   8379    "tcp" []
  , makeService "cruise-diags"   8379    "udp" []
  , makeService "cruise-update"  8380    "tcp" []
  , makeService "cruise-update"  8380    "udp" []
  , makeService "m2mservices"    8383    "tcp" []
  , makeService "m2mservices"    8383    "udp" []
  , makeService "cvd"            8400    "tcp" []
  , makeService "cvd"            8400    "udp" []
  , makeService "sabarsd"        8401    "tcp" []
  , makeService "sabarsd"        8401    "udp" []
  , makeService "abarsd"         8402    "tcp" []
  , makeService "abarsd"         8402    "udp" []
  , makeService "admind2"        8403    "tcp" []
  , makeService "admind2"        8403    "udp" []
  , makeService "svcloud"        8404    "tcp" []
  , makeService "svbackup"       8405    "tcp" []
  , makeService "dlpx-sp"        8415    "tcp" []
  , makeService "espeech"        8416    "tcp" []
  , makeService "espeech"        8416    "udp" []
  , makeService "espeech-rtp"    8417    "tcp" []
  , makeService "espeech-rtp"    8417    "udp" []
  , makeService "cybro-a-bus"    8442    "tcp" []
  , makeService "cybro-a-bus"    8442    "udp" []
  , makeService "pcsync-https"   8443    "tcp" []
  , makeService "pcsync-https"   8443    "udp" []
  , makeService "pcsync-http"    8444    "tcp" []
  , makeService "pcsync-http"    8444    "udp" []
  , makeService "copy"           8445    "tcp" []
  , makeService "copy-disc"      8445    "udp" []
  , makeService "npmp"           8450    "tcp" []
  , makeService "npmp"           8450    "udp" []
  , makeService "nexentamv"      8457    "tcp" []
  , makeService "cisco-avp"      8470    "tcp" []
  , makeService "pim-port"       8471    "tcp" []
  , makeService "otv"            8472    "tcp" []
  , makeService "otv"            8472    "udp" []
  , makeService "vp2p"           8473    "tcp" []
  , makeService "vp2p"           8473    "udp" []
  , makeService "noteshare"      8474    "tcp" []
  , makeService "noteshare"      8474    "udp" []
  , makeService "fmtp"           8500    "tcp" []
  , makeService "fmtp"           8500    "udp" []
  , makeService "cmtp-mgt"       8501    "tcp" []
  , makeService "cmtp-av"        8501    "udp" []
  , makeService "rtsp-alt"       8554    "tcp" []
  , makeService "rtsp-alt"       8554    "udp" []
  , makeService "d-fence"        8555    "tcp" []
  , makeService "d-fence"        8555    "udp" []
  , makeService "enc-tunnel"     8567    "tcp" []
  , makeService "enc-tunnel"     8567    "udp" []
  , makeService "asterix"        8600    "tcp" []
  , makeService "asterix"        8600    "udp" []
  , makeService "canon-cpp-disc" 8609    "udp" []
  , makeService "canon-mfnp"     8610    "tcp" []
  , makeService "canon-mfnp"     8610    "udp" []
  , makeService "canon-bjnp1"    8611    "tcp" []
  , makeService "canon-bjnp1"    8611    "udp" []
  , makeService "canon-bjnp2"    8612    "tcp" []
  , makeService "canon-bjnp2"    8612    "udp" []
  , makeService "canon-bjnp3"    8613    "tcp" []
  , makeService "canon-bjnp3"    8613    "udp" []
  , makeService "canon-bjnp4"    8614    "tcp" []
  , makeService "canon-bjnp4"    8614    "udp" []
  , makeService "imink"          8615    "tcp" []
  , makeService "monetra"        8665    "tcp" []
  , makeService "monetra-admin"  8666    "tcp" []
  , makeService "msi-cps-rm"     8675    "tcp" []
  , makeService "msi-cps-rm-disc"8675    "udp" []
  , makeService "sun-as-jmxrmi"  8686    "tcp" []
  , makeService "sun-as-jmxrmi"  8686    "udp" []
  , makeService "openremote-ctrl"8688    "tcp" []
  , makeService "vnyx"           8699    "tcp" []
  , makeService "vnyx"           8699    "udp" []
  , makeService "nvc"            8711    "tcp" []
  , makeService "dtp-net"        8732    "udp" []
  , makeService "ibus"           8733    "tcp" []
  , makeService "ibus"           8733    "udp" []
  , makeService "dey-keyneg"     8750    "tcp" []
  , makeService "mc-appserver"   8763    "tcp" []
  , makeService "mc-appserver"   8763    "udp" []
  , makeService "openqueue"      8764    "tcp" []
  , makeService "openqueue"      8764    "udp" []
  , makeService "ultraseek-http" 8765    "tcp" []
  , makeService "ultraseek-http" 8765    "udp" []
  , makeService "amcs"           8766    "tcp" []
  , makeService "amcs"           8766    "udp" []
  , makeService "dpap"           8770    "tcp" []
  , makeService "dpap"           8770    "udp" []
  , makeService "msgclnt"        8786    "tcp" []
  , makeService "msgclnt"        8786    "udp" []
  , makeService "msgsrvr"        8787    "tcp" []
  , makeService "msgsrvr"        8787    "udp" []
  , makeService "acd-pm"         8793    "tcp" []
  , makeService "acd-pm"         8793    "udp" []
  , makeService "sunwebadmin"    8800    "tcp" []
  , makeService "sunwebadmin"    8800    "udp" []
  , makeService "truecm"         8804    "tcp" []
  , makeService "truecm"         8804    "udp" []
  , makeService "dxspider"       8873    "tcp" []
  , makeService "dxspider"       8873    "udp" []
  , makeService "cddbp-alt"      8880    "tcp" []
  , makeService "cddbp-alt"      8880    "udp" []
  , makeService "galaxy4d"       8881    "tcp" []
  , makeService "secure-mqtt"    8883    "tcp" []
  , makeService "secure-mqtt"    8883    "udp" []
  , makeService "ddi-tcp-1"      8888    "tcp" []
  , makeService "ddi-udp-1"      8888    "udp" []
  , makeService "ddi-tcp-2"      8889    "tcp" []
  , makeService "ddi-udp-2"      8889    "udp" []
  , makeService "ddi-tcp-3"      8890    "tcp" []
  , makeService "ddi-udp-3"      8890    "udp" []
  , makeService "ddi-tcp-4"      8891    "tcp" []
  , makeService "ddi-udp-4"      8891    "udp" []
  , makeService "ddi-tcp-5"      8892    "tcp" []
  , makeService "ddi-udp-5"      8892    "udp" []
  , makeService "ddi-tcp-6"      8893    "tcp" []
  , makeService "ddi-udp-6"      8893    "udp" []
  , makeService "ddi-tcp-7"      8894    "tcp" []
  , makeService "ddi-udp-7"      8894    "udp" []
  , makeService "ospf-lite"      8899    "tcp" []
  , makeService "ospf-lite"      8899    "udp" []
  , makeService "jmb-cds1"       8900    "tcp" []
  , makeService "jmb-cds1"       8900    "udp" []
  , makeService "jmb-cds2"       8901    "tcp" []
  , makeService "jmb-cds2"       8901    "udp" []
  , makeService "manyone-http"   8910    "tcp" []
  , makeService "manyone-http"   8910    "udp" []
  , makeService "manyone-xml"    8911    "tcp" []
  , makeService "manyone-xml"    8911    "udp" []
  , makeService "wcbackup"       8912    "tcp" []
  , makeService "wcbackup"       8912    "udp" []
  , makeService "dragonfly"      8913    "tcp" []
  , makeService "dragonfly"      8913    "udp" []
  , makeService "twds"           8937    "tcp" []
  , makeService "ub-dns-control" 8953    "tcp" []
  , makeService "cumulus-admin"  8954    "tcp" []
  , makeService "cumulus-admin"  8954    "udp" []
  , makeService "sunwebadmins"   8989    "tcp" []
  , makeService "sunwebadmins"   8989    "udp" []
  , makeService "http-wmap"      8990    "tcp" []
  , makeService "http-wmap"      8990    "udp" []
  , makeService "https-wmap"     8991    "tcp" []
  , makeService "https-wmap"     8991    "udp" []
  , makeService "bctp"           8999    "tcp" []
  , makeService "bctp"           8999    "udp" []
  , makeService "cslistener"     9000    "tcp" []
  , makeService "cslistener"     9000    "udp" []
  , makeService "etlservicemgr"  9001    "tcp" []
  , makeService "etlservicemgr"  9001    "udp" []
  , makeService "dynamid"        9002    "tcp" []
  , makeService "dynamid"        9002    "udp" []
  , makeService "ogs-client"     9007    "udp" []
  , makeService "ogs-server"     9008    "tcp" []
  , makeService "pichat"         9009    "tcp" []
  , makeService "pichat"         9009    "udp" []
  , makeService "sdr"            9010    "tcp" []
  , makeService "tambora"        9020    "tcp" []
  , makeService "tambora"        9020    "udp" []
  , makeService "panagolin-ident"9021    "tcp" []
  , makeService "panagolin-ident"9021    "udp" []
  , makeService "paragent"       9022    "tcp" []
  , makeService "paragent"       9022    "udp" []
  , makeService "swa-1"          9023    "tcp" []
  , makeService "swa-1"          9023    "udp" []
  , makeService "swa-2"          9024    "tcp" []
  , makeService "swa-2"          9024    "udp" []
  , makeService "swa-3"          9025    "tcp" []
  , makeService "swa-3"          9025    "udp" []
  , makeService "swa-4"          9026    "tcp" []
  , makeService "swa-4"          9026    "udp" []
  , makeService "versiera"       9050    "tcp" []
  , makeService "fio-cmgmt"      9051    "tcp" []
  , makeService "glrpc"          9080    "tcp" []
  , makeService "glrpc"          9080    "udp" []
  , makeService "emc-pp-mgmtsvc" 9083    "tcp" []
  , makeService "aurora"         9084    "tcp" []
  , makeService "aurora"         9084    "udp" []
  , makeService "ibm-rsyscon"    9085    "tcp" []
  , makeService "ibm-rsyscon"    9085    "udp" []
  , makeService "net2display"    9086    "tcp" []
  , makeService "net2display"    9086    "udp" []
  , makeService "classic"        9087    "tcp" []
  , makeService "classic"        9087    "udp" []
  , makeService "sqlexec"        9088    "tcp" []
  , makeService "sqlexec"        9088    "udp" []
  , makeService "sqlexec-ssl"    9089    "tcp" []
  , makeService "sqlexec-ssl"    9089    "udp" []
  , makeService "websm"          9090    "tcp" []
  , makeService "websm"          9090    "udp" []
  , makeService "xmltec-xmlmail" 9091    "tcp" []
  , makeService "xmltec-xmlmail" 9091    "udp" []
  , makeService "XmlIpcRegSvc"   9092    "tcp" []
  , makeService "XmlIpcRegSvc"   9092    "udp" []
  , makeService "copycat"        9093    "tcp" []
  , makeService "hp-pdl-datastr" 9100    "udp" ["pdl-datastream"]
  , makeService "bacula-dir"     9101    "tcp" []
  , makeService "bacula-dir"     9101    "udp" []
  , makeService "bacula-fd"      9102    "tcp" []
  , makeService "bacula-fd"      9102    "udp" []
  , makeService "bacula-sd"      9103    "tcp" []
  , makeService "bacula-sd"      9103    "udp" []
  , makeService "peerwire"       9104    "tcp" []
  , makeService "peerwire"       9104    "udp" []
  , makeService "xadmin"         9105    "tcp" []
  , makeService "xadmin"         9105    "udp" []
  , makeService "astergate"      9106    "tcp" []
  , makeService "astergate-disc" 9106    "udp" []
  , makeService "astergatefax"   9107    "tcp" []
  , makeService "mxit"           9119    "tcp" []
  , makeService "mxit"           9119    "udp" []
  , makeService "grcmp"          9122    "tcp" []
  , makeService "grcp"           9123    "tcp" []
  , makeService "dddp"           9131    "tcp" []
  , makeService "dddp"           9131    "udp" []
  , makeService "apani1"         9160    "tcp" []
  , makeService "apani1"         9160    "udp" []
  , makeService "apani2"         9161    "tcp" []
  , makeService "apani2"         9161    "udp" []
  , makeService "apani3"         9162    "tcp" []
  , makeService "apani3"         9162    "udp" []
  , makeService "apani4"         9163    "tcp" []
  , makeService "apani4"         9163    "udp" []
  , makeService "apani5"         9164    "tcp" []
  , makeService "apani5"         9164    "udp" []
  , makeService "sun-as-jpda"    9191    "tcp" []
  , makeService "sun-as-jpda"    9191    "udp" []
  , makeService "wap-wsp"        9200    "tcp" []
  , makeService "wap-wsp"        9200    "udp" []
  , makeService "wap-wsp-wtp"    9201    "tcp" []
  , makeService "wap-wsp-wtp"    9201    "udp" []
  , makeService "wap-wsp-s"      9202    "tcp" []
  , makeService "wap-wsp-s"      9202    "udp" []
  , makeService "wap-wsp-wtp-s"  9203    "tcp" []
  , makeService "wap-wsp-wtp-s"  9203    "udp" []
  , makeService "wap-vcard"      9204    "tcp" []
  , makeService "wap-vcard"      9204    "udp" []
  , makeService "wap-vcal"       9205    "tcp" []
  , makeService "wap-vcal"       9205    "udp" []
  , makeService "wap-vcard-s"    9206    "tcp" []
  , makeService "wap-vcard-s"    9206    "udp" []
  , makeService "wap-vcal-s"     9207    "tcp" []
  , makeService "wap-vcal-s"     9207    "udp" []
  , makeService "rjcdb-vcards"   9208    "tcp" []
  , makeService "rjcdb-vcards"   9208    "udp" []
  , makeService "almobile-system"9209    "tcp" []
  , makeService "almobile-system"9209    "udp" []
  , makeService "oma-mlp"        9210    "tcp" []
  , makeService "oma-mlp"        9210    "udp" []
  , makeService "oma-mlp-s"      9211    "tcp" []
  , makeService "oma-mlp-s"      9211    "udp" []
  , makeService "serverviewdbms" 9212    "tcp" []
  , makeService "serverviewdbms" 9212    "udp" []
  , makeService "serverstart"    9213    "tcp" []
  , makeService "serverstart"    9213    "udp" []
  , makeService "ipdcesgbs"      9214    "tcp" []
  , makeService "ipdcesgbs"      9214    "udp" []
  , makeService "insis"          9215    "tcp" []
  , makeService "insis"          9215    "udp" []
  , makeService "acme"           9216    "tcp" []
  , makeService "acme"           9216    "udp" []
  , makeService "fsc-port"       9217    "tcp" []
  , makeService "fsc-port"       9217    "udp" []
  , makeService "teamcoherence"  9222    "tcp" []
  , makeService "teamcoherence"  9222    "udp" []
  , makeService "Mon"            9255    "tcp" []
  , makeService "Mon"            9255    "udp" []
  , makeService "traingpsdata"   9277    "udp" []
  , makeService "pegasus"        9278    "tcp" []
  , makeService "pegasus"        9278    "udp" []
  , makeService "pegasus-ctl"    9279    "tcp" []
  , makeService "pegasus-ctl"    9279    "udp" []
  , makeService "pgps"           9280    "tcp" []
  , makeService "pgps"           9280    "udp" []
  , makeService "swtp-port1"     9281    "tcp" []
  , makeService "swtp-port1"     9281    "udp" []
  , makeService "swtp-port2"     9282    "tcp" []
  , makeService "swtp-port2"     9282    "udp" []
  , makeService "callwaveiam"    9283    "tcp" []
  , makeService "callwaveiam"    9283    "udp" []
  , makeService "visd"           9284    "tcp" []
  , makeService "visd"           9284    "udp" []
  , makeService "n2h2server"     9285    "tcp" []
  , makeService "n2h2server"     9285    "udp" []
  , makeService "n2receive"      9286    "udp" []
  , makeService "cumulus"        9287    "tcp" []
  , makeService "cumulus"        9287    "udp" []
  , makeService "armtechdaemon"  9292    "tcp" []
  , makeService "armtechdaemon"  9292    "udp" []
  , makeService "storview"       9293    "tcp" []
  , makeService "storview"       9293    "udp" []
  , makeService "armcenterhttp"  9294    "tcp" []
  , makeService "armcenterhttp"  9294    "udp" []
  , makeService "armcenterhttps" 9295    "tcp" []
  , makeService "armcenterhttps" 9295    "udp" []
  , makeService "vrace"          9300    "tcp" []
  , makeService "vrace"          9300    "udp" []
  , makeService "sphinxql"       9306    "tcp" []
  , makeService "sphinxapi"      9312    "tcp" []
  , makeService "secure-ts"      9318    "tcp" []
  , makeService "secure-ts"      9318    "udp" []
  , makeService "guibase"        9321    "tcp" []
  , makeService "guibase"        9321    "udp" []
  , makeService "mpidcmgr"       9343    "tcp" []
  , makeService "mpidcmgr"       9343    "udp" []
  , makeService "mphlpdmc"       9344    "tcp" []
  , makeService "mphlpdmc"       9344    "udp" []
  , makeService "ctechlicensing" 9346    "tcp" []
  , makeService "ctechlicensing" 9346    "udp" []
  , makeService "fjdmimgr"       9374    "tcp" []
  , makeService "fjdmimgr"       9374    "udp" []
  , makeService "boxp"           9380    "tcp" []
  , makeService "boxp"           9380    "udp" []
  , makeService "d2dconfig"      9387    "tcp" []
  , makeService "d2ddatatrans"   9388    "tcp" []
  , makeService "adws"           9389    "tcp" []
  , makeService "otp"            9390    "tcp" []
  , makeService "fjinvmgr"       9396    "tcp" []
  , makeService "fjinvmgr"       9396    "udp" []
  , makeService "mpidcagt"       9397    "tcp" []
  , makeService "mpidcagt"       9397    "udp" []
  , makeService "sec-t4net-srv"  9400    "tcp" []
  , makeService "sec-t4net-srv"  9400    "udp" []
  , makeService "sec-t4net-clt"  9401    "tcp" []
  , makeService "sec-t4net-clt"  9401    "udp" []
  , makeService "sec-pc2fax-srv" 9402    "tcp" []
  , makeService "sec-pc2fax-srv" 9402    "udp" []
  , makeService "git"            9418    "tcp" []
  , makeService "git"            9418    "udp" []
  , makeService "tungsten-https" 9443    "tcp" []
  , makeService "tungsten-https" 9443    "udp" []
  , makeService "wso2esb-console"9444    "tcp" []
  , makeService "wso2esb-console"9444    "udp" []
  , makeService "mindarray-ca"   9445    "tcp" []
  , makeService "sntlkeyssrvr"   9450    "tcp" []
  , makeService "sntlkeyssrvr"   9450    "udp" []
  , makeService "ismserver"      9500    "tcp" []
  , makeService "ismserver"      9500    "udp" []
  , makeService "sma-spw"        9522    "udp" []
  , makeService "mngsuite"       9535    "tcp" []
  , makeService "mngsuite"       9535    "udp" []
  , makeService "laes-bf"        9536    "tcp" []
  , makeService "laes-bf"        9536    "udp" []
  , makeService "trispen-sra"    9555    "tcp" []
  , makeService "trispen-sra"    9555    "udp" []
  , makeService "ldgateway"      9592    "tcp" []
  , makeService "ldgateway"      9592    "udp" []
  , makeService "cba8"           9593    "tcp" []
  , makeService "cba8"           9593    "udp" []
  , makeService "msgsys"         9594    "tcp" []
  , makeService "msgsys"         9594    "udp" []
  , makeService "pds"            9595    "tcp" []
  , makeService "pds"            9595    "udp" []
  , makeService "mercury-disc"   9596    "tcp" []
  , makeService "mercury-disc"   9596    "udp" []
  , makeService "pd-admin"       9597    "tcp" []
  , makeService "pd-admin"       9597    "udp" []
  , makeService "vscp"           9598    "tcp" []
  , makeService "vscp"           9598    "udp" []
  , makeService "robix"          9599    "tcp" []
  , makeService "robix"          9599    "udp" []
  , makeService "micromuse-ncpw" 9600    "tcp" []
  , makeService "micromuse-ncpw" 9600    "udp" []
  , makeService "streamcomm-ds"  9612    "tcp" []
  , makeService "streamcomm-ds"  9612    "udp" []
  , makeService "iadt-tls"       9614    "tcp" []
  , makeService "erunbook_agent" 9616    "tcp" ["erunbook-agent"]
  , makeService "erunbook_server"9617    "tcp" ["erunbook-server"]
  , makeService "condor"         9618    "tcp" []
  , makeService "condor"         9618    "udp" []
  , makeService "odbcpathway"    9628    "tcp" []
  , makeService "odbcpathway"    9628    "udp" []
  , makeService "uniport"        9629    "tcp" []
  , makeService "uniport"        9629    "udp" []
  , makeService "peoctlr"        9630    "tcp" []
  , makeService "peocoll"        9631    "tcp" []
  , makeService "mc-comm"        9632    "udp" []
  , makeService "pqsflows"       9640    "tcp" []
  , makeService "xmms2"          9667    "tcp" []
  , makeService "xmms2"          9667    "udp" []
  , makeService "tec5-sdctp"     9668    "tcp" []
  , makeService "tec5-sdctp"     9668    "udp" []
  , makeService "client-wakeup"  9694    "tcp" []
  , makeService "client-wakeup"  9694    "udp" []
  , makeService "ccnx"           9695    "tcp" []
  , makeService "ccnx"           9695    "udp" []
  , makeService "board-roar"     9700    "tcp" []
  , makeService "board-roar"     9700    "udp" []
  , makeService "l5nas-parchan"  9747    "tcp" []
  , makeService "l5nas-parchan"  9747    "udp" []
  , makeService "board-voip"     9750    "tcp" []
  , makeService "board-voip"     9750    "udp" []
  , makeService "rasadv"         9753    "tcp" []
  , makeService "rasadv"         9753    "udp" []
  , makeService "tungsten-http"  9762    "tcp" []
  , makeService "tungsten-http"  9762    "udp" []
  , makeService "davsrc"         9800    "tcp" []
  , makeService "davsrc"         9800    "udp" []
  , makeService "sstp-2"         9801    "tcp" []
  , makeService "sstp-2"         9801    "udp" []
  , makeService "davsrcs"        9802    "tcp" []
  , makeService "davsrcs"        9802    "udp" []
  , makeService "sapv1"          9875    "tcp" []
  , makeService "sapv1"          9875    "udp" []
  , makeService "sd"             9876    "tcp" []
  , makeService "kca-service"    9878    "udp" []
  , makeService "cyborg-systems" 9888    "tcp" []
  , makeService "cyborg-systems" 9888    "udp" []
  , makeService "gt-proxy"       9889    "tcp" []
  , makeService "gt-proxy"       9889    "udp" []
  , makeService "monkeycom"      9898    "tcp" []
  , makeService "monkeycom"      9898    "udp" []
  , makeService "sctp-tunneling" 9899    "udp" []
  , makeService "iua"            9900    "tcp" []
  , makeService "iua"            9900    "udp" []
  , makeService "enrp"           9901    "udp" []
  , makeService "multicast-ping" 9903    "tcp" []
  , makeService "multicast-ping" 9903    "udp" []
  , makeService "domaintime"     9909    "tcp" []
  , makeService "domaintime"     9909    "udp" []
  , makeService "sype-transport" 9911    "tcp" []
  , makeService "sype-transport" 9911    "udp" []
  , makeService "apc-9950"       9950    "tcp" []
  , makeService "apc-9950"       9950    "udp" []
  , makeService "apc-9951"       9951    "tcp" []
  , makeService "apc-9951"       9951    "udp" []
  , makeService "apc-9952"       9952    "tcp" []
  , makeService "apc-9952"       9952    "udp" []
  , makeService "acis"           9953    "tcp" []
  , makeService "acis"           9953    "udp" []
  , makeService "hinp"           9954    "tcp" []
  , makeService "alljoyn-stm"    9955    "tcp" []
  , makeService "alljoyn-mcm"    9955    "udp" []
  , makeService "alljoyn"        9956    "udp" []
  , makeService "odnsp"          9966    "tcp" []
  , makeService "odnsp"          9966    "udp" []
  , makeService "xybrid-rt"      9978    "tcp" []
  , makeService "dsm-scm-target" 9987    "tcp" []
  , makeService "dsm-scm-target" 9987    "udp" []
  , makeService "nsesrvr"        9988    "tcp" []
  , makeService "osm-appsrvr"    9990    "tcp" []
  , makeService "osm-appsrvr"    9990    "udp" []
  , makeService "osm-oev"        9991    "tcp" []
  , makeService "osm-oev"        9991    "udp" []
  , makeService "palace-1"       9992    "tcp" []
  , makeService "palace-1"       9992    "udp" []
  , makeService "palace-2"       9993    "tcp" []
  , makeService "palace-2"       9993    "udp" []
  , makeService "palace-3"       9994    "tcp" []
  , makeService "palace-3"       9994    "udp" []
  , makeService "palace-4"       9995    "tcp" []
  , makeService "palace-4"       9995    "udp" []
  , makeService "palace-5"       9996    "tcp" []
  , makeService "palace-5"       9996    "udp" []
  , makeService "palace-6"       9997    "tcp" []
  , makeService "palace-6"       9997    "udp" []
  , makeService "distinct32"     9998    "tcp" []
  , makeService "distinct32"     9998    "udp" []
  , makeService "distinct"       9999    "tcp" []
  , makeService "distinct"       9999    "udp" []
  , makeService "ndmp"           10000    "tcp" []
  , makeService "ndmp"           10000    "udp" []
  , makeService "scp-config"     10001    "tcp" []
  , makeService "scp-config"     10001    "udp" []
  , makeService "documentum"     10002    "tcp" []
  , makeService "documentum"     10002    "udp" []
  , makeService "documentum_s"   10003    "tcp" ["documentum-s"]
  , makeService "documentum_s"   10003    "udp" ["documentum-s"]
  , makeService "emcrmirccd"     10004    "tcp" []
  , makeService "emcrmird"       10005    "tcp" []
  , makeService "mvs-capacity"   10007    "tcp" []
  , makeService "mvs-capacity"   10007    "udp" []
  , makeService "octopus"        10008    "tcp" []
  , makeService "octopus"        10008    "udp" []
  , makeService "swdtp-sv"       10009    "tcp" []
  , makeService "swdtp-sv"       10009    "udp" []
  , makeService "rxapi"          10010    "tcp" []
  , makeService "zabbix-agent"   10050    "tcp" []
  , makeService "zabbix-agent"   10050    "udp" []
  , makeService "zabbix-trapper" 10051    "tcp" []
  , makeService "zabbix-trapper" 10051    "udp" []
  , makeService "qptlmd"         10055    "tcp" []
  , makeService "itap-ddtp"      10100    "tcp" []
  , makeService "itap-ddtp"      10100    "udp" []
  , makeService "ezmeeting-2"    10101    "tcp" []
  , makeService "ezmeeting-2"    10101    "udp" []
  , makeService "ezproxy-2"      10102    "tcp" []
  , makeService "ezproxy-2"      10102    "udp" []
  , makeService "ezrelay"        10103    "tcp" []
  , makeService "ezrelay"        10103    "udp" []
  , makeService "swdtp"          10104    "tcp" []
  , makeService "swdtp"          10104    "udp" []
  , makeService "bctp-server"    10107    "tcp" []
  , makeService "bctp-server"    10107    "udp" []
  , makeService "nmea-0183"      10110    "tcp" []
  , makeService "nmea-0183"      10110    "udp" []
  , makeService "nmea-onenet"    10111    "udp" []
  , makeService "netiq-endpoint" 10113    "tcp" []
  , makeService "netiq-endpoint" 10113    "udp" []
  , makeService "netiq-qcheck"   10114    "tcp" []
  , makeService "netiq-qcheck"   10114    "udp" []
  , makeService "netiq-endpt"    10115    "tcp" []
  , makeService "netiq-endpt"    10115    "udp" []
  , makeService "netiq-voipa"    10116    "tcp" []
  , makeService "netiq-voipa"    10116    "udp" []
  , makeService "iqrm"           10117    "tcp" []
  , makeService "iqrm"           10117    "udp" []
  , makeService "bmc-perf-sd"    10128    "tcp" []
  , makeService "bmc-perf-sd"    10128    "udp" []
  , makeService "bmc-gms"        10129    "tcp" []
  , makeService "qb-db-server"   10160    "tcp" []
  , makeService "qb-db-server"   10160    "udp" []
  , makeService "snmptls"        10161    "tcp" []
  , makeService "snmpdtls"       10161    "udp" []
  , makeService "snmptls-trap"   10162    "tcp" []
  , makeService "snmpdtls-trap"  10162    "udp" []
  , makeService "trisoap"        10200    "tcp" []
  , makeService "trisoap"        10200    "udp" []
  , makeService "rsms"           10201    "tcp" []
  , makeService "rscs"           10201    "udp" []
  , makeService "apollo-relay"   10252    "tcp" []
  , makeService "apollo-relay"   10252    "udp" []
  , makeService "axis-wimp-port" 10260    "tcp" []
  , makeService "axis-wimp-port" 10260    "udp" []
  , makeService "blocks"         10288    "tcp" []
  , makeService "blocks"         10288    "udp" []
  , makeService "cosir"          10321    "tcp" []
  , makeService "hip-nat-t"      10500    "udp" []
  , makeService "MOS-lower"      10540    "tcp" []
  , makeService "MOS-lower"      10540    "udp" []
  , makeService "MOS-upper"      10541    "tcp" []
  , makeService "MOS-upper"      10541    "udp" []
  , makeService "MOS-aux"        10542    "tcp" []
  , makeService "MOS-aux"        10542    "udp" []
  , makeService "MOS-soap"       10543    "tcp" []
  , makeService "MOS-soap"       10543    "udp" []
  , makeService "MOS-soap-opt"   10544    "tcp" []
  , makeService "MOS-soap-opt"   10544    "udp" []
  , makeService "printopia"      10631    "tcp" []
  , makeService "gap"            10800    "tcp" []
  , makeService "gap"            10800    "udp" []
  , makeService "lpdg"           10805    "tcp" []
  , makeService "lpdg"           10805    "udp" []
  , makeService "nbd"            10809    "tcp" []
  , makeService "nmc-disc"       10810    "udp" []
  , makeService "helix"          10860    "tcp" []
  , makeService "helix"          10860    "udp" []
  , makeService "bveapi"         10880    "tcp" []
  , makeService "bveapi"         10880    "udp" []
  , makeService "rmiaux"         10990    "tcp" []
  , makeService "rmiaux"         10990    "udp" []
  , makeService "irisa"          11000    "tcp" []
  , makeService "irisa"          11000    "udp" []
  , makeService "metasys"        11001    "tcp" []
  , makeService "metasys"        11001    "udp" []
  , makeService "origo-sync"     11103    "tcp" []
  , makeService "netapp-icmgmt"  11104    "tcp" []
  , makeService "netapp-icdata"  11105    "tcp" []
  , makeService "sgi-lk"         11106    "tcp" []
  , makeService "sgi-lk"         11106    "udp" []
  , makeService "sgi-dmfmgr"     11109    "tcp" []
  , makeService "sgi-soap"       11110    "tcp" []
  , makeService "vce"            11111    "tcp" []
  , makeService "vce"            11111    "udp" []
  , makeService "dicom"          11112    "tcp" []
  , makeService "dicom"          11112    "udp" []
  , makeService "suncacao-snmp"  11161    "tcp" []
  , makeService "suncacao-snmp"  11161    "udp" []
  , makeService "suncacao-jmxmp" 11162    "tcp" []
  , makeService "suncacao-jmxmp" 11162    "udp" []
  , makeService "suncacao-rmi"   11163    "tcp" []
  , makeService "suncacao-rmi"   11163    "udp" []
  , makeService "suncacao-csa"   11164    "tcp" []
  , makeService "suncacao-csa"   11164    "udp" []
  , makeService "suncacao-websvc"11165    "tcp" []
  , makeService "suncacao-websvc"11165    "udp" []
  , makeService "snss"           11171    "udp" []
  , makeService "oemcacao-jmxmp" 11172    "tcp" []
  , makeService "t5-straton"     11173    "tcp" []
  , makeService "oemcacao-rmi"   11174    "tcp" []
  , makeService "oemcacao-websvc"11175    "tcp" []
  , makeService "smsqp"          11201    "tcp" []
  , makeService "smsqp"          11201    "udp" []
  , makeService "dcsl-backup"    11202    "tcp" []
  , makeService "wifree"         11208    "tcp" []
  , makeService "wifree"         11208    "udp" []
  , makeService "memcache"       11211    "tcp" []
  , makeService "memcache"       11211    "udp" []
  , makeService "imip"           11319    "tcp" []
  , makeService "imip"           11319    "udp" []
  , makeService "imip-channels"  11320    "tcp" []
  , makeService "imip-channels"  11320    "udp" []
  , makeService "arena-server"   11321    "tcp" []
  , makeService "arena-server"   11321    "udp" []
  , makeService "atm-uhas"       11367    "tcp" []
  , makeService "atm-uhas"       11367    "udp" []
  , makeService "tempest-port"   11600    "tcp" []
  , makeService "tempest-port"   11600    "udp" []
  , makeService "intrepid-ssl"   11751    "tcp" []
  , makeService "intrepid-ssl"   11751    "udp" []
  , makeService "lanschool"      11796    "tcp" []
  , makeService "lanschool-mpt"  11796    "udp" []
  , makeService "xoraya"         11876    "tcp" []
  , makeService "xoraya"         11876    "udp" []
  , makeService "x2e-disc"       11877    "udp" []
  , makeService "sysinfo-sp"     11967    "tcp" []
  , makeService "sysinfo-sp"     11967    "udp" []
  , makeService "entextxid"      12000    "tcp" []
  , makeService "entextxid"      12000    "udp" []
  , makeService "entextnetwk"    12001    "tcp" []
  , makeService "entextnetwk"    12001    "udp" []
  , makeService "entexthigh"     12002    "tcp" []
  , makeService "entexthigh"     12002    "udp" []
  , makeService "entextmed"      12003    "tcp" []
  , makeService "entextmed"      12003    "udp" []
  , makeService "entextlow"      12004    "tcp" []
  , makeService "entextlow"      12004    "udp" []
  , makeService "dbisamserver1"  12005    "tcp" []
  , makeService "dbisamserver1"  12005    "udp" []
  , makeService "dbisamserver2"  12006    "tcp" []
  , makeService "dbisamserver2"  12006    "udp" []
  , makeService "accuracer"      12007    "tcp" []
  , makeService "accuracer"      12007    "udp" []
  , makeService "accuracer-dbms" 12008    "tcp" []
  , makeService "accuracer-dbms" 12008    "udp" []
  , makeService "ghvpn"          12009    "udp" []
  , makeService "edbsrvr"        12010    "tcp" []
  , makeService "vipera"         12012    "tcp" []
  , makeService "vipera"         12012    "udp" []
  , makeService "vipera-ssl"     12013    "tcp" []
  , makeService "vipera-ssl"     12013    "udp" []
  , makeService "rets-ssl"       12109    "tcp" []
  , makeService "rets-ssl"       12109    "udp" []
  , makeService "nupaper-ss"     12121    "tcp" []
  , makeService "nupaper-ss"     12121    "udp" []
  , makeService "cawas"          12168    "tcp" []
  , makeService "cawas"          12168    "udp" []
  , makeService "hivep"          12172    "tcp" []
  , makeService "hivep"          12172    "udp" []
  , makeService "linogridengine" 12300    "tcp" []
  , makeService "linogridengine" 12300    "udp" []
  , makeService "rads"           12302    "tcp" []
  , makeService "warehouse-sss"  12321    "tcp" []
  , makeService "warehouse-sss"  12321    "udp" []
  , makeService "warehouse"      12322    "tcp" []
  , makeService "warehouse"      12322    "udp" []
  , makeService "italk"          12345    "tcp" []
  , makeService "italk"          12345    "udp" []
  , makeService "tsaf"           12753    "tcp" []
  , makeService "tsaf"           12753    "udp" []
  , makeService "netperf"        12865    "tcp" []
  , makeService "i-zipqd"        13160    "tcp" []
  , makeService "i-zipqd"        13160    "udp" []
  , makeService "bcslogc"        13216    "tcp" []
  , makeService "bcslogc"        13216    "udp" []
  , makeService "rs-pias"        13217    "tcp" []
  , makeService "rs-pias"        13217    "udp" []
  , makeService "emc-vcas-tcp"   13218    "tcp" []
  , makeService "emc-vcas-udp"   13218    "udp" []
  , makeService "powwow-client"  13223    "tcp" []
  , makeService "powwow-client"  13223    "udp" []
  , makeService "powwow-server"  13224    "tcp" []
  , makeService "powwow-server"  13224    "udp" []
  , makeService "doip-data"      13400    "tcp" []
  , makeService "doip-disc"      13400    "udp" []
  , makeService "nbdb"           13785    "tcp" []
  , makeService "nbdb"           13785    "udp" []
  , makeService "nomdb"          13786    "tcp" []
  , makeService "nomdb"          13786    "udp" []
  , makeService "dsmcc-config"   13818    "tcp" []
  , makeService "dsmcc-config"   13818    "udp" []
  , makeService "dsmcc-session"  13819    "tcp" []
  , makeService "dsmcc-session"  13819    "udp" []
  , makeService "dsmcc-passthru" 13820    "tcp" []
  , makeService "dsmcc-passthru" 13820    "udp" []
  , makeService "dsmcc-download" 13821    "tcp" []
  , makeService "dsmcc-download" 13821    "udp" []
  , makeService "dsmcc-ccp"      13822    "tcp" []
  , makeService "dsmcc-ccp"      13822    "udp" []
  , makeService "bmdss"          13823    "tcp" []
  , makeService "ucontrol"       13894    "tcp" []
  , makeService "ucontrol"       13894    "udp" []
  , makeService "dta-systems"    13929    "tcp" []
  , makeService "dta-systems"    13929    "udp" []
  , makeService "medevolve"      13930    "tcp" []
  , makeService "scotty-ft"      14000    "tcp" []
  , makeService "scotty-ft"      14000    "udp" []
  , makeService "sua"            14001    "tcp" []
  , makeService "sua"            14001    "udp" []
  , makeService "scotty-disc"    14002    "udp" []
  , makeService "sage-best-com1" 14033    "tcp" []
  , makeService "sage-best-com1" 14033    "udp" []
  , makeService "sage-best-com2" 14034    "tcp" []
  , makeService "sage-best-com2" 14034    "udp" []
  , makeService "vcs-app"        14141    "tcp" []
  , makeService "vcs-app"        14141    "udp" []
  , makeService "icpp"           14142    "tcp" []
  , makeService "icpp"           14142    "udp" []
  , makeService "gcm-app"        14145    "tcp" []
  , makeService "gcm-app"        14145    "udp" []
  , makeService "vrts-tdd"       14149    "tcp" []
  , makeService "vrts-tdd"       14149    "udp" []
  , makeService "vcscmd"         14150    "tcp" []
  , makeService "vad"            14154    "tcp" []
  , makeService "vad"            14154    "udp" []
  , makeService "cps"            14250    "tcp" []
  , makeService "cps"            14250    "udp" []
  , makeService "ca-web-update"  14414    "tcp" []
  , makeService "ca-web-update"  14414    "udp" []
  , makeService "hde-lcesrvr-1"  14936    "tcp" []
  , makeService "hde-lcesrvr-1"  14936    "udp" []
  , makeService "hde-lcesrvr-2"  14937    "tcp" []
  , makeService "hde-lcesrvr-2"  14937    "udp" []
  , makeService "hydap"          15000    "tcp" []
  , makeService "hydap"          15000    "udp" []
  , makeService "v2g-secc"       15118    "udp" []
  , makeService "xpilot"         15345    "tcp" []
  , makeService "xpilot"         15345    "udp" []
  , makeService "3link"          15363    "tcp" []
  , makeService "3link"          15363    "udp" []
  , makeService "cisco-snat"     15555    "tcp" []
  , makeService "cisco-snat"     15555    "udp" []
  , makeService "bex-xr"         15660    "tcp" []
  , makeService "bex-xr"         15660    "udp" []
  , makeService "ptp"            15740    "tcp" []
  , makeService "ptp"            15740    "udp" []
  , makeService "2ping"          15998    "udp" []
  , makeService "programmar"     15999    "tcp" []
  , makeService "fmsas"          16000    "tcp" []
  , makeService "fmsascon"       16001    "tcp" []
  , makeService "gsms"           16002    "tcp" []
  , makeService "alfin"          16003    "udp" []
  , makeService "jwpc"           16020    "tcp" []
  , makeService "jwpc-bin"       16021    "tcp" []
  , makeService "sun-sea-port"   16161    "tcp" []
  , makeService "sun-sea-port"   16161    "udp" []
  , makeService "solaris-audit"  16162    "tcp" []
  , makeService "etb4j"          16309    "tcp" []
  , makeService "etb4j"          16309    "udp" []
  , makeService "pduncs"         16310    "tcp" []
  , makeService "pduncs"         16310    "udp" []
  , makeService "pdefmns"        16311    "tcp" []
  , makeService "pdefmns"        16311    "udp" []
  , makeService "netserialext1"  16360    "tcp" []
  , makeService "netserialext1"  16360    "udp" []
  , makeService "netserialext2"  16361    "tcp" []
  , makeService "netserialext2"  16361    "udp" []
  , makeService "netserialext3"  16367    "tcp" []
  , makeService "netserialext3"  16367    "udp" []
  , makeService "netserialext4"  16368    "tcp" []
  , makeService "netserialext4"  16368    "udp" []
  , makeService "connected"      16384    "tcp" []
  , makeService "connected"      16384    "udp" []
  , makeService "xoms"           16619    "tcp" []
  , makeService "vtp"            16666    "udp" []
  , makeService "newbay-snc-mc"  16900    "tcp" []
  , makeService "newbay-snc-mc"  16900    "udp" []
  , makeService "sgcip"          16950    "tcp" []
  , makeService "sgcip"          16950    "udp" []
  , makeService "intel-rci-mp"   16991    "tcp" []
  , makeService "intel-rci-mp"   16991    "udp" []
  , makeService "amt-soap-http"  16992    "tcp" []
  , makeService "amt-soap-http"  16992    "udp" []
  , makeService "amt-soap-https" 16993    "tcp" []
  , makeService "amt-soap-https" 16993    "udp" []
  , makeService "amt-redir-tcp"  16994    "tcp" []
  , makeService "amt-redir-tcp"  16994    "udp" []
  , makeService "amt-redir-tls"  16995    "tcp" []
  , makeService "amt-redir-tls"  16995    "udp" []
  , makeService "isode-dua"      17007    "tcp" []
  , makeService "isode-dua"      17007    "udp" []
  , makeService "soundsvirtual"  17185    "tcp" []
  , makeService "soundsvirtual"  17185    "udp" []
  , makeService "chipper"        17219    "tcp" []
  , makeService "chipper"        17219    "udp" []
  , makeService "avdecc"         17221    "tcp" []
  , makeService "avdecc"         17221    "udp" []
  , makeService "cpsp"           17222    "udp" []
  , makeService "integrius-stp"  17234    "tcp" []
  , makeService "integrius-stp"  17234    "udp" []
  , makeService "ssh-mgmt"       17235    "tcp" []
  , makeService "ssh-mgmt"       17235    "udp" []
  , makeService "db-lsp"         17500    "tcp" []
  , makeService "db-lsp-disc"    17500    "udp" []
  , makeService "ea"             17729    "tcp" []
  , makeService "ea"             17729    "udp" []
  , makeService "zep"            17754    "tcp" []
  , makeService "zep"            17754    "udp" []
  , makeService "zigbee-ip"      17755    "tcp" []
  , makeService "zigbee-ip"      17755    "udp" []
  , makeService "zigbee-ips"     17756    "tcp" []
  , makeService "zigbee-ips"     17756    "udp" []
  , makeService "sw-orion"       17777    "tcp" []
  , makeService "biimenu"        18000    "tcp" []
  , makeService "biimenu"        18000    "udp" []
  , makeService "radpdf"         18104    "tcp" []
  , makeService "racf"           18136    "tcp" []
  , makeService "opsec-cvp"      18181    "tcp" []
  , makeService "opsec-cvp"      18181    "udp" []
  , makeService "opsec-ufp"      18182    "tcp" []
  , makeService "opsec-ufp"      18182    "udp" []
  , makeService "opsec-sam"      18183    "tcp" []
  , makeService "opsec-sam"      18183    "udp" []
  , makeService "opsec-lea"      18184    "tcp" []
  , makeService "opsec-lea"      18184    "udp" []
  , makeService "opsec-omi"      18185    "tcp" []
  , makeService "opsec-omi"      18185    "udp" []
  , makeService "ohsc"           18186    "tcp" []
  , makeService "ohsc"           18186    "udp" []
  , makeService "opsec-ela"      18187    "tcp" []
  , makeService "opsec-ela"      18187    "udp" []
  , makeService "checkpoint-rtm" 18241    "tcp" []
  , makeService "checkpoint-rtm" 18241    "udp" []
  , makeService "iclid"          18242    "tcp" []
  , makeService "clusterxl"      18243    "tcp" []
  , makeService "gv-pf"          18262    "tcp" []
  , makeService "gv-pf"          18262    "udp" []
  , makeService "ac-cluster"     18463    "tcp" []
  , makeService "ac-cluster"     18463    "udp" []
  , makeService "rds-ib"         18634    "tcp" []
  , makeService "rds-ib"         18634    "udp" []
  , makeService "rds-ip"         18635    "tcp" []
  , makeService "rds-ip"         18635    "udp" []
  , makeService "ique"           18769    "tcp" []
  , makeService "ique"           18769    "udp" []
  , makeService "infotos"        18881    "tcp" []
  , makeService "infotos"        18881    "udp" []
  , makeService "apc-necmp"      18888    "tcp" []
  , makeService "apc-necmp"      18888    "udp" []
  , makeService "igrid"          19000    "tcp" []
  , makeService "igrid"          19000    "udp" []
  , makeService "j-link"         19020    "tcp" []
  , makeService "opsec-uaa"      19191    "tcp" []
  , makeService "opsec-uaa"      19191    "udp" []
  , makeService "ua-secureagent" 19194    "tcp" []
  , makeService "ua-secureagent" 19194    "udp" []
  , makeService "keysrvr"        19283    "tcp" []
  , makeService "keysrvr"        19283    "udp" []
  , makeService "keyshadow"      19315    "tcp" []
  , makeService "keyshadow"      19315    "udp" []
  , makeService "mtrgtrans"      19398    "tcp" []
  , makeService "mtrgtrans"      19398    "udp" []
  , makeService "hp-sco"         19410    "tcp" []
  , makeService "hp-sco"         19410    "udp" []
  , makeService "hp-sca"         19411    "tcp" []
  , makeService "hp-sca"         19411    "udp" []
  , makeService "hp-sessmon"     19412    "tcp" []
  , makeService "hp-sessmon"     19412    "udp" []
  , makeService "fxuptp"         19539    "tcp" []
  , makeService "fxuptp"         19539    "udp" []
  , makeService "sxuptp"         19540    "tcp" []
  , makeService "sxuptp"         19540    "udp" []
  , makeService "jcp"            19541    "tcp" []
  , makeService "jcp"            19541    "udp" []
  , makeService "mle"            19788    "udp" []
  , makeService "iec-104-sec"    19998    "tcp" []
  , makeService "dnp-sec"        19999    "tcp" []
  , makeService "dnp-sec"        19999    "udp" []
  , makeService "dnp"            20000    "tcp" []
  , makeService "dnp"            20000    "udp" []
  , makeService "microsan"       20001    "tcp" []
  , makeService "microsan"       20001    "udp" []
  , makeService "commtact-http"  20002    "tcp" []
  , makeService "commtact-http"  20002    "udp" []
  , makeService "commtact-https" 20003    "tcp" []
  , makeService "commtact-https" 20003    "udp" []
  , makeService "openwebnet"     20005    "tcp" []
  , makeService "openwebnet"     20005    "udp" []
  , makeService "ss-idi-disc"    20012    "udp" []
  , makeService "ss-idi"         20013    "tcp" []
  , makeService "opendeploy"     20014    "tcp" []
  , makeService "opendeploy"     20014    "udp" []
  , makeService "nburn_id"       20034    "tcp" ["nburn-id"]
  , makeService "nburn_id"       20034    "udp" ["nburn-id"]
  , makeService "tmophl7mts"     20046    "tcp" []
  , makeService "tmophl7mts"     20046    "udp" []
  , makeService "mountd"         20048    "tcp" []
  , makeService "mountd"         20048    "udp" []
  , makeService "nfsrdma"        20049    "tcp" []
  , makeService "nfsrdma"        20049    "udp" []
  , makeService "tolfab"         20167    "tcp" []
  , makeService "tolfab"         20167    "udp" []
  , makeService "ipdtp-port"     20202    "tcp" []
  , makeService "ipdtp-port"     20202    "udp" []
  , makeService "ipulse-ics"     20222    "tcp" []
  , makeService "ipulse-ics"     20222    "udp" []
  , makeService "emwavemsg"      20480    "tcp" []
  , makeService "emwavemsg"      20480    "udp" []
  , makeService "track"          20670    "tcp" []
  , makeService "track"          20670    "udp" []
  , makeService "athand-mmp"     20999    "tcp" []
  , makeService "athand-mmp"     20999    "udp" []
  , makeService "irtrans"        21000    "tcp" []
  , makeService "irtrans"        21000    "udp" []
  , makeService "rdm-tfs"        21553    "tcp" []
  , makeService "dfserver"       21554    "tcp" []
  , makeService "dfserver"       21554    "udp" []
  , makeService "vofr-gateway"   21590    "tcp" []
  , makeService "vofr-gateway"   21590    "udp" []
  , makeService "tvpm"           21800    "tcp" []
  , makeService "tvpm"           21800    "udp" []
  , makeService "webphone"       21845    "tcp" []
  , makeService "webphone"       21845    "udp" []
  , makeService "netspeak-is"    21846    "tcp" []
  , makeService "netspeak-is"    21846    "udp" []
  , makeService "netspeak-cs"    21847    "tcp" []
  , makeService "netspeak-cs"    21847    "udp" []
  , makeService "netspeak-acd"   21848    "tcp" []
  , makeService "netspeak-acd"   21848    "udp" []
  , makeService "netspeak-cps"   21849    "tcp" []
  , makeService "netspeak-cps"   21849    "udp" []
  , makeService "snapenetio"     22000    "tcp" []
  , makeService "snapenetio"     22000    "udp" []
  , makeService "optocontrol"    22001    "tcp" []
  , makeService "optocontrol"    22001    "udp" []
  , makeService "optohost002"    22002    "tcp" []
  , makeService "optohost002"    22002    "udp" []
  , makeService "optohost003"    22003    "tcp" []
  , makeService "optohost003"    22003    "udp" []
  , makeService "optohost004"    22004    "tcp" []
  , makeService "optohost004"    22004    "udp" []
  , makeService "optohost005"    22005    "tcp" []
  , makeService "optohost005"    22005    "udp" []
  , makeService "dcap"           22125    "tcp" []
  , makeService "gsidcap"        22128    "tcp" []
  , makeService "cis"            22305    "udp" []
  , makeService "cis-secure"     22343    "tcp" []
  , makeService "cis-secure"     22343    "udp" []
  , makeService "WibuKey"        22347    "tcp" []
  , makeService "WibuKey"        22347    "udp" []
  , makeService "CodeMeter"      22350    "tcp" []
  , makeService "CodeMeter"      22350    "udp" []
  , makeService "caldsoft-backup"22537    "tcp" []
  , makeService "vocaltec-wconf" 22555    "tcp" []
  , makeService "vocaltec-phone" 22555    "udp" []
  , makeService "talikaserver"   22763    "tcp" []
  , makeService "talikaserver"   22763    "udp" []
  , makeService "aws-brf"        22800    "tcp" []
  , makeService "aws-brf"        22800    "udp" []
  , makeService "brf-gw"         22951    "tcp" []
  , makeService "brf-gw"         22951    "udp" []
  , makeService "inovaport1"     23000    "tcp" []
  , makeService "inovaport1"     23000    "udp" []
  , makeService "inovaport2"     23001    "tcp" []
  , makeService "inovaport2"     23001    "udp" []
  , makeService "inovaport3"     23002    "tcp" []
  , makeService "inovaport3"     23002    "udp" []
  , makeService "inovaport4"     23003    "tcp" []
  , makeService "inovaport4"     23003    "udp" []
  , makeService "inovaport5"     23004    "tcp" []
  , makeService "inovaport5"     23004    "udp" []
  , makeService "inovaport6"     23005    "tcp" []
  , makeService "inovaport6"     23005    "udp" []
  , makeService "gntp"           23053    "tcp" []
  , makeService "s102"           23272    "udp" []
  , makeService "elxmgmt"        23333    "tcp" []
  , makeService "elxmgmt"        23333    "udp" []
  , makeService "novar-dbase"    23400    "tcp" []
  , makeService "novar-dbase"    23400    "udp" []
  , makeService "novar-alarm"    23401    "tcp" []
  , makeService "novar-alarm"    23401    "udp" []
  , makeService "novar-global"   23402    "tcp" []
  , makeService "novar-global"   23402    "udp" []
  , makeService "aequus"         23456    "tcp" []
  , makeService "aequus-alt"     23457    "tcp" []
  , makeService "areaguard-neo"  23546    "tcp" []
  , makeService "med-ltp"        24000    "tcp" []
  , makeService "med-ltp"        24000    "udp" []
  , makeService "med-fsp-rx"     24001    "tcp" []
  , makeService "med-fsp-rx"     24001    "udp" []
  , makeService "med-fsp-tx"     24002    "tcp" []
  , makeService "med-fsp-tx"     24002    "udp" []
  , makeService "med-supp"       24003    "tcp" []
  , makeService "med-supp"       24003    "udp" []
  , makeService "med-ovw"        24004    "tcp" []
  , makeService "med-ovw"        24004    "udp" []
  , makeService "med-ci"         24005    "tcp" []
  , makeService "med-ci"         24005    "udp" []
  , makeService "med-net-svc"    24006    "tcp" []
  , makeService "med-net-svc"    24006    "udp" []
  , makeService "filesphere"     24242    "tcp" []
  , makeService "filesphere"     24242    "udp" []
  , makeService "vista-4gl"      24249    "tcp" []
  , makeService "vista-4gl"      24249    "udp" []
  , makeService "ild"            24321    "tcp" []
  , makeService "ild"            24321    "udp" []
  , makeService "hid"            24322    "udp" []
  , makeService "intel_rci"      24386    "tcp" ["intel-rci"]
  , makeService "intel_rci"      24386    "udp" ["intel-rci"]
  , makeService "tonidods"       24465    "tcp" []
  , makeService "tonidods"       24465    "udp" []
  , makeService "flashfiler"     24677    "tcp" []
  , makeService "flashfiler"     24677    "udp" []
  , makeService "proactivate"    24678    "tcp" []
  , makeService "proactivate"    24678    "udp" []
  , makeService "tcc-http"       24680    "tcp" []
  , makeService "tcc-http"       24680    "udp" []
  , makeService "cslg"           24754    "tcp" []
  , makeService "assoc-disc"     24850    "udp" []
  , makeService "find"           24922    "tcp" []
  , makeService "find"           24922    "udp" []
  , makeService "icl-twobase1"   25000    "tcp" []
  , makeService "icl-twobase1"   25000    "udp" []
  , makeService "icl-twobase2"   25001    "tcp" []
  , makeService "icl-twobase2"   25001    "udp" []
  , makeService "icl-twobase3"   25002    "tcp" []
  , makeService "icl-twobase3"   25002    "udp" []
  , makeService "icl-twobase4"   25003    "tcp" []
  , makeService "icl-twobase4"   25003    "udp" []
  , makeService "icl-twobase5"   25004    "tcp" []
  , makeService "icl-twobase5"   25004    "udp" []
  , makeService "icl-twobase6"   25005    "tcp" []
  , makeService "icl-twobase6"   25005    "udp" []
  , makeService "icl-twobase7"   25006    "tcp" []
  , makeService "icl-twobase7"   25006    "udp" []
  , makeService "icl-twobase8"   25007    "tcp" []
  , makeService "icl-twobase8"   25007    "udp" []
  , makeService "icl-twobase9"   25008    "tcp" []
  , makeService "icl-twobase9"   25008    "udp" []
  , makeService "icl-twobase10"  25009    "tcp" []
  , makeService "icl-twobase10"  25009    "udp" []
  , makeService "sauterdongle"   25576    "tcp" []
  , makeService "idtp"           25604    "tcp" []
  , makeService "vocaltec-hos"   25793    "tcp" []
  , makeService "vocaltec-hos"   25793    "udp" []
  , makeService "tasp-net"       25900    "tcp" []
  , makeService "tasp-net"       25900    "udp" []
  , makeService "niobserver"     25901    "tcp" []
  , makeService "niobserver"     25901    "udp" []
  , makeService "nilinkanalyst"  25902    "tcp" []
  , makeService "nilinkanalyst"  25902    "udp" []
  , makeService "niprobe"        25903    "tcp" []
  , makeService "niprobe"        25903    "udp" []
  , makeService "bf-game"        25954    "udp" []
  , makeService "bf-master"      25955    "udp" []
  , makeService "scscp"          26133    "tcp" []
  , makeService "scscp"          26133    "udp" []
  , makeService "ezproxy"        26260    "tcp" []
  , makeService "ezproxy"        26260    "udp" []
  , makeService "ezmeeting"      26261    "tcp" []
  , makeService "ezmeeting"      26261    "udp" []
  , makeService "k3software-svr" 26262    "tcp" []
  , makeService "k3software-svr" 26262    "udp" []
  , makeService "k3software-cli" 26263    "tcp" []
  , makeService "k3software-cli" 26263    "udp" []
  , makeService "exoline-tcp"    26486    "tcp" []
  , makeService "exoline-udp"    26486    "udp" []
  , makeService "exoconfig"      26487    "tcp" []
  , makeService "exoconfig"      26487    "udp" []
  , makeService "exonet"         26489    "tcp" []
  , makeService "exonet"         26489    "udp" []
  , makeService "imagepump"      27345    "tcp" []
  , makeService "imagepump"      27345    "udp" []
  , makeService "jesmsjc"        27442    "tcp" []
  , makeService "jesmsjc"        27442    "udp" []
  , makeService "kopek-httphead" 27504    "tcp" []
  , makeService "kopek-httphead" 27504    "udp" []
  , makeService "ars-vista"      27782    "tcp" []
  , makeService "ars-vista"      27782    "udp" []
  , makeService "astrolink"      27876    "tcp" []
  , makeService "tw-auth-key"    27999    "tcp" []
  , makeService "tw-auth-key"    27999    "udp" []
  , makeService "nxlmd"          28000    "tcp" []
  , makeService "nxlmd"          28000    "udp" []
  , makeService "pqsp"           28001    "tcp" []
  , makeService "voxelstorm"     28200    "tcp" []
  , makeService "voxelstorm"     28200    "udp" []
  , makeService "siemensgsm"     28240    "tcp" []
  , makeService "siemensgsm"     28240    "udp" []
  , makeService "a27-ran-ran"    28119    "udp" []
  , makeService "otmp"           29167    "tcp" []
  , makeService "otmp"           29167    "udp" []
  , makeService "bingbang"       29999    "tcp" []
  , makeService "ndmps"          30000    "tcp" []
  , makeService "pago-services1" 30001    "tcp" []
  , makeService "pago-services1" 30001    "udp" []
  , makeService "pago-services2" 30002    "tcp" []
  , makeService "pago-services2" 30002    "udp" []
  , makeService "kingdomsonline" 30260    "tcp" []
  , makeService "kingdomsonline" 30260    "udp" []
  , makeService "ovobs"          30999    "tcp" []
  , makeService "ovobs"          30999    "udp" []
  , makeService "autotrac-acp"   31020    "tcp" []
  , makeService "yawn"           31029    "udp" []
  , makeService "xqosd"          31416    "tcp" []
  , makeService "xqosd"          31416    "udp" []
  , makeService "tetrinet"       31457    "tcp" []
  , makeService "tetrinet"       31457    "udp" []
  , makeService "lm-mon"         31620    "tcp" []
  , makeService "lm-mon"         31620    "udp" []
  , makeService "dsx_monitor"    31685    "tcp" ["dsx-monitor"]
  , makeService "gamesmith-port" 31765    "tcp" []
  , makeService "gamesmith-port" 31765    "udp" []
  , makeService "iceedcp_tx"     31948    "tcp" ["iceedcp-tx"]
  , makeService "iceedcp_tx"     31948    "udp" ["iceedcp-tx"]
  , makeService "iceedcp_rx"     31949    "tcp" ["iceedcp-rx"]
  , makeService "iceedcp_rx"     31949    "udp" ["iceedcp-rx"]
  , makeService "iracinghelper"  32034    "tcp" []
  , makeService "iracinghelper"  32034    "udp" []
  , makeService "t1distproc60"   32249    "tcp" []
  , makeService "t1distproc60"   32249    "udp" []
  , makeService "apm-link"       32483    "tcp" []
  , makeService "apm-link"       32483    "udp" []
  , makeService "sec-ntb-clnt"   32635    "tcp" []
  , makeService "sec-ntb-clnt"   32635    "udp" []
  , makeService "DMExpress"      32636    "tcp" []
  , makeService "DMExpress"      32636    "udp" []
  , makeService "filenet-powsrm" 32767    "tcp" []
  , makeService "filenet-powsrm" 32767    "udp" []
  , makeService "filenet-tms"    32768    "tcp" []
  , makeService "filenet-tms"    32768    "udp" []
  , makeService "filenet-rpc"    32769    "tcp" []
  , makeService "filenet-rpc"    32769    "udp" []
  , makeService "filenet-nch"    32770    "tcp" []
  , makeService "filenet-nch"    32770    "udp" []
  , makeService "filenet-rmi"    32771    "tcp" []
  , makeService "filenet-rmi"    32771    "udp" []
  , makeService "filenet-pa"     32772    "tcp" []
  , makeService "filenet-pa"     32772    "udp" []
  , makeService "filenet-cm"     32773    "tcp" []
  , makeService "filenet-cm"     32773    "udp" []
  , makeService "filenet-re"     32774    "tcp" []
  , makeService "filenet-re"     32774    "udp" []
  , makeService "filenet-pch"    32775    "tcp" []
  , makeService "filenet-pch"    32775    "udp" []
  , makeService "filenet-peior"  32776    "tcp" []
  , makeService "filenet-peior"  32776    "udp" []
  , makeService "filenet-obrok"  32777    "tcp" []
  , makeService "filenet-obrok"  32777    "udp" []
  , makeService "mlsn"           32801    "tcp" []
  , makeService "mlsn"           32801    "udp" []
  , makeService "retp"           32811    "tcp" []
  , makeService "idmgratm"       32896    "tcp" []
  , makeService "idmgratm"       32896    "udp" []
  , makeService "aurora-balaena" 33123    "tcp" []
  , makeService "aurora-balaena" 33123    "udp" []
  , makeService "diamondport"    33331    "tcp" []
  , makeService "diamondport"    33331    "udp" []
  , makeService "dgi-serv"       33333    "tcp" []
  , makeService "speedtrace"     33334    "tcp" []
  , makeService "speedtrace-disc"33334    "udp" []
  , makeService "snip-slave"     33656    "tcp" []
  , makeService "snip-slave"     33656    "udp" []
  , makeService "turbonote-2"    34249    "tcp" []
  , makeService "turbonote-2"    34249    "udp" []
  , makeService "p-net-local"    34378    "tcp" []
  , makeService "p-net-local"    34378    "udp" []
  , makeService "p-net-remote"   34379    "tcp" []
  , makeService "p-net-remote"   34379    "udp" []
  , makeService "dhanalakshmi"   34567    "tcp" []
  , makeService "profinet-rt"    34962    "tcp" []
  , makeService "profinet-rt"    34962    "udp" []
  , makeService "profinet-rtm"   34963    "tcp" []
  , makeService "profinet-rtm"   34963    "udp" []
  , makeService "profinet-cm"    34964    "tcp" []
  , makeService "profinet-cm"    34964    "udp" []
  , makeService "ethercat"       34980    "tcp" []
  , makeService "ethercat"       34980    "udp" []
  , makeService "heathview"      35000    "tcp" []
  , makeService "kitim"          35354    "tcp" []
  , makeService "altova-lm"      35355    "tcp" []
  , makeService "altova-lm-disc" 35355    "udp" []
  , makeService "guttersnex"     35356    "tcp" []
  , makeService "openstack-id"   35357    "tcp" []
  , makeService "allpeers"       36001    "tcp" []
  , makeService "allpeers"       36001    "udp" []
  , makeService "febooti-aw"     36524    "tcp" []
  , makeService "kastenxpipe"    36865    "tcp" []
  , makeService "kastenxpipe"    36865    "udp" []
  , makeService "neckar"         37475    "tcp" []
  , makeService "neckar"         37475    "udp" []
  , makeService "unisys-eportal" 37654    "tcp" []
  , makeService "unisys-eportal" 37654    "udp" []
  , makeService "gdrive-sync"    37483    "tcp" []
  , makeService "galaxy7-data"   38201    "tcp" []
  , makeService "galaxy7-data"   38201    "udp" []
  , makeService "fairview"       38202    "tcp" []
  , makeService "fairview"       38202    "udp" []
  , makeService "agpolicy"       38203    "tcp" []
  , makeService "agpolicy"       38203    "udp" []
  , makeService "sruth"          38800    "tcp" []
  , makeService "secrmmsafecopya"38865    "tcp" []
  , makeService "turbonote-1"    39681    "tcp" []
  , makeService "turbonote-1"    39681    "udp" []
  , makeService "safetynetp"     40000    "tcp" []
  , makeService "safetynetp"     40000    "udp" []
  , makeService "cscp"           40841    "tcp" []
  , makeService "cscp"           40841    "udp" []
  , makeService "csccredir"      40842    "tcp" []
  , makeService "csccredir"      40842    "udp" []
  , makeService "csccfirewall"   40843    "tcp" []
  , makeService "csccfirewall"   40843    "udp" []
  , makeService "ortec-disc"     40853    "udp" []
  , makeService "fs-qos"         41111    "tcp" []
  , makeService "fs-qos"         41111    "udp" []
  , makeService "tentacle"       41121    "tcp" []
  , makeService "crestron-cip"   41794    "tcp" []
  , makeService "crestron-cip"   41794    "udp" []
  , makeService "crestron-ctp"   41795    "tcp" []
  , makeService "crestron-ctp"   41795    "udp" []
  , makeService "crestron-cips"  41796    "tcp" []
  , makeService "crestron-ctps"  41797    "tcp" []
  , makeService "candp"          42508    "tcp" []
  , makeService "candp"          42508    "udp" []
  , makeService "candrp"         42509    "tcp" []
  , makeService "candrp"         42509    "udp" []
  , makeService "caerpc"         42510    "tcp" []
  , makeService "caerpc"         42510    "udp" []
  , makeService "recvr-rc"       43000    "tcp" []
  , makeService "recvr-rc-disc"  43000    "udp" []
  , makeService "reachout"       43188    "tcp" []
  , makeService "reachout"       43188    "udp" []
  , makeService "ndm-agent-port" 43189    "tcp" []
  , makeService "ndm-agent-port" 43189    "udp" []
  , makeService "ip-provision"   43190    "tcp" []
  , makeService "ip-provision"   43190    "udp" []
  , makeService "noit-transport" 43191    "tcp" []
  , makeService "shaperai"       43210    "tcp" []
  , makeService "shaperai-disc"  43210    "udp" []
  , makeService "eq3-update"     43439    "tcp" []
  , makeService "eq3-config"     43439    "udp" []
  , makeService "ew-mgmt"        43440    "tcp" []
  , makeService "ew-disc-cmd"    43440    "udp" []
  , makeService "ciscocsdb"      43441    "tcp" []
  , makeService "ciscocsdb"      43441    "udp" []
  , makeService "z-wave-s"       44123    "tcp" []
  , makeService "pmcd"           44321    "tcp" []
  , makeService "pmcd"           44321    "udp" []
  , makeService "pmcdproxy"      44322    "tcp" []
  , makeService "pmcdproxy"      44322    "udp" []
  , makeService "cognex-dataman" 44444    "tcp" []
  , makeService "domiq"          44544    "udp" []
  , makeService "rbr-debug"      44553    "tcp" []
  , makeService "rbr-debug"      44553    "udp" []
  , makeService "EtherNet/IP-2"  44818    "tcp" ["EtherNet-IP-2"]
  , makeService "EtherNet/IP-2"  44818    "udp" ["EtherNet-IP-2"]
  , makeService "m3da"           44900    "tcp" []
  , makeService "m3da-disc"      44900    "udp" []
  , makeService "asmp"           45000    "tcp" []
  , makeService "asmp-mon"       45000    "udp" []
  , makeService "asmps"          45001    "tcp" []
  , makeService "synctest"       45045    "tcp" []
  , makeService "invision-ag"    45054    "tcp" []
  , makeService "invision-ag"    45054    "udp" []
  , makeService "eba"            45678    "tcp" []
  , makeService "eba"            45678    "udp" []
  , makeService "dai-shell"      45824    "tcp" []
  , makeService "qdb2service"    45825    "tcp" []
  , makeService "qdb2service"    45825    "udp" []
  , makeService "ssr-servermgr"  45966    "tcp" []
  , makeService "ssr-servermgr"  45966    "udp" []
  , makeService "sp-remotetablet"46998    "tcp" []
  , makeService "mediabox"       46999    "tcp" []
  , makeService "mediabox"       46999    "udp" []
  , makeService "mbus"           47000    "tcp" []
  , makeService "mbus"           47000    "udp" []
  , makeService "winrm"          47001    "tcp" []
  , makeService "jvl-mactalk"    47100    "udp" []
  , makeService "dbbrowse"       47557    "tcp" []
  , makeService "dbbrowse"       47557    "udp" []
  , makeService "directplaysrvr" 47624    "tcp" []
  , makeService "directplaysrvr" 47624    "udp" []
  , makeService "ap"             47806    "tcp" []
  , makeService "ap"             47806    "udp" []
  , makeService "bacnet"         47808    "tcp" []
  , makeService "bacnet"         47808    "udp" []
  , makeService "nimcontroller"  48000    "tcp" []
  , makeService "nimcontroller"  48000    "udp" []
  , makeService "nimspooler"     48001    "tcp" []
  , makeService "nimspooler"     48001    "udp" []
  , makeService "nimhub"         48002    "tcp" []
  , makeService "nimhub"         48002    "udp" []
  , makeService "nimgtw"         48003    "tcp" []
  , makeService "nimgtw"         48003    "udp" []
  , makeService "nimbusdb"       48004    "tcp" []
  , makeService "nimbusdbctrl"   48005    "tcp" []
  , makeService "3gpp-cbsp"      48049    "tcp" []
  , makeService "isnetserv"      48128    "tcp" []
  , makeService "isnetserv"      48128    "udp" []
  , makeService "blp5"           48129    "tcp" []
  , makeService "blp5"           48129    "udp" []
  , makeService "com-bardac-dw"  48556    "tcp" []
  , makeService "com-bardac-dw"  48556    "udp" []
  , makeService "iqobject"       48619    "tcp" []
  , makeService "iqobject"       48619    "udp" []
  , makeService "matahari"       49000    "tcp" []
  ]
